/*
 * @(#)guide.h	1.68 90/07/09 Copyright 1989 Sun Microsystems.
 *
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef guide_guide_DEFINED
#define guide_guide_DEFINED

#include	<c_varieties.h>

/*
 * Miscellaneous definitions.
 */
#define OK	0
#define	ERROR	-1
#define	TRUE	1
#define	FALSE	0

/*
 * String appended to an application name to form the GIL file.
 */
#define	G_GIL_SUFFIX_STRING	".G"

/*
 * GIL version string.  The version number will be incremented when
 * the format of GIL files is changed.
 */
#define	G_VERSION		2
#define	G_VERSION_PREFIX	";GIL-"

/*
 * Public enumerated types.
 */

/*
 * GUIDE object types.
 */
typedef enum {
	G_BASE_WINDOW,
	G_BUTTON,
	G_CANVAS_PANE,
	G_CONTROL_AREA,
	G_GAUGE,
	G_MENU,
	G_MESSAGE,
	G_POPUP_WINDOW,
	G_SCROLLING_LIST,
	G_SETTING,
	G_SLIDER,
	G_TERM_PANE,
	G_TEXT_FIELD,
	G_TEXT_PANE,
}               G_TYPES;

/*
 * Button types.
 */
typedef enum {
	G_NORMAL,
	G_ABBREVIATED,
}		G_BUTTON_TYPES;

/*
 * Drawing models.
 */
typedef enum {
	G_XVIEW,
	G_XWINDOWS,
	G_POSTSCRIPT,
}		G_DRAWING_MODELS;

/*
 * Label types.
 */
typedef enum {
	G_STRING,
	G_GLYPH,
}               G_LABEL_TYPES;

/*
 * Layout types for the G_LAYOUT attribute.
 */
typedef enum {
	G_HORIZONTAL,
	G_VERTICAL,
}               G_LAYOUT_TYPES;

/*
 * Menu types.
 */
typedef enum {
	G_COMMAND_MENU,
	G_EXCLUSIVE_MENU,
	G_NONEXCLUSIVE_MENU,
}		G_MENU_TYPES;

/*
 * Setting types.
 */
typedef enum {
	G_EXCLUSIVE,
	G_NONEXCLUSIVE,
	G_CHECK,
	G_STACK,
}		G_SETTING_TYPES;

/*
 * Event types.
 */
typedef enum {
	G_KEYBOARD,
	G_KEYBOARD_LEFT,
	G_KEYBOARD_RIGHT,
	G_KEYBOARD_TOP,
	G_MOUSE,
	G_MOUSE_DRAG,
	G_MOUSE_ENTER,
	G_MOUSE_EXIT,
	G_MOUSE_MOVE,
}		G_EVENT_TYPES;

/*
 * Text types.
 */
typedef enum {
	G_ALPHANUMERIC,
	G_NUMERIC,
}		G_TEXT_TYPES;

/*
 * GUIDE object attributes.
 */
typedef enum {
	G_BACKGROUND_COLOR,
	G_BUTTON_TYPE,
	G_CHOICES,
	G_CHOICE_LABEL_TYPES,
	G_CHOICE_COLORS,
	G_COLUMNS,
	G_CONSTANT_WIDTH,
	G_DONE_HANDLER,
	G_DRAWING_MODEL,
	G_EVENTS,
	G_EVENT_HANDLER,
	G_FOREGROUND_COLOR,
	G_HEIGHT,
	G_HELP,
	G_HSCROLL,
	G_ICON,
	G_ICON_MASK,
	G_LABEL,
	G_LABEL_TYPE,
	G_LABEL_BOLD,
	G_LAYOUT_TYPE,
	G_MAPPED,
	G_MAX_VALUE,
	G_MENU_HANDLER,
	G_MENU_ITEM_COLORS,
	G_MENU_ITEM_DEFAULTS,
	G_MENU_ITEM_HANDLERS,
	G_MENU_ITEM_LABELS,
	G_MENU_ITEM_LABEL_TYPES,
	G_MENU_ITEM_MENUS,
	G_MENU_NAME,
	G_MENU_TITLE,
	G_MENU_TYPE,
	G_MIN_VALUE,
	G_MULTIPLE_SELECTIONS,
	G_NAME,
	G_NOTIFY_HANDLER,
	G_ORIENTATION,
	G_OWNER,
	G_PINNABLE,
	G_PINNED,
	G_READ_ONLY,
	G_REPAINT_PROC,
	G_RESIZABLE,
	G_ROWS,
	G_SCROLLABLE_HEIGHT,
	G_SCROLLABLE_WIDTH,
	G_SELECTION_REQUIRED,
	G_SETTING_TYPE,
	G_SHOW_BORDER,
	G_SHOW_ENDBOXES,
	G_SHOW_FOOTER,
	G_SHOW_RANGE,
	G_SHOW_VALUE,
	G_SLIDER_WIDTH,
	G_STORED_LENGTH,
	G_TEXT_TYPE,
	G_TICKS,
	G_TYPE,
	G_USER_DATA,
	G_VALUE_LENGTH,
	G_VALUE_X,
	G_VALUE_Y,
	G_VSCROLL,
	G_WIDTH,
	G_X,
	G_Y,
}               G_ATTRS;

/*
 * Public function declarations.
 */
EXTERN_FUNCTION( G_ATTRS	*G_object_attrs,	(G_TYPES) );
EXTERN_FUNCTION( char		*G_attr_to_string,	(G_ATTRS) );
EXTERN_FUNCTION( G_ATTRS	G_string_to_attr,	(char *) );

EXTERN_FUNCTION( char		*G_drawing_model_to_string, (G_DRAWING_MODELS) );
EXTERN_FUNCTION( G_DRAWING_MODELS	G_string_to_drawing_model, (char *) );

EXTERN_FUNCTION( char		*G_event_type_to_string, (G_EVENT_TYPES) );
EXTERN_FUNCTION( G_EVENT_TYPES	G_string_to_event_type, (char *) );

EXTERN_FUNCTION( char		*G_label_type_to_string, (G_LABEL_TYPES) );
EXTERN_FUNCTION( G_LABEL_TYPES	G_string_to_label_type, (char *) );

EXTERN_FUNCTION( char		*G_layout_value_to_string, (G_LAYOUT_TYPES) );
EXTERN_FUNCTION( G_LAYOUT_TYPES	G_string_to_layout_type, (char *) );

EXTERN_FUNCTION( char		*G_button_type_to_string, (G_BUTTON_TYPES) );
EXTERN_FUNCTION( G_BUTTON_TYPES	G_string_to_button_type, (char *) );

EXTERN_FUNCTION( char		*G_menu_type_to_string, (G_MENU_TYPES) );
EXTERN_FUNCTION( G_MENU_TYPES	G_string_to_menu_type, (char *) );

EXTERN_FUNCTION( char		*G_setting_type_to_string, (G_SETTING_TYPES) );
EXTERN_FUNCTION( G_SETTING_TYPES	G_string_to_setting_type, (char *) );

EXTERN_FUNCTION( char		*G_text_type_to_string, (G_TEXT_TYPES) );
EXTERN_FUNCTION( G_TEXT_TYPES	G_string_to_text_type, (char *) );

EXTERN_FUNCTION( char		*G_type_to_string, (G_TYPES) );
EXTERN_FUNCTION( G_TYPES	G_string_to_type, (char *) );

#endif ~guide_guide_DEFINED
