
/*                                                                   
 *     Blockette manipulation structures                             
 */                                                                  
                                                                     
#define SEEDVER 21     /* Version 2.1 of the SEED format */          
                                                                     
#define CKP(a) if (!(a)) return(FALSE) /* Procedure to unwind on error */      
#define FOE(fs)  if ( (fs) == -1) printf("Fseek error\n");
                                                                     
#define VOLFLT double                                                

#define S_ID struct _s_id_
S_ID             
{                      
      char station[5];            /* Station id                     */
      char location[2];           /* Location id                    */
      char channel[3];            /* Channel id                     */
      char res[2];                /* reserved */
};

#define S_TIME struct _s_time_
S_TIME         
{                    
      unsigned short int year;     /* year (ie 1987)                 */
      unsigned short int day;      /* day of the year (Jan 1, is 1)  */
      char hour;                   /* hour of the day (0-23)         */
      char minute;                 /* minute of day (0-59)           */
      char seconds;                /* seconds of day (0-59,60 for    */
                                   /*                 leap seconds)  */
      char unused;                 /* get on word boundry            */
      unsigned short int frac_sec; /* .0001 seconds (0-9999)         */
};

#define S_DATA struct _s_data_
S_DATA
{                  
      unsigned short int num_samples;    /* Number of samples in record    */
      short int sample_rate;             /* Sample rate factor             */ 
      short int sample_rate_multiplier;  /* Sample rate multiplier factor  */
      /* this gives an bus alignment error */
      /* long int flags;	*/       /* all the flags together         */ 
      unsigned char  f1;
      unsigned char  f2;
      unsigned char  f3;
      unsigned char  f4;
      /* this gives an bus alignment error */
      /* long int num_frac_sec_corr; */  /* Number of .0001 sec time       */
      short int t1;
      short int t2;
      unsigned short int begin_data;     /* Byte number of beginning of    */
                                         /*   correction (first byte of    */
                                         /*   rec is 0)                    */
      unsigned short int begin_blockette;/* Byte number of beginning of    */
                                         /*   first blockette (0 if none)  */
};

#define CITED_SOURCE	struct _cited_source
CITED_SOURCE {
       INT              Source_ID;
       UBYTE		*Cited_Source_Key;
       UBYTE		*Name_Publication;   /* Name of publication */
       UBYTE		*Date_Published;     /* Date published */
       UBYTE            *Publisher_Name;     /* Name of publisher */
       CITED_SOURCE     *Next;
};
	

#define EVENT_ENTRY  struct _event_entry
#define MAXMAGN 6
EVENT_ENTRY {
	STDTIME		Origin_Time;
	CITED_SOURCE	*Info_Source;
	DOUBLE		Lattitude;
	DOUBLE		Longitude;
	DOUBLE		Depth;
	INT		Number_Magnitudes;
	VOLFLT		Magnitude[MAXMAGN];
	UBYTE		*Magnitude_Type[MAXMAGN];
	INT		Magnitude_Source[MAXMAGN];
 	EVENT_ENTRY	*Next;	
};

#define GDSN_ENTRY  struct _gdsn
GDSN_ENTRY {
	 INT    nst;
         INT    *stlog;
         INT    *datatype[4];
         INT    *nch[4];
         INT    *stID;
         INT    *format;
         UBYTE   **inst_type;
         EVENT_ENTRY *Root_Event; 
         EVENT_ENTRY *Tail_Event; 
         INT    last_block;
};

/*
#define POSEIDON_ENTRY  struct _pos_entry
POSEIDON_ENTRY {
	 INT    Number_Channels;
	 INT    StID;
         UBYTE  *Chname[MAX_CHANNELS];
};
*/
                                                                     
#define BLOCKETTE struct _blockette                                  
BLOCKETTE {                                                          
       UBYTE           *BlkBuff;       /* Pointer to the current blockette */  
       ULONG           BufLen;         /* Max Length of the buffer */  
       UWORD           BlkType;        /* The blockette id number */  
       UWORD           BlkVer;         /* The version of the blockette */      
       UBYTE           BlkRec;         /* Record type of blockette */
       ULONG           PackPos;        /* Number of bytes this blockette */    
       UWORD           ParsePos;       /* Where is the parser now? */
};                                                                   
       
extern ULONG   DefBlkLen;              /* Default blockette length */
extern ULONG   DefBlkExt;              /* Blockette Extend Increment */
                                                                      
#define LOGICAL_RECORD struct _logical_record                        
LOGICAL_RECORD {                                                     
       UBYTE           *RecBuff;       /* Pointer to current record */
       ULONG           RecNum;         /* Record number of current record */   
       UWORD           NextPos;        /* Next position to write to */
       UWORD           RecLen;         /* logical rec len power of 2 exp */    
       UWORD           RecSize;        /* The size of the record */ 
       BOOL            Continue;       /* Is this a continuation record */     
       UBYTE           RecType;        /* What is the control header type */   
       BOOL            (*WriteService)(); /* Subroutine to dump buffers */     
       BOOL            (*ReadService)(); /* Routine to read next */  
       ULONG           BlockExp;       /* Exponent for block buffer */
       ULONG           BlockMax;       /* The number of bytes of buffer */     
       ULONG           BlockSize;      /* Size of last read */       
       ULONG           BlockPos;       /* Current position */       
       UBYTE           *BlockBuffer;   /* Pointer to buffer */     
};                                                                
                             
#define ABBREV struct _abbrev 
ABBREV {                     
       UBYTE           *Abbrev_Key;    /* How do we access this? */ 
       UWORD           Abbrev_ID;      /* Numeric id for writing headers */    
       UBYTE           *Comment;       /* Text for abbreviation */
       UWORD           Use_Count;      /* How many times refered to */
       ABBREV          *Next;                                        
};                                                                   
                                                                    
#define UNIT struct _unit                                          
UNIT {                                                           
       UBYTE           *Unit_Key;      /* Key for the units */  
       UWORD           Unit_ID;        /* What code units? */  
       UBYTE           *Comment;       /* Description of Unit */   
       UWORD           Use_Count;      /* Number of times refered to */
       UNIT            *Next;                                         
};                                                                   

                                                                    
#define COMMENT        struct _comment                             
COMMENT { 
       UWORD           Comment_ID;     /* Comment numeric ident */
       UBYTE           Class;          /* Class Code */
       UBYTE           *Text;          /* Text of comment */
       UNIT            *Level;         /* The unit of the level */
       UWORD           Use_Count;      /* Number of times refered to */
       COMMENT         *Next; 
};


#define FORMAT struct _format 
#define MAXKEYS 10           
FORMAT {                     
       UBYTE           *Format_Key;    /* What is name of format */
       UBYTE           *Format_Name;   /* Description of format */
       UWORD           Format_ID;      /* Id of format */
       UWORD           Family;  
       UWORD           NumKeys;
       UBYTE           *Keys[MAXKEYS];
       UWORD           Use_Count;      /* Number of times refered to */
       FORMAT          *Next; 
};  

#define LOC_KEY_ID  struct _loc_abr_id
LOC_KEY_ID {                       
       ABBREV          *Root_Abbrev; 
       ABBREV          *Tail_Abbrev;
       UNIT            *Root_Unit; 
       UNIT            *Tail_Unit;
       FORMAT          *Root_Format;
       FORMAT          *Tail_Format;
};
       
#define COMMENT_ENTRY struct _comment_entry  
COMMENT_ENTRY {
       STDTIME         Start_Comment; 
       STDTIME         End_Comment;   
       COMMENT         *Comment;     
       LONG            Comment_Level;
       COMMENT_ENTRY   *Next;       
};                                 

                                  
#define RESPONSE struct _response_pointer
                                        
#define ZEROS_POLES struct _zeros_poles
                                      
ZEROS_POLES {                        
       UBYTE           Function_Type;
       UWORD           Cascade;     
       UNIT            *Response_In;   
       UNIT            *Response_Out; 
       VOLFLT          A0_Normalization;
       VOLFLT          Norm_Frequency; 
       UWORD           Complex_Zeros; 
       VOLFLT          *Zero_Real_List;
       VOLFLT          *Zero_Imag_List;
       VOLFLT          *Zero_Real_Errors; 
       VOLFLT          *Zero_Imag_Errors;
       UWORD           Complex_Poles;   
       VOLFLT          *Pole_Real_List; 
       VOLFLT          *Pole_Imag_List;
       VOLFLT          *Pole_Real_Errors; 
       VOLFLT          *Pole_Imag_Errors; 
       UBYTE           *Cite_Name; 
};                                
                                 
#define COEFFICIENTS struct _coefficients    
COEFFICIENTS {     
       UBYTE           Function_Type;       
       UWORD           Cascade;            
       UNIT            *Response_In;      
       UNIT            *Response_Out;    
       UWORD           Numerators;      
       VOLFLT          *Numer_Coeff;   
       VOLFLT          *Numer_Error;  
       UWORD           Denominators; 
       VOLFLT          *Denom_Coeff;
       VOLFLT          *Denom_Error; 
       UBYTE           *Cite_Name; 
};                                
                                 
#define RESP_GENERIC struct _resp_generic 
RESP_GENERIC {                           
       UBYTE           Function_Type;   
       UWORD           Cascade;        
       UNIT            *Response_In;  
       UNIT            *Response_Out;
       UWORD           Corners;     
       VOLFLT          *Corner_Freq;
       VOLFLT          *Corner_Slope;
       UBYTE           *Cite_Name;  
};                                 
                                  
#define RESP_LIST struct _resp_list    
                               
RESP_LIST {                   
       UWORD           Cascade;
       UNIT            *Response_In;
       UNIT            *Response_Out;
       UWORD           Number_Responses; 
       VOLFLT          *Frequency;      
       VOLFLT          *Amplitude;     
       VOLFLT          *Amplitude_Error;
       VOLFLT          *Phase_Angle;   
       VOLFLT          *Phase_Error;  
       UBYTE           *Cite_Name;  
};                                 
                                     
#define DECIMATION struct _decimation 
       
DECIMATION {
       UWORD           Cascade;
       VOLFLT          Input_Rate;     /* Input sample rate (sps) */ 
       LONG            Factor;         /* Decimation Factor */      
       LONG            Offset;         /* Decimation Offset */     
       VOLFLT          Delay;          /* Estimated Delay (secs) */
       VOLFLT          Correction;     /* Correction Applied (secs) */
       UBYTE           *Cite_Name;                                   
};                                                                  

#define SENSITIVITY struct _sensitivity                            
SENSITIVITY {                                                     
       UWORD           Cascade;        /* Zero if final capping sens */
       VOLFLT          Sensitivity;    /* Accumulated average sens */ 
       VOLFLT          Frequency;      /* Frequency of sens */       
       UWORD           Num_Cals;       /* Number of calibrations */ 
       VOLFLT          *Cal_Value;     /* Value of cal */          
       VOLFLT          *Cal_Freq;      /* Frequency of cal */     
       STDTIME         *Cal_Time;      /* Time of above cal */   
       UBYTE           *Cite_Name;                             
};                                                            

RESPONSE {                                                   
       union   {                                            
               ZEROS_POLES     *PZ;                        
               COEFFICIENTS    *CO;                       
               RESP_LIST       *RL;                      
               RESP_GENERIC    *GR;                     
               DECIMATION      *DM;                    
               SENSITIVITY     *SENS;                 
       } ptr;                                        
       UBYTE           type;                        
       RESPONSE        *Next;                      
};                                                

#define         RESP_ZEROS_POLES 'P'             
#define         RESP_COEFFICIENTS 'C'           
#define  	RESP_RESP_LIST 'L'             

/* #define  RESP_GENERIC 'G'       */

#define  	RESP_SENSITIVITY 'S'
#define         RESP_DECIMATION 'D'

#define        CHANNEL_DATA struct _channel_data                     
CHANNEL_DATA {                                                       
       UWORD           Year;                                         
       UWORD           Day;                                          
       STDTIME	       Start_Time;
       STDTIME	       End_Time;
       UBYTE           *DataFile;     /* Data file list by hour */  
       ULONG           Total_Recs;     /* Total number of records */ 
       ULONG           Total_Entries;  /* What is the total */       
       CHANNEL_DATA    *Next;          /* Next day */                
       ULONG           RecNum;         /* Starting Record number */
};                                
                                 
#define CHANNEL_ENTRY struct _channel_entry
CHANNEL_ENTRY {                           
       UBYTE           *Location;        
       UBYTE           *Identifier;     
       UWORD           SubChannel;     
       ABBREV          *Instrument;   
       UBYTE           *Optional;    
       UNIT            *Signal_Response;  
       UNIT            *Calibration_Input;
       VOLFLT          Lattitude;        
       VOLFLT          Longitude;       
       VOLFLT          Elevation;      
       VOLFLT          Local_Depth;   
       VOLFLT          Azimuth;      
       VOLFLT          Dip;         
       FORMAT          *Format_Type;
       UBYTE           Data_Exp;   
       VOLFLT          Sample_Rate; 
       VOLFLT          Max_Drift;   
       ULONG           Channel_Flags;
               
#define CFG_TRIGGERED  0x00000001      /* Data is triggered */ 
#define CFG_CONTINUOUS 0x00000002      /* Data recorded continuously */
#define CFG_HEALTHCHAN 0x00000004      /* State of health channel */  
#define CFG_GEODATA    0x00000008      /* Geophysical data */        
#define CFG_ENVIRON    0x00000010      /* Weather/Environmental data */ 
#define CFG_FLAGS      0x00000020      /* Flags/Switch information */  
#define CFG_SYNTH      0x00000040      /* Data is synthesized */      
#define CFG_CALIN      0x00000080      /* Channel is a cal input signal */
#define CFG_EXPERIMENT 0x00000100      /* Channel is experimental */ 
#define CFG_MAINTENANCE        0x00000200      /* Maintenance underway on chann
el */      
#define CFG_BEAM       0x00000400      /* Data is a beam synthesis */
           
       UWORD           Number_Comments;                             
       COMMENT_ENTRY   *Root_Comment;                              
       COMMENT_ENTRY   *Tail_Comment;                             
       STDTIME         Effective_Start;                          
       STDTIME         Effective_End;     
       UBYTE           Update_Flag;      
       CHANNEL_ENTRY   *Update_Channel; 
       CHANNEL_ENTRY   *Root_Channel;  /* First channel of this set */
       RESPONSE        *Root_Response;        
       RESPONSE        *Tail_Response;       
       CHANNEL_ENTRY   *Next;               
       CHANNEL_DATA    *Root_Data;         
       CHANNEL_DATA    *Tail_Data;        
       ULONG           Use_Count;        
};                                      

#define STATION_ENTRY struct _station_table 
STATION_ENTRY {                            
       ULONG           RecNum;                /* Where was this written */    
       UBYTE           *Call_Letters;    
       UWORD           Station_Number;  
       VOLFLT          Lattitude;      
       VOLFLT          Longitude;     
       VOLFLT          Elevation;    
       UBYTE           *Site_Name;  
       ABBREV          *Station_Owner; 
       UWORD           Long_Order;    
       UWORD           Word_Order;   
       UWORD           Number_Channels; 
       UWORD           Number_Comments;
       CHANNEL_ENTRY   *Root_Channel; 
       CHANNEL_ENTRY   *Tail_Channel;
       COMMENT_ENTRY   *Root_Comment;
       COMMENT_ENTRY   *Tail_Comment; 
       STDTIME         Effective_Start;  
       STDTIME         Effective_End;   
       UBYTE           Update_Flag;    
       STATION_ENTRY   *Update_Station;
       STATION_ENTRY   *Root_Station; 
       STATION_ENTRY   *Next;        
       ULONG           Use_Count;   
       STDTIME         Established;
};

#define BLOCK73 struct _block_73
BLOCK73 {
        INT             number_data_pieces;
	UBYTE		*Station_ID;
	UBYTE		*Location_ID;
	UBYTE		*Channel_ID;
	STDTIME		Start_Time;
	INT		Start_Sequence_No;
	INT		Start_Sub_Sequence_No;
        BLOCK73         *Next;
};


#define BLOCK74 struct _block_74
BLOCK74 {
	UBYTE		*Station_ID;
	UBYTE		*Location_ID;
	UBYTE		*Channel_ID;
	STDTIME		Start_Time;
	INT		Start_Sequence_No;
	INT		Start_Sub_Sequence_No;
	INT		Updated_Start_Sequence_No;
	STDTIME		End_Time;
	INT		End_Sequence_No;
	INT		End_Sub_Sequence_No;
	INT		Updated_End_Sequence_No;
	INT		Accelerators;
	BYTE		File_Name[60];
	BLOCK74		*Next;
};

#define TIME_SPAN  struct _time_span
TIME_SPAN {
       STDTIME         Start_Time;
       STDTIME         End_Time;
       BYTE            Flag;
};

#define TIME_ENTRY struct _time_entry
TIME_ENTRY {
       TIME_SPAN       *Time_Span_ID;
       EVENT_ENTRY     *Root_Event;
       EVENT_ENTRY     *Tail_Event;
       BLOCK74	       *Root_Block74;
       BLOCK74	       *Tail_Block74;
       INT	        RecNum;
       TIME_ENTRY      *Next;
};


#define VOLUME_ID struct _volume_id
VOLUME_ID {                       
       UBYTE           LenExp;         /* Record length exponent */  
       STDTIME         Vol_Begin;      /* Beginning time of volume */
       STDTIME         End_Volume;     /* End time of volume */ 
       ABBREV          *Root_Abbrev; 
       ABBREV          *Tail_Abbrev;
       UNIT            *Root_Unit; 
       UNIT            *Tail_Unit;
       FORMAT          *Root_Format;
       FORMAT          *Tail_Format;
       COMMENT         *Root_Comment;
       COMMENT         *Tail_Comment;
       CITED_SOURCE    *Root_Cited_Source;
       CITED_SOURCE    *Tail_Cited_Source;
       STATION_ENTRY   *Root_Station;
       STATION_ENTRY   *Tail_Station;
       STDTIME         Effective_Start; 
       STDTIME         Effective_End;  
       TIME_ENTRY      *Root_Time_Entry;
       TIME_ENTRY      *Tail_Time_Entry;
};                                    

extern VOLUME_ID       *VI;         
extern STDTIME         _ztime,_etime;
extern STATION_ENTRY   *Current_Station,*First_Station; 
extern CHANNEL_ENTRY   *Current_Channel,*First_Channel; 
                                                       
/******************Forward Definitions With Typing************************/    
/*
BLOCKETTE      *CreateBlockette();   
VOID           SetupBlockette( BLOCKETTE *, UWORD, UWORD);       
BOOL           ExtendBlockette(BLOCKETTE *,ULONG);           
BOOL           AppendBlockette(BLOCKETTE *,UBYTE *,ULONG);  
BOOL           PackString(BLOCKETTE *,UBYTE *,ULONG,BOOL); 
BOOL           PackChar(BLOCKETTE *,UBYTE,BOOL);          
BOOL           PackVariable(BLOCKETTE *,UBYTE *,ULONG,BOOL); 
BOOL           PackInteger(BLOCKETTE *,LONG,LONG,BOOL);     
BOOL           PackFixed(BLOCKETTE *,VOLFLT,LONG,LONG,BOOL); 
BOOL           PackExponent(BLOCKETTE *,VOLFLT,LONG);       
BOOL           PackTime(BLOCKETTE *,STDTIME);              
                                                          
LOGICAL_RECORD *CreateRecordBuff(UWORD, BOOL (*)());    
BOOL           FlushRecord(LOGICAL_RECORD *);            
BOOL           NoiseRecord(LOGICAL_RECORD *);          
BOOL           NewHeader(LOGICAL_RECORD *,UBYTE);     
BOOL           RecordPack(BLOCKETTE *,LOGICAL_RECORD *); 
         
VOID           InitializeVolume(STDTIME, STDTIME);      
BOOL           VolumeBlockette(BLOCKETTE *,LOGICAL_RECORD *,BOOL); 
UBYTE          *AllocString(UBYTE *);                             
UBYTE          *FixKey(UBYTE *);                                 
        
ABBREV         *SearchAbbrev(UBYTE *);                          
ABBREV         *SearchAbbrevIdx(LONG);                         
ABBREV         *InitAbbrev();                                 
BOOL           LinkInAbbrev(ABBREV *);                       
BOOL           AddAbbrev(UBYTE *,UBYTE *);                  
BOOL           AbbrevBlockette(ABBREV *,BLOCKETTE *,LOGICAL_RECORD *); 
BOOL           DumpAbbrevBlockettes(BLOCKETTE *,LOGICAL_RECORD *);    
BOOL           PackAbbrev(BLOCKETTE *,ABBREV *);                     
VOID           CountAbbrev(ABBREV *);                               
         
UNIT           *SearchUnit(UBYTE *);                               
UNIT           *SearchUnitIdx(LONG);                              
UNIT           *InitUnit();                                      
BOOL           LinkInUnit(UNIT *);                              
BOOL           AddUnit(UBYTE *,UBYTE *);                       
         
BOOL           UnitBlockette(UNIT *,BLOCKETTE *,LOGICAL_RECORD *); 
BOOL           DumpUnitBlockettes(BLOCKETTE *,LOGICAL_RECORD *);  
BOOL           PackUnit(BLOCKETTE *,UNIT *);                     
VOID           CountUnit(ABBREV *);                             
         
COMMENT        *SearchComment(UWORD);                  
COMMENT        *InitComment();                        
BOOL           LinkInComment(COMMENT *);                     
BOOL           AddComment(UWORD,UBYTE,UBYTE *,UBYTE *);     
BOOL           CommentBlockette(COMMENT *,BLOCKETTE *,LOGICAL_RECORD *);
BOOL           DumpCommentBlockettes(BLOCKETTE *,LOGICAL_RECORD *);    
BOOL           PackComment(BLOCKETTE *,COMMENT *);                    
VOID           CountComment(COMMENT *);                              
         
FORMAT         *SearchFormat(UBYTE *);                              
FORMAT         *SearchFormatIdx(LONG);                             
FORMAT         *InitFormat();                                     
BOOL           LinkInFormat(FORMAT *);                           
BOOL           AddFormat(UBYTE *,UBYTE *,UWORD,UBYTE,UBYTE *[]);
BOOL           FormatBlockette(FORMAT *,BLOCKETTE *,LOGICAL_RECORD *);
BOOL           DumpFormatBlockettes(BLOCKETTE *,LOGICAL_RECORD *);   
BOOL           PackFormat(BLOCKETTE *, FORMAT *);                   
VOID           CountFormat(FORMAT *);                              
         
STATION_ENTRY  *SearchStation(UBYTE *);                           
STATION_ENTRY  *InitStation();                                   
BOOL           LinkInStation(STATION_ENTRY *);                  
BOOL           StationBlockette(STATION_ENTRY *,BLOCKETTE *,LOGICAL_RECORD *); 
         
CHANNEL_ENTRY  *SearchChannel(STATION_ENTRY *,UBYTE *,UBYTE *);      
CHANNEL_ENTRY  *InitChannel();                                      
BOOL           LinkInChannel(STATION_ENTRY *,CHANNEL_ENTRY *);     
BOOL           ChannelBlockette(CHANNEL_ENTRY *,BLOCKETTE *,LOGICAL_RECORD *); 
ULONG          CnvChanFlags(UBYTE *);                             
        
ZEROS_POLES    *InitZerosPoles(UWORD,UWORD);                     
SENSITIVITY    *InitSensitivity(UWORD);                         
COEFFICIENTS   *InitCoefficients(UWORD,UWORD);                 
RESPONSE       *AddResponse(CHANNEL_ENTRY *);                 
      
BOOL           LinkInZerosPoles(CHANNEL_ENTRY *,ZEROS_POLES *); 
BOOL           LinkInSensitivity(CHANNEL_ENTRY *,SENSITIVITY *); 
BOOL           LinkInCoefficients(CHANNEL_ENTRY *,COEFFICIENTS *); 
BOOL           SensBlockette(SENSITIVITY *,BLOCKETTE *,LOGICAL_RECORD *);      
BOOL           ZerosBlockette(ZEROS_POLES *,BLOCKETTE *,LOGICAL_RECORD *);     
BOOL           CoefBlockette(COEFFICIENTS *,BLOCKETTE *,LOGICAL_RECORD *);     
BOOL           RespBlockette(RESPONSE *,BLOCKETTE *,LOGICAL_RECORD *);
COMMENT_ENTRY  *InitCom();                                           
BOOL           LinkInStatCom(STATION_ENTRY *,COMMENT_ENTRY *);      
BOOL           LinkInChanCom(CHANNEL_ENTRY *,COMMENT_ENTRY *);     
BOOL           AddStatCom(STATION_ENTRY *,STDTIME,STDTIME,UWORD,ULONG); 
BOOL           AddChanCom(CHANNEL_ENTRY *,STDTIME,STDTIME,UWORD,ULONG);
BOOL           ComBlockette(COMMENT_ENTRY *,BLOCKETTE *,LOGICAL_RECORD *,BOOL);

*/
/* Reader definitions */                                              
/*
#ifdef RDBUFFSUP                                                     
LOGICAL_RECORD         *RecordReadBuff(UWORD,UWORD,BOOL (*)());     
BOOL           GetARecord(LOGICAL_RECORD *);                       
BOOL           GetBytes(UBYTE *,UBYTE *,int,int,int,LOGICAL_RECORD *); 
BOOL           ParseABlockette(BLOCKETTE *);                          
BOOL           ParseBlockettes(UBYTE,LOGICAL_RECORD *,BLOCKETTE *);  
        
#endif */                     /* RDBUFFSUP */                       
/*
BOOL           UnpackString(BLOCKETTE *,UBYTE **,ULONG );          
BOOL           UnpackChar(BLOCKETTE *,UBYTE *);                   
BOOL           UnpackVariable(BLOCKETTE *,UBYTE **,UWORD,BOOL ); 
BOOL           UnpackInteger(BLOCKETTE *,LONG *,LONG,BOOL );    
BOOL           UnpackFixed(BLOCKETTE *,VOLFLT *,LONG, LONG,BOOL ); 
BOOL           UnpackExponent(BLOCKETTE *,VOLFLT *,LONG );        
BOOL           UnpackTime(BLOCKETTE *,STDTIME *);                
BOOL           UnpackAbbrev(BLOCKETTE *,ABBREV **);             
BOOL           UnpackComment(BLOCKETTE *,COMMENT_ENTRY **);    
BOOL           UnpackFormat(BLOCKETTE *,FORMAT **);           
BOOL           UnpackUnit(BLOCKETTE *,UNIT **);              
BOOL           ParseEventPhases(BLOCKETTE *);               
BOOL           ParseHypoCenterInfo(BLOCKETTE *);           
BOOL           ParseTimeSpanID(BLOCKETTE *);              
BOOL           ParseTimeSpanDataIndex(BLOCKETTE *);      
BOOL           ParseStationHeader(BLOCKETTE *);         
BOOL           ParseNetworkHeader(BLOCKETTE *);        
BOOL           ParseStationIndex(BLOCKETTE *);        
BOOL           ParseSpanIndex(BLOCKETTE *);          
BOOL           ParseFormatDictionary(BLOCKETTE *);  
BOOL           ParseCommentDictionary(BLOCKETTE *);
BOOL           ParseSourceDictionary(BLOCKETTE *); 
BOOL           ParseAbbrevDictionary(BLOCKETTE *);
BOOL           ParseUnitsDictionary(BLOCKETTE *);
BOOL           ParseBeamDictionary(BLOCKETTE *);
BOOL           ParseStationID(BLOCKETTE *);    
BOOL           ParseStationComment(BLOCKETTE *); 
BOOL           ChanSame(CHANNEL_ENTRY *,CHANNEL_ENTRY *);       
BOOL           ParseChannelID(BLOCKETTE *);                    
BOOL           ParseZerosPoles(BLOCKETTE *);                  
BOOL           ParseCoefficients(BLOCKETTE *);               
BOOL           ParseResponseList(BLOCKETTE *);              
BOOL           ParseGenericResponse(BLOCKETTE *);          
BOOL           ParseSensitivity(BLOCKETTE *);             
BOOL           ParseChannelComment(BLOCKETTE *);         
*/

/* Blockette types */                                                
                                                                    
/* #define SEED_STATION_HEADER    5                          */
#define SEED_NETWORK_HEADER    10                                    
#define SEED_STATION_INDEX     11                                    
#define SEED_SPAN_INDEX        12                            
#define SEED_FORMAT_DICT       30                                    
#define SEED_COMMENT_DICT      31                                    
#define SEED_SOURCE_DICT       32                                    
#define SEED_ABBREV_DICT       33                                    
#define SEED_UNIT_DICT         34                                    
#define SEED_BEAM_DICT         35                                    
#define SEED_STATION_HEADER    50                                    
#define SEED_STATION_COMMENT   51                                    
#define SEED_CHANNEL_HEADER    52                                    
#define SEED_ZEROS_POLES       53                                    
#define SEED_COEFFICIENTS      54                                    
#define SEED_RESPONSE_LIST     55                                    
#define SEED_GENERIC_RESPONSE  56                                    
#define SEED_DECIMATION        57                            
#define SEED_GAIN_SENSITIVITY  58                                    
#define SEED_CHANNEL_COMMENT   59                                    
#define SEED_TIME_SPAN_HEADER  70                                    
#define SEED_HYPOCENTER        71                            
#define SEED_EVENT_PHASES      72                                    
#define SEED_TIME_SPAN_INDEX   73                                    
#define SEED_TIME_SERIES_INDEX 74
