/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sun/dkio.h>
#include <dirent.h>
#include <strings.h>
#include <ctype.h>

#include "conver.h"
#include "seed.h"
#include "user2.h"

#define LOOP_OVER_TIME_ENTRIES   for(te=VI->Root_Time_Entry;\
                                     te!=NULL;\
                                     te=te->Next)
#define LOOP_OVER_BLOCK74        for(bl74=te->Root_Block74;\
                                     bl74 != NULL;\
                                     bl74=bl74->Next)

extern char   *CD_year[];
extern char   *CD_month[];
extern int     CD_day[];
extern char   *CD_evtfile[];
extern int     SEED, GDSN;
extern FILE    *uf;
extern char    path_name[];
extern int     INP_SDF;

path_elements  dir_path;
int            CD_YEARS, NODIR, NDAYS, mdays[31], LPYEAR, GEOSCOPE;
TIME_ENTRY     *te;

int   days_tab[12] = { 31,28, 31, 30, 31, 30, 31, 31, 30, 31, 30 ,31 };
static char   month_tab[12][4] = {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG",
                                  "SEP","OCT","NOV","DEC"};


void find_years (path)
char *path;
{
        int           i,  yr;
        DIR           *dirp;
        struct dirent *dp;
	
        NODIR=FALSE;

        if (INP_SDF == TRUE) {

                NODIR = TRUE;
		if((uf = (FILE *) fopen(dir_path.file,"r"))==NULL) {
			printf("File %s not opened.\n",dir_path.file);
			exit(1);
		}
                parse_seedfile();

	        printf("\n\n\n\t\tBe patient .... some initialization time is needed !\n\n\n");
                return;
	}

        


        /* Open directory with data */
	if ( ( dirp = opendir(path)) == NULL) {
              printf("(find_years) dir %s not mounted\n",path);
	      exit(1);
  	}

	dp = (struct dirent *) malloc (sizeof(struct dirent)); 
	i = 0;

        /* Loop over subdirectories to determine the years  */
	for ( dp=readdir(dirp); dp!=NULL; dp=readdir(dirp) ) {

                if(strlen(dp->d_name)!=4) continue;
                sscanf(dp->d_name,"%d",&yr);
                if(yr<1980 || yr>1999) continue;

	        CD_year[i] = (char *) malloc(5);
                strcpy ( CD_year[i], dp->d_name );
                i++;
	}
        /* Close directory */
	closedir ( dirp );

	CD_YEARS = i;


        /* If no subdirectories, my guess is it's Geoscope */

        if(CD_YEARS==0) {

	     if ( ( dirp = opendir(path)) == NULL) {
                   printf("(find_years) dir %s not mounted\n",path);
	           exit(1);
  	     }

             /* Loop over subdirectories to determine the years  */
             for ( dp=readdir(dirp); dp!=NULL; dp=readdir(dirp) ) {
                   if(strcmp(dp->d_name,"seed.dat") == NULL ) {
                              NODIR = TRUE;
                              strcpy(dir_path.year,".");
                              strcpy(dir_path.month,".");
                              strcpy(dir_path.file,"seed.dat");
                              GEOSCOPE = TRUE;
                   }
             }

             /* Close directory */
	     closedir ( dirp );
    
        }

        if (NODIR==TRUE) {

                fill_path_name(4);
		if((uf = (FILE *) fopen(path_name,"r"))==NULL) {
			printf("File %s not opened.\n",path_name);
			exit(1);
		}
                parse_seedfile();
	}

        
	printf("\n\n\n\t\tBe patient .... some initialization time is needed !\n\n\n");

}


void find_months( path ) 
char *path;
{
	int            i, m, dom, cd_year;
	int            MONTH[13];
        DIR           *dirp;
        struct dirent *dp;

        for(i=0;i<12;i++) CD_month[i] = (char *) NULL;
        for(i=0;i<12;i++) MONTH[i] = FALSE;

        i=0;
        if ( NODIR == TRUE ) {
            LOOP_OVER_TIME_ENTRIES {
                 sscanf(dir_path.year,"%d",&cd_year);
                 if ( te->Time_Span_ID->Start_Time.year != cd_year ) continue;
                 if(te->Time_Span_ID->Start_Time.day==0) continue;
                 m = day_of_month ( te->Time_Span_ID->Start_Time.day, &dom,
                                    te->Time_Span_ID->Start_Time.year);
                 if(m<1||m>31) continue;

                 if(MONTH[m]==TRUE) continue;

                 CD_month[i] = (char *) malloc(5);
                 strcpy ( CD_month[i], month_tab[m-1] );
                 MONTH[m] = TRUE;
                 i++;

            }
            return;
        }


        /* Open subdirectory */
	if( ( dirp = opendir(path)) == NULL) {
               printf("(find_months) dir %s not opened\n",path);
	       exit(1);
  	}

	i=0;
	for ( dp=readdir(dirp); dp!=NULL; dp=readdir(dirp) ){

             if(strlen(dp->d_name)!=3) continue;
             CD_month[i] = (char *) malloc(5);
             strcpy ( CD_month[i], dp->d_name );
             i++;
	}

        /* Close subdirectory */
	closedir ( dirp );
}



char
*capitalize ( string )
char *string;
{
	int  k, len;
        char *retstr;

	len = strlen(string);
        retstr = (char *) malloc((len+1) * (sizeof(char)) );

        for ( k=0; k < len; k++ ) 
                retstr[k]  = toupper(string[k]);
        retstr[k] = '\0';

        return(retstr);
}

char 
*capitalize_off ( string )
char *string;
{
	int  k, len;
        char *retstr;

	len = strlen(string);
        retstr = (char *) malloc((len+1) * (sizeof(char)) );

        for ( k=0; k < len; k++ ) 
                retstr[k]  = tolower(string[k]);
        retstr[k] = '\0';

        return(retstr);
}

int
sort_days ( el1, el2 )
int  *el1, *el2;
{
        return(*el1 - *el2); 
}

int
sort_days2 ( el1, el2 )
char  *el1, *el2;
{
        char dum[4];
        int jd1, jd2;

        strncpy( dum, el1+2, 3);
        dum[3] = '\0';
        sscanf ( dum, "%d", &jd1 );
        strncpy( dum, el2+2, 3);
        dum[3] = '\0';
        sscanf ( dum, "%d", &jd2 );

        return(jd1 - jd2); 
}




void find_days(path_name)
char *path_name;
{
        DIR           *dirp;
        struct dirent *dp;
        int           i, jd, year, DAY[31], cd_year, m, dom;
        char          *jul_day[31], dum[4];

        void          lpyr();
        int           day_of_month(); 
	

        for ( i=0;i<31;i++ ) CD_day[i] = 0;
        for ( i=0;i<31;i++ ) DAY[i] = FALSE;

        i=0;
        if ( NODIR == TRUE ) {
            LOOP_OVER_TIME_ENTRIES {
                 sscanf(dir_path.year,"%d",&cd_year);
                 if ( te->Time_Span_ID->Start_Time.year != cd_year ) continue;
                 if(te->Time_Span_ID->Start_Time.day==0) continue;
                 m = day_of_month ( te->Time_Span_ID->Start_Time.day, &dom,
                                    te->Time_Span_ID->Start_Time.year);
                 if(m<1||m>31) continue;
                 if ( strcmp(dir_path.month,month_tab[m-1]) != NULL ) continue;

                 if(DAY[dom]==TRUE) continue;
                 CD_day[i] =  dom;
                 DAY[dom] = TRUE;
                 i++;
           }
           NDAYS = i;
           SEED = TRUE;
           GDSN = FALSE;

           qsort( &CD_day[0], NDAYS, sizeof(int), sort_days);
           return;
        }


/* Open directory with data */

	if ( ( dirp = opendir((char *)path_name)) == NULL) {
              printf("(find_days) dir %s not mounted\n",path_name);
	      exit(1);
  	}

	dp = (struct dirent *) malloc (sizeof(struct dirent)); 

	i = 0;

/* Loop over subdirectories to determine the days  */

	for ( dp=readdir(dirp); dp!=NULL; dp=readdir(dirp) ) {

                if(strlen(dp->d_name)!=11) continue;

	        jul_day[i] = (char *) malloc(12);
                strcpy ( jul_day[i], dp->d_name );
                strncpy( dum, &dp->d_name[2], 3);
                dum[3] = '\0';
                sscanf ( dum, "%d", &jd );

                strncpy( dum, &dp->d_name[0], 2);
                dum[2] = '\0';
                sscanf ( dum, "%d", &year );

/* Convert julian day to day of month */

                day_of_month ( jd, &mdays[i], year );
                CD_day[i] = mdays[i];
                CD_evtfile[i] = (char *) malloc(15);
                strcpy ( CD_evtfile[i], dp->d_name );
                i++;
	}

/* Sort CD_day and CD_evtfile in the same order */
        NDAYS = i;
/*
        qsort( &CD_day[0], NDAYS, sizeof(int), sort_days);
        qsort( &CD_evtfile[0], NDAYS, sizeof(char *), sort_days2);
*/

/* Determine SEED or GDSN formatted CDROM */

        if ( strncmp(&CD_evtfile[0][8],"evt",3) == 0  || 
             strncmp(&CD_evtfile[0][8],"EVT",3) == 0  ) {
                 GDSN = TRUE;
                 SEED = FALSE;
        }
        if ( strncmp(&CD_evtfile[0][8],"sed",3) == 0  || 
             strncmp(&CD_evtfile[0][8],"SED",3) == 0  ) {
                 SEED = TRUE;
                 GDSN = FALSE;
        }
        if( SEED == FALSE && GDSN == FALSE ) {
             printf("Can't find GDSN or SEED files on CDROM.\n");
             exit(1);
        }

/* Close directory */

	closedir ( dirp );

}


int 
day_of_month ( jd, day, year ) 
int jd, *day, year;
{
        int i, iday, maxdays;

        iday = jd;
        lpyr ( year );
	
    	if ( LPYEAR ) {
              maxdays = 366;
              days_tab[1] = 29;
        }
        else {
              maxdays = 365;
              days_tab[1] = 28;
        }

	if ( jd < 1 || jd > maxdays ) {
                   printf("jul day %d not correct \n", jd );
                   return(-1);
        }

	if ( jd<=31 ) {
              *day = jd;
              return(1);
        }

        for(i=0;i<12;i++) {
                *day = iday;
		iday -= days_tab[i];
                if ( iday <= days_tab[i+1] ) {
                           *day = iday;
                           break;
                }
        }

	return(i+2);  /* return month */
          	
}

void lpyr ( year )
int year;
{
	LPYEAR = FALSE;

        if((year%4) == 0 ) LPYEAR=TRUE;

}

int md[] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

int
julian_day ( month, day ) 
int month, day;
{
	int i;


	if ( month > 1 ) {
		for(i=1;i<month;i++) {
			day += md[i];
                }
                if(month>2)
	                if(LPYEAR==TRUE) day++;
	}

	return day;
}
