/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
/****                                                                       ***/
/****                          DECODE_DATA.C                                ***/
/****                                                                       ***/
/******************************************************************************/

#include "conver.h"
#include "seed.h"
#include "user2.h"
#include "define.h"
#include "prototyp.h"
#include <math.h>
#include <malloc.h>

#define l_bt(C)                ((C>>4)&(~(~0<<4)))
#define r_bt(C)                ((C)&(~(~0<<4)))

#define LOOP_OVER_DATA_BLOCKS  for ( block=0; block < datablocks; block++ ) 

/* Macro to check on succes of malloc */
#define CHM(ret) if (ret == NULL ) { printf("Malloc failed.\n"); exit(1); }

extern int            stbuf[];
extern int            DTYP;
extern int            DATA_ON_HDISC;
extern int            SEED, GDSN, DEBUG, LRECL, SEL_DAY;
extern int            channel_order();
extern char           Orientation, BandCode;
extern double         evttimeinsec;
extern STATION_STRUCT *stinfo[]; 
extern FILE           *uf;

int      *xdata[MAX_CHANNELS];
int      SRO, DWWSSN, RSTN, GRF, ECH, GSC, MULTIPLEXED;
int	 DATA_ALLOCATED, DATA_BUF_ALLOCATED, SEED_DATA_BUF_ALLOCATED[6];
int      day_in_first_header, malloc_size, NDP;
int      NBL[MAX_STATIONS][MAX_CHANNELS];
int sdnumblocks[6];

char     *dfile[MAX_STATIONS+1], *sfile[MAX_STATIONS+1][4];
char     *data_buf, *seed_data_buf[5], loc_orientation;
float    *ydata[MAX_CHANNELS];

STDTIME  htime;
S_ID    *did;
S_TIME  *dtime;
S_DATA  *ddata;


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*     +                GDSN specific functions                +             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

void 
analyze_gdsn_data( stc )
int stc;
{
	int        trig, block;
	int        datablocks, ns;
	int        ns_old, ch, Ntriggers, t, s1, s2;
        char       *char_ptr;
        double     time_in_seconds, time_in_seconds_old, Ts;
        float      samples_per_second, tb, te, dt;
        STDTIME    btime, etime;

        /* Initialize some values  */
        ns_old                   = 0;
        time_in_seconds_old      = 0;
        Ts                       = 0;
        trig                     = 0;
        day_in_first_header      = 5000;

        /* determine number of datablocks to read   */
        datablocks = (stinfo[stc]->datablocks[DTYP]) -3;


        /* read number of channels */
        stinfo[stc]->number_channels[DTYP] = get_number_of_channels (data_buf); 

        /*  read number of samples per second */
        samples_per_second = get_samples_per_second ( data_buf, &samples_per_second);


        /* Loop over datablocks (2000 bytes)  */

        NDP=0;  /* sample_counter */

        LOOP_OVER_DATA_BLOCKS {

            char_ptr = (char *) ( data_buf + block*2000 );

            /* Extract header time and fill structure htime  */
            /* ns is number of samples per channel           */

            parse_header ( stc, char_ptr, &time_in_seconds, &ns );
            if ( block == 0 ) day_in_first_header = htime.day;

            /* Ts is time in seconds per block of 2000 bytes  */
            Ts = ((float)(ns_old)) / samples_per_second; 

            /* Check for new trigger ! */
            if ( fabs(time_in_seconds_old  + (double)Ts  - time_in_seconds) > 0.1) {
                      trig++;   /*  trigger counter */
                      store_trigger ( stc, trig, NDP, samples_per_second ,(int)0 );
            }

            time_in_seconds_old = time_in_seconds;
            ns_old = ns;


            /*  Total number of samples  */
	    NDP += ns;

        }  


       NDP -= 1;
       Ntriggers = trig;

       /* Store last sample number for all triggers */

       for ( t=1; t< Ntriggers; t++) {

                stinfo[stc]->data_entry[DTYP]->trigger[t]->last_sample_no =
                      stinfo[stc]->data_entry[DTYP]->trigger[t+1]->first_sample_no -1; 
       }

       stinfo[stc]->data_entry[DTYP]->trigger[Ntriggers]->last_sample_no = NDP;


       /* Determine time of last sample for all triggers */

       for ( t=1; t<= Ntriggers; t++) {

                htime = stinfo[stc]->data_entry[DTYP]->trigger[t]->start_time;
                s1 = stinfo[stc]->data_entry[DTYP]->trigger[t]->first_sample_no;
                s2 = stinfo[stc]->data_entry[DTYP]->trigger[t]->last_sample_no;
                          
                Ts = (s2-s1 +1)/ samples_per_second;
                time_of_last_sample ( stc, t, Ts, (int)0 );
       }

       /* Determine number of (empty) samples between end and start of 
                                                                two triggers */

       for ( t=1; t< Ntriggers; t++) {

                if ( Ntriggers == 1 ) break;

                btime = stinfo[stc]->data_entry[DTYP]->trigger[t+1]->start_time;
                etime = stinfo[stc]->data_entry[DTYP]->trigger[t]->end_time;
                          
                dt = stinfo[stc]->data_entry[DTYP]->dt;

                /* convert time structures into seconds */
		tb = time_to_sec ( &btime );
		te = time_to_sec ( &etime );

		stinfo[stc]->data_entry[DTYP]->trigger[t]->dN =
                                            (int) (((te-tb)/dt)-1);
       }
}
            

/*   Convert 20 bytes header to usefull information   */

void 
parse_header ( stc,char_ptr, time_in_seconds, ns )
int     stc, *ns;
char   *char_ptr;
double *time_in_seconds;
{

    htime.year   = l_bt(*(char_ptr+2))+1980;
    htime.day    = r_bt(*(char_ptr+2))*100 + l_bt(*(char_ptr+3))*10 
                                           + r_bt(*(char_ptr+3));
    htime.hour   = l_bt(*(char_ptr+4))*10  + r_bt(*(char_ptr+4));
    htime.minute = l_bt(*(char_ptr+5))*10  + r_bt(*(char_ptr+5));
    htime.second = l_bt(*(char_ptr+6))*10  + r_bt(*(char_ptr+6));
    htime.msec   = l_bt(*(char_ptr+7))*100 + r_bt(*(char_ptr+7))*10 
                                           + l_bt(*(char_ptr+8)) ;

    *ns          = r_bt(*(char_ptr+12))*100 + l_bt(*(char_ptr+13))*10 
                                            + r_bt(*(char_ptr+13));

    *ns /= stinfo[stc]->number_channels[DTYP];

    *time_in_seconds = time_to_sec ( &htime );

    if ( htime.day > day_in_first_header ) *time_in_seconds += 86400;

}


/*  Calculate # of samples per second.   */

float 
get_samples_per_second ( char_ptr )
char *char_ptr;
{
        int idum, idum1, idum2;
	float sps;

        idum  = (int) 10;
        idum1 = (5 - (int) ( l_bt(*(char_ptr+11))));
        idum2 = (int) (r_bt(*(char_ptr+11))*10 + l_bt(*(char_ptr+12)));

        sps = (float) (idum2 * pow ((double) idum, (double) idum1 ));
        return (sps);

}

int 
get_number_of_channels ( ptr )
char *ptr;
{
        int nch;

        nch = (int) (l_bt(*(ptr+10))*10 + r_bt(*(ptr+10)));
        return (nch);
}


void 
get_channels_gdsn ( stc )
int stc;
{

        int            nch;
        off_t          offset;
        char           *tbuf;

        /*numblocks = stinfo[stc]->datablocks[DTYP]; */

        if ((tbuf = (char *) malloc ( 20 * sizeof(char) ) ) ==NULL ) {
           exit(1);
        }

        offset = stinfo[stc]->station_log; 
        if ( SHORTP_CHOSEN ) 
                  offset += 0;
        else if (LONGP_CHOSEN )  
                   offset += stinfo[stc]->datablocks[1];
        else if (INTERMP_CHOSEN ) 
                   offset += (stinfo[stc]->datablocks[1] + stinfo[stc]->datablocks[2]);
        offset += 3;
        offset *= 2000;
        fseek ( uf, offset, 0 );
        if ((fread ( tbuf, sizeof(char), 20, uf )) == -1) {
                       exit(1);
        }

        nch = (int) (l_bt(*(tbuf+10))*10 + r_bt(*(tbuf+10)));
        stinfo[stc]->number_channels[DTYP] = nch;

}


/* Read data for station stc and type SP, LP or IP from event file, and
   store them in a tmp-file on disk for faster I/O later on !            */

void 
read_data ( stc )
int stc;
{

        int            numblocks, n ;
        off_t          offset;
	FILE           *hdf;

        numblocks = stinfo[stc]->datablocks[DTYP];

	if ( DATA_BUF_ALLOCATED == TRUE ) {
             free ( data_buf );
             DATA_BUF_ALLOCATED = FALSE;
        }
        
        malloc_size =  numblocks * 2000 * sizeof(char);
        if((data_buf = (char *) malloc(malloc_size) ) == NULL) {
             printf("Malloc failed.\n");
             exit(1);
        }
        else DATA_BUF_ALLOCATED = TRUE;


        if ( DATA_ON_HDISC == FALSE ) {

               offset = stinfo[stc]->station_log; 
               if ( SHORTP_CHOSEN ) 
                        offset += 0;
               else if (LONGP_CHOSEN )  
                        offset += stinfo[stc]->datablocks[1];
               else if (INTERMP_CHOSEN ) 
                        offset += (stinfo[stc]->datablocks[1] + stinfo[stc]->datablocks[2]);
               offset += 3;
               offset *= 2000;

               /* Seek right position in file and read data */
               fseek ( uf, offset, 0 );
               if ((n=fread ( data_buf, sizeof(char), numblocks*2000, uf )) == -1) {
                       printf("No data read in.\n");
                       exit(1);
               }


               dfile[stc] = (char *) create_tmp_file ( stc, (char) NULL, (char) NULL );

               if ( (hdf = (FILE *) fopen ( dfile[stc],"w")) == NULL ) {
                       printf("(1) File %s not opened.\n",dfile[stc]); 
                       printf("Are you allowed to write on %s ?\n\n",getenv("PWD"));
                       exit(1);
               }
               fwrite(data_buf, sizeof(char),numblocks*2000,hdf);
	       fclose(hdf);
        }

	else {

               if ( (hdf = (FILE *) fopen ( dfile[stc],"r")) == NULL ) {
                       printf("(2) File %s not opened.\n",dfile[stc]); 
                       exit(1);
	       }

               if ((n=fread ( data_buf, sizeof(char), numblocks*2000, hdf )) == -1) {
                       printf("No data read in.\n");
                       exit(1);
               }
               fclose(hdf);
	}


}


/* Arrays with gain factors */

float    sro_gf[]  = { 1024, 512, 256, 128, 64, 32, 16, 8, 4, 2, 1, 
                       .5, .25, .125, .0625, .03125 };
float    grf_gf[]  = { 4096, 2048, 1024, 512, 256, 128, 64, 32, 16, 8,
                       4, 2, 1, .5, .25, .125, .0625, .03125 };
float    ech_gf[]  = { 1, 0.5, 0.25, 0.125, 0.0625, 0.03125, 0.015625, 0.00781250 };
float    rstn_gf[] = { 1, 4, 16, 128 }; 



void 
decode_data( stc )
int stc;
{
        short int  word;
	int        mant, gain_index, x, word_offset, block;
	int        samples_per_channel, datablocks, plotbuf_size, i, ns;
	int        j, s, ch;
        char       *char_ptr;
        double     time_in_seconds;
        float      dt;
	short int       *buf;


     /* determine number of datablocks to read   */
        datablocks = (stinfo[stc]->datablocks[DTYP]) -3;

        buf = (short int *) data_buf;
        char_ptr = (char *) data_buf;
        stinfo[stc]->number_channels[DTYP] = get_number_of_channels (char_ptr); 

     /* determine size of plotbuffer(s) and allocate memory   */
	plotbuf_size = (datablocks * 990 * 2)/ stinfo[stc]->number_channels[DTYP];


        while ( DATA_ALLOCATED >= 0 ) {
		    free ( xdata[DATA_ALLOCATED] );
		    free ( ydata[DATA_ALLOCATED] );
		    DATA_ALLOCATED--;
        }

        for ( i=0; i< stinfo[stc]->number_channels[DTYP]; i++)  {
              CHM ((xdata[i] = (int *) malloc ( sizeof(int) * plotbuf_size )));
              CHM ((ydata[i] = (float *) malloc ( sizeof(float) * plotbuf_size )));
	      DATA_ALLOCATED = i;
        }

    /*  define data decoding type  */

        SRO = DWWSSN = RSTN = GRF = ECH = GSC = False;

        switch ( stinfo[stc]->formatID ) {

                 case 1 : SRO = True;
                          if(DEBUG)
                          printf("(2)Data in SRO format. \n");
                          break;
                 case 2 : DWWSSN = True;
                          if(DEBUG)
                          printf("(2)Data in DWWSSN format. \n");
                          break;
                 case 3 : RSTN = True;
                          if(DEBUG)
                          printf("(2)Data in RSTN format. \n");
                          break;
                 case 4 : GRF = True;
                          if(DEBUG)
                          printf("(2)Data in GRF format. \n");
                          break;
                 case 5 : GSC = True;
                          if(DEBUG)
                          printf("(2)Data in GSC format. \n");
                          break;
                 case 6 : ECH = TRUE; 
                          if(DEBUG)
                          printf("(2)Data in ECH format. \n");
                          break;
                 default: printf("ID %d not yet implemented\n",stinfo[stc]->formatID);
                          break;
        }


   /*  Extract # of samples per second */
        char_ptr = (char *) buf;
        /* samples_per_second = get_samples_per_second (char_ptr); */


   /*   Loop over datablocks (2000 bytes)  */

        x=0;  /* sample_counter */

        LOOP_OVER_DATA_BLOCKS {

            word_offset = block*1000;
            char_ptr = (char *) (buf+word_offset);

            parse_header ( stc,char_ptr, &time_in_seconds, &ns );
            samples_per_channel = ns;

        /*  Loop over samples */
            for ( j=word_offset+10, s=0; s<samples_per_channel ; s++) {

             /*  Loop over channels to decode abd demultiplex the data   */

                 for ( ch=0; ch<stinfo[stc]->number_channels[DTYP]; ch++ ) {

                    word =  *(buf+j);

                    if (SRO) {
                       mant = (int)(word & SRO_MANTISSA_MASK);
                       gain_index = (int)( (word & SRO_GAIN_MASK) >> SRO_GAIN_SHIFT );
                       if(mant>=2048)mant -= 4096;
                       *(ydata[ch]+x) = (float) (mant * sro_gf[gain_index]);
                       *(xdata[ch]+x) =  x;
                    }
                    else if ( DWWSSN ) {
		       *(ydata[ch]+x) = (float) word;	
                       *(xdata[ch]+x) = x;
                    }
                    else if ( RSTN ) {
                       mant = (int)(word & RSTN_MANTISSA_MASK);
                       gain_index = (int)( (word & RSTN_GAIN_MASK) >> RSTN_GAIN_SHIFT );
                       mant -= 8191;
                       *(ydata[ch]+x) = (float) (mant * rstn_gf[gain_index]);
                       *(xdata[ch]+x) = x;
                    }
                    else if ( GRF ) {
                       mant = (int)((word & GRF_MANTISSA_MASK) >> GRF_MANTISSA_SHIFT);
                       gain_index = (int)( (word & GRF_GAIN_MASK) );
                       if(mant>=2048) mant -= 4096;
                       *(ydata[ch]+x) = (float) (mant * grf_gf[gain_index]);
                       *(xdata[ch]+x) = x;
                    }
                    else if ( ECH || GSC) {
                       mant = (int)(word & SRO_MANTISSA_MASK);
                       gain_index = (int)( (word & SRO_GAIN_MASK) >> SRO_GAIN_SHIFT );
                       mant -= 2048;
                       *(ydata[ch]+x) = (float) (mant * ech_gf[gain_index]);
                       *(xdata[ch]+x) = x;
                    }
                    else {
                       printf("data not yet decoded\n");
		       exit(1);
		    }

                    j++;
               }
                    x++;
           }
        }  

    /* End of loop over datablocks  */

}



/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*     +                SEED specific functions                +             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

int 
read_seed_data ( stc )
int stc;
{
        int            trig,  pt, loop_counter, RETVAL, el, nb, CHUSED;
int pp;
	FILE           *hdf;
        long int       offset;
        char chc, orient_code;

        RETVAL = FALSE;
        trig = 0;

        if ( Orientation == 'A' ) loop_counter = 3;
        else                      loop_counter = 1;

        /* Check fotr multiplexed data */
        MULTIPLEXED = check_for_multiplex ( stc, BandCode );

        /* While loop over channels */
        while ( loop_counter >= 1 ) {

        pt = channel_order ( loop_counter, &loc_orientation );

        /* Do this while loop only once in case data are multiplexed. */
        if ( MULTIPLEXED == TRUE ) {
             pt=0;
             loop_counter = 0;
             loc_orientation = ' ';
             chc = ' ';
        }

        if ( SEED_DATA_BUF_ALLOCATED[pt] == TRUE ) {
                 free ( seed_data_buf[pt] );
                 SEED_DATA_BUF_ALLOCATED[pt] = FALSE;
        }
        el = 0;
        sdnumblocks[pt] = 0;
        trig=0;

        LOOP_OVER_TIME_ENTRIES {
		LOOP_OVER_BLOCK74 {

                     /* Check if this blockette 74 is needed */
                     if ( strcmp( bl74->Station_ID, stinfo[stc]->station_code) != NULL ) 
                               continue;
                     if ( bl74->Channel_ID[0] != BandCode ) 
                               continue;
                     if ( bl74->Channel_ID[2] != loc_orientation)
                               continue;
                     if ( time_entry->Time_Span_ID->Start_Time.day != SEL_DAY )
                               continue; 
                     trig++;

                     if ( stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig]->bool == FALSE ) continue;



                     if ( MULTIPLEXED == FALSE ) {

                         orient_code = bl74->Channel_ID[2]; 
                         CHUSED = FALSE;

                         if ( orient_code == 'Z' ) {
                              if ( !(loc_orientation == 'N' || loc_orientation == 'E') ) {
                                 chc = 'Z';
                                 CHUSED = TRUE;
                              }
                          }
                          else if ( orient_code == 'N' ) {
                              if ( !(loc_orientation == 'Z' || loc_orientation == 'E') ) {
                                 chc = 'N';
                                 CHUSED = TRUE;
                              }
                          }
                          else if ( orient_code == 'E' ) {
                              if ( !(loc_orientation == 'Z' || loc_orientation == 'N') ) {
                                 chc = 'E';
                                 CHUSED = TRUE;
                              }
                          }
                          else {
                               printf("Orientation code %c not used.\n",orient_code);
                               CHUSED = FALSE;
                          }

                          if ( CHUSED == FALSE ) {
                               printf("%c NOT USED ????\n",orient_code);
                               continue;   /* Next bl74 or time_entry */
                          }

                      } 

                     RETVAL = TRUE;

                     sdnumblocks[pt] += bl74->End_Sequence_No - bl74->Start_Sequence_No +1;
                     nb = bl74->End_Sequence_No - bl74->Start_Sequence_No +1;

                     /* Allocate memory for data buffer */
                     if ( SEED_DATA_BUF_ALLOCATED[pt] == FALSE ) {
                         if ( ( seed_data_buf[pt] = (char *)
                            malloc ( nb * LRECL * sizeof(char)) ) == NULL ) {
                                   printf("Malloc failed in read_seed_data. \n");
                                   exit(1);
                         }
                     }

                     /* Increase memory for data buffer */
                     if ( SEED_DATA_BUF_ALLOCATED[pt] == TRUE ) {
                         if ( ( seed_data_buf[pt] = (char *)
                                   realloc ( (char *) seed_data_buf[pt],(unsigned) sdnumblocks[pt]*LRECL * sizeof(char)) ) == NULL ) {
                                   printf("Realloc failed in read_seed_data. \n");
                         }
                     }
                     SEED_DATA_BUF_ALLOCATED[pt] = TRUE;


                     offset = (bl74->Start_Sequence_No-1) * LRECL;
                     fseek ( uf, offset, 0 );
                     printf("Reading data for %s %s (%d bytes)  stc=%d  pt=%d el=%d  trig=%d\n", bl74->Station_ID, bl74->Channel_ID,nb*LRECL,stc,pt,el,trig);

                     if ((fread ( seed_data_buf[pt]+el, sizeof(char), nb*LRECL, uf )) == -1) {
                             printf("No data read in.\n");
                             exit(1);
                     }

   
/*
                     if(sfile[stc][pt]==(char *) NULL) {
                           sfile[stc][pt] = (char *) create_tmp_file ( stc, BandCode, chc );
                           printf("create file %s\n",sfile[stc][pt]);
                     }

                     if((hdf = (FILE *) fopen ( sfile[stc][pt],"a"))==NULL) {
                                printf("(4) File %s not opened.\n",sfile[stc][pt]);
                                exit(1);
                     } 

                     printf("(1) Writing/Appending  data for %s %s  to file %s\n", 
                            bl74->Station_ID, bl74->Channel_ID, sfile[stc][pt]);

printf("el = %d\n",el);
                     if((fwrite (&seed_data_buf[pt][el], sizeof(char),nb*LRECL,hdf))!=nb*LRECL) {
                                 printf("Write failed.....\n");
                                 exit(1);
                     }
                     fclose(hdf);
*/
                     el += nb*LRECL;



       	        }
	    }
            loop_counter--;
        }
        return(RETVAL);
}


int 
decode_seed_data( stc )
int stc;
{
	int        block, trig;
	int        datablocks, plotbuf_size,ns;
	int        index, zz;
        int        byte_offset, FOUND, loop_counter, dac, pt, bod, MSIZE;
        char       decode[60], *ptr_to_data;


        /* Initialize some variables */
        FOUND                   =FALSE;
	dac                       = -1;


        if ( Orientation == 'A' ) loop_counter = 3;
        else                      loop_counter = 1;

        /* Check fotr multiplexed data */
        MULTIPLEXED = check_for_multiplex ( stc, BandCode );

        while ( DATA_ALLOCATED >= 0 ) {
		    free ( xdata[DATA_ALLOCATED] );
		    free ( ydata[DATA_ALLOCATED] );
		    DATA_ALLOCATED--;
        }

        /* While loop over channels */

        while ( loop_counter >= 1 ) {

             pt = channel_order ( loop_counter, &loc_orientation );
             FOUND = FALSE;

             /* Do this while loop only once in case data are multiplexed. */

             if ( MULTIPLEXED == TRUE ) {
                  pt=0;
                  loop_counter = 0;
                  loc_orientation = ' '; 
             }
             trig=0;
     
             LOOP_OVER_TIME_ENTRIES {

		LOOP_OVER_BLOCK74 {

                     if ( strcmp( bl74->Station_ID, stinfo[stc]->station_code) != NULL ) 
                               continue;
                     if ( bl74->Channel_ID[0] != BandCode ) 
                               continue;
                     if ( bl74->Channel_ID[2] != loc_orientation)
                               continue;
                     if ( time_entry->Time_Span_ID->Start_Time.day != SEL_DAY )
                               continue; 
                     trig++;

                     if ( stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig]->bool == FALSE ) continue;

                     FOUND=TRUE;
                     break;
                }
                if(FOUND==TRUE) break;
             }

             /* No station/channel found: get out from here  */
             if(FOUND==FALSE)  return(FALSE);

             FOUND=FALSE;


	     LOOP_OVER_STATIONS {
            
                  if (strcmp(stinfo[stc]->station_code,sty->Call_Letters)!=0)
                       continue;

	          LOOP_OVER_CHANNELS {

		      if ( !(chy->Identifier[0] == BandCode &&
                           chy->Identifier[2] == loc_orientation )) continue;

                      FOUND = TRUE;
                      break;
                  }
                  if(FOUND==TRUE) break;
             }

             /* determine number of datablocks to read   */
             datablocks = bl74->End_Sequence_No - bl74->Start_Sequence_No +1;
             datablocks = sdnumblocks[pt];


             /* determine size of plotbuffer(s) and allocate memory   */
	     plotbuf_size = (datablocks * LRECL/2 );


             /* For multiplexed data as for SSB, 3 data arrays xdata and ydata */
             /* must be allocated here ! */
             /* Be aware that for Steim compressed data the size of xdata and
                ydata probably has to grow ! */

             if ( MULTIPLEXED == TRUE ) {

                  pt=0;
                  dac++;
                  CHM ((xdata[0] = (int *) malloc ( sizeof(int) * plotbuf_size/3 )));
                  CHM ((ydata[0] = (float *) malloc ( sizeof(float) * plotbuf_size/3 )));
                  DATA_ALLOCATED = dac;
                  dac++;
                  CHM ((xdata[1] = (int *) malloc ( sizeof(int) * plotbuf_size/3 )));
                  CHM ((ydata[1] = (float *) malloc ( sizeof(float) * plotbuf_size/3 )));
                  DATA_ALLOCATED = dac;
                  dac++;
                  CHM ((xdata[2] = (int *) malloc ( sizeof(int) * plotbuf_size/3 )));
                  CHM ((ydata[2] = (float *) malloc ( sizeof(float) * plotbuf_size/3)));
                  DATA_ALLOCATED = dac;
             }

             else {      /* Data not multiplexed  */
                 dac++;
          
                 CHM ((xdata[pt] = (int *) malloc ( sizeof(int) * plotbuf_size )));
                 CHM ((ydata[pt] = (float *) malloc ( sizeof(float) * plotbuf_size )));
                 DATA_ALLOCATED = dac;
             }
     
             /*  define data decoding type  */

             strcpy ( decode, chy->Format_Type->Format_Name);
             decode[5] = '\0';

 
             /*  do the decoding, that is: fill arrays xdata and ydata with the
                 correct values.                                             */

             index = 0;
	     data_buf = seed_data_buf[pt];
             MSIZE = 0;

             for(block=0;block<datablocks;block++) {

                 byte_offset = block*LRECL;

                 did =   (S_ID *) (data_buf+byte_offset+8);
                 dtime = (S_TIME *) (data_buf+12+byte_offset+8);
                 ddata = (S_DATA *) (data_buf+22+byte_offset+8);


	         ns = (int) ddata->num_samples;

                 MSIZE += ns;

                 if(MSIZE >= plotbuf_size ) {
                    plotbuf_size += (MSIZE-plotbuf_size+20) ;
                    CHM((xdata[pt] = (int *) realloc((char *)xdata[pt],(unsigned)plotbuf_size*sizeof(int))));
                  
                    CHM((ydata[pt] = (float *) realloc((char *)ydata[pt],(unsigned)plotbuf_size*sizeof(float))));
                  
                 }
             
                 ptr_to_data = (char *)( data_buf + byte_offset + (int) ddata->begin_data);

                 /* begin of data: needed for steim decompression */
                 bod = ddata->begin_data;

	         if (strncmp (decode, "ASRO",4) == 0) 
		    decode_sro (ptr_to_data, ns, index, pt);
	         else if (strcmp (decode, "SRO G") == 0)
		    decode_sro (ptr_to_data, ns, index, pt);
	         else if (strcmp (decode, "GRAEF") == 0)
		    decode_graef (ptr_to_data, ns, index, pt);
	         else if (strcmp (decode, "DWWSS") == 0)
		    decode_dwwssn (ptr_to_data, ns, index, pt);
	         else if (strncmp (decode, "RSTN",4) == 0)
		    decode_rstn (ptr_to_data, ns, index, pt);
	         else if (strcmp (decode, "GEOSC") == 0)
		    decode_geoscope (ptr_to_data, ns, index, pt, chy->Format_Type->Format_Name);
	         else if (strcmp (decode, "Geosc") == 0)
		    decode_geoscope (ptr_to_data, ns, index, pt, chy->Format_Type->Format_Name);
                 else if (strncmp(decode,"ECHY",4) == 0) 
		    decode_echery (ptr_to_data, ns, index, pt);
                 else if (strncmp(decode,"Steim",5) == 0) 
		    decode_steim (ptr_to_data, ns, index, pt, bod);
                 else if (strncmp(decode,"STEIM",5) == 0) 
		    decode_steim (ptr_to_data, ns, index, pt, bod);
                 else {
                    printf("Format %s not included \n", decode);
                    exit(1);
                 }

                 index += ns;
            }
     
            loop_counter--;
       }


       return(TRUE);

}




/* Get number of different channels for stations stc in SEED volume */

int
get_channels_seed ( stc )
int stc;
{
        int            numc=0;

        LOOP_OVER_STATIONS {
            if ( strcmp( sty->Call_Letters, stinfo[stc]->station_code) == NULL ) {
                 LOOP_OVER_CHANNELS {

                     if ( HBB_CHOSEN == TRUE && 
                         ( chy->Identifier[0] == 'H' || chy->Identifier[0] == 'S' ))
                                   numc++; 
	             else if ( BB_CHOSEN == TRUE && chy->Identifier[0] == 'B' ) 
                                   numc++;
	             else if ( MP_CHOSEN == TRUE && chy->Identifier[0] == 'M' ) 
                                   numc++;
	             else if ( LP_CHOSEN == TRUE && chy->Identifier[0] == 'L' ) 
                                   numc++;
	             else if ( VLP_CHOSEN == TRUE && chy->Identifier[0] == 'V' ) 
                                   numc++;
	             else if ( ULP_CHOSEN == TRUE && chy->Identifier[0] == 'U' ) 
                                   numc++;
                 }
            }

        }

        return(numc);
}

int
analyze_seed_data ( stc )
int stc;
{
	int        trig;
	int        FOUND, CHUSED;
	int        Ntriggers, t, s1, s2;
	int        pt, loop_counter,  np;
        char       orient_code, *buf, *buff, chc;
        double f_sam_rate;
        float      tb, te, dt, Ts;
        STDTIME    btime, etime;
	FILE       *hdf;
        long offset;
        float sps;


        FOUND = FALSE;
        if ( Orientation == 'A' ) loop_counter = 3;
        else                      loop_counter = 1;

        /* Check fotr multiplexed data */
        MULTIPLEXED = check_for_multiplex ( stc, BandCode );

        /* While loop over channels */
        while ( loop_counter >= 1 ) {

        pt = channel_order ( loop_counter, &loc_orientation );

        /* Do this while loop only once in case data are multiplexed. */

        if ( MULTIPLEXED == TRUE ) {
             pt=0;
             loop_counter = 0;
             loc_orientation = ' ';
             chc = 'X';
        }
       
        NBL[stc][pt] = 0;
        trig = 0;
        NDP=0;
        np=0;

        LOOP_OVER_TIME_ENTRIES {

	      LOOP_OVER_BLOCK74 {

                     if ( strcmp( bl74->Station_ID, stinfo[stc]->station_code) != NULL ) 
                               continue;
                     if ( bl74->Channel_ID[0] != BandCode ) 
                               continue;
                     if ( bl74->Channel_ID[2] != loc_orientation)
                               continue;
                     if ( time_entry->Time_Span_ID->Start_Time.day != SEL_DAY )
                               continue; 


                     if ( MULTIPLEXED == FALSE ) {

                         orient_code = bl74->Channel_ID[2]; 
                         CHUSED = FALSE;

                         if ( orient_code == 'Z' ) {
                              if ( !(loc_orientation == 'N' || loc_orientation == 'E') ) {
                                 chc = 'Z';
                                 CHUSED = TRUE;
                              }
                          }
                          else if ( orient_code == 'N' ) {
                              if ( !(loc_orientation == 'Z' || loc_orientation == 'E') ) {
                                 chc = 'N';
                                 CHUSED = TRUE;
                              }
                          }
                          else if ( orient_code == 'E' ) {
                              if ( !(loc_orientation == 'Z' || loc_orientation == 'N') ) {
                                 chc = 'E';
                                 CHUSED = TRUE;
                              }
                          }
                          else {
                               printf("Orientation code %c not used.\n",orient_code);
                               CHUSED = FALSE;
                          }

                          if ( CHUSED == FALSE ) continue;   /* Next bl74 or time_entry */

                      } 

                     trig++;
                     FOUND = TRUE;


                     if ((buf = (char *) malloc(200) ) == NULL ) {
                                   printf("Malloc failed. \n");
                                   exit(1);
                     };

                     /* Read first data record */

                     offset = (bl74->Start_Sequence_No-1) * LRECL;
                     fseek ( uf, offset, 0 );

                     if ((fread ( buf, sizeof(char), 100, uf )) == -1) {
                             printf("No data read in.\n");
                             exit(1);
                     }

                     data_buf = buf;
                     ddata = (S_DATA *) (data_buf+22+8);

                     /*  Extract # of samples per second */

	             if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier > 0) 
                        f_sam_rate = ddata->sample_rate * ddata->sample_rate_multiplier;
                     if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier < 0) 
                        f_sam_rate = (-1) * ddata->sample_rate / ddata->sample_rate_multiplier;
                     if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier > 0)  
                        f_sam_rate = (-1) * ddata->sample_rate_multiplier/ (float)(ddata->sample_rate);
                     if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier < 0)   
                        f_sam_rate = 1./( (float)(ddata->sample_rate)/ ( (float)ddata->sample_rate_multiplier) );

                     sps = (float)(f_sam_rate);

                     tb = time_to_sec3 ( bl74->Start_Time ); 
                     te = time_to_sec3 ( bl74->End_Time ); 


                     htime.day =    (int)bl74->Start_Time.day;
                     htime.hour =   (int)bl74->Start_Time.hour;
                     htime.minute = (int)bl74->Start_Time.minute;
                     htime.second = (int)bl74->Start_Time.second;
                     htime.msec =   (int)bl74->Start_Time.msec/10;

                     np += NDP;
                     store_trigger ( stc, trig, np, sps, pt );
                     free(buf);

                     /* NDP is number of sample for next (!) (available) trigger */
                     NDP = (te-tb)*sps;
                     NDP += 1;

                     
/*

                     numblocks = bl74->End_Sequence_No - bl74->Start_Sequence_No +1;
                     NBL[stc][pt] += numblocks;
                     if(numblocks <= 0 ) {
                                printf("Something wrong. Numblocks <= 0. \n");
		                return(FALSE);
                     }


                     malloc_size = numblocks * LRECL * sizeof(char);
                     if ((buff = (char *) malloc(malloc_size) ) == NULL ) {
                                   printf("Malloc failed. \n");
                                   exit(1);
                     };

                     offset = (bl74->Start_Sequence_No-1) * LRECL;
                     fseek ( uf, offset, 0 );
                     printf("Reading data for %s %s (%d bytes)  stc=%d  pt=%d\n", bl74->Station_ID, bl74->Channel_ID,numblocks*LRECL,stc,pt);

                     if ((fread ( buff, sizeof(char), numblocks*LRECL, uf )) == -1) {
                             printf("No data read in.\n");
                             exit(1);
                     }

   
                     if(sfile[stc][pt]==(char *) NULL) {
                           sfile[stc][pt] = (char *) create_tmp_file ( stc, BandCode, chc );
                           printf("create file %s\n",sfile[stc][pt]);
                     }

                     if((hdf = (FILE *) fopen ( sfile[stc][pt],"a"))==NULL) {
                                printf("(4) File %s not opened.\n",sfile[stc][pt]);
                                exit(1);
                     } 

                     printf("(1) Writing/Appending  data for %s %s  to file %s\n", 
                            bl74->Station_ID, bl74->Channel_ID, sfile[stc][pt]);

                     if((fwrite (buff, sizeof(char),numblocks*LRECL,hdf))!=numblocks*LRECL) {
                                 printf("Write failed.....\n");
                                 exit(1);
                     }

                     fclose(hdf);
                     free(buff);
*/


               }  /* end of loop over time_entries */

        }  /* end of loop over bl74  */


       if(FOUND==FALSE) return(FALSE);

       Ntriggers = trig;

       for ( t=1; t< Ntriggers; t++) {

                stinfo[stc]->seed_entry[DTYP][pt]->trigger[t]->last_sample_no =
                      stinfo[stc]->seed_entry[DTYP][pt]->trigger[t+1]->first_sample_no -1; 
       }

       stinfo[stc]->seed_entry[DTYP][pt]->trigger[Ntriggers]->last_sample_no = 
       stinfo[stc]->seed_entry[DTYP][pt]->trigger[Ntriggers]->first_sample_no + NDP;


/* Determine time of last sample for all triggers */

       for ( t=1; t<= Ntriggers; t++) {

                htime = stinfo[stc]->seed_entry[DTYP][pt]->trigger[t]->start_time;
                s1 = stinfo[stc]->seed_entry[DTYP][pt]->trigger[t]->first_sample_no;

                s2 = stinfo[stc]->seed_entry[DTYP][pt]->trigger[t]->last_sample_no;

                          
                Ts = (float)(s2-s1 +1)/ sps;
                time_of_last_sample ( stc, t, Ts, pt );
       }

/* Determine time of last sample for all triggers */

       for ( t=1; t< Ntriggers; t++) {

                if ( Ntriggers == 1 ) break;

                btime = stinfo[stc]->seed_entry[DTYP][pt]->trigger[t+1]->start_time;
                etime = stinfo[stc]->seed_entry[DTYP][pt]->trigger[t]->end_time;
                          
                dt = stinfo[stc]->seed_entry[DTYP][pt]->dt;

                tb = time_to_sec (&btime);
                te = time_to_sec (&etime);

		stinfo[stc]->seed_entry[DTYP][pt]->trigger[t]->dN =
                                            (int) (((te-tb)/dt)-1);
       }


       printf("Analyzing done ...\n");

       loop_counter--;

   } /* end of while loop */
   return(TRUE);
}


/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  */
/*    +            MIXED                          +                          */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  */

double
time_to_sec ( sttime )
STDTIME *sttime;
{
	double sec;

        sec = (double) sttime->hour * 3600  + 
              (double) sttime->minute *60 +
              (double) sttime->second + 
              (double) sttime->msec/1000;

        return(sec);
}

double
time_to_sec2 ( sttime )
STDTIME *sttime;
{
	double sec;

        sec = (double) sttime->hour * 3600  + 
              (double) sttime->minute *60 +
              (double) sttime->second + 
              (double) sttime->msec/10000;

        return(sec);
}

double
time_to_sec3 ( sttime )
STDTIME *sttime;
{
	double sec;

        sec = (double) sttime->day * 86400 +
              (double) sttime->hour * 3600  + 
              (double) sttime->minute *60 +
              (double) sttime->second + 
              (double) sttime->msec/1000;

        return(sec);
}

char 
*create_tmp_file ( stc, typ, chan )
int  stc;
char typ, chan;
{
      char *flname, dum[9], code[6];
      int  i, j;
      
      flname = (char *) malloc(10);
      strcpy ( flname, "dtmp");

      j=0;
      for(i=0;i<strlen(stinfo[stc]->station_code);i++) {
            if(stinfo[stc]->station_code[i] == ' ' ) continue;
            code[j++]=stinfo[stc]->station_code[i];
      }
      code[j]='\0';

      if(GDSN)
         sprintf ( dum ,"%s", code);
      if(SEED) {
         if ( chan != ' ')
             sprintf ( dum ,"%s.%c%c", code, typ, chan);
         else if ( chan == ' ' )  /* for multiplexed data */
             sprintf ( dum ,"%s.%c", code, typ);
      }

      strcat ( flname, dum );
      return ( flname );
}



void
time_of_last_sample ( stc,trig,ns,pt )
int     stc, trig, pt;
float   ns;
{

        if(DEBUG)
      printf("beg time %d %d %d %d %d\n",
                      htime.day,htime.hour,htime.minute,htime.second,htime.msec);

      ns = ns*1000;
      htime.msec += ns;

      while ( htime.msec > 999 ) {
            htime.msec -= 1000;
            htime.second += 1;
      }
      while ( htime.second > 59 ) {
            htime.second -= 60;
            htime.minute += 1;
      }
      while ( htime.minute > 59 ) {
            htime.minute -= 60;
            htime.hour += 1;
      }
      while ( htime.hour > 23 ) {
            htime.hour -= 24;
            htime.day += 1;
      }
        if(DEBUG)
      printf("last time %d %d %d %d %d\n",
                    htime.day,htime.hour,htime.minute,htime.second,htime.msec);

      if(GDSN)
         copy_time ( &stinfo[stc]->data_entry[DTYP]->trigger[trig]->end_time, htime );
      if(SEED) 
            copy_time ( &stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig]->end_time, htime );


}

void 
copy_time ( time1, time2 )
STDTIME *time1, time2;
{
	time1->year = time2.year;
	time1->day = time2.day;
	time1->hour = time2.hour;
	time1->minute = time2.minute;
	time1->second = time2.second;
	time1->msec = time2.msec;
}


/*  Store trigger information.    */

void 
store_trigger ( stc,trig, x, sps, pt)
int   stc, trig, x, pt;
float sps;
{

      if ( trig > MAX_TRIGGERS ) {
                  printf("Increase MAX_TRIGEERS in glob.h \n");
                  exit(1);
      }

      if(GDSN) {

            if ( DTYP < 1 || DTYP > 3 ) {
                  printf("No data ? Program terminated in store_trigger(). \n");
                  exit(1);
            }

            if ( stinfo[stc]->data_entry[DTYP] == NULL ) 
                 stinfo[stc]->data_entry[DTYP] = (DATA_TYPE_STRUCT *) InitDataTypeStruct();

            stinfo[stc]->data_entry[DTYP]->trigger[trig] = (TRIGGER_STRUCT *) InitTriggerStruct();

            stinfo[stc]->data_entry[DTYP]->Ntriggers = trig;  
            stinfo[stc]->data_entry[DTYP]->dt = 1/sps;  
            stinfo[stc]->data_entry[DTYP]->trigger[trig]->first_sample_no = x;

            copy_time ( &stinfo[stc]->data_entry[DTYP]->trigger[trig]->start_time,
                       htime );

      }

      else if (SEED) {

            if ( DTYP < 4 || DTYP > 10 ) {
                  printf("No data ? Program terminated in store_trigger(). \n");
                  exit(1);
            }

            if(DEBUG)
                printf("in store trigger loc_orientat = (%c)\n",loc_orientation);


            if ( stinfo[stc]->seed_entry[DTYP][pt] == NULL ) {
                stinfo[stc]->seed_entry[DTYP][pt] = (DATA_TYPE_STRUCT *) InitDataTypeStruct();
                if ( stinfo[stc]->seed_entry[DTYP][pt] == NULL ) {
                     printf("InitDataTypeStruct.....failed !!!\n");
                     exit(1);
                }
            }

            /*stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig] = NULL;*/
            stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig] = (TRIGGER_STRUCT *) InitTriggerStruct();

            stinfo[stc]->seed_entry[DTYP][pt]->Ntriggers = trig;  
            stinfo[stc]->seed_entry[DTYP][pt]->dt = 1/sps;  
            stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig]->first_sample_no = x;

            copy_time ( &stinfo[stc]->seed_entry[DTYP][pt]->trigger[trig]->start_time,
                       htime );

      }

}


void 
init_filenames()
{
	int i,j;

	for ( i=0; i<= MAX_STATIONS; i++ ) {
              if(GDSN) 
                        dfile[i] = (char *) NULL;
              
              if(SEED) {
                        for ( j=0; j<4 ; j++ ) 
                        sfile[i][j] = (char *) NULL;
              }
        }
}

void
remove_dtmp_files()
{
	int i,j;
	
        printf("Cleaning up .....\n");

	for ( i=0; i<= MAX_STATIONS; i++ ) {

              if(GDSN) {
                    if ( dfile[i] == NULL ) continue;
                    else unlink(dfile[i]);
                    dfile[i] = (char *) NULL;
              }
              if(SEED) {
                    for(j=0;j<4;j++) {
                        if ( sfile[i][j] == NULL ) continue;
                        else unlink(sfile[i][j]);
                        sfile[i][j] = (char *) NULL;
                    }
              }
	}
}

