/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/

#define MANTISSA_MASK 0x0fff			/* mask for mantissa */
#define GAIN3_MASK 0x7000			/* mask for gainrange factor */
#define GAIN4_MASK 0xf000			/* mask for gainrange factor */
#define SHIFT 12				/* # bits in mantissa */
#define MAX12 0x7ff				/* maximum 12 bit positive # */
#define MAX24 0x7fffff				/* maximum 24 bit positive # */

extern float *ydata[];
extern int   *xdata[];

void decode_geoscope (data_ptr, nsamp, index, pt, name)
char *data_ptr;
int nsamp, index, pt;
char  *name;
{
	short *temp;				/* recovers sample from SEED */
	int mantissa;				/* mantissa from SEED data */
	int gainrange;				/* gain range factor */
	int base;				/* base for exponentiation */
	int add2gr;				/* added to gainrange factor */
	int mult;				/* multiplier for terms above */
	int add2result;				/* added to result above */
	int exponent;				/* total exponent */
	double sample;				/* sample value */
	int demux;				/* demux pointer */
	int i, j, k;				/* counter */
	union   {				/* byte */
		char c[4];
		int  i;
		} data;

        int code;
	base = 2;
	add2gr = 0;
	mult = -1;
	add2result = 0;

        if ( strncmp(name,"Geoscope gain-range on 3 bits",29) == 0)
             code =1; 
        else if ( strncmp(name,"Geoscope-3 byte",15) == 0)
             code =2; 
        else if ( strncmp(name,"Geoscope gain range on 4 bits",29) == 0)
             code =3; 
        else { printf("name not found  %s\n",name);
               exit(1);
             }

	for (i = 0; i < nsamp; i++) {

		for (j = 0; j < 3; j++) {

			switch (code) {

			case 1:
				temp = (short *) data_ptr;

				/* recover mantissa and gain range factor */
				mantissa = ((int) (*temp & MANTISSA_MASK));
				gainrange = ((unsigned int) (*temp & GAIN3_MASK)) >> SHIFT;
	
				/* calculate exponent from gain range factor and format definition */
				exponent = gainrange;

				/* calculate sample from mantissa and exponent */
				sample = ((double) (mantissa-2048)) / pow((double) base, (double) exponent);

				break;
			case 2:
				data.i = 0;
				for (k=0;k<3;k++) data.c[1+k] = data_ptr[k];

				mantissa = data.i;
/*
printf("(1)j=%d  data.i = %x  data.char = (%x)(%x)(%x)(%x)\n",j,data.i,data.c[0],data.c[1],data.c[2],data.c[3]);
printf("(2)mantis = %d %x\n",mantissa,mantissa);
*/
				/* take 2's complement for mantissa */
				if (mantissa > MAX24) mantissa -= 2 * (MAX24 + 1);

				/* calculate sample from mantissa and exponent */
				sample = (double) mantissa;
/*
printf("(3)sample = %lf  mantis = %d %x\n",sample,mantissa,mantissa);
if(i==6) exit(1);
*/
				break;
			case 3:
				temp = (short *) data_ptr;
				/* recover mantissa and gain range factor */
				mantissa = ((int) (*temp & MANTISSA_MASK));
				gainrange = ((unsigned int) (*temp & GAIN4_MASK)) >> SHIFT;

				/* calculate exponent from gain range factor and format definition */
				exponent = gainrange ;

				/* take 2's complement for mantissa */
				/* if (mantissa > MAX12) mantissa -= 2 * (MAX12 + 1); */

				/* calculate sample from mantissa and exponent */
				sample =( (double) (mantissa-2048)) / pow ((double) base, (double) exponent);

				break;
			}

			/* save sample in seismic data array */

/* printf("store sample: j=%d  i=%d  index=%d  sample=%f\n",j,i,index,sample); */
		        *(ydata[j]+index+i) = (float) sample;
		        *(xdata[j]+index+i) = index+i;

			switch(code) {
				case 1: case 3: data_ptr += 2; break;
				case 2: data_ptr += 3; break;
			} 
		}
	}
}
