/******************************************************************************/
/****                                                                       ***/
/****    Original version modified and extended by Reinoud Sleeman.         ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/

#define MASK 0x00000003			/* mask for bottom 2 bits */
#define SHIFT 2				/* # bits per decompres flag */
#define MAX8  0x7f			/* maximum  8 bit positive # */
#define MAX16 0x7fff			/* maximum 16 bit positive # */
#define MAX32 0x7fffffff		/* maximum 32 bit positive # */
#define ELEMENTS_FRAME 16		/* # elements per data frame */


extern float *ydata[];
extern int   *xdata[];
extern int   LRECL;
extern int usegval;



void decode_steim (data_ptr, nsamples, index, pt, bod)
char *data_ptr;
int nsamples;
int index, pt, bod;
{
	unsigned short int *temp_u2byte;	/* temp for byte swapping */
	short int *temp_2byte;			/* temp for byte swapping */
	int *temp_4byte;			/* temp for byte swapping */
	char *temp_byte;			/* temp for decompression */
	int temp;				/* temp for decompression */
	double sample;				/* temp for decompression */
	double last_value;			/* last value, previous blk */
	int initial_value;			/* first value, this blk */
	int final_value;			/* last value, this blk */

	int i;					/* counter */
	int j;					/* counter */
	int k;					/* counter */
	int compression_int;			/* coded compression flags */
	int compression_flag[ELEMENTS_FRAME];	/* compression flags */
	int num_frames;				/* number of frames in block */
	int counter;				/* counter */
	/* int bod; */				/* beginning-of-data byte # */

        int s=0;

/*                 +=======================================+                 */
/*=================| get first, last values, # of frames   |=================*/
/*                 +=======================================+                 */

	/* find last value from current seismogram DOESN'T WORK YET */
/*
	if (index != 0) last_value = (double) *(seismic_data_ptr - 1);
	else last_value = 0;
*/
	last_value = 0;

	/* recover this block's intial and final sample values from 1st frame */
	temp_4byte = (int *) (data_ptr + 4);
	/* if (byteswap == TRUE) *temp_4byte = swap_4byte (*temp_4byte); */
	initial_value = (int) *temp_4byte;

	temp_4byte = (int *) (data_ptr + 8);
	/* if (byteswap == TRUE) *temp_4byte = swap_4byte (*temp_4byte); */
	final_value = (int) *temp_4byte;

	num_frames = (LRECL - bod) / (ELEMENTS_FRAME * 4); 


/*                 +=======================================+                 */
/*=================|           process each frame          |=================*/
/*                 +=======================================+                 */

	counter = 0;

	/* process each frame */
	while (counter < nsamples)
	{
		for (i = 0; i < num_frames; i++)
		{

			/* get decompression flags */
			temp_4byte = (int *) data_ptr;
			/* if (byteswap == TRUE) *temp_4byte = swap_4byte (*temp_4byte); */
			compression_int = (int) *temp_4byte;

			for (j = 15; j >= 0; j--)
			{
				compression_flag[j] = (compression_int & MASK);
				compression_int >>= SHIFT;
			}

			data_ptr += 4;

/*                 +=======================================+                 */
/*=================|         decompress each frame         |=================*/
/*                 +=======================================+                 */

			/* decompress this frame of data */
			for (j = 1; j < 16; j++)
			{
				switch (compression_flag[j])
				{
					case 1:						/* type 1, 4 differences */
						for (k = 0; k < 4; k++, data_ptr += 1)
						{
							temp_byte = (char *) data_ptr;
							temp = (int) *temp_byte;
							if (temp > MAX8) temp -= 2 * (MAX8 + 1);

							/* should do some checking here */
							if ((i == 0) && (j == 3) && (k == 0)) 
								last_value = initial_value - temp;

							sample = (double) temp + last_value;
                       usegval = compression_flag[j];
							*(ydata[pt]+index+s) = (float) sample;
                                                        *(xdata[pt]+index+s) = index+s;
                                                        s++;
							last_value = sample;
							counter++;
						}
						break;

					case 2:						/* type 2, 2 differences */
						for (k = 0; k < 2; k++, data_ptr += 2)
						{
							temp_2byte = (short *) data_ptr;
							/* if (byteswap == TRUE)
								*temp_2byte = swap_2byte (*temp_2byte);
                                                        */
							temp = (int) *temp_2byte;
							if (temp > MAX16) temp -= 2 * (MAX16 + 1);

							/* should do some checking here */
							if ((i == 0) && (j == 3) && (k == 0)) 
								last_value = initial_value - temp;

							sample = (double) temp + last_value;
                       usegval = compression_flag[j];
							*(ydata[pt]+index+s) = (float) sample;
                                                        *(xdata[pt]+index+s) = index+s;
                                                        s++;
							last_value = sample;
							counter++;
						}
						break;

					case 3:						/* type 3, 1 differences */
						for (k = 0; k < 1; k++, data_ptr += 4)
						{
							temp_4byte = (int *) data_ptr;
							/* if (byteswap == TRUE)
								*temp_4byte = swap_4byte (*temp_4byte);
                                                        */
							temp = (int) *temp_4byte;
							if (temp > MAX32) temp -= 2 * (MAX32 + 1);

							/* should do some checking here */
							if ((i == 0) && (j == 3) && (k == 0)) 
								last_value = initial_value - temp;

							sample = (double) temp + last_value;
                       usegval = compression_flag[j];
							*(ydata[pt]+index+s) = (float) sample;
                                                        *(xdata[pt]+index+s) = index+s;
                                                        s++;
							last_value = sample;
							counter++;
						}
						break;

					case 0:						/* type 0, not data */
					default:
						data_ptr += 4;
						break;
				}								/* switch loop */
			}									/* single frame for loop */
		}										/* all frames for loop */
	}											/* while loop */
}												/* subprocedure */
