/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include <signal.h>
#include "user.h"

extern void remove_dtmp_files();
extern int  DEBUG;
int usegval;

void
xview_errors(object, list )
Xv_object    object;
Attr_avlist  list;
{
	Attr_avlist attr;
	int n=0;

	printf("BLZ 430 - 433 XView Vol. 7\n");
	printf("Error: %s\n",xv_error_format(object,list));

	for(attr = list; *attr && n < 5; attr = attr_next(attr) ) {

		switch ((int) attr[0]) {

		    case ERROR_BAD_ATTR:
			    printf("bad attribute %s", attr_name(attr[1]));
			    break;
		    case ERROR_BAD_VALUE:
			    printf("bad value (0x%x) for attribute %s",
			        attr[1], attr_name(attr[2]));
			    break;
		    case ERROR_INVALID_OBJECT:
			    printf( "invalid object (%s)", (char *) attr[1]);
			    break;
		    case ERROR_STRING:
			    printf( "error string (%s)", (char *) attr[1]);
			    break;
		}
	}


}

void
xlib_errors(display, xerror)
Display     *display;
XErrorEvent *xerror;
{
	char msg[80];

	XGetErrorText ( display, xerror->error_code, msg, 80);
	fprintf("Error detected: %s\n",msg);
}



void
my_sigint()
{
	printf("\n\n\tProgram interrupted.\n");
        remove_dtmp_files();
	signal(SIGFPE, SIG_DFL );
        exit(1);
}

void
my_sigquit()
{
	printf("\n\n\tProgram quit.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_sigill()
{
	printf("\n\n\tProgram quit.\n");
        printf("\tIllegal instruction.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_sigtrap()
{
	printf("\n\n\tProgram quit.\n");
        printf("\tTrace trap.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_sigabrt()
{
	printf("\n\n\tProgram aborted.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_sigemt()
{
	printf("\n\n\tProgram exited.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_sigfpe()
{
	printf("\n\n\tFloating point error.\n");
        remove_dtmp_files();
	signal(SIGFPE, SIG_DFL );
        exit(1);
}

void
my_sigbus()
{
	printf("\n\n\tBus error.\n");
        remove_dtmp_files();
        signal(SIGBUS, SIG_DFL );
        exit(1);
}

void
my_sigsegv()
{
	printf("\n\n\tSegmentation violation.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_sigsys()
{
	printf("\n\n\tProgram exited.\n");
        remove_dtmp_files();
        exit(1);
}

void
my_siglost()
{
	printf("\n\n\tProgram exited.\n");
        remove_dtmp_files();
        exit(1);
}



void
setup_error_handlers()
{

	signal(SIGQUIT, my_sigquit );
	signal(SIGILL, my_sigill );
	signal(SIGTRAP, my_sigtrap );
	signal(SIGABRT, my_sigabrt );
	signal(SIGEMT, my_sigemt );
	signal(SIGFPE, my_sigfpe );
	signal(SIGBUS, my_sigbus );
	signal(SIGSEGV, my_sigsegv ); 
	signal(SIGSYS, my_sigsys );
	signal(SIGLOST, my_siglost );
	signal(SIGINT, my_sigint );

	if( strncmp("sleeman",getenv("USER"),7)==NULL) DEBUG = TRUE;
        else DEBUG = FALSE;
}
