/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include "conver.h" 
#include "seed.h"
#include "user2.h" 

#define LOOP_OVER_TIME_ENTRIES for ( time_entry=VI->Root_Time_Entry;\
                                     time_entry!=NULL; \
                                     time_entry=time_entry->Next)
#define LOOP_OVER_BLOCK74      for ( bl74=time_entry->Root_Block74;\
                                     bl74 != NULL;\
                                     bl74=bl74->Next)

extern char          *evline[10];
extern evinfo         event[10];
extern STATION_STRUCT *stinfo[];

extern int           Nevents, NST, selevt, SEL_DAY;
extern int HBB_DATA, BB_DATA, MP_DATA, LP_DATA, VLP_DATA, ULP_DATA;



/* Fill event lines with event information in SEED volume  */

void
fill_evlines( inday )
int inday;
{
	TIME_ENTRY *te;
	EVENT_ENTRY *ev;
	int	evno, evmonth, year, jd, day;

	Nevents = 0;

	for(te = VI->Root_Time_Entry; te!=NULL; te=te->Next ) {


           if(te->Time_Span_ID->Start_Time.day == inday ) {


             for(ev=te->Root_Event,evno=0;ev!=NULL;ev=ev->Next,evno++) {

                year = ev->Origin_Time.year;
                jd = ev->Origin_Time.day;
                evmonth = day_of_month ( jd, &day, year );
/* printf("year = %d  jd= %d day = %d  evmonth = %d\n",year,jd,day,evmonth); */
                ev->Origin_Time.day = day;
                evline[evno] = (char *)malloc(81);
 
                if(ev->Number_Magnitudes == 1 )
                        sprintf ( evline[evno], "%d %d  %d     %02d:%02d:%02d.%d   %5.2lf  %5.2lf    %5.1lf  %5.1lf ", 
                                 ev->Origin_Time.year,
                                 evmonth,
                                 ev->Origin_Time.day,
                                 ev->Origin_Time.hour,
                                 ev->Origin_Time.minute,
                                 ev->Origin_Time.second,
                                 ev->Origin_Time.msec,
 	                         ev->Lattitude,
 	                         ev->Longitude,
 	                         ev->Depth,
	                         ev->Magnitude[0]
                                 );
                else if(ev->Number_Magnitudes == 2 )
                        sprintf ( evline[evno], "%d %d  %d     %02d:%02d:%02d.%d   %5.2lf  %5.2lf    %5.2lf  %5.1lf  %5.1lf ", 
                                 ev->Origin_Time.year,
                                 evmonth,
                                 ev->Origin_Time.day,
                                 ev->Origin_Time.hour,
                                 ev->Origin_Time.minute,
                                 ev->Origin_Time.second,
                                 ev->Origin_Time.msec,
 	                         ev->Lattitude,
 	                         ev->Longitude,
 	                         ev->Depth,
	                         ev->Magnitude[0],
	                         ev->Magnitude[1]
                                 );
                else
                        sprintf ( evline[evno], "%d %d  %d     %02d:%02d:%02d.%d   %5.2lf  %5.2lf    %5.2lf ", 
                                 ev->Origin_Time.year,
                                 evmonth,
                                 ev->Origin_Time.day,
                                 ev->Origin_Time.hour,
                                 ev->Origin_Time.minute,
                                 ev->Origin_Time.second,
                                 ev->Origin_Time.msec,
 	                         ev->Lattitude,
 	                         ev->Longitude,
 	                         ev->Depth
                                 );
                 

                 sscanf ( evline[evno],"%d %d %d %d:%d:%d.%d %f %f %f %f %f",
                         &event[evno].time.year, &event[evno].month, &event[evno].day_of_month,
                         &event[evno].time.hour, &event[evno].time.minute,
                         &event[evno].time.second, &event[evno].time.msec,
                         &event[evno].lat, &event[evno].lon,
                         &event[evno].dep, &event[evno].mb, &event[evno].ms );
/* printf("event %s\n",evline[evno]); */
		Nevents++;
	     }

        }
    }

return;
/*
    if(Nevents==0) {
      evno = 0;
      Nevents=1;
      evline[evno] = (char *)malloc(81);
      printf("Give time: (dag:uur:minuut:sec.msec)  ");
      scanf("%d:%d:%d:%d.%d",&p1,&p2,&p3,&p4,&p5);
      printf("Give lat , lon and depth:  ");
      scanf("%f %f %f",&f1,&f2,&f3);
      sprintf(evline[evno],"%d %d  %d     %02d:%02d:%02d.%d   %5.2lf  %5.2lf    %5.2lf ",
              p1,p1,p1,p2,p3,p4,p5,f1,f2,f2); 
      
    }
*/

}


/* Count number of stations in SEED volume */

void 
count_nst()
{
	STATION_ENTRY *ste;
	int i;

	NST=0;
	for ( ste=VI->Root_Station; ste!=NULL; ste=ste->Next)
		NST++;

         /*  Allocate memory for station information */
         for ( i=0; i<NST; i++ ) { 
                 stinfo[i] = ( STATION_STRUCT *) NULL ;
                 stinfo[i] = ( STATION_STRUCT *) init_stinfo ();
                 if(stinfo == NULL ) {
                    printf("Error in count_st\n");
                    exit(1);
                 }
		 stinfo[i]->cdorder = i;

	 }
}


/* Fill part of structure stinfo with station information in SEED volume  */

void 
fill_stinfo()
{
        int            p;
	STATION_ENTRY  *ste;


	for ( ste=VI->Root_Station, p=0; ste!=NULL; ste=ste->Next, p++) {

		strcpy ( stinfo[p]->station_code, ste->Call_Letters );
		stinfo[p]->flat = ste->Lattitude;
		stinfo[p]->flon = ste->Longitude;
		stinfo[p]->felev = ste->Elevation;


                calc_refpt ( event[selevt].lat, event[selevt].lon, selevt );
                calc_epi_float (stinfo[p]->flat,stinfo[p]->flon,p);
	}


}




/* Determine type of data for station stcode in SEED volume */

void 
det_data_type( stcode )
char *stcode;
{
        int      p;
	int      hbb_flag, bb_flag, mp_flag, lp_flag, vlp_flag, ulp_flag;

	STATION_ENTRY *ste;
        CHANNEL_ENTRY *che;
        BLOCK74       *bl74;
        TIME_ENTRY    *time_entry;

        for ( ste=VI->Root_Station, p=0; ste!=NULL; ste=ste->Next, p++) 
                if ( strcmp(ste->Call_Letters, stcode) == NULL ) break;
        

        LOOP_OVER_TIME_ENTRIES {
             LOOP_OVER_BLOCK74 {
 
                     if ( strncmp( bl74->Station_ID,stcode,strlen(stcode)) != NULL )
                               continue;
                     if ( time_entry->Time_Span_ID->Start_Time.day != SEL_DAY )
                               continue;

                     /* Only high gain seismometer */

		     if ( bl74->Channel_ID[1] != 'H' ) continue;

	             hbb_flag = bb_flag = mp_flag = lp_flag = vlp_flag = ulp_flag = FALSE;

                     if ( bl74->Channel_ID[0] == 'H' || bl74->Channel_ID[0] == 'S' ) {
                                  hbb_flag = TRUE;
                                  stinfo[p]->hbb = TRUE;
                     }
                     if ( bl74->Channel_ID[0] == 'B' ) {
                                  bb_flag = TRUE;
                                  stinfo[p]->bb = TRUE;
                     }
                     if ( bl74->Channel_ID[0] == 'M' ){
                                  mp_flag = TRUE;
                                  stinfo[p]->mp = TRUE;
                     }

                     /* For geoscope cdrom only BB and MP data can be handled so far */
                     if(!GEOSCOPE) {
                         if ( bl74->Channel_ID[0] == 'L' ) {
                                  lp_flag = TRUE;
                                  stinfo[p]->lp = TRUE;
                         }
                         if ( bl74->Channel_ID[0] == 'V' ) {
                                  vlp_flag = TRUE;
                                  stinfo[p]->vlp = TRUE;
                         }
                         if ( bl74->Channel_ID[0] == 'U' ){
                                  ulp_flag = TRUE;
                                  stinfo[p]->ulp = TRUE;
                         }
                     }

		     HBB_DATA |= hbb_flag;
		     BB_DATA |= bb_flag;
		     MP_DATA |= mp_flag;
		     LP_DATA |= lp_flag;
		     VLP_DATA |= vlp_flag;
		     ULP_DATA |= ulp_flag;

             }


        }

}
