/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#define PI 3.14159265

#include "user.h"
#include "popup_bp_ui.h"
#include "popup_hp_ui.h"

popup_bp_popup1_objects         *popup_bp_popup1;
popup_hp_popup1_objects         *popup_hp_popup1;

float        upper_freq, lower_freq, corner_freq;

extern double                           TPB, TPE;
extern OLsunic1_base_window_objects     *bfr;
extern STATION_STRUCT                   *stinfo[];
extern int                              stbuf[];


void
bp_filter_coeff ( freq2, freq1 , a, b, dt )
float freq1, freq2, a[], b[], dt;
{
	float dum2, dum1, prod;

	dum1 = tan( PI * freq1 * dt );
	dum2 = tan( PI * freq2 * dt );
        prod = (1+dum1) * (1+dum2);

	a[0] = 0.0;
	a[1] = -dum2/prod;
        a[2] = -a[1];
        b[0] = 1.0;
        b[1] = ((1+dum1)*(1-dum2) + (1-dum1)*(1+dum2)) / prod;
        b[2] = (-1) * (1-dum1)*(1-dum2)/prod;
	

}

void
hp_filter_coeff (  freq1 , a, b, dt )
float freq1, a[], b[], dt;
{
	float dum1;

	dum1 = tan( PI * freq1 * dt );

	a[0] = 1/(1+dum1);
        a[1] = -a[0];
        b[0] = 0.0;
        b[1] = (1-dum1)/(1+dum1);
}



/*
 * Menu handler for `filter_menu (band pass filter)'.
 */
Menu_item
bp_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
                                            xv_get(item, XV_KEY_DATA, INSTANCE);
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
	        xv_set ( popup_bp_popup1->popup1, XV_SHOW, TRUE, NULL);
                xv_set ( bfr->base_window, 
                         FRAME_SHOW_FOOTER, TRUE,
                         FRAME_LEFT_FOOTER, "Type in upper corner frequency, then RETURN:",
                         FRAME_RIGHT_FOOTER, " ",
                         NULL);
		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}

/*
 * Menu handler for `filter_menu (high pass filter)'.
 */
Menu_item
hp_handler(item, op)
Menu_item	item;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
                                            xv_get(item, XV_KEY_DATA, INSTANCE);
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
	        xv_set ( popup_hp_popup1->popup1, XV_SHOW, TRUE, NULL);
		break;

	case MENU_NOTIFY_DONE:
		break;
	}
	return item;
}


/*
 * Notify callback function for `textfield1'.
 */
Panel_setting
read_lower_freq(item, event)
Panel_item	item;
Event		*event;
{
	popup_bp_popup1_objects	*ip = (popup_bp_popup1_objects *) 
                                           xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);
	
        sscanf ( value, "%f", &lower_freq );
        
        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, "Type in upper corner frequency, then RETURN:",
                 FRAME_RIGHT_FOOTER, " ",
                 NULL);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield2'.
 */
Panel_setting
read_upper_freq(item, event)
Panel_item	item;
Event		*event;
{
	popup_bp_popup1_objects	*ip = (popup_bp_popup1_objects *) 
                                            xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);

	
        sscanf ( value, "%f", &upper_freq );

        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, "Dismiss popup window by clicking LEFT mouse button on pushpin:",
                 FRAME_RIGHT_FOOTER, " ",
                 NULL);
	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield1'.
 */
Panel_setting
read_corner_freq(item, event)
Panel_item	item;
Event		*event;
{
	popup_hp_popup1_objects	*ip = (popup_hp_popup1_objects *) 
                                          xv_get(item, XV_KEY_DATA, INSTANCE);
	char *	value = (char *) xv_get(item, PANEL_VALUE);
	
        sscanf ( value, "%f", &corner_freq );
        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, "Dismiss popup window by clicking LEFT mouse button on pushpin:",
                 FRAME_RIGHT_FOOTER, " ",
                 NULL);
	return panel_text_notify(item, event);
}


/*
 * Done callback function for `popup1'.
 */
void
do_highpass(frame)
	Frame		frame;
{
	xv_set(frame, XV_SHOW, FALSE, 0);
        HP_FILTER = TRUE;
        BP_FILTER = FALSE;
        graphics_loop ( TPB, TPE ); 
}



/*
 * Done callback function for `popup1'.
 */
void
do_bandpass(frame)
	Frame		frame;
{

        if ( upper_freq != -1 && lower_freq != -1 ) {
	     xv_set(frame, XV_SHOW, FALSE, 0);
             BP_FILTER = TRUE;
             HP_FILTER = FALSE;
             graphics_loop ( TPB, TPE ); 
        }
        else {
                xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, "Type in lower and upper corner frequency :",
                 FRAME_RIGHT_FOOTER, " ",
                 NULL);
        }
}


bp_filter_data ( y, np, c, d )
float *y;
int np;
float c[], d[];
{
	float  x0, x1, x2;
	int i;
	
        /* start values */
        x0 = *y;
        x1 = *(y+1);

	/* n=0 */         *y = 0.0;
        /* n=1 */         *(y+1) = c[1]*x0 + d[1]*(*y);

	for ( i=2; i<np; i++ ) {
                x2 = *(y+i);
		*(y+i) = c[1]*x1 + c[2]*x0 + d[1]*(*(y+i-1)) + d[2]*(*(y+i-2));
                x0 = x1;
                x1 = x2;
	}
}

hp_filter_data ( y, np, c, d )
float *y;
int np;
float c[], d[];
{
	float  x0, x1;
	int i;
	
        /* start values */
        x0 = *y;
	/* n=0 */         *y = c[0]* x0 ;

	for ( i=1; i<np; i++ ) {
                x1 = *(y+i);
		*(y+i) = c[0]*x1 + c[1]*x0 + d[1]*(*(y+i-1));
                x0 = x1;
	}
}


void
nyquist ( dt, freq, stc )
float dt, *freq;
int stc;
{
             if ( (dt * (*freq) >= 0.5) ) {
                 printf("Freq. out of Nyquist interval for %s\n",
                         stinfo[stbuf[stc]]->station_code);
                 *freq = (0.5/dt)*0.95;
                 printf("Freq. set to %f\n",*freq);
             }
}

