/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
/****                                                                       ***/
/****                          MAIN.C                                       ***/
/****                                                                       ***/
/******************************************************************************/
#include "user.h"
#include "pop_ui.h"
#include "popup_bp_ui.h"
#include "popup_hp_ui.h"
#include "popup_log_ui.h"
#include "pop_evt_ui.h"
#include <X11/cursorfont.h>

char                    starray[MAX_STATIONS][6], stch[10], *val[7];
char                    label[48], path_name[240] , cbuf[10000], Orientation;
char                    *CD_year[20], *CD_month[12], *CD_evtfile[31];
char                    message_5[60], BandCode;

int   			i, nit, stbuf[MAX_STATIONS+1],UC;
int                     plchoice, nst_selected=0, cursor_shape;
int 			SHORTP_CHOSEN, LONGP_CHOSEN, INTERMP_CHOSEN;
int                     HBB_CHOSEN, BB_CHOSEN, MP_CHOSEN, LP_CHOSEN, VLP_CHOSEN;
int			ULP_CHOSEN, SEARCH_STATIONS, ACTIVE, DEBUG, NOSTAT;
int                     SEED, GDSN, DATA_ON_HDISC, DTYP, WELCOME, IPF_OPEN, PATIENT;
int                     SDATA_ON_HDISC[MAX_STATIONS+1][MAX_CHANNELS+1];
int                     i, nit, julday, chyear, selevt, num_channels;
int                     CD_day[31], NCH, pch, SEL_DAY, INP_SDF;
float                   epidist_max;
double                  evttimeinsec;

FILE                    *uf;
Cursor 			cursor;
Pixmap                	px;
Attr_attribute	        INSTANCE;
Xv_Window 		canvas_pw;
Display			*display;
XGCValues		gcval;
XID		        xid, bfr_xid;
GC	                gc, revgc;

OLsunic1_base_window_objects    *bfr;
pop_popup1_objects              *pop_popup1;
pop_evt_popup1_objects          *pop_evt_popup1;

extern path_elements         	dir_path;
extern int 	        	NST, SELS,NODIR, SET_BL74;
extern int                      DATA_ALLOCATED, DATA_BUF_ALLOCATED, SEED_DATA_BUF_ALLOCATED[];
extern float                    lower_freq, upper_freq;
extern int                      Nevents;
extern char                     *evline[];
extern double                   TPB, TPE;

extern popup_bp_popup1_objects  *popup_bp_popup1;
extern popup_hp_popup1_objects  *popup_hp_popup1;
extern popup_log_popup_log_objects  *popup_log_popup_log;
extern STATION_STRUCT 	        *stinfo[];

extern void                     setup_error_handlers();
extern void                     my_sigquit(), my_sigill(), my_sigtrap();
extern void                     my_sigfpe(),  my_sigbus(), my_sigsegv();
extern void                     my_sigint(), my_sigemt(), my_siglost();
extern void                     my_sigsys(), my_sigabrt();
extern void                     xview_errors();
extern void                     xlib_errors();
extern void                     remove_dtmp_files();
extern void                     init_filenames();



void
show_selection(n, str)
int n;
char *str;
{
	char message[41];

        switch(n) {

             case 2:
	             strcpy ( message, "Year:     ");
	             strcat ( message, dir_path.year );
	             xv_set ( bfr->message3, PANEL_LABEL_STRING, message, NULL);
                     break;

             case 3:
	             strcpy ( message, "Year/Month:   ");
	             strcat ( message, dir_path.year );
	             strcat ( message, "  ");
	             strcat ( message, dir_path.month );
	             xv_set ( bfr->message3, PANEL_LABEL_STRING, message, NULL);
                     break;

             case 4:
	             strcpy ( message, "Year/Month/Day:   ");
	             strcat ( message, dir_path.year );
	             strcat ( message, "  ");
	             strcat ( message, dir_path.month );
	             strcat ( message, "  ");
	             strcat ( message,  str);
	             xv_set ( bfr->message3, PANEL_LABEL_STRING, message, NULL);
                     break;
        }
}



void 
main(argc, argv)
int	argc;
char	**argv;
{
	void                            initialize_graphics_context ();
	void                            repaint_proc();
	void                            repaint_proc2();
        void                            canvas_event_proc();
        void                            cdlook_version(), scan_input();
	OLsunic1_base_window_objects    *OLsunic1_base_window;


        /* set up handlers for errors first ! */

        setup_error_handlers();

	DATA_ALLOCATED = -1;
	DATA_BUF_ALLOCATED = FALSE;
	SEED_DATA_BUF_ALLOCATED[0] = FALSE;
	SEED_DATA_BUF_ALLOCATED[1] = FALSE;
	SEED_DATA_BUF_ALLOCATED[2] = FALSE;
	SEED_DATA_BUF_ALLOCATED[3] = FALSE;
	IPF_OPEN = FALSE;
	WELCOME = TRUE;
	HP_FILTER = BP_FILTER = FALSE;
	SELS=0;
	PASS1 = TRUE;
	PATIENT = TRUE;
	SET_BL74=FALSE;
        upper_freq = lower_freq = -1;

        cdlook_version();

        scan_input();

	find_years ( dir_path.dev_name );

	/* Initialize XView, set up error routines for internal XView errors
					       and Xlib or Xserever errors .  */

	xv_init ( XV_INIT_ARGC_PTR_ARGV, &argc, argv,
	                      XV_ERROR_PROC, xview_errors,
	                      XV_X_ERROR_PROC, xlib_errors ,
	                      0);

	INSTANCE = xv_unique_key();

	/* Initialize user interface components.  */

	OLsunic1_base_window = OLsunic1_base_window_objects_initialize(NULL, NULL);

	xv_set ( OLsunic1_base_window->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_LEFT_FOOTER, "Select year by clicking RIGHT mouse button :",
			     FRAME_RIGHT_FOOTER, " ",
                             NULL);

        pop_popup1 = 
          pop_popup1_objects_initialize(NULL,OLsunic1_base_window->base_window);
        popup_bp_popup1 = 
          popup_bp_popup1_objects_initialize(NULL,OLsunic1_base_window->base_window);
        popup_hp_popup1 = 
          popup_hp_popup1_objects_initialize(NULL,OLsunic1_base_window->base_window);
        popup_log_popup_log= 
          popup_log_popup_log_objects_initialize(NULL,OLsunic1_base_window->base_window);
        pop_evt_popup1 = 
          pop_evt_popup1_objects_initialize(NULL, OLsunic1_base_window->base_window);



	bfr = OLsunic1_base_window;

        deactivate_buttons (0);

	/* Repaint procedure for plots ! */
        xv_set(bfr->canvas1, CANVAS_X_PAINT_WINDOW, TRUE,
                             CANVAS_REPAINT_PROC, repaint_proc,
                             0);

	canvas_pw  = canvas_paint_window(OLsunic1_base_window->canvas1);

	xv_set ( canvas_pw, WIN_CONSUME_EVENTS, WIN_NO_EVENTS,  NULL,
			    WIN_EVENT_PROC, canvas_event_proc,
                            NULL);

        /* Initialize temp. filenames */
        init_filenames();

	/* Initialize graphics contexts */
	initialize_graphics_context ( OLsunic1_base_window );


	/* Turn control over to XView.  */
	xv_main_loop(OLsunic1_base_window->base_window);
	exit(1);
    
}


void
scan_input()
{
	char answer[40];

/*
        fprintf(stdout,"\nAre data stored on CD-ROM  [1]  or in a single SEED volume  (2)  ?  -  ");
        gets(answer);
*/
        fprintf(stdout,"\nAre data stored on CD-ROM  [1]  or in a single SEED volume  (2)  ?\n");
        fprintf(stdout,"\nDo NOT consider GEOSCOPE CD-ROM's as single SEED volumes !       -  ");
        gets(answer);

        if ( strlen(answer) == 0 || answer[0] == '1') {

               fprintf(stdout,"\nGive pathname to CD-ROM [/cdrom] :  ");
               gets(answer);
               if ( strlen(answer) != 0 ) strcpy( dir_path.dev_name, answer );
               else strcpy ( dir_path.dev_name, "/cdrom");
               INP_SDF = FALSE;
       
        }
        else {
               fprintf(stdout,"\nGive (path)name of SEED volume:  ");
               gets(answer);
	       strcpy ( dir_path.dev_name, ".");
	       strcpy ( dir_path.year, ".");
	       strcpy ( dir_path.month, ".");
	       strcpy ( dir_path.file, answer);
               INP_SDF = TRUE;
        }

        if ( DEBUG ) {
               fprintf(stderr,"\nDebug mode is on.  OK ? [n] -   ");
               gets(answer);
               if ( strlen(answer) == 0 ) DEBUG = FALSE; 
               else  if ( answer[0] == 'n' ) DEBUG = FALSE;
        }

	return;
}




deactivate_buttons (Q )
int Q;
{

        BP_FILTER = FALSE;
        HP_FILTER = FALSE;
        if(Q==7) return;

        /* deactivate action buttons */
        xv_set ( bfr->writehd_button, PANEL_INACTIVE, TRUE, NULL);
        xv_set ( bfr->filter_button, PANEL_INACTIVE, TRUE, NULL);
        xv_set ( bfr->zoom_button, PANEL_INACTIVE, TRUE, NULL);
        xv_set ( bfr->response_button, PANEL_INACTIVE, TRUE, NULL);

        /* deactivate selection buttons */
        xv_set ( bfr->component_button, PANEL_INACTIVE, TRUE, NULL);
        if(Q==6) return;
        xv_set ( bfr->data_type_button, PANEL_INACTIVE, TRUE, NULL);
        if(Q==5) return;
        xv_set ( bfr->station_button, PANEL_INACTIVE, TRUE, NULL); 
        if(Q==4) return;
        xv_set ( bfr->event_button, PANEL_INACTIVE, TRUE, NULL);
        if(Q==3) return;
        xv_set ( bfr->day_button, PANEL_INACTIVE, TRUE, NULL);
        if(Q==2) return;
        xv_set ( bfr->month_button, PANEL_INACTIVE, TRUE, NULL);
        if(Q==1) return;

        if(Q==0) return;
}




void
set_flags ( VAL )
int VAL;
{
        SPDATA        = VAL;   LPDATA       = VAL;  IPDATA         = VAL;
        HBB_DATA      = VAL;   BB_DATA      = VAL;  MP_DATA        = VAL;
        LP_DATA       = VAL;   VLP_DATA     = VAL;  ULP_DATA       = VAL;
	SHORTP_CHOSEN = VAL;   LONGP_CHOSEN = VAL;  INTERMP_CHOSEN = VAL;
        HBB_CHOSEN    = VAL;   BB_CHOSEN    = VAL;  MP_CHOSEN      = VAL;
        LP_CHOSEN     = VAL;   VLP_CHOSEN   = VAL;  ULP_CHOSEN     = VAL;
}



void 
initialize_graphics_context ( OLsunic1_base_window )
OLsunic1_base_window_objects	*OLsunic1_base_window;
{
	XWindowAttributes  win_at;

	display          = (Display *) xv_get ( canvas_pw, XV_DISPLAY );
	xid              = (XID) xv_get(canvas_pw,XV_XID);
	gcval.foreground = BlackPixel(display,DefaultScreen(display));
	gcval.background = WhitePixel(display,DefaultScreen(display));
	gc               = XCreateGC(display,xid,GCForeground|GCBackground, &gcval);
	gcval.foreground = WhitePixel(display,DefaultScreen(display));
	gcval.background = BlackPixel(display,DefaultScreen(display));
        gcval.graphics_exposures = TRUE;
	revgc            = XCreateGC(display,xid,GCForeground|GCBackground|GCGraphicsExposures, &gcval);


	XGetWindowAttributes ( display, xid, &win_at);
	px = XCreatePixmap ( display,xid, win_at.width, win_at.height, win_at.depth);
        XCopyArea(display, xid, px, gc, 0,0, win_at.width, win_at.height);
        XClearWindow ( display, xid );
}



void
cdlook_version()
{
        printf("\n\n\n\t\t\t\t    CDLOOK\n");
 	printf("\n\n\t        Written by Reinoud Sleeman, Orfeus Data Center.\n");

 	printf("\n\n   /----------------------------------------------------------------------\\\n");
	printf(" >>    February 1992     Version 1.9     ODC   Utrecht, The Netherlands    <<\n ");
        printf("  \\----------------------------------------------------------------------/\n");
        printf("\n           A way of monitoring waveforms of digital earthquake data from: \n\n");
        printf("      * -  Event oriented data on CD-ROM's as distributed by ORFEUS,\n");
        printf("           NEIC and POSEIDON.\n");
        printf("      * -  SEED volumes, version 2.1    \n");
        printf("      * -  Triggered data on GEOSCOPE CDROM's  (Broad Band and Mid Period data). \n");  
        printf("\n\n\t   Please send all your suggestions, remarks or questions to:\n");
        printf("\n\t\t\tReinoud Sleeman\n");
        printf("\t\t\tOrfeus Data Center\n");
        printf("\t\t\tBudapestlaan 4   \n");
        printf("\t\t\t3584 CD Utrecht \n");
        printf("\t\t\tThe Netherlands \n\n");
        printf("\t\t\tE-mail:  sleeman@eurydice.geof.ruu.nl\n");
        printf("\t\t\tFAX: (int).30.535030\n");
        printf("\t\t\tTEL: (int).30.535143\n\n");
	printf("    -----------------------------------------------------------------   \n\n\n");

}




/****** ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ **************/
/*      +             Callback procedures                      +              */
/****** ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ **************/


/* Notify callback function for `station_button'.  */

void
station_notify(item, event)
Panel_item	item;
Event		*event;
{
	OLsunic1_base_window_objects	*ip = (OLsunic1_base_window_objects *) 
                                            xv_get(item, XV_KEY_DATA, INSTANCE);

	int i, nrows;
	char *stlabel[MAX_STATIONS];
	
        nrows = (int)xv_get ( pop_popup1->list1, PANEL_LIST_NROWS, NULL);

	for ( i=nrows; i>=0; i-- ) 
              xv_set ( pop_popup1->list1, PANEL_LIST_DELETE, i, NULL);

        nst_selected = 0;


	for ( i=0; i< NST;i++ ) {
		stlabel[i] = (char *) malloc(30);
		sprintf(stlabel[i],"%-5s  %7.3f ",stinfo[i]->station_code,stinfo[i]->epi);

	        xv_set (pop_popup1->list1, PANEL_LIST_INSERT, i, 
                                           PANEL_LIST_STRING, i, stlabel[i],
                                           NULL); 
                stinfo[i]->selected = FALSE;
	}

	xv_set ( pop_popup1->popup1, XV_SHOW, TRUE, NULL);

        xv_set ( ip->base_window, 
                      FRAME_SHOW_FOOTER, TRUE,
                      FRAME_LEFT_FOOTER,
                      "Select / Deselect stations by clicking LEFT mouse button:",
	              FRAME_RIGHT_FOOTER, " ",
                      NULL);
}



/* Notify callback function for `list1'.  */

int
list_notify(item, string, client_data, op, event)
Panel_item	item;
char		*string;
Xv_opaque	client_data;
Panel_list_op	op;
Event		*event;
{
	pop_popup1_objects	*ip = (pop_popup1_objects *) 
                                           xv_get(item, XV_KEY_DATA, INSTANCE);

	int  len, r, i;
	char *stcode;

	stcode = (char *) malloc(6);
	
	switch(op) {
	case PANEL_LIST_OP_DESELECT:
                sscanf( string, "%s", stcode );
                nst_selected--;
		break;

	case PANEL_LIST_OP_SELECT:
                sscanf( string, "%s", stcode );
                nst_selected++;
		break;

	case PANEL_LIST_OP_VALIDATE:
		break;

	case PANEL_LIST_OP_DELETE:
		break;
	}


	for ( i=0; i<NST; i++ ) {
        
             len = strlen(stcode);

             if ( strncmp ( stcode, stinfo[i]->station_code, len ) == NULL ) {
                       if(stinfo[i]->selected ==  TRUE )
                                 stinfo[i]->selected = FALSE;
                       else
                                 stinfo[i]->selected = TRUE;
             }

	}

        r=0;
	for ( i=0; i<NST; i++ ) {
                    if(stinfo[i]->selected ==  TRUE) {
                          strcpy(starray[r],stinfo[i]->station_code);
		          r++;
		}
	}

        xv_set ( bfr->base_window, 
                      FRAME_SHOW_FOOTER, TRUE,
                      FRAME_LEFT_FOOTER,
                      "When you have finished your station selection dismiss popup\
                       window by clicking LEFT mouse button on pushpin:",
	              FRAME_RIGHT_FOOTER, " ",
                      NULL);

	return XV_OK;
}




/* Notify callback function for `zoom_button'.  */

void
zoom_notify(item, event)
Panel_item	item;
Event		*event;
{
      if(DEBUG) printf("Here it comes.\n");
}



/*  Notify callback function for `response_button'.  */

void
response_notify(item, event)
Panel_item	item;
Event		*event;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
                                            xv_get(item, XV_KEY_DATA, INSTANCE);
	
}



/* Event callback function for `quit_button'.  */

void 
quit_proc(item, event)
Panel_item	item;
Event		*event;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	    xv_get(item, XV_KEY_DATA, INSTANCE);

	xv_destroy_safe(ip);

        remove_dtmp_files();
        printf("\nProgram exited.\n");
	exit(1);
}


/* Event callback function for `zoom_button'.  */

void
zoom_proc(item, event)
Panel_item	item;
Event		*event;
{
	OLsunic1_base_window_objects	*ip = (OLsunic1_base_window_objects *) 
                                            xv_get(item, XV_KEY_DATA, INSTANCE);
	
        void canvas_event_proc();

        panel_default_handle_event(item, event); 

        if(event_is_down(event)) {

            cursor_shape = XC_crosshair;
	    cursor = XCreateFontCursor(display,cursor_shape);
            XDefineCursor(display,xid,cursor);

            xv_set(canvas_pw, WIN_CONSUME_EVENTS, 
                                      WIN_NO_EVENTS,
                                      LOC_MOVE, WIN_MOUSE_BUTTONS, 
                                      NULL,
                          WIN_EVENT_PROC, canvas_event_proc,
                          NULL);

            xv_set ( ip->base_window, 
                      FRAME_SHOW_FOOTER, TRUE,
                      FRAME_LEFT_FOOTER,
                      "Press LEFT mouse button to specify first time boundary .....",
	              FRAME_RIGHT_FOOTER, " ",
                      NULL);
          }


}


/*  Done callback function for `popup1'.  */

void
station_menu_dismissed(frame)
Frame	frame;
{
        if(DEBUG) fputs("pop: station_menu_dismissed\n", stderr);

	xv_set(frame, XV_SHOW, FALSE, 0);

        xv_set ( bfr->base_window, 
                      FRAME_SHOW_FOOTER, TRUE,
                      FRAME_LEFT_FOOTER,
                      "Select data type by clicking RIGHT mouse button:",
	              FRAME_RIGHT_FOOTER, " ",
                      NULL);

        xv_set ( bfr->data_type_button, PANEL_INACTIVE, FALSE, NULL);
        deactivate_buttons (6); 

}




/*  Event callback function for `response_button'.  */

void
response_proc(item, event)
Panel_item	item;
Event		*event;
{
        int r;
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
                                            xv_get(item, XV_KEY_DATA, INSTANCE);
	
        if ( event_is_down(event) ) {

             xv_set ( ip->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, "  ",
                 FRAME_RIGHT_FOOTER, "WRITING response ...",
                 NULL);
             for(r=0;r<nst_selected;r++) {
                    /*printf("extracting response for %d\n", stbuf[r]);*/
                    extract_response(stbuf[r]);
             }
        }

        xv_set ( ip->base_window, 
              FRAME_SHOW_FOOTER, TRUE,
              FRAME_RIGHT_FOOTER,"",
              FRAME_LEFT_FOOTER,
              "DONE! Channel response written to outputfile(s). Continue with selection or quit.",
              NULL);

        panel_default_handle_event(item, event);
}




/****** ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ **************/
/*      +              Menu handlers                           +              */
/****** ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ **************/




void 
fill_path_name (n)
int n;
{
        if ( n == 0 ) return;
	strcpy ( path_name, dir_path.dev_name);
	if ( n == 1 ) return;
	strcat ( path_name,"/");
	strcat ( path_name, dir_path.year);
	if ( n == 2 ) return;
	strcat ( path_name,"/");
	strcat ( path_name, dir_path.month);
	if ( n == 3 ) return;
	strcat ( path_name,"/");
	strcat ( path_name, dir_path.file);
	if ( n == 4 ) return;
}


static int 
sort_on_epidist ( el1, el2 )
STATION_STRUCT  **el1, **el2;
{
	if ( (*el1)->epi < (*el2)->epi ) return(-1);
	if ( (*el1)->epi == (*el2)->epi ) return(0);
	if ( (*el1)->epi > (*el2)->epi ) return(1);
}

static int 
sort_on_cdorder ( el1, el2 )
STATION_STRUCT  **el1, **el2;
{
	if ( (*el1)->cdorder < (*el2)->cdorder ) return(-1);
	if ( (*el1)->cdorder == (*el2)->cdorder ) return(0);
	if ( (*el1)->cdorder > (*el2)->cdorder ) return(1);
}


/* Menu handler for `year_menu_handler (1980)'.  */

Menu_item 
select_months(item, op)
Menu_item	item;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	                                    xv_get(item, XV_KEY_DATA, INSTANCE);

	static char *status_message = "You have selected: ";

	WELCOME = FALSE;

	switch (op) {
	case MENU_DISPLAY:

               /* Display message log */

               /*
	       xv_set ( popup_log_popup_log->popup_log, 
                              XV_X, 200,
                              XV_Y, 200,
                              XV_SHOW, FALSE, NULL);
	       */

		/*  Activate year-buttons which are present on CD-ROM */

		ACTIVE = FALSE;
		xv_set ( item, MENU_INACTIVE, TRUE, 0);

		strcpy ( label, xv_get(item,MENU_STRING) );
		for ( i=0; i<11; i++ ) {
			if ( CD_year[i] == NULL ) continue;
			if ( strcmp( label,CD_year[i] ) == 0 )
				ACTIVE = TRUE;
		}
		if ( ACTIVE )
			xv_set ( item, MENU_INACTIVE, FALSE, 0);
		break;

	case MENU_DISPLAY_DONE:

		break;

	case MENU_NOTIFY:

		xv_set(ip->message2, PANEL_LABEL_STRING, status_message, NULL);
		break;

	case MENU_NOTIFY_DONE:

		strcpy ( dir_path.year, xv_get(item,MENU_STRING) );
                sscanf ( dir_path.year, "%d", &chyear );

                show_selection(2, (char *) NULL );
		fill_path_name(2);
		find_months ( path_name );
		SEARCH_STATIONS = TRUE;

	        xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_LEFT_FOOTER, "Select month by clicking RIGHT mouse button :",
			     FRAME_RIGHT_FOOTER, " ",
                             NULL);

                xv_set ( ip->month_button, PANEL_INACTIVE, FALSE, NULL);
                deactivate_buttons (2);
		break;
	}
	return item;
}


/* Menu handler for `month_menu_handler (JAN)'.  */

Menu_item 
select_days(item, op)
Menu_item	item;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	                                   xv_get(item, XV_KEY_DATA, INSTANCE);
	static int         NO_CAPIT;

	switch (op) {
	case MENU_DISPLAY:

		/*  Activate month-buttons which are present on CD-ROM */

		ACTIVE = FALSE;
		NO_CAPIT = FALSE;
		xv_set ( item, MENU_INACTIVE, TRUE, 0);

		strcpy ( label, xv_get(item,MENU_STRING) );

		for ( i=0; i<12; i++ ) {
			if ( CD_month[i] == NULL ) continue;

			if ( CD_month[i][0] >= 97 && CD_month[i][0] <=122 ) {
                                NO_CAPIT = TRUE;
				if ( strncmp( label, capitalize(CD_month[i]), 3 ) == 0 )
					ACTIVE = TRUE;
			}
			if ( CD_month[i][0] >= 65 && CD_month[i][0] <=90 ) {
				if ( strncmp( label, CD_month[i], 3 ) == 0 )
					ACTIVE = TRUE;
			}
		}
		if ( ACTIVE )
			xv_set ( item, MENU_INACTIVE, FALSE, 0);

		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
		break;

	case MENU_NOTIFY_DONE:
		strcpy ( dir_path.month, xv_get(item,MENU_STRING) );

                if ( NO_CAPIT == TRUE ) 
                    strcpy ( dir_path.month ,capitalize_off ( dir_path.month ));
              
                show_selection(3, (char *) NULL );
		fill_path_name(3);
		find_days ( path_name );

		SEARCH_STATIONS = TRUE;
	        xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_LEFT_FOOTER, "Select day by clicking RIGHT mouse button :",
			     FRAME_RIGHT_FOOTER, " ",
                             NULL);

                xv_set ( ip->day_button, PANEL_INACTIVE, FALSE, NULL);
                deactivate_buttons (3);
		break;
	}
	return item;
}

/* Menu handler for `day_menu_handler'.  */

static char   month_tab[12][4] = {"JAN","FEB","MAR","APR","MAY","JUN","JUL","AUG",
                                  "SEP","OCT","NOV","DEC"};

Menu
show_days(menu, op)
Menu		menu;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	                                  xv_get(menu, XV_KEY_DATA, INSTANCE);
	Menu_item       mi;
	Menu            show_events();
	char            *day_label[31];
	char            user_info[80];
	char            dum[9];
        int             selday, m;

        void    init_val ();


	switch (op) {
	case MENU_DISPLAY:
                if(DEBUG)
		fputs("OLsunic1: show_days: MENU_DISPLAY\n", stderr);


		nit = xv_get(menu,MENU_NITEMS);
		for(i=nit;i>0;i--) {
			xv_set(menu,MENU_REMOVE,i,0);
			xv_destroy(xv_get(menu,MENU_NTH_ITEM, i));
		}


		for ( i=0; i<31; i++ ) {
			if ( CD_day[i] == 0 ) continue;

			day_label[i] = (char *) malloc(5);
			sprintf ( day_label[i], "%2d", CD_day[i] );

			mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
			    MENU_STRING, day_label[i],
			    MENU_VALUE, CD_evtfile[i],
			    MENU_NOTIFY_PROC, show_events,
			    MENU_RELEASE_IMAGE, 
			    NULL);
			xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
		}

		break;

	case MENU_DISPLAY_DONE:
                if(DEBUG)
		fputs("OLsunic1: show_days: MENU_DISPLAY_DONE\n", stderr);
		break;

	case MENU_NOTIFY_DONE:
                if(DEBUG)
		fputs("OLsunic1: show_days: MENU_NOTIFY_DONE\n", stderr);

		break;

	case MENU_NOTIFY:

	        xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_LEFT_FOOTER, " ",
			     FRAME_RIGHT_FOOTER, " Wait ... scanning events ",
                             /* FRAME_BUSY, TRUE, */
                             NULL);

                if(NODIR==FALSE)
		          strcpy ( dir_path.file, (char *)
		              xv_get( (int)xv_get(menu,MENU_SELECTED_ITEM), 
		              MENU_VALUE ) );

		strcpy ( user_info, (char *)
		             xv_get( (int)xv_get(menu,MENU_SELECTED_ITEM), 
		             MENU_STRING ) );

/*
		strcpy ( lab, (char *)
		           xv_get( (int)xv_get(menu,MENU_SELECTED_ITEM), 
		           MENU_STRING ) );
*/
                sscanf(user_info,"%d",&selday); 

                for(m=0;m<12;m++) {
                      if ( islower(dir_path.month[0]) != 0 ) {
                           if(strcmp(capitalize(dir_path.month),month_tab[m-1])==NULL) 
                                   break;
                      }
                      else {
                           if(strcmp(dir_path.month,month_tab[m-1])==NULL) 
                                   break;
                      }
                }

                julday = julian_day ( m, selday );
                SEL_DAY = julday;

                sprintf(dum,"    (%03d)",julday);
                strcat(user_info,dum);

                show_selection(4, user_info);
		fill_path_name(4);

if(NODIR==FALSE) {
		if ( IPF_OPEN == TRUE )  fclose(uf);

                
		if((uf = (FILE *) fopen(path_name,"r"))==NULL) {
			printf("File %s not opened.\n",path_name);
			exit(1);
		}
		else {
                        IPF_OPEN = TRUE;
		}
}



                if ( GDSN ) {
			if((fread ( cbuf, sizeof(char), 10000, uf))!=10000) {
				printf("Not read from file %s\n",path_name);
				exit(1);
			}
		       parse_indexlog ();
		       parse_eventlog ();

                }
                else if ( SEED ) {

	               xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_LEFT_FOOTER, " ",
			     FRAME_RIGHT_FOOTER, "Analyzing SEED volume .... ",
                             FRAME_BUSY, TRUE,
                             NULL);

                       /* skip parsing of the seed volume in case input is single file */  

                       if(NODIR==FALSE)
                               parse_seedfile (); 
		       strncpy ( dum, &dir_path.file[2], 3); dum[3] = '\0';
                       sscanf ( &dum[0], "%d", &julday );


                       fill_evlines(julday);
                       count_nst();
                }

                if(Nevents==0) {
                         UC=0;
                         init_val ();
                         xv_set ( pop_evt_popup1->popup1, XV_SHOW, TRUE, NULL);
                }
                else {
	                 xv_set ( ip->base_window, 
                                  FRAME_SHOW_FOOTER, TRUE,
	                          FRAME_LEFT_FOOTER, "Select event by clicking RIGHT mouse button: ",
		                  FRAME_RIGHT_FOOTER, " ",
                                  FRAME_BUSY, FALSE,
                                  NULL);
                         xv_set ( ip->event_button, PANEL_INACTIVE, FALSE, NULL);
                         deactivate_buttons (4);
                }

/*
                       count_nst();
                }
*/

		SEARCH_STATIONS = TRUE;

		nit = xv_get(menu,MENU_NITEMS);
		for(i=nit;i>0;i--){
			xv_set(menu,MENU_REMOVE,i,0);
			xv_destroy(xv_get(menu,MENU_NTH_ITEM, i));
		}

		break;
	}
	return menu;
}


void
init_val()
{

          if ( val[1] == (char *) NULL ) {
                      val[1] = (char *) malloc(6);
                      strcpy ( val[1], "000");
          }
          if ( val[2] == (char *) NULL ) {
                      val[2] = (char *) malloc(6);
                      strcpy ( val[2], "00");
          }
          if ( val[3] == (char *) NULL ) {
                      val[3] = (char *) malloc(6); 
                      strcpy ( val[3], "00");
          }
          if ( val[4] == (char *) NULL ) {
                      val[4] = (char *) malloc(6); 
                      strcpy ( val[4], "00.0");
          }
          if ( val[5] == (char *) NULL ) {
                      val[5] = (char *) malloc(6); 
                      strcpy ( val[5], "00.0");
          }
          if ( val[6] == (char *) NULL ) {
                      val[6] = (char *) malloc(6); 
                      strcpy ( val[6], "00.0");
          }
}

/* Menu handler for `event_menu_handler'.  */

Menu 
show_events(menu, op)
Menu		menu;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	    xv_get(menu, XV_KEY_DATA, INSTANCE);
	Menu_item       mi;
	char            user_info[80];
        void            recalc_epi_dist();



	switch (op) {
	case MENU_DISPLAY:

		nit = xv_get(menu,MENU_NITEMS);
		for(i=nit;i>0;i--) {
			xv_set(menu,MENU_REMOVE,i,0);
			xv_destroy(xv_get(menu,MENU_NTH_ITEM, i));

		}

		xv_set ( menu, MENU_TITLE_ITEM,
		    "           DATE       TIME          LAT   \
				LON      DEPTH     MB     MS   ", 
		    NULL);


		for ( i=0; i< Nevents;i++ ) {
			mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
			    MENU_STRING, evline[i],
			    MENU_VALUE, i,
			    MENU_RELEASE_IMAGE,
			    NULL);

			xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
		}


		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
                if(DEBUG)
		fputs("OLsunic1: show_events: MENU_NOTIFY\n", stderr);
		break;

	case MENU_NOTIFY_DONE:

	        xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_RIGHT_FOOTER, "Wait ... scanning stationlogs on CD-ROM. ",
	                     FRAME_LEFT_FOOTER, " ",
                             FRAME_BUSY, TRUE,
                             NULL);

		strcpy ( selected_evline, 
		    (char *) xv_get ( (int)xv_get(menu,MENU_SELECTED_ITEM),
		    MENU_STRING));
		selevt = (int) xv_get ( (int)xv_get(menu,MENU_SELECTED_ITEM),
		    MENU_VALUE);

		fill_path_name(4);


                /* sort stations on cdrom order */
		qsort ( &stinfo[0], NST, sizeof(STATION_STRUCT *), sort_on_cdorder );

                if ( GDSN )  {
		    strncpy ( user_info, &selected_evline[16], 51);
		    user_info[51]='\0'; 
                }
                if ( SEED ) {
		    strncpy ( user_info, &selected_evline[14], 51);
		    user_info[51]='\0'; 
                }

		xv_set(ip->message4, PANEL_LABEL_STRING, user_info, NULL);

		if ( GDSN ) parse_netlog ();

		if ( SEARCH_STATIONS == TRUE ) {

                        if ( GDSN ) {
			      for(i=0; i<NST; i++) {
                                      stinfo[i]->selected = FALSE;
                                      parse_stationlog(i);
				      SEARCH_STATIONS = FALSE;
                              }
			}
                        else if ( SEED ) {
                                      fill_stinfo ();
				      SEARCH_STATIONS = FALSE;
			}

			qsort ( &stinfo[0], NST, sizeof(STATION_STRUCT *), sort_on_epidist );
		}
                else {
			for(i=0; i<NST; i++) 
                                recalc_epi_dist (i); 

			qsort ( &stinfo[0], NST, sizeof(STATION_STRUCT *), sort_on_epidist );
		}

	        xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_RIGHT_FOOTER, " ",
	                     FRAME_LEFT_FOOTER, "Pop up station menu by clicking LEFT mouse button:",
                             FRAME_BUSY, FALSE,
                             NULL);

                xv_set ( ip->station_button, PANEL_INACTIVE, FALSE, NULL);
                deactivate_buttons (5); 

		nit = xv_get(menu,MENU_NITEMS);
		for(i=nit;i>0;i--){
			xv_set(menu,MENU_REMOVE,i,0);
			xv_destroy(xv_get(menu,MENU_NTH_ITEM, i));
		}
		break;
	}
	return menu;
}



/* Menu handler for `data_type_menu_handler (Short Period)'.  */

Menu 
select_components(menu, op)
Menu     	menu;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	                                    xv_get(menu, XV_KEY_DATA, INSTANCE);
        Menu_item mi;

	char type[40], user_info[60];
	int spflag, ipflag, lpflag, nit, i, r;
	void set_flags();

        epidist_max = -100;
        set_flags ( FALSE );

        for ( r=0; r<nst_selected; r++ ) {

              for (i=0; i<NST; i++) 
                  if ( strcmp(starray[r], stinfo[i]->station_code) == 0 ) break;

                  stbuf[r] = i;
                  if(stinfo[i]->epi > epidist_max)
                                 epidist_max = stinfo[i]->epi;

                  if ( GDSN ) {
                          spflag = lpflag = ipflag = FALSE;

                          if ( stinfo[i]->datablocks[1] != 0 ) spflag = TRUE;
                          if ( stinfo[i]->datablocks[2] != 0 ) lpflag = TRUE;
                          if ( stinfo[i]->datablocks[3] != 0 ) ipflag = TRUE;

                          SPDATA |= spflag;
	                  LPDATA |= lpflag;
		          IPDATA |= ipflag;
                  }

		  if ( SEED ) {
                          det_data_type ( starray[r] );  
                          	
                  }

	}



	switch (op) {
	case MENU_DISPLAY:
		nit = xv_get(menu,MENU_NITEMS);
		for(i=nit;i>0;i--) {
			xv_set(menu,MENU_REMOVE,i,0);
			xv_destroy(xv_get(menu,MENU_NTH_ITEM, i));

		}
      

                if ( GDSN ) {
                     if ( SPDATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Short Period",
                                     MENU_VALUE, 1,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                     if ( LPDATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Long Period",
                                     MENU_VALUE, 2,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                     if ( IPDATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Intermediate Period",
                                     MENU_VALUE, 3,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                }

                if ( SEED ) {
                     if ( HBB_DATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "High Broad Band Data",
                                     MENU_VALUE, 4,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                     if ( BB_DATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Broad Band Data",
                                     MENU_VALUE, 5,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                     if ( MP_DATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Mid Period Data",
                                     MENU_VALUE, 6,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                     if ( LP_DATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Long Period Data",
                                     MENU_VALUE, 7,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }
                     if ( VLP_DATA == TRUE ) {
		          mi = (Menu_item) xv_create ( XV_NULL, MENUITEM,
		                     MENU_STRING, "Very Long Period Data",
                                     MENU_VALUE, 8,
		                     MENU_RELEASE_IMAGE,
		                     NULL);

		          xv_set ( menu, MENU_APPEND_ITEM, mi, NULL );
                     }

		}
		break;

	case MENU_DISPLAY_DONE:
                if(DEBUG) printf("select_components MENU_DISP_DONE\n");
		break;

	case MENU_NOTIFY:
                if(DEBUG) printf("select_components MENU_NOTIFY\n");
		break;

	case MENU_NOTIFY_DONE:
                if(DEBUG) printf("select_components MENU_NOTIFY_DONE\n");

		strcpy ( type, 
		    (char *) xv_get ( (int)xv_get(menu,MENU_SELECTED_ITEM),
		    MENU_STRING));
		DTYP = (int) xv_get ( (int)xv_get(menu,MENU_SELECTED_ITEM),
		    MENU_VALUE);

                if ( nst_selected == 1)
		      strcpy ( user_info, "Station/Type/Comp:     ");
                else  strcpy ( user_info, "Type/Comp:  ");

                if ( nst_selected == 1)
		      strcat ( user_info, stinfo[stbuf[0]]->station_code );
		strcat ( user_info, "  ");



                switch ( DTYP ) {
                        case 1:
				SHORTP_CHOSEN = TRUE;
				strcat ( user_info, "SP");
                                break;
      
                        case 2:
				LONGP_CHOSEN = TRUE;
				strcat ( user_info, "LP");
                                break;

                        case 3:
				INTERMP_CHOSEN = TRUE;
				strcat ( user_info, "IP");
                                break;
                        case 4:
                                HBB_CHOSEN = TRUE;
				strcat ( user_info, "HBB");
                                BandCode = 'S';
                                break;
                        case 5:
                                BB_CHOSEN = TRUE;
				strcat ( user_info, "BB");
                                BandCode = 'B';
                                break;
                        case 6:
                                MP_CHOSEN = TRUE;
				strcat ( user_info, "MP");
                                BandCode = 'M';
                                break;
                        case 7:
                                LP_CHOSEN = TRUE;
				strcat ( user_info, "LP");
                                BandCode = 'L';
                                break;
                        case 8:
                                VLP_CHOSEN = TRUE;
				strcat ( user_info, "VLP");
                                BandCode = 'V';
                                break;
                        case 9:
                                ULP_CHOSEN = TRUE;
				strcat ( user_info, "ULP");
                                BandCode = 'U';
                                break;

                        default:
				strcat ( user_info, "No Per.");
                                BandCode = 'Q';
                                break;
                }

                if ( nst_selected == 1 && GDSN == TRUE ) {

	               if ( data_type_avail_for_station ( stbuf[0] ) == FALSE )
                                  break;

		       get_channels_gdsn (stbuf[0]);

		       num_channels = stinfo[stbuf[0]]->number_channels[DTYP];
                }
                if ( nst_selected == 1 && SEED == TRUE) {

                       num_channels = get_channels_seed ( stbuf[0] );

                }

                xv_set ( ip->component_button, PANEL_INACTIVE, FALSE, NULL);
                deactivate_buttons (7);

                xv_set ( ip->base_window, 
                       FRAME_SHOW_FOOTER, TRUE,
	               FRAME_LEFT_FOOTER, "Select component by clicking RIGHT mouse button",
	               FRAME_RIGHT_FOOTER, " ",
                       NULL);



                strcpy ( message_5, user_info );
                xv_set ( ip->message5, PANEL_LABEL_STRING, user_info, NULL);
		break;
	}
	return menu;
}


/* Menu handler for `component_menu_handler (vertical      Z)'.  */

Menu_item  
mkplot(item, op)
Menu_item	item;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *)
	                                    xv_get(item, XV_KEY_DATA, INSTANCE);

	char      type[20], dum[20], user_info[40];
	int       r, ev_julday, evmonth;
	double    ev_in_seconds, wte, wtb;
        float     msec;
        STDTIME   evtime;


	switch (op) {
	case MENU_DISPLAY:


		strcpy ( type, xv_get(item,MENU_STRING));
                if(DEBUG) printf("type = %s\n",type);

		xv_set(item,MENU_INACTIVE,TRUE,0);

		if ( type[0] == 'Z' ) {
				xv_set(item,MENU_INACTIVE,FALSE,0);
		}
                else if ( type[0] == 'N' || type[0] == 'E' ) {
			if ( num_channels == 3 )
				xv_set(item,MENU_INACTIVE,FALSE,0);
                        if ( nst_selected > 1 )
				xv_set(item,MENU_INACTIVE,FALSE,0);
		}
		else if ( type[0] == 'A')   {
			if ( num_channels == 3  && nst_selected == 1 )
				xv_set(item,MENU_INACTIVE,FALSE,0);
		}

                        

		break;

	case MENU_DISPLAY_DONE:

		break;

	case MENU_NOTIFY:

		strcpy ( type, xv_get(item,MENU_STRING));
                if(DEBUG) printf("type = %s\n",type);

                if ( type[0] ==  'Z' ) Orientation = 'Z' ;
                else if ( type[0] ==  'N' ) Orientation = 'N' ;
                else if ( type[0] ==  'E' ) Orientation = 'E' ;
                else if ( type[0] ==  'A' ) Orientation = 'A' ;

	        xv_set ( ip->base_window, 
                             FRAME_SHOW_FOOTER, TRUE,
	                     FRAME_RIGHT_FOOTER, "Be patient .....reading data from CD-ROM.",
	                     FRAME_LEFT_FOOTER, " ",
                             FRAME_BUSY, TRUE,
                             NULL);

		if ( type[0] ==  'A' )  NCH = 3;
                else  NCH = 1;

                if ( type[0] ==  'Z' ) pch = 0;
                if ( type[0] ==  'N' ) pch = 1;
                if ( type[0] ==  'E' ) pch = 2;


                strcpy(user_info, message_5 );

                if ( type[0] != 'A' ) {
                	sprintf ( dum, "%c", type[0] );
                	strcat ( user_info, dum );
                        xv_set ( ip->message5, PANEL_LABEL_STRING, user_info, NULL);
	        }
                else
                        xv_set ( ip->message5, PANEL_LABEL_STRING, user_info, NULL);



	        sscanf(selected_evline,"%*d %d %d %2d:%2d:%f",&evmonth,&evtime.day,
	               &evtime.hour,&evtime.minute,&msec);

                if(DEBUG) printf("selected evtline: %s\n",selected_evline);

	        evtime.second = (int)floor((double)msec);
                evtime.msec = (int)(fmod((double)msec,(double)1)*1000);


                printf("(1)selected evtime:  %d  %d:%d:%d.%d\n",
	            evtime.day,evtime.hour,evtime.minute,evtime.second,evtime.msec);
                ev_julday = julian_day ( evmonth, evtime.day );

                if(DEBUG) printf("evjulday = %d\n",ev_julday);

	        ev_in_seconds = 
                    (double) ev_julday * 86400  +
                    (double) evtime.hour * 3600  +
	            (double) evtime.minute * 60 +
	            (double) evtime.second +
	            (double) evtime.msec/1000;


                /* Set begin of plot equal to begin time of event,
                   and endtime to some big value.                  */

	        wtb = ev_in_seconds;
                wte = wtb + 7200;    /* 2 hours extra  */

/*
		if ( selevt+1 <  Nevents ){

	             sscanf(evline[selevt+1],"%*d %*d %d %2d:%2d:%2d.%d",&evtime.day,
	               &evtime.hour,&evtime.minute,&evtime.second,&evtime.msec);

                printf("(2)selected evtime:  %d  %d:%d:%d.%d\n",
	            evtime.day,evtime.hour,evtime.minute,evtime.second,evtime.msec);
                     ev_julday2 = julian_day ( evmonth, evtime.day );

	             ev_in_seconds = 
                         (double) ev_julday2 * 86400  +
                         (double) evtime.hour * 3600  +
	                 (double) evtime.minute * 60 +
	                 (double) evtime.second +
	                 (double) evtime.msec/1000;

                     wte = ev_in_seconds;
 
                     if ( (wte - wtb) > 7200 )  wte = wtb + 7200;
                }
*/

	        evttimeinsec = (double) ev_in_seconds;

		/*  Read the data, write them to hard disc and decode them */

                DATA_ON_HDISC = FALSE;

                remove_dtmp_files();
                init_filenames();

                NOSTAT = TRUE;

                if ( GDSN ) {
                     for(r=0;r<nst_selected;r++) {

			  if ( data_type_avail_for_station ( stbuf[r] ) == FALSE )
                                  continue;

                          NOSTAT = FALSE;
		          read_data (stbuf[r]);
                          analyze_gdsn_data (stbuf[r]);
                     }
                }


                if ( SEED ) {
                     for(r=0;r<nst_selected;r++) {
                         if ( analyze_seed_data ( stbuf[r] ) == TRUE ) {
                              NOSTAT = FALSE;
                         }
                        
/*
			  if ( read_seed_data ( stbuf[r] ) == TRUE ) {
                               NOSTAT = FALSE;
                               analyze_seed_data ( stbuf[r] ); 
                          }
*/
                     }

                }
                if(NOSTAT == TRUE ) {
                     printf("No data for this station(s) on this day.\n"); 
                }

                DATA_ON_HDISC = TRUE;


		/*  Start the graphics part ! */

		graphics_loop (wtb,wte );

                xv_set ( ip->writehd_button, PANEL_INACTIVE, FALSE, NULL);
                xv_set ( ip->filter_button, PANEL_INACTIVE, FALSE, NULL);
                xv_set ( ip->zoom_button, PANEL_INACTIVE, FALSE, NULL);
                /*if(GDSN)*/
                xv_set ( ip->response_button, PANEL_INACTIVE, FALSE, NULL);

		break;

	case MENU_NOTIFY_DONE:
		break;
	}

	return item;
}

Menu_item
create_sacfile(item, op)
Menu_item	item;
Menu_generate	op;
{
	OLsunic1_base_window_objects *ip = (OLsunic1_base_window_objects *) 
                                           xv_get(item, XV_KEY_DATA, INSTANCE);
	
	switch (op) {
	case MENU_DISPLAY:
		break;

	case MENU_DISPLAY_DONE:
		break;

	case MENU_NOTIFY:
		break;

	case MENU_NOTIFY_DONE:

                sac_loop ( TPB, TPE );

                xv_set ( bfr->base_window, 
                      FRAME_SHOW_FOOTER, TRUE,
                      FRAME_LEFT_FOOTER,
                      "WRITING done ! Continue with selection, action or quit ",
	              FRAME_RIGHT_FOOTER, " ",
                      NULL);
		break;
	}
	return item;
}


/*
 * Notify callback function for `textfield1'.
 */
Panel_setting
ui_evjday(item, event)
Panel_item	item;
Event		*event;
{
	pop_evt_popup1_objects	*ip = (pop_evt_popup1_objects *)
                                               xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);

        if(strlen(value)!=0) strcpy(val[1],value);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield2'.
 */
Panel_setting
ui_evhr(item, event)
Panel_item	item;
Event		*event;
{
	pop_evt_popup1_objects	*ip = (pop_evt_popup1_objects *) 
                                               xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);
	
        if(strlen(value)!=0) strcpy(val[2],value);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield3'.
 */
Panel_setting
ui_evmin(item, event)
Panel_item	item;
Event		*event;
{
	pop_evt_popup1_objects	*ip = (pop_evt_popup1_objects *)
                                               xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);
	
        if(strlen(value)!=0) strcpy(val[3],value);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield4'.
 */
Panel_setting
ui_evsec(item, event)
Panel_item	item;
Event		*event;
{
	pop_evt_popup1_objects	*ip = (pop_evt_popup1_objects *)
                                               xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);
	
        if(strlen(value)!=0) strcpy(val[4],value);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield5'.
 */
Panel_setting
ui_evlat(item, event)
Panel_item	item;
Event		*event;
{
	pop_evt_popup1_objects	*ip = (pop_evt_popup1_objects *)
                                               xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);
	
        if(strlen(value)!=0) strcpy(val[5],value);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `textfield6'.
 */
Panel_setting
ui_evlon(item, event)
Panel_item	item;
Event		*event;
{
	pop_evt_popup1_objects	*ip = (pop_evt_popup1_objects *)
                                               xv_get(item, XV_KEY_DATA, INSTANCE);
	char *value = (char *) xv_get(item, PANEL_VALUE);
	
        if(strlen(value)!=0) strcpy(val[6],value);

	return panel_text_notify(item, event);
}

/*
 * Notify callback function for `evtok'.
 */
void
evt_known(item, event)
Panel_item      item;
Event           *event;
{
        pop_evt_popup1_objects  *ip = (pop_evt_popup1_objects *)
                                          xv_get(item, XV_KEY_DATA, INSTANCE);
        int evno;
        int jd, hr, min,m;
        int yr,dom;
        float sec;
        double lat, lon;

        evno=0;
        Nevents=1;
        evline[evno] = (char *)malloc(81);

	sscanf(val[1],"%d",&jd);
	sscanf(val[2],"%d",&hr);
	sscanf(val[3],"%d",&min);
	sscanf(val[4],"%f",&sec);
	sscanf(val[5],"%lf",&lat);
	sscanf(val[6],"%lf",&lon);
        
        sscanf(dir_path.year,"%d",&yr);
        m = day_of_month (jd, &dom, yr);

        sprintf(evline[evno],"%s %d  %d     %02d:%02d:%05.2f   %5.2lf  %5.2lf     ",
                dir_path.year,m,dom,hr,min,sec,lat,lon);


        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_RIGHT_FOOTER, " ",
	         FRAME_LEFT_FOOTER, "Select event by clicking RIGHT mouse button: ",
                 FRAME_BUSY, FALSE,
                 NULL);

        xv_set ( bfr->event_button, PANEL_INACTIVE, FALSE, NULL);
        deactivate_buttons (4); 
}


/*
 * Done callback function for `popup event window'.
 */
void
pop_evt_dismissed(frame)
Frame           frame;
{
        int evno;
        int jd, hr, min,m;
        int yr,dom;
        float sec;
        double lat, lon;
        fputs("pop_evt: pop_evt_dismissed\n", stderr);

        evno=0;
        Nevents=1;
        evline[evno] = (char *)malloc(81);

	sscanf(val[1],"%d",&jd);
	sscanf(val[2],"%d",&hr);
	sscanf(val[3],"%d",&min);
	sscanf(val[4],"%f",&sec);
	sscanf(val[5],"%lf",&lat);
	sscanf(val[6],"%lf",&lon);
        
        sscanf(dir_path.year,"%d",&yr);
        m = day_of_month (jd, &dom, yr);

        sprintf(evline[evno],"%s %d  %d     %02d:%02d:%05.2f   %5.2lf  %5.2lf    ",
                dir_path.year,m,dom,hr,min,sec,lat,lon);

        xv_set(frame, XV_SHOW, FALSE, 0);

        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_RIGHT_FOOTER, " ",
	         FRAME_LEFT_FOOTER, "Select event by clicking RIGHT mouse button: ",
                 FRAME_BUSY, FALSE,
                 NULL);

        xv_set ( bfr->event_button, PANEL_INACTIVE, FALSE, NULL);
        deactivate_buttons (4); 

}
