/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include "conver.h"
#include "seed.h"
#include "user2.h"

#define PI           3.14159265359
#define HALFPI        .5*PI
#define TWOPI         (2.0*PI)
#define SQRT2         1.41421356
#define SQRT3         1.73205081
#define DEG           (180./PI)        /* Degrees/Radian */
#define RAD           (PI/180.)        /* Radians/Degree */
#define EQTRAD        6378.163
#define POLRAD        6356.177
#define REARTH        6371.
#define FLATTENING    (EQTRAD-POLRAD)/EQTRAD

#define LOOP_OVER_STATIONS     for ( sty=VI->Root_Station;\
                                     sty!=NULL;\
                                     sty=sty->Next)
#define LOOP_OVER_CHANNELS     for ( chy=sty->Root_Channel;\
                                     chy!=NULL;\
                                     chy=chy->Next)


float          st0,ct0, phi0 ;
int            Nevents, NST;
char           *evline[10];
char           *stline[MAX_STATIONS];
evinfo         event[10];
STATION_STRUCT *stinfo[MAX_STATIONS];

void           calc_epi ();

extern char cbuf[], BandCode;
extern FILE *uf;
extern int  selevt, julday, chyear;
extern int  DTYP, GDSN, SEED;
extern char orientation;
extern int channel_order();

void parse_indexlog()
{
	int      nst, i, st, ist, iyear, iday;
        int      p1, p2, p3, p4;

     /*  Check date !   */
         sscanf(&cbuf[9],"%*d %d %d %d",&ist, &iyear, &iday);

     /*  Scan for number of stations in this file */
         sscanf(&cbuf[2400],"%*s %*s %*s %d",&nst);
         if ( nst > MAX_STATIONS ) {
		printf("Increase MAX_STATIONS in xnetdefine.h !\n");
		exit(1);
	 }

         if ( nst != ist ) 
                fprintf(stdout,"WARNING: Number of stations in indexlog and stationlog are not corresponding !\n");

         NST = nst;

     /*  Allocate memory for station information */
         for ( i=0; i<NST; i++ ) { 
                 stinfo[i] = ( STATION_STRUCT *) init_stinfo ();
		 stinfo[i]->cdorder = i;
	 }


	 if ( nst <= 24 ) {    /*  ONE COLUMN  */

              for ( i=0; i< nst; i++) {
                     sscanf(&cbuf[(i+1)*80],"%d %d %d %d",
                            &p1, &p2, &p3, &p4);

                     stinfo[i]->station_log = p1;
                     stinfo[i]->datablocks[1] = p2;
                     stinfo[i]->datablocks[2] = p3;
                     stinfo[i]->datablocks[3] = p4;
	      }
	
	 }
         else {               /*   TWO COLUMNS  */

               for(st=0;st<24;st++) {
                     sscanf(&cbuf[(st+1)*80],"%d %d %d %d",
                            &p1, &p2, &p3, &p4);
                     stinfo[st]->station_log = p1;
                     stinfo[st]->datablocks[1] = p2;
                     stinfo[st]->datablocks[2] = p3;
                     stinfo[st]->datablocks[3] = p4;
	       }
	       for(st=24;st<nst;st++) {
                     sscanf(&cbuf[(st-23)*80],"%*d %*d %*d %*d %d %d %d %d",
                            &p1, &p2, &p3, &p4);
                     stinfo[st]->station_log = p1;
                     stinfo[st]->datablocks[1] = p2;
                     stinfo[st]->datablocks[2] = p3;
                     stinfo[st]->datablocks[3] = p4;
	       }
	 }

}


void parse_eventlog()
{
	int i, log_offset;
        char line[81], word[20];
        float msec;


	if(NST>36)  log_offset = 6000;
        else        log_offset = 4000;

        i=0;
        do {
             
		strncpy ( line, &cbuf[log_offset+(i+7)*80], 80 );
                line[80] ='\0';
                strcpy(word,"");
                sscanf(line,"%s",word);
                if ( strlen(word) != 0 ) {
                     evline[i] = (char *)malloc(81);
                     strcpy ( evline[i], line );

                     sscanf ( evline[i],"%d %d %d %d:%d:%f %f %f %f %f %3f",
                            &event[i].time.year, &event[i].month, &event[i].day_of_month,
                            &event[i].time.hour, &event[i].time.minute,
                            &msec,
                            &event[i].lat, &event[i].lon,
                            &event[i].dep, &event[i].mb, &event[i].ms );

                     event[i].time.second = (int)floor((double)msec);
                     event[i].time.msec = (int)(fmod((double)msec,(double)1)*1000);
              

                     i++;
                }

                 
        } while ( (strlen(word) != 0) && i < 10 );

        Nevents = i;


}

void parse_netlog()
{

	int i, format;
        char line[81], stcode[8];

        i=0;

        for ( i=0; i< NST; i++) {
      
            if ( i < 18 ) {
             
		strncpy ( line, &cbuf[2000+ (i+7)*80], 80 );
                line[80] ='\0';

                stline[i] = (char *)malloc(81);
                strcpy ( stline[i], line );
                sscanf(line,"%*d %05s %*s %d",stcode, &format);
                strcpy ( stinfo[i]->station_code , stcode );
                stinfo[i]->formatID = format;
                 
            } 
            else if ( i >= 18 && i<36 ) {

		strncpy ( line, &cbuf[2000+ ((i-18)+7)*80], 80 );
                line[80] ='\0';

                stline[i] = (char *)malloc(81);
                strcpy ( stline[i], line );
                sscanf(line,"%*d %*s %*s %*d %*d %*d %05s %*s %d",stcode, &format);
                strcpy ( stinfo[i]->station_code , stcode );
                stinfo[i]->formatID = format;

	    }
            else if ( i >= 36 ) {

		strncpy ( line, &cbuf[4000+ ((i-36)+7)*80], 80 );
                line[80] ='\0';

                stline[i] = (char *)malloc(81);
                strcpy ( stline[i], line );
                sscanf(line,"%*d %05s %*s %d",stcode, &format);
                strcpy ( stinfo[i]->station_code , stcode );
                stinfo[i]->formatID = format;

	    }
        }

}


void parse_stationlog(st)
int st;
{
        int      chans,  ncor, dif1, dif2, evday, hour, evhour;
	int      i,j,nl,k,day[6],time[6], eq;
        long     offset;
	char     buff[2000], line[81], ch, word1[15], sgn[6];
        char     inst[20], slat[20], slon[20], selev[20];
        float    temp[6];


         offset = (stinfo[st]->station_log -1 ) * 2000;
         fseek ( uf, offset, 0 );
         
         fread ( buff, sizeof(char), 2000, uf );

         sscanf(&buff[12],"%d", &chans);

	 strncpy ( line, &buff[4*80], 80 );
         line[80] ='\0';

         sscanf(line,"%d %*s %s %s %s %s ",
                         &stinfo[st]->stationID,inst, slat, slon, selev);
         strcpy ( stinfo[st]->inst , inst);
         strcpy ( stinfo[st]->lat , slat);
         strcpy ( stinfo[st]->lon , slon);
         strcpy ( stinfo[st]->elev , selev);
         printf("station: %s  lat: %s  lon: %s\n",stinfo[st]->station_code, slat, slon);

         ch = selev[ strlen (selev)-1 ];
         if(ch == 'M' || ch == 'm') {
                 selev[ strlen (selev)-1 ] = '\0';
                 sscanf (selev, "%f", &stinfo[st]->felev);
         }

         calc_refpt ( event[selevt].lat, event[selevt].lon, selevt );
         calc_epi (slat,slon,st);

         /* j indicates the line with "TIME CORRECTIONS: ", and is 11 or 12 */
	 for(i=11;i<13;i++) {
		sscanf((buff+i*80),"%s",line);
                if(strncmp(line,"TIME",4)==NULL) j=i;
         }
 	 sscanf((buff+j*80),"%*s %*s %d",&ncor);
         if ( (ncor < 0) || (ncor > 6) )  ncor = 0;


         /*  Return if there are no time corrections  */
         if ( ncor == 0 ) return;

         for ( i=0; i < ncor; i++ ) {

             /*  Read line - store time corrections in array temp  */
             if ( i <= 2 ) nl = i;
             else nl = i-3;

             /* The value k is determined above for the line */
             strncpy(line,(buff+(j+2+nl)*80),80);
             line[80] = '\0';

             if ( i <= 2 ) sscanf (line,"%*s %*s %*s %s", word1);
             else  sscanf (line,"%*s %*s %*s %*s %*s %*s %*s %s", word1);

             if ( (strlen(word1)) != 0 ) {
                 if ( i <=2 )
                     sscanf (line, "%*s %d %d %f/%c", &day[i], &time[i], &temp[i],&sgn[i]);
                 else
                     sscanf (line, "%*s %*s %*s %*s %*s %d %d %f/%c", &day[i], &time[i], &temp[i], &sgn[i]);
             }

         }  /*  End do loop over time corrections  */

	 /* Time corrections are zero if sgn == 0 */
         for(i=0;i<ncor;i++)
             if(sgn[i]==0) temp[i]=0;

         /*  Return if there is only one time correction  */
         stinfo[st]->time_cor = temp[0];
         if ( ncor == 1 ) return;

         /*  Check if all time corrections are equal  */
         i = 0;
         do {
             if ( (fabs (temp[i] - temp[i+1])) < 1.0E-06 ) eq = 0;
             else eq = 1;
             i++;
         } while ( i < (ncor-1) && eq == 0 );

         /*  Return if all time corrections are eqaul  */
         stinfo[st]->time_cor  = temp[0];
         if ( eq == 0 ) return;
     
         /*  Select the time correction closest to evtime  */
         evhour = (int)event[selevt].time.hour;
         k = 0;
         dif1 = 24;
         evday = julian_day( event[selevt].month, event[selevt].day_of_month);

         for ( i=0; i < ncor; i++ ) {
             if ( day[i] == evday ) {
                 hour = time[i] / 100;
                 dif2 = abs ( evhour - hour );
                 if ( dif2 < dif1 ) {
                     dif1 = dif2;
                     k = i;
                 }
             }
         }
         stinfo[st]->time_cor = temp[k];


}


void
recalc_epi_dist (st)
int st;
{
         calc_refpt ( event[selevt].lat, event[selevt].lon, selevt );
         calc_epi (stinfo[st]->lat,stinfo[st]->lon,st);
}



void
extract_response(st)
int st;
{
	char sdum[81], dummy, part[80], buff[180];
	char outfile_name[19], chcode[4] ;
	int i,  Nz, y, channel[2], line, cal_year[2], cal_day[2];
        int NZS=0, NPS=0, checkvariable=0;
        int numpoles[4], numzeros[4], ch, NCH, Jday, Year;
        double poles_x[3][20], poles_y[3][20], zeros_x[3][20], zeros_y[3][20];
	long int  orig;
        double cal_freq[2], ppp, calibration[2], av_calibration[2], A0[4];
        double norm_freq[4], sens[4];
        FILE *fp[3];
        STATION_ENTRY *sty;
        CHANNEL_ENTRY *chy;
        char *find, scode[6];
        int MULTIPLEXED, numcals;



	MULTIPLEXED = FALSE;
        numcals = 0;

        if(GDSN) {
	     fseek(uf, (long) ((stinfo[st]->station_log -1) * 2000), 0);
             fread(buff,sizeof(char),160,uf);
             sscanf(&buff[80],"%*s %d %*s %d",&Jday,&Year);

             /* read number of channels */

	     fseek(uf, (long) ((stinfo[st]->station_log) * 2000), 0);
             fread(buff,sizeof(char),16,uf);
             buff[16] = '\0';
             sscanf(&buff[12],"%d",&NCH);
        }

        if ( SEED ) {
             NCH = get_channels_seed ( st );
             MULTIPLEXED = check_for_multiplex ( st, BandCode );
             Jday = julday;
             Year = chyear;
        }

        /* open outputfile(s) */

        switch ( DTYP ) {
                 case 1: strcpy ( chcode, "SP"); break;
                 case 2: strcpy ( chcode, "LP"); break;
                 case 3: strcpy ( chcode, "IP"); break;
                 case 4: strcpy ( chcode, "HB"); break;
                 case 5: strcpy ( chcode, "BB"); break;
                 case 6: strcpy ( chcode, "MP"); break;
                 case 7: strcpy ( chcode, "LP"); break;
                 case 8: strcpy ( chcode, "VL"); break;
                 case 9: strcpy ( chcode, "UL"); break;
	}
    
        for(ch=0;ch<NCH;ch++) {

             if(MULTIPLEXED==TRUE) {
                 if(ch==0) chcode[2] = '1';
                 if(ch==1) chcode[2] = '2';
                 if(ch==2) chcode[2] = '3';
                 chcode[3]='\0';
             }
             else {
                 if(ch==0) chcode[2] = 'Z';
                 if(ch==1) chcode[2] = 'N';
                 if(ch==2) chcode[2] = 'E';
                 chcode[3]='\0';
             }

             /* remove spaces from station name */

             strcpy(scode,stinfo[st]->station_code);
             if((find = strchr(scode,' '))!=NULL) *find = '\0';
             sprintf(outfile_name,"RESP.%s.%s",scode,chcode);

             if((fp[ch] = fopen(outfile_name,"a"))==NULL) {
                 printf("\tWarning: Outputfile %s not able for writing. \n",
                         outfile_name);
                 return;
             }
             else 
                 printf("Response output file: %s\n",outfile_name);

	     if((fprintf(fp[ch],"==========   CHANNEL RESPONSE DATA   =======\n"))==NULL) printf("Write error\n");
             fprintf(fp[ch],"Station:     %s\n",scode);
             fprintf(fp[ch],"Channel:     %s\n",chcode);
             fprintf(fp[ch],"Start date:  %d,%03d\n",Year,Jday);
             fprintf(fp[ch],"End date:    %d,%03d,23,59,59.999\n",Year,Jday);
	     fprintf(fp[ch],"============================================\n");
        }


        if(GDSN) {

	   fseek(uf,(long) ((stinfo[st]->station_log) * 2000), 0);
	   fseek(uf,(long)(8*80), 1);

           for(ch=0;ch<NCH;ch++) {

              channel[0] = 0; channel[1] = 0;
              calibration[0] = 0.0; calibration[1] = 0.0;
              av_calibration[0] = 0.0; av_calibration[1] = 0.0;
              cal_freq[0] = 0.0; cal_freq[1] = 0.0;
              cal_year[0] = 0; cal_year[1] = 0;
              cal_day[0] = 0; cal_day[1] = 0;

              for(line=0;line<=1;line++) {
                strcpy(sdum,"");
                fread(sdum,sizeof(*sdum),80,uf);
                strcpy(part,"0");strncpy(part,&sdum[5],6); part[5]='\0';
                /* read channel no */
                sscanf(part,"%d",&channel[line]);
                strcpy(part,"0");strncpy(part,&sdum[12],16); part[16]='\0';
                /* read calibration date */
                sscanf(part,"%d %d",&cal_year[line],&cal_day[line]);
                strcpy(part,"0.0");strncpy(part,&sdum[28],17); part[17]='\0';
                /* read calibration value */
                sscanf(part,"%lf",&calibration[line]);
                strcpy(part,"0.0");strncpy(part,&sdum[46],19); part[19]='\0';
                /* read sensitivity */
                sscanf(part,"%lf",&av_calibration[line]);
                strcpy(part,"0.0");strncpy(part,&sdum[65],13); part[13]='\0';
                /* read frequency */
                sscanf(part,"%lf",&cal_freq[line]);

/*
                if ( av_calibration[line] != 0.0 ) {
                         printf("Sensitivity:   %lf\n",av_calibration[line]);
                         printf("Frequency:     %lf\n",cal_freq[line]);
	        }
*/
         
                if ( calibration[line] != 0.0  ) {
                    numcals++;
/*
                    transfer[ch]->calib[line] = calibration[line];
                    transfer[ch]->cal_freq[line] = cal_freq[line];
                    transfer[ch]->cal_time[line].year = cal_year[line];
                    transfer[ch]->cal_time[line].day = cal_day[line];
*/
		      
                }
/*
                else 
                    transfer[ch]->cal_freq[line] = cal_freq[line];
*/
	      }
	  }


          /* read a0  */

	  fseek(uf, (long) ((stinfo[st]->station_log+1) * 2000), 0);
          do {
	      fscanf(uf,"%s", sdum);
              if ( checkvariable++ > 20 ) {
                  printf("Something wrong in first datalog for station %s\n",stinfo[st]->station_code);
                  printf("A0 not found.\n");
                  return;
              }
          } while ( ((strcmp(sdum,"A0")) != 0) && ((strcmp(sdum,"AO")) != 0) );

          for(ch=0;ch<NCH;ch++) {
		fscanf(uf,"%lf", &ppp);
                A0[ch] = ppp;
                /* Is this ok ? */
                norm_freq[ch] = cal_freq[0];
		sens[ch] = av_calibration[0]*1e6;

          }


          /* Do loop over poles and zeroes */

	  do {
	     fscanf(uf, "%s", sdum);

	     if( (sdum[0] == 'P')) {
		dummy = 'P';

                for(ch=0;ch<NCH;ch++) {
                        numpoles[ch] = NPS +1;  /* start value is 0 */
			fscanf(uf,"%lf,", &ppp);
                        poles_x[ch][NPS] = ppp;
			fscanf(uf,"%lf", &ppp);
                        poles_y[ch][NPS] = ppp;
		}
                NPS++;
	     }
	     else if ( sdum[0] == 'Z' ) {
		dummy = 'Z';
		orig = ftell(uf);
		fscanf(uf, "%s", sdum);
		if (sdum[0] == 'S' && sdum[1] == '*' && sdum[2] == '*'){
			Nz = sdum[3] - 48;
			for(y=0;y<Nz;y++) {
                           for(ch=0;ch<NCH;ch++) {
                                numzeros[ch] = NZS +1;
                        	zeros_x[ch][NZS] = 0;
                        	zeros_y[ch][NZS] = 0;
			   }
                           NZS++;
		        }
		        dummy = '^';    /* break ! */ 
		        break;
	        }
			 
		else {
		  fseek(uf,orig,0);
                  for(ch=0;ch<NCH;ch++) {
                        numzeros[ch] = NZS +1;
			fscanf(uf,"%lf,", &ppp);
                        zeros_x[ch][NZS] = ppp;
			fscanf(uf,"%lf", &ppp);
                        zeros_y[ch][NZS] = ppp;
		  }
		  NZS++;
	       }
	     }
 
	  } while ( dummy == 'P' || dummy == 'Z' );


          for(ch=0;ch<NCH;ch++) {

             if(ch==0) chcode[2] = 'Z';
             if(ch==1) chcode[2] = 'N';
             if(ch==2) chcode[2] = 'E';
             chcode[3]='\0';

	     fprintf (fp[ch],"+                  +--------------------");
	     fprintf (fp[ch],"-------------------+                  +\n");
	     fprintf (fp[ch],"+                  |   Instrument response, %5s ch %3s",
		     stinfo[st]->station_code,chcode);
	     fprintf (fp[ch],"   |                  +\n");
	     fprintf (fp[ch],"+                  +--------------------");
	     fprintf (fp[ch],"-------------------+                  +\n");
	     fprintf (fp[ch],"\n");
     
	     fprintf (fp[ch],"Response type:                         Poles and zeroes\n");
             fprintf (fp[ch],"Transfer function type:                Laplace Transform (Rad/sec)\n"); 
	     fprintf (fp[ch],"AO normalization factor:               %G\n", A0[ch]);
	     fprintf (fp[ch],"Normalization frequency (Hz):          %G\n", norm_freq[ch]);
	     fprintf (fp[ch],"Number of zeroes:                      %d\n",numzeros[ch]);
	     fprintf (fp[ch],"Number of poles:                       %d\n",numpoles[ch]);
	     fprintf (fp[ch],"Complex zeroes:\n");
	     fprintf (fp[ch],"  i  real          imag \n");
	     for (i = 0; i < numzeros[ch]; i++)
		     fprintf (fp[ch],"%3d % E % E \n",
                                  i, zeros_x[ch][i], zeros_y[ch][i]);
	     fprintf (fp[ch],"Complex poles:\n");
	     fprintf (fp[ch],"  i  real          imag   \n");
	     for (i = 0; i < numpoles[ch]; i++)
		     fprintf (fp[ch],"%3d % E % E \n",
                                  i, poles_x[ch][i], poles_y[ch][i]);
	     fprintf (fp[ch],"\n");


	     fprintf (fp[ch],"+                  +--------------------");
	     fprintf (fp[ch],"-------------------+                  +\n");
	     fprintf (fp[ch],"+                  |   Channel sensitivity, %5s ch %3s",
		     stinfo[st]->station_code,chcode);
	     fprintf (fp[ch],"   |                  +\n");
	     fprintf (fp[ch],"+                  +--------------------");
	     fprintf (fp[ch],"-------------------+                  +\n");
	     fprintf (fp[ch],"\n");

	     fprintf (fp[ch],"Sensitivity (Counts/Meter):            %E\n", sens[ch]);
	     fprintf (fp[ch],"Frequency of sensitivity (Hz):         %E\n", norm_freq[ch]);

/*
	     fprintf (fp[ch],"Number of calibrations:                %d\n",
			     type58->number_calibrations);
	     fprintf (fp[ch],"Calibrations:\n");
	     fprintf (fp[ch]," i, sensitivity, frequency, time of calibration\n");
	     for (i = 0; i < type58->number_calibrations; i++)
			     fprintf (fp[ch],"%3d % E % E %s\n", i, 
			     type58->calibration[i].sensitivity, 
			     type58->calibration[i].frequency,
			     type58->calibration[i].time);
	     }
*/
	     fprintf (fp[ch],"\n\n");
	
             fclose(fp[ch]);
        }

    } /* End of GDSN  */

    if(SEED) {

        /* Check fotr multiplexed data */
        MULTIPLEXED = check_for_multiplex ( st, BandCode );


        LOOP_OVER_STATIONS {
                if (strcmp(stinfo[st]->station_code,sty->Call_Letters)!=0)
                              continue;

                LOOP_OVER_CHANNELS {
                         if ( !(chy->Identifier[0] == BandCode ))
                               continue; 

                         if(MULTIPLEXED) {
                                  if(chy->SubChannel==1) ch=0;
                                  if(chy->SubChannel==2) ch=1;
                                  if(chy->SubChannel==3) ch=2;
                         }
                         else {
                                  if ( chy->Identifier[2] == 'Z' ) ch =0;
                                  if ( chy->Identifier[2] == 'N' ) ch =1;
                                  if ( chy->Identifier[2] == 'E' ) ch =2;
                         }
                              
	
                         print_response ( fp[ch], sty, chy);
                         fclose(fp[ch]);

               }

       }

    } /* end of SEED */

}

/*********************************************************************/
/*                                                                   */
/*   CALC. EPICENTRAL DISTANCE                                       */


calc_refpt ( evlat, evlon, i  )
float evlat, evlon;
int i;
{
    float       ggtogc();
    void        refpt();


    refpt ( ggtogc(evlat*RAD), evlon*RAD, i );
}

void calc_epi ( lat, lon, i )
char *lat, *lon;
int i;
{
    float        ggtogc();
    char         ch;


      /*  Determine floating point values stlat and stlon  */

          ch = lat[strlen(lat)-1];
          lat[ strlen (lat)-1 ] = '\0';
          sscanf (lat, "%f", &stinfo[i]->flat);
          if ( ch == 'S' ) stinfo[i]->flat = -stinfo[i]->flat;

          ch = lon[ strlen (lon)-1 ];
          lat[ strlen (lon)-1 ] = '\0';
          sscanf (lon, "%f", &stinfo[i]->flon);
          if ( ch == 'W' ) stinfo[i]->flon = -stinfo[i]->flon;


     /*  Determine the distance to the selected event  */
         delaz ( ggtogc(stinfo[i]->flat*RAD), stinfo[i]->flon*RAD, 
                                &stinfo[i]->epi, &stinfo[i]->back );
         stinfo[i]->epi *= DEG;
         stinfo[i]->back *= DEG;



}

void calc_epi_float ( lat, lon, i )
float lat, lon;
int i;
{
    float        ggtogc();


     /*  Determine the distance to the selected event  */
         delaz ( ggtogc(stinfo[i]->flat*RAD), stinfo[i]->flon*RAD, 
                                &stinfo[i]->epi, &stinfo[i]->back );
         stinfo[i]->epi *= DEG;
         stinfo[i]->back *= DEG;



}



/* stores the coordinates (geocentric latitude and */
/* longitude) of a refernce point on the earth's surface */

void refpt(lat, lon, i)
float lat, lon;
int i;
{
     st0 = cos(lat);
     ct0 = sin(lat);
     phi0 = lon;
}

/* calculates the geocentric distance, and */
/* azimuth from the reference point to a given */
/* secondary point, and the back azimuth from */
/* the secondary point to the reference point */

delaz(lat, lon, delta, az1)
float lat, lon, *delta, *az1;
{
    float ct1, st1, sdlon, cdlon, cdelt, sdelt,az;

    ct1 = sin(lat);
    st1 = cos(lat);
    sdlon = sin(lon - phi0);
    cdlon = cos(lon - phi0);
    cdelt = st0*st1*cdlon + ct0*ct1;
    cvrtop (st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, &az); 
    *delta = atan2(sdelt, cdelt);
    cvrtop (st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, &az);
    *az1 = az;
    if (*az1 < 0) *az1 +=  TWOPI;
}                        

/* calculates the geocentric coordinates of a */
/* secondary point, given its geocentric distance */
/* and azimuth from the reference point */

back (delta, az0, lat, lon)
float delta, az0, *lat, *lon;
{    
    float sdelt, cdelt, cz0, ct1; 
    float st1, dlon;
    float sign();

    sdelt = sin(delta);
    cdelt = cos(delta);
    cz0 = cos(az0);
    ct1 = st0*sdelt*cz0 + ct0*cdelt;
    cvrtop (st0*cdelt-ct0*sdelt*cz0,sdelt*sin(az0), &st1, &dlon);
    *lat = atan2(ct1, st1);
    *lon = phi0 + dlon ;
    if (fabs(*lon) > PI) *lon = *lon - sign(TWOPI, *lon);
}

/* cvrtop - convert from rectangular to polar coordinates */
cvrtop(x, y, r, theta)
float  x, y; /* (input)*/
float  *r, *theta;    /* (output - may overlay x, y) */
{
    float rad;
    float myhypot();

    rad = myhypot(x, y);
    *theta = atan2(y, x);
    *r = rad;
}

/* cv_lat - convert between geographic and geocentric latitude */
#define c1    ((1.-FLATTENING)*(1.-FLATTENING))
#define c2    (HALFPI*(1./c1 -1.))

/*  converts from geographic to geocentric latitude */

float ggtogc (lat)
float lat;
{
    float gg;
    float sign();

    if ((HALFPI-fabs(lat)) < .02) 
       gg = lat/c1 -sign(c2, lat);
    else 
       gg = atan(c1 * tan(lat));
    return gg;
}

/*  converts from geocentric to geographic latitude */
float gctogg(lat)
float lat;
{
    float gc;

    if ((HALFPI - fabs(lat)) < .02) gc = c1*(lat + sign(c2, lat));
    else gc = atan(tan(lat)/c1);
    return gc;
}

/* myhypot - euclidian distance, accurately and avoiding overflow */
float myhypot(a, b)
float a, b;
{
    float l, s, t;
    float euc;

    l = fabs(a);
    s = fabs(b);
    if (s > l) {
       t = s;
       s = l;
       l = t;
    }
    if (l == 0.0) euc = 0.0;
    else {
       s = s/l;
       euc = l*sqrt(1.0 + s*s);
    }
    return euc;
}

float sign (a, b)
float a, b;
{
    if (b > 0) return a;
    else if (b < 0) return -a;
}


/*                                                                   */
/*********************************************************************/
