/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include "conver.h"
#include "seed.h"
#include "user2.h"
#include "define.h"
#include "prototyp.h"
#include "icon_bitmap1"
#include <X11/cursorfont.h>
#include <xview/font.h>

#define fmax(a,b) if(b>a) a=b
#define fmin(a,b) if(b<a) a=b

extern OLsunic1_base_window_objects    *bfr;

extern XID                xid;
extern GC                 gc, revgc;
extern Display            *display; 
extern Xv_Window          canvas_pw;
extern Cursor             cursor;
extern Pixmap             px;

extern int                NDP, GDSN, SEED, DTYP, NCH, WELCOME, PATIENT, DEBUG;
extern int                *xdata[];
extern float              *ydata[];
extern int                nst_selected, pch, julday, chyear;
extern int                stbuf[];
extern int                cursor_shape;
extern STATION_STRUCT     *stinfo[];
extern float              upper_freq, lower_freq, corner_freq;
extern char               Orientation, BandCode;


Winfor         sw[4];
int            screen, XT1, XT2;
int            X0, Y0, chi, SELS, X1, Y1, CHI;
int            DATA_FLAG, MULTIPLEXED;
unsigned long  bg, fg;

float          min_y[MAX_STATIONS][6], max_y[MAX_STATIONS][6];
float          delta_epi, epi_max, epi_min;
float          xev_conv, val, AMAX[4], SCALEY[4], PLS;
STDTIME2        pltime;
double         TPB, TPE;

Xv_Font        font, font_ori;


void 
graphics_loop (wtb,wte )
double wtb, wte;                /*  wtb, wte :  time window boundaries  */
{
	double 	          TP1, TP2, TPb, TPe;
	float		  dt, fdum, x_offset, y_offset,scale_x, scale_y;
	float             c[3], d[3];
	int               p1,  np,  tr, NP, Ntr, loop_counter, pt;
        int               sp1[3], snp[3], npt, r1,  shft, np2;
        int               stc, ch, ALL_DATA;
	char              user_info[40], loc_orientation;
	void              nyquist();
	int               data_type_avail_for_station();
        STDTIME           *start_time;


	screen = DefaultScreen(display);
	bg = WhitePixel(display,screen);
	fg = BlackPixel(display,screen);

        /* Set base frame in busy mode */
        xv_set ( bfr->base_window, FRAME_BUSY, FALSE, NULL);

        /* Initialize some variables */
        TPb                  = 1e15;
        TPe                  = -1e15;
        epi_max              = 0;
        epi_min              = 180;
        ALL_DATA             = FALSE;
        shft                 = 15;


        /* Loop over selected station(s):
         *   -  determine which elements in xdata and ydata are in interval
         *      wtb, wte. 
         *      TP1 and TP2 are time boundaries of actual plot.
         *   -  determine max. epicentral distance.          
         *   -  read the data, decode them and determine max. and min. value
         */

        for ( stc = 0; stc < nst_selected; stc++ ) { 

	      TP1 = wtb;
	      TP2 = wte;

              if(GDSN) {

                  /* See if selected station has selected data type, and
                     if channel is available.                         */

	          if ( data_type_avail_for_station(stbuf[stc],0) == FALSE ) 
                        continue;
                  if (pch>0 && stinfo[stbuf[stc]]->number_channels[DTYP] == 1 )
                        continue;

                  find_data_points ( stc, wtb, wte, &p1, &np, &TP1, &TP2,(int)0 );

                  if(DATA_FLAG == FALSE) {
                    printf("No data for %s [plotje.c(1)]\n",stinfo[stbuf[stc]]->station_code);
                  }

                  ALL_DATA |= DATA_FLAG;

                  if ( TP1 < TPb ) TPb = TP1;
                  if ( TP2 > TPe ) TPe = TP2;
              }

              if(SEED) {

                 /* Check fotr multiplexed data */
                 MULTIPLEXED = check_for_multiplex ( stbuf[stc], BandCode );


                 /* Do while loop once for each selected channel */
                 if(Orientation == 'A' ) loop_counter = 3;
                 else                    loop_counter = 1;

                 while ( loop_counter >= 1 ) {

                     pt = channel_order ( loop_counter, &loc_orientation );

                     /* Do loop only once for multiplexed data */
                     if(MULTIPLEXED == TRUE ) {
                          pt=0;
                          loop_counter=0;
                     }

                     if ( data_type_avail_for_station(stbuf[stc],pt) == FALSE ) {
                                loop_counter--;
                                continue;
                     }

                     find_data_points ( stc, wtb, wte, &sp1[pt], &snp[pt], &TP1, &TP2, pt );
                     ALL_DATA |= DATA_FLAG;
                     if ( TP1 < TPb ) TPb = TP1;
                     if ( TP2 > TPe ) TPe = TP2;
                     loop_counter--;
                 }
	      }


              /* Max. and min. value for epicentral distance */
	      if(stinfo[stbuf[stc]]->epi > epi_max ) epi_max = stinfo[stbuf[stc]]->epi;
	      if(stinfo[stbuf[stc]]->epi < epi_min ) epi_min = stinfo[stbuf[stc]]->epi;

        }


        /* Skip plot part of this routine, if no data are available,
           and continue with selection.                              */

        if( ALL_DATA == FALSE ) {
            continue_screen();
            goto NO_DATA;
        }

        fdum = (float)((epi_max-epi_min)/10);
        epi_max += fdum;
        epi_min -= fdum;
	delta_epi = epi_max - epi_min;


        /* Clear (sub)window(s) and map them  */
	XClearWindow(display,xid);
	draw_plot_window ();


        /* This statement  is needed for filling pltime ! */
	start_time = (STDTIME *) sec_to_time ( TPb );

	/* Loop over selected stations to plot the data */

        for ( stc=0; stc<nst_selected; stc++) {

               TP1 = wtb;
               TP2 = wte;

               if(GDSN) {

	          if ( data_type_avail_for_station(stbuf[stc], 0) == FALSE ) 
                       continue;
                  if (pch>0 && stinfo[stbuf[stc]]->number_channels[DTYP] == 1 )
                        continue;

                  find_data_points ( stc, wtb, wte, &p1, &np, &TP1, &TP2 ,(int)0);
                  if ( np < 0 || DATA_FLAG == FALSE ) continue;

                  read_data (stbuf[stc]);
                  decode_data (stbuf[stc]);
               }

               if(SEED) {

                  if(Orientation == 'A' ) loop_counter = 3;
                  else                    loop_counter = 1;

                  /* Check fotr multiplexed data */
                  MULTIPLEXED = check_for_multiplex ( stbuf[stc], BandCode );
      
                  while ( loop_counter >= 1 ) {

                      pt = channel_order ( loop_counter, &loc_orientation );

                      if(MULTIPLEXED==TRUE) {
                             pt=0;
                             loop_counter=0;
                      }

                      if ( data_type_avail_for_station(stbuf[stc],pt) == FALSE )  {
                             loop_counter--;
                             continue;
                      }

                      find_data_points ( stc, wtb, wte, &sp1[pt], &snp[pt], &TP1, &TP2, pt );

                      loop_counter--;
                 }

                 /* Check for data */

                 if ( read_seed_data (stbuf[stc]) == FALSE ) continue;
                 if ( decode_seed_data (stbuf[stc]) == FALSE ) continue;
	      }


              /* Loop over channels: filter and/or find max. and min. values */

	       for ( ch=0; ch < NCH; ch++) {

                    chi = (NCH==1) ? pch:ch ;

                    if(SEED) {

                          if(MULTIPLEXED==TRUE) {
                              chi = 0;
                              p1=sp1[0];
                              np=snp[0];
                          }
                          else {
                              p1=sp1[chi];
                              np=snp[chi];
                          }

                          if(np<0) continue;
                    }

                    if ( GDSN )
		         dt  = stinfo[stbuf[stc]]->data_entry[DTYP]->dt;
                    if ( SEED )
		         dt  = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->dt;

                    if ( BP_FILTER == TRUE ) {

                         /* Check nyquist frequency */

                         nyquist ( dt, &upper_freq, stc );
                         nyquist ( dt, &lower_freq, stc );

                         bp_filter_coeff ( upper_freq, lower_freq, c, d, dt );
                         bp_filter_data ( ydata[chi]+p1, np, c, d);
                    }
                    if ( HP_FILTER == TRUE ) {

                         nyquist ( dt, &corner_freq, stc );

                         hp_filter_coeff ( corner_freq, c, d, dt );
                         hp_filter_data ( ydata[chi]+p1, np, c, d);
                    }


                    chi = (NCH==1) ? pch:ch ;

                    get_point_limits ( ydata[chi]+p1, np, &min_y[stc][chi], &max_y[stc][chi]);
                    if ( min_y[stc][chi] == 0 &&  max_y[stc][chi] == 0 ) {
                         max_y[stc][chi] = 1;
                         min_y[stc][chi] = -1;
                    }


                    /* scale factor for x-axis */
		    NP = (int)((TPe-TPb)/dt) -1;
		    scale_x = (float)(NP)/(float)sw[ch].width;


                    /* scale factor for y-axis */
                    if ( nst_selected == 1 ) {
		        scale_y = (max_y[stc][chi] - min_y[stc][chi]) / sw[ch].height;

                        AMAX[chi] = max_y[stc][chi];
                        SCALEY[chi] = (float)scale_y;

                    }
                    else  {
		        scale_y = (max_y[stc][chi] - min_y[stc][chi]) * 10 / sw[ch].height;
                    }

                    /* Protect for dividing by zero */
                    if(scale_y == 0 ) printf("WARNING SCALE_Y = 0 !!\n");
                    if(scale_y == 0 ) scale_y = 1.0;



                    /* ofset in y-direction */

                    if ( nst_selected == 1 ) {
                          y_offset = (max_y[stc][chi]/scale_y) + sw[ch].y;;
                    }
                    else  {
                          y_offset = (int)((1 - ((stinfo[stbuf[stc]]->epi - epi_min)/
                                                 delta_epi)) * sw[ch].height);
			  y_offset += sw[ch].y;

                          XDrawString ( display, xid, gc, 
                                       (int)(X1 +35 +shft ), (int)(y_offset), 
                                        stinfo[stbuf[stc]]->station_code,
                                        strlen(stinfo[stbuf[stc]]->station_code) ); 
                          shft *= (-1);
                    }

                    if(MULTIPLEXED == TRUE) chi = 0;

                    if ( GDSN )
		          Ntr = stinfo[stbuf[stc]]->data_entry[DTYP]->Ntriggers;
                    if ( SEED )
		          Ntr = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->Ntriggers;

np2=0;

		    for(tr=1;tr<=Ntr;tr++) {

                        if(GDSN) {

			    if(stinfo[stbuf[stc]]->data_entry[DTYP]->trigger[tr]->bool == FALSE ) 
                                                                    continue;
			    x_offset = (int) ((( stinfo[stbuf[stc]]->data_entry[DTYP]->trigger[tr]->t1 - TPb)/dt)/scale_x);
                            x_offset += sw[ch].x;
    
			    r1 = stinfo[stbuf[stc]]->data_entry[DTYP]->trigger[tr]->R1;
			    npt = stinfo[stbuf[stc]]->data_entry[DTYP]->trigger[tr]->npoints;
                        }
 
                        if(SEED) {
                            if(MULTIPLEXED == TRUE) chi = 0;
			    if(stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->bool == FALSE ) {
                 np2 = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->last_sample_no;
                 np2 += 1;
                                                                    continue;
}
			    x_offset = (int) ((( stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->t1 - TPb)/dt)/scale_x);

                            x_offset += sw[ch].x;

			    r1 = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->R1;
                 r1 = r1 - np2;
			    npt = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->npoints;
                        }

                        chi = (NCH==1) ? pch:ch ;
			draw_scaled_points ( scale_x, x_offset, 
			                  scale_y, y_offset, npt, ydata[chi]+r1);
		   }
 
             } /* end of loop over stations */


	}

	draw_tics ( TPb, TPe );

        TPB = (double) pltime.day * 86400 +
              (double) pltime.hour * 3600 +
              (double) pltime.minute * 60 +
              (double) pltime.second;

        TPE = TPB + PLS;

	sprintf( user_info, "Start time of plot : %02d:%02d:%6.3lf",
                                              pltime.hour,pltime.minute,
	                                      pltime.second);
	xv_set(bfr->start_time_message, PANEL_LABEL_STRING, user_info, NULL);

        xv_set ( canvas_pw, WIN_CONSUME_EVENTS,
                                 WIN_NO_EVENTS,
                                 LOC_MOVE, 
                                 NULL,
                            WIN_EVENT_PROC, canvas_event_proc,
                            NULL);


        /* jump to this position if no data are available */

NO_DATA:

        xv_set ( bfr->base_window, FRAME_BUSY, FALSE, NULL);

        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, "Continue with selection, action or quit.",
                 FRAME_RIGHT_FOOTER, " ",
                 NULL);

}

int
check_for_multiplex ( st, bc )
int st;
char bc;
{
	int val;

	val = FALSE;

        LOOP_OVER_STATIONS {
             if ( strncmp(sty->Call_Letters,stinfo[st]->station_code,strlen(stinfo[st]->station_code)) == NULL ) {
                  LOOP_OVER_CHANNELS {
                            if(chy->Identifier[0] != bc ) continue;
                            if(chy->Next == NULL ) break;
                            if(chy->SubChannel!=chy->Next->SubChannel) val = TRUE;
                  }
             }    
             
        }

	return(val);
}

int 
channel_order ( lc, lo )
int lc;
char *lo;
{
        int pt;

        if ( Orientation == 'A')  {
                 if(lc == 3) *lo = 'Z';
                 if(lc == 2) *lo = 'N';
                 if(lc == 1) *lo = 'E';
        }
        else *lo = Orientation;

        if(*lo == 'Z' ) pt=0;
        if(*lo == 'N' ) pt=1;
        if(*lo == 'E' ) pt=2;

	return(pt);
}


int
data_type_avail_for_station(i,pt)
int i, pt;
{
          if(GDSN) {
              if (( stinfo[i]->datablocks[1] == 0) && ( DTYP == 1 ) ) return(FALSE);
              if (( stinfo[i]->datablocks[2] == 0) && ( DTYP == 2 ) ) return(FALSE);
              if (( stinfo[i]->datablocks[3] == 0) && ( DTYP == 3 ) ) return(FALSE);
              return(TRUE);
          }

          if(SEED) {
              if(stinfo[i]->seed_entry[DTYP][pt] == NULL ) return(FALSE);
              return(TRUE);
          }
}


void
draw_plot_window ()
{
	int               ch, y;
	XWindowAttributes win_at;

	XGetWindowAttributes(display,xid,&win_at);

	Y0 = (int)(0.05 * (float)win_at.height);
	X0 = (int)(0.12 * (float)win_at.width);

	win_at.height =(int)( 0.9 * (float)win_at.height );
	win_at.width =(int)( 0.8 * (float)win_at.width );

	X1 = X0 + win_at.width;
	Y1 = Y0 + win_at.height;

	XDrawLine(display,xid,gc,X0,Y0,X1,Y0);
	XDrawLine(display,xid,gc,X1,Y0,X1,Y1);
	XDrawLine(display,xid,gc,X1,Y1,X0,Y1);
	XDrawLine(display,xid,gc,X0,Y1,X0,Y0);

	win_at.height = (int)((float)(win_at.height)/NCH);

	y=Y0;

	for ( ch=0; ch < NCH; ch++) {

		if(ch>0) {
                            y + = win_at.height;
	                    XDrawLine(display,xid,gc,X0, y, X1, y );
                }

	        sw[ch].x = X0;
                sw[ch].y = y;
                sw[ch].height = win_at.height;
                sw[ch].width = win_at.width;
	}

}


void 
get_point_limits (  val, np,  min_y, max_y)
/*****************************************************************/
/*     val            -   pointer to first point to plot         */
/*     np             -   number of points to plot               */
/*     min_y, max_y   -   min/max values of array ydata[comp]    */
/*****************************************************************/
float *val;
float *min_y, *max_y;
int np;
{
	int i;
	*min_y = *max_y = *val;

	for ( i=0; i< np; i++ ) {
		if( *val < *min_y ) *min_y = *val ;
		if( *val > *max_y ) *max_y = *val ;
                val++;
	}
}

void
draw_scaled_points ( scale_x, base_x, scale_y, base_y, n_points, val)
/*****************************************************************************/
/*   win        -       (sub)window to plot  on                              */
/*   scale_x,y  -       scale factors in x and y direction                   */
/*   base_x,y   -       offsets in x and y direction                         */
/*   n_points   -       number of points to plot                             */
/*   val        -       pointer to first point to plot                       */
/*****************************************************************************/
float scale_x, base_x, scale_y, base_y;
int n_points;
float *val;
{
	int i;
	int x2, y2, x1, y1;
	XWindowAttributes win_at;

	x1 = (int)( base_x );
        y1 = (int)( base_y - (*val/scale_y));

	for ( i=1; i<(n_points-1); i++ ) {

                val++;
		x2 = (int)( base_x + (i/scale_x) );
                y2 = (int)( base_y - (*val/scale_y));
		XDrawLine(display,xid,gc,x1,y1,x2,y2);
		x1=x2;
		y1=y2;
	}

	XGetWindowAttributes ( display, xid, &win_at);
	XFillRectangle ( display, px, gc, 0, 0,win_at.width, win_at.height );
	XCopyArea ( display, xid,px, gc, 0, 0, win_at.width, win_at.height, 0, 0);

}



void
canvas_event_proc (window, event, arg )
Xv_Window window;
Event     *event;
Notify_arg arg;
{

	int  yy, xx, ch, startx, starty;
	XWindowAttributes win_at;
	double time_in_sec, t1, t2, wte, wtb, ti;
	char dummy[30];
	float sec;


	switch(event_action(event)) {

	case ACTION_SELECT:

                if ( event_is_down(event) ) {

                    SELS++;
                    if(SELS==1) {

                          XGetWindowAttributes(display,xid,&win_at);
                          startx  = event_x(event);
                          XDrawLine(display, xid,gc,startx,Y0,startx,Y1);
                          XT1 = startx - X0;
	                  XCopyArea ( display, xid,px, gc, 0, 0, win_at.width, win_at.height, 0, 0);

                          xv_set ( canvas_pw, WIN_CONSUME_EVENTS, 
                                                   WIN_NO_EVENTS, 
                                                   WIN_MOUSE_BUTTONS, LOC_MOVE ,
                                                   NULL,
                                              NULL); 

                          xv_set ( bfr->base_window,
                                     FRAME_SHOW_FOOTER, TRUE,
                                     FRAME_LEFT_FOOTER, 
                                     "Press left mouse button to specify second time boundary .....",
                                     FRAME_RIGHT_FOOTER, " ",
                                     NULL);
                    }
 
                    if(SELS==2){

                          SELS=0;

                          XGetWindowAttributes(display,xid,&win_at);
                          startx  = event_x(event);
                          XDrawLine(display,xid,gc,startx,Y0,startx,Y1);
                          XT2 = startx -X0;
	                  XCopyArea ( display, xid,px, gc, 0, 0, win_at.width, win_at.height, 0, 0);
                     

                          time_in_sec = (double) pltime.day * 86400 +
                                        (double) pltime.hour * 3600 +
                                        (double) pltime.minute * 60 +
                                        (double) pltime.second; 

		          t1 = (double)((XT1*PLS)/(X1-X0));
		          t2 = (double)((XT2*PLS)/(X1-X0));

                          if ( t1 > t2 ) {
				     ti = t1;
                                     t2 = t1;
                                     t1 = ti;
                          }
                          wtb = time_in_sec + t1;
                          wte = time_in_sec + t2;

	                  XCopyArea ( display, xid,px, gc, 0, 0, win_at.width, win_at.height, 0, 0);

		          xv_set ( bfr->base_window, 
				     FRAME_SHOW_FOOTER, TRUE,
				     FRAME_LEFT_FOOTER, "  ",
                                     FRAME_RIGHT_FOOTER, " ",
                                     NULL);

                          /* zoom boundaries has been set:
                             disassociate zoom possibility from plotwindow */

                          xv_set ( canvas_pw, WIN_CONSUME_EVENTS, 
                                                      WIN_NO_EVENTS ,
                                                      NULL,
                                              NULL); 

                          /* disassociate crosshair cursor from plotwindow */

		          XUndefineCursor (display, xid);

                          graphics_loop ( wtb, wte);
                     }


                }
		break;

	case ACTION_ADJUST:
		return;

	case ACTION_MENU:
		return;

        case LOC_MOVE:
 
                yy = event_y(event);
                xx = event_x(event);

                if ( xx < X0 || xx > X1 || yy < Y0 || yy > Y1 ) {
                     sprintf( dummy, " ");
	             xv_set(bfr->message7, PANEL_LABEL_STRING, dummy, NULL);
                     return;
                }

              
                if ( NCH == 1 ) {

                      if( (yy >= Y0)  && (yy < Y1) ) {
                            ch = pch;
                            val = AMAX[ch] - SCALEY[ch]*(yy - Y0 );
                      }
                      else val=0;
                }
                else {

                      if( (yy >= sw[0].y)  && (yy < (sw[0].y+sw[0].height) ) ) ch = 0;
                      else if( (yy >= sw[1].y)  && (yy < sw[2].y) ) ch = 1;
                      else if( (yy >= sw[2].y)  && (yy <= Y1) ) ch = 2;
                      else ch = -1;

                      if ( ch != -1 ) 
                           val = AMAX[ch] - SCALEY[ch]*(yy - sw[ch].y );
                      else val = 0.0;
                }

                if ( nst_selected > 1 ) val = 0.0;


                sec = (float)((xx - X0) * xev_conv );
                if ( sec<0 ) sec = 0;
                if ( sec>PLS) sec=PLS;

                /* sprintf( dummy, "sec : % 7.1f   ampl: %03d", sec, (int)val ); */
                sprintf( dummy, "sec : % 7.1f   ampl: % 8.2E", sec, val );
	        xv_set(bfr->message7, PANEL_LABEL_STRING, dummy, NULL);

                return;

	default:

                if ( WELCOME ) welcome_screen();
		return;
	}

        repaint_proc  ( (Canvas)NULL, window, 
                        (Display *) xv_get(window, XV_DISPLAY),
                         xv_get(window, XV_XID),
                        (Xv_xrectlist *) NULL );

}

void
repaint_proc (canvas,paint_window,dpy,xwin,xrects)
Canvas       canvas;
Xv_Window    paint_window;
Display      *dpy;
Window       xwin;
Xv_xrectlist *xrects;
{

	XWindowAttributes win_at;

	XGetWindowAttributes(display,xid,&win_at);
	XCopyArea(display,xid,px,gc,0,0,win_at.width,
		    win_at.height,0,0);

}



void
find_data_points ( stc, wtb, wte , R, npoints, TP1, TP2, pt )
/*************************************************************************/
/*     Find out what points I have to plot.                              */
/*          wtb, wte   -      window boundaries                          */
/*          R          -      first point ( element ) to plot            */
/*          npoints    -      number of points to plot                   */
/*          TP1, TP2   -      the final time limits of the plot          */
/*************************************************************************/
double	wtb, wte, *TP1, *TP2;
int 	*R, *npoints, stc, pt;
{
	STDTIME  TS1, TS2;
	double   t1, t2, tp1, tp2;
	int      t, nt, P1, P2, class_type;
	int      bool,S, MSEC;
        long int R1, R2, p1, p2;
	float    dt;

        S=stbuf[stc];

        if(GDSN) {
 	      nt = stinfo[S]->data_entry[DTYP]->Ntriggers ;
              MSEC = 1000;
        }
        if(SEED) {
 	      nt = stinfo[S]->seed_entry[DTYP][pt]->Ntriggers ;
              MSEC = 1000;
        }

	*TP1 = 1e15;
	*TP2 = -1e15;
        R1 = 99999999;
        R2 = -99999999;

	p1 = P1 = 0;

	/* Loop over number of triggers */

        DATA_FLAG = FALSE;  /* Flag to check of this station has data for the
                               selected event.                                */

	for ( t=1; t<=nt; t++) {

                if(GDSN) {
		   TS1 = stinfo[S]->data_entry[DTYP]->trigger[t]->start_time ;
		   TS2 = stinfo[S]->data_entry[DTYP]->trigger[t]->end_time ;
		   P1  = stinfo[S]->data_entry[DTYP]->trigger[t]->first_sample_no;
		   P2  = stinfo[S]->data_entry[DTYP]->trigger[t]->last_sample_no;
		   dt  = stinfo[S]->data_entry[DTYP]->dt;
                }
                if(SEED) {
		   TS1 = stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->start_time ;
		   TS2 = stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->end_time ;
		   P1  = stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->first_sample_no;
		   P2  = stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->last_sample_no;
		   dt  = stinfo[S]->seed_entry[DTYP][pt]->dt;
                }


		if ( dt == 0.0 ) {
			printf("dt = 0.0 . Cannot work with this value!\n");
			exit(1);
		}


		t1 =  (double) TS1.day * 86400  + 
                      (double) TS1.hour * 3600  + 
		      (double) TS1.minute *60 +
		      (double) TS1.second + 
		      (double) TS1.msec/MSEC;

		t2 =  (double) TS2.day * 86400  + 
                      (double) TS2.hour * 3600  + 
		      (double) TS2.minute *60 +
		      (double) TS2.second + 
		      (double) TS2.msec/MSEC;

                /* Apply time correction */
                t1 += (double) stinfo[S]->time_cor;
                t2 += (double) stinfo[S]->time_cor;

                if(DEBUG) {
                            printf("start_time trigger: %d %d %d.%d\n",
                                      TS1.hour,TS1.minute,TS1.second,TS1.msec);
                            printf("end_time trigger: %d %d %d.%d\n",
                                      TS2.hour,TS2.minute,TS2.second,TS2.msec);
                            printf("t1 = %lf  t2 = %lf  wtb=%lf  wte=%lf\n",
                                      t1,t2,wtb,wte);
                }

		class_type = determine_class ( wtb, wte, t1, t2 );

		switch ( class_type ) {
		case 1:
			printf("No data for station %s (trigger %d)\n", stinfo[S]->station_code,t);
			p1=9999999;
			p2=-999999;
			tp1 =  1e15;
			tp2 = -1e15;
			bool = FALSE;
			break;
		case 2:
			p1 = P1;
			p2 = P1 + (wte-t1)/dt;
			tp1 =  t1;
			tp2 = wte;
			bool = TRUE;
			break;
		case 3:
			p1 = P1;
			p2 = P2;
			tp1 = t1;
			tp2 = t2;
			bool = TRUE;
			break;
		case 4:
			p1 = P1 + (wtb-t1)/dt;
			p2 = P2;
			tp1 =  wtb;
			tp2 =  t2;
			bool = TRUE;
			break;
		case 5:
			p1=9999999;
			p2=-999999;
			tp1 =  1e15;
			tp2 = -1e15;
			bool = FALSE;
			break;
		case 6:
			p1 = P1 + (wtb-t1)/dt;
			p2 = P1 + (wte-t1)/dt;
			tp1 = wtb;
			tp2 = wte;
			bool = TRUE;
			break;
		default:
			printf("This should never occur !\n");
		}

                if(GDSN) {
		    stinfo[S]->data_entry[DTYP]->trigger[t]->R1 = (int)p1;
		    stinfo[S]->data_entry[DTYP]->trigger[t]->npoints = (int)(p2-p1);
		    stinfo[S]->data_entry[DTYP]->trigger[t]->bool = bool;
		    stinfo[S]->data_entry[DTYP]->trigger[t]->t1 = tp1;
                }
                if(SEED) {
		    stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->R1 = (int)p1;
		    stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->npoints = (int)(p2-p1);
		    stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->bool = bool;
		    stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->t1 = tp1;

                }

                /*    R1 is first element to plot , npoints is total number of
                 *    elements to plot.
                 */

		if ( p1 < R1 ) R1 = p1;
		if ( p2 > R2 ) R2 = p2;


		/*    TP1 and TP2 are the final time limits of the plot, in seconds */

		if ( tp1 < *TP1 ) *TP1 = tp1;
		if ( tp2 > *TP2 ) *TP2 = tp2;
	}

	*R = (int)R1;
	*npoints = (int)(R2-R1);

/* R is the first sample of the trigger to read in seed_data_buf ! */
        if(SEED) {
             for(t=1;t<=nt;t++) {
                 if ( stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->bool == FALSE ) 
                      continue;
                 *R = (int)R1 - stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->first_sample_no;
/*printf("Updating R trig: %d!!!  R1=%d  R=%d  stc:%d  pt:%d\n",t,(int)R1,*R,stc,pt);*/
                 if(*R<0) {
                       printf("Error in sample number.\n");
                       exit(1);
                 }
                 break;
             }
        }


        /* Check for data flag */

        if(GDSN)
 	      nt = stinfo[S]->data_entry[DTYP]->Ntriggers ;
        if(SEED)
 	      nt = stinfo[S]->seed_entry[DTYP][pt]->Ntriggers ;

	for ( t=1; t<=nt; t++) {
              if(GDSN) 
	         DATA_FLAG |= stinfo[S]->data_entry[DTYP]->trigger[t]->bool;
              if(SEED) 
	         DATA_FLAG |= stinfo[S]->seed_entry[DTYP][pt]->trigger[t]->bool;
        }

}

int
determine_class ( wtb, wte, t1, t2 )
/********************************************************************/
/* Determine  what part of the data can be seen through the window. */
/*     wtb, wte    -    window boundaries                           */
/*     t1,  t2     -    start and end time of data                  */
/********************************************************************/
double wtb, wte, t1, t2;
{


	if ( (wtb < t1) && ( wte <= t1 ) )                                    return(1);
	if ( (wtb < t1) && ( (wte > t1) && (wte <= t2) ) )                    return(2);
	if ( (wtb <= t1) && (wte >= t2) )                                     return(3);
	if ( ((wtb >= t1) && ( wtb < t2 ) ) && (wte > t2) )                   return(4);
	if ( ( wtb >= t2 )  && (wte > t2) )                                   return(5);
	if ( ((wtb >= t1) && ( wtb < t2 ) ) && ((wte > t1) && ( wte <= t2) )) return(6);
        return(9999);
}


void
draw_tics ( t1, t2 )
double 	t1, t2;
{
	int               i, Ntics, x2, y2, x1, y1, y_offset, ch, stc;
        float               delta_amp, amp_min, amp_max; 
	float             dx, ylen, st, dy, xlen, i0, TY, TX, k0;
        float             height, width;
	char              time[9], y_text[50], *x_text = "Seconds";


/*      PLS:      total seconds of plot                                     */
/*      TX:       time step between tics on horizontal axis                 */
/*      TY:       step between tics on vertical axis                        */
/*      dx, dy:   number of pixels between tics, in horizontal and vertical */
/*                direction                                                 */
/*      Ntics:    number of tics                                            */


	PLS = t2 - t1;

        if ( PLS <= 1 )         TX = .1;
        else if ( PLS <= 10 )   TX = 1;
        else if ( PLS <= 60 )   TX = 5;
        else if ( PLS <= 120 )  TX = 15;
        else if ( PLS <= 900 )  TX = 60;
        else if ( PLS <= 1800 ) TX = 120;
        else if ( PLS <= 3600 ) TX = 300;
        else                    TX = 600;

        if ( delta_epi < 0.11 )      TY = 0.005;
        else if ( delta_epi <= 0.4 ) TY = 0.05;
        else if ( delta_epi <= 1 )   TY = 0.2; 
        else if ( delta_epi <= 3 )   TY = 0.5; 
        else if ( delta_epi <= 5 )   TY = 1; 
        else if ( delta_epi <= 10 )  TY = 2; 
        else if ( delta_epi <= 30 )  TY = 4; 
        else if ( delta_epi <= 90 )  TY = 5; 
        else                         TY = 10;


        /* Print text x and y axes */

        XDrawString ( display, xid, gc, X1+30, Y1+20, x_text, strlen(x_text) ); 

        if ( nst_selected > 1 ) {
                strcpy ( y_text, "Epicentral Distance ");
                XDrawString ( display, xid, gc, 5, 15, y_text, strlen(y_text) ); 
	}
        else {
                stc=0;   /* stc = 0  if only one station has been selected */

                strcpy ( y_text, "Station: ");
                strcat ( y_text, stinfo[stbuf[stc]]->station_code);
                XDrawString ( display, xid, gc, 5, 15, y_text, strlen(y_text) ); 
        }
  
        width = (float)(X1-X0);
        height = (float)(Y1-Y0);

        xev_conv = (float)(PLS)/width;
	dx = (width/PLS)*TX;

	ylen = (float) height/27;
        xlen = (float)(width/100);


        /* X tics */

	Ntics = (int) floor(PLS/TX);

	for ( i=0; i<= Ntics; i++ ) {

		st = (float)(TX*i);
		sprintf(time,"%5.2f",st);
		x1 = (int)(X0 + dx*i);
                if ( x1 > X1 ) break;
		x2 = x1;
		y1 = Y0;
		y2 = (int)(Y0+ylen);
		XDrawLine(display,xid,gc,x1,y1,x2,y2);

		y1 = Y1;
		y2 = (int)(Y1-ylen);
		XDrawLine(display,xid,gc,x1,y1,x2,y2);
                XDrawString ( display, xid, gc, (int)(x1-20),
                             (int)(Y1+20), time, strlen(time) ); 

	}

        /* Y tics indicating epicentral distance */

        if ( nst_selected > 1 ) {

                dy = (float)((height/delta_epi)*TY);
	        Ntics = (int) floor(delta_epi/TY);

                /* i0; offset (in epi. dist) to first round value */
                i0 = (float) ((floor(epi_min/TY)*TY) + TY - epi_min);

                /* k0; value (in epi. dist) to first round value */
                k0 = (float) ( (floor(epi_min/TY)*TY) + TY ); 

                /* y_offset; offset (in pixels) to first round value */
                y_offset = (int) (( 1 - i0/delta_epi)* height);
		y_offset += Y0;


	        for ( i=0; i<= Ntics; i++ ) {

		        st = (float)(TY*i + k0);
			if ( st > epi_max ) break;
		        sprintf(time,"%6.3f",st);
		        y1 = (int)(y_offset - dy*i ); 
		        y2 = y1;
		        x1 = X0;
		        x2 = (int)(x1+xlen);
		        XDrawLine(display,xid,gc,x1,y1,x2,y2);

                        XDrawString ( display, xid, gc, (int)(X0-45),
                         (int)( y1 + 5), time, strlen(time) ); 

	        }
         }

        /* Y tics indicating amplitude  */

        else {

            for(ch=0;ch<NCH;ch++) {

                chi = ( NCH == 1 ) ? pch:ch ;
                /* if(MULTIPLEXED) chi=0; */

                amp_min = min_y[stc][chi];
                amp_max = max_y[stc][chi];
                delta_amp = amp_max - amp_min;

                if ( delta_amp == 0 ) {
                     printf("This should never occur. \n");
                     printf("stc=%d chi=%d station: %s miny=%f maxy=%f\n",
                             stc,chi,stinfo[stbuf[stc]]->station_code,
                             min_y[stc][chi], max_y[stc][chi]);
                     exit(1);
                }

                chi = ( NCH == 1 ) ? pch:ch ;
                if(NCH==1) {
	              TY = (float) ((float)delta_amp/8);
                      y1 = (sw[ch].y + sw[ch].height/2);
                      if ( chi == 0 ) strcpy ( time, " Z");
                      if ( chi == 1 ) strcpy ( time, " N/S");
                      if ( chi == 2 ) strcpy ( time, " E/W");
                      XDrawString ( display, xid, gc, (int)(X1+15),
                                     (int)(y1 + 5), time, strlen(time) ); 
                }
                if(NCH==3) {
	              TY = (float) ((float)delta_amp/4);
                      y1 = (sw[ch].y + sw[ch].height/2);
                      if ( chi == 0 ) strcpy ( time, " Z");
                      if ( chi == 1 ) strcpy ( time, " N/S");
                      if ( chi == 2 ) strcpy ( time, " E/W");
                      XDrawString ( display, xid, gc, (int)(X1+15),
                                     (int)(y1 + 5), time, strlen(time) ); 
	        }

	        Ntics = (int) floor((float)delta_amp/TY);
                dy = (float)(sw[ch].height*(TY/delta_amp));

                /* i0; offset (in amplitude) to first round value */
                i0 = (float) ((floor(amp_min/TY)*TY) + TY - amp_min);

                /* k0; value (in amplitude) to first round value */
                k0 = (float) ( (floor(amp_min/TY)*TY) + TY );

                /* y_offset; offset (in pixels) to first round value */
                y_offset = (int) (( 1 - i0/delta_amp)* sw[ch].height);
		y_offset += sw[ch].y;

	        for ( i=0; i<= Ntics; i++ ) {

		        st = (float)(TY*i + k0);
			if ( st > amp_max ) break;
		        /* sprintf(time,"%+8.2f",st); */
		        sprintf(time,"%8.2E",st);
		        y1 = (int)(y_offset - dy*i ); 
                        if ( y1 > (sw[ch].y + sw[ch].height) ) continue;
                        if ( y1 <  sw[ch].y ) continue;
		        y2 = y1;
		        x1 = X0;
		        x2 = (int)(x1+xlen);
		        XDrawLine(display,xid,gc,x1,y1,x2,y2);

                        XDrawString ( display, xid, gc, (int)(X0-65),
                                     (int)(y1 + 5), time, strlen(time) ); 

	        }
           }
     }
}

STDTIME 
*sec_to_time ( T1 )
double T1;
{

	STDTIME *sttime;
	double frac;

	sttime = (STDTIME *) malloc ( sizeof(STDTIME));

	sttime->day = 0;
	sttime->hour = 0;
	sttime->minute = 0;
	sttime->second = 0;
	sttime->msec = 0;

        pltime.year = chyear;

	while ( T1 > 86400 ) {
		sttime->day += 1;
		T1 -= 86400;
	}
        pltime.day = sttime->day;
	while ( T1 > 3600 ) {
		sttime->hour += 1;
		T1 -= 3600;
	}
	pltime.hour = sttime->hour;
	while ( T1 > 60 ) {
		sttime->minute += 1;
		T1 -= 60;
	}
	pltime.minute = sttime->minute;
	sttime->second = (int)T1;

	pltime.second = (double)T1;
/*
	pltime.second = sttime->second;
*/

/* modf is not portable ? */

/*
	frac = (double) modf(T1,&ip);

	sttime->msec = (int)((float)(frac*1000));
	pltime.msec = sttime->msec;
*/

        return(sttime);
}
