/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include <stdio.h>
#include "conver.h"
#include "seed.h"
#include "define.h"

extern FILE *uf;
extern int LRECL;


S_ID    *did;
S_TIME  *dtime;
S_DATA  *ddata;


repair_seed()
{

     double tb, te, ftcor, dt, f_sam_rate;
     float sps;
     int day, hour, minute, second, msec, ns;
     char *buf, *data_buf;
     long offset, tcor;


/** Repair start and end time of blockettes 74 for station SSB between day
    213 and 234 of 1988 for ORFEUS-882 CDROM                              **/


    printf("REPAIR routine .....................................\n");


    LOOP_OVER_TIME_ENTRIES {

         LOOP_OVER_BLOCK74 {

             if ( strncmp( bl74->Station_ID, "SSB", 3) != NULL )
                               continue;
             if ( time_entry->Time_Span_ID->Start_Time.day < 213  ||
                  time_entry->Time_Span_ID->Start_Time.day > 234 )
                               continue;


             if ((buf = (char *) malloc(200) ) == NULL ) {
                         printf("Malloc failed. \n");
                         exit(1);
             };

             /* Read first data record */
             offset = (bl74->Start_Sequence_No-1) * LRECL;
             fseek ( uf, offset, 0 );

             if ((fread ( buf, sizeof(char), 100, uf )) == -1) {
                      printf("No data read in.\n");
                      exit(1);
             }

             data_buf = buf;

             did =   (S_ID *) (data_buf+8);
             dtime = (S_TIME *) (data_buf+12+8);
             ddata = (S_DATA *) (data_buf+22+8);

             /*printf("%s %s  %d:%d:%d\n",did->station, did->channel,dtime->hour,dtime->minute,dtime->seconds);*/


/*
             day =    (int)dtime->day;
             hour =   (int)dtime->hour;
             minute = (int)dtime->minute;
             second = (int)dtime->seconds;
             msec =   (int)dtime->frac_sec;

             tb = (double) day * 86400 +
                  (double) hour * 3600  +
                  (double) minute *60 +
                  (double) second +
                  (double) msec/10000;
*/

             tb = (double) dtime->day * 86400 +
                  (double) dtime->hour * 3600  +
                  (double) dtime->minute *60 +
                  (double) dtime->seconds +
                  (double) dtime->frac_sec/10000;


             /* Apply time correction to header time */

             tcor = (long)ddata->t1*65536 + (long)ddata->t2;
             ftcor = (double) tcor;
             ftcor /= 10000;

             tb = tb + (double) ftcor;

/*printf("REPAIR(1): %03d:%02d:%02d:%02d.%04d\n",bl74->Start_Time.day,
bl74->Start_Time.hour,bl74->Start_Time.minute,bl74->Start_Time.second,bl74->Start_Time.msec);*/

             day = hour = minute = second = msec = 0;

             while ( tb > 86400 ) {
                     day += 1;
                     tb -= 86400;
             }
             while ( tb > 3600 ) {
                     hour += 1;
                     tb -= 3600;
             }
             while ( tb > 60 ) {
                     minute += 1;
                     tb -= 60;
             }

             second = (int)floor(tb);
             msec = (int)(fmod(tb,(double)1)*10000 +0.0001);

             bl74->Start_Time.day = (int)day;
             bl74->Start_Time.hour = (int)hour;
             bl74->Start_Time.minute = (int)minute;
             bl74->Start_Time.second = (int)second;
             bl74->Start_Time.msec = (int)msec;
/*printf("REPAIR(2): %03d:%02d:%02d:%02d.%04d\n",bl74->Start_Time.day,
bl74->Start_Time.hour,bl74->Start_Time.minute,bl74->Start_Time.second,bl74->Start_Time.msec); */



             /* Read last data record */
             offset = (bl74->End_Sequence_No-1) * LRECL;
             fseek ( uf, offset, 0 );

             if ((fread ( buf, sizeof(char), 100, uf )) == -1) {
                      printf("No data read in.\n");
                      exit(1);
             }

             data_buf = buf;
             dtime = (S_TIME *) (data_buf+12+8);
             ddata = (S_DATA *) (data_buf+22+8);

/*
             day =    (int)dtime->day;
             hour =   (int)dtime->hour;
             minute = (int)dtime->minute;
             second = (int)dtime->seconds;
             msec =   (int)dtime->frac_sec;

             te = (double) day * 86400 +
                  (double) hour * 3600  +
                  (double) minute *60 +
                  (double) second +
                  (double) msec/10000;
*/

             te = (double) dtime->day * 86400 +
                  (double) dtime->hour * 3600  +
                  (double) dtime->minute *60 +
                  (double) dtime->seconds +
                  (double) dtime->frac_sec/10000;

             /* Apply time correction to header time */

             tcor = (long)ddata->t1*65536 + (long)ddata->t2;
             ftcor = (double) tcor;
             ftcor /= 10000;

             te = te + (double) ftcor;

	     ns = (int) ddata->num_samples;

             /*  Extract # of samples per second */

	     if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier > 0) 
                 f_sam_rate = ddata->sample_rate * ddata->sample_rate_multiplier;
             if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier < 0) 
                 f_sam_rate = (-1) * ddata->sample_rate / ddata->sample_rate_multiplier;
             if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier > 0)  
                 f_sam_rate = (-1) * ddata->sample_rate_multiplier/ (float)(ddata->sample_rate);
             if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier < 0)   
                 f_sam_rate = 1./( (float)(ddata->sample_rate)/ ( (float)ddata->sample_rate_multiplier) );

             sps = (float)(f_sam_rate);

             dt = (double)(ns-1)/sps;

	     te = te + dt;

             day = hour = minute = second = msec = 0;

             while ( te > 86400 ) {
                     day += 1;
                     te -= 86400;
             }
             while ( te > 3600 ) {
                     hour += 1;
                     te -= 3600;
             }
             while ( te > 60 ) {
                     minute += 1;
                     te -= 60;
             }

             second = (int)floor(te);
             msec = (int)(fmod(te,(double)1)*1000 +0.0001);

             bl74->End_Time.day = (int)day;
             bl74->End_Time.hour = (int)hour;
             bl74->End_Time.minute = (int)minute;
             bl74->End_Time.second = (int)second;
             bl74->End_Time.msec = (int)msec;

/*printf("REPAIR(3): %03d:%02d:%02d:%02d.%04d\n",bl74->End_Time.day,
bl74->End_Time.hour,bl74->End_Time.minute,bl74->End_Time.second,bl74->End_Time.msec);*/

	     free(buf);

        }
    }
}
