/******************************************************************************/
/****                                                                       ***/
/****    Original version modified and extended by Reinoud Sleeman.         ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/

#include "conver.h"
#include "seed.h"                                           
                                                                       
 VOLUME_ID      Root_Volume;
 VOLUME_ID      *VI = &Root_Volume; 
                                                                       
STATION_ENTRY  *Current_Station=NULL,                                  
               *First_Station=NULL;                                    
                                                                       
CHANNEL_ENTRY  *Current_Channel=NULL,                                  
               *First_Channel=NULL;                                    
          
STDTIME _ztime = {0,0,0,0,0,0};                                         
STDTIME _etime = {9999,1,0,0,0,0};                                   
                                                                     
ULONG  DefBlkLen = 8192;            /* Default blockette buffer len */
ULONG  DefBlkExt = 2048;            /* Default blockette extension */ 


LOC_KEY_ID *UI[10];
BOOL       MERGE_FILES;
extern BOOL EXTRACT;


/*********************Blockette Creation Support**********************/
/*                                                                   
 *     Initialize a blockette structure for use (allocate the blockette
 *     buffer and set up the initial information)                     
 */                                                                  
                                                                     
BLOCKETTE *CreateBlockette()                                         
{                                                                    
       BLOCKETTE *InBlock;                                           
                                                                     
       InBlock = (BLOCKETTE *) malloc(sizeof(BLOCKETTE));            
       InBlock->BufLen = DefBlkLen;                    /* Default */ 
       InBlock->BlkBuff = (UBYTE *)malloc(InBlock->BufLen);     /* Allocate memory */   
       InBlock->BlkType = 0;                   /* Init remaining fields */     
       InBlock->BlkVer = 0;                                          
       InBlock->PackPos = 0;                                         
                                                                     
       return(InBlock);                /* Success */                 
}                                                                    
                                                                     
/*                                                                   
 *     Set up a blockette structure to contain data                  
 */                                                                  
                                                                     
VOID SetupBlockette(InBlock, BlockType, BlockVer)
BLOCKETTE *InBlock;
UWORD BlockType;
UWORD BlockVer;
{                                                                    
       InBlock->BlkType = BlockType;                                 
       InBlock->BlkVer = BlockVer;                                   
       InBlock->PackPos = 0;                                        
}                                                                    
                                                                     
/*                                                                   
 *     Extend a blockette                                            
 */                                                                  
                                                                     
BOOL ExtendBlockette(InBlock, Increments)            
BLOCKETTE *InBlock;
ULONG Increments;
{                                                                    
       ULONG NewLen;                                                 
       UBYTE *NewBuff;                                               
                                                                     
       if (Increments<=0) Increments = 1;      /* Default case */    
                                                                     
       NewLen = InBlock->BufLen + (DefBlkExt * Increments);          
       NewBuff = (UBYTE *)realloc(InBlock->BlkBuff, NewLen);  
       if (NewBuff==NULL) {                                          
               fprintf(stderr,"Could not extend a blockette\n");     
               return(FALSE);                                        
       }                                                             
       InBlock->BlkBuff = NewBuff;                                   
       InBlock->BufLen = NewLen;                                     
                                                                     
       return(TRUE);                                                 
}                                                                    
                                                                     
/*********************Create Standard Structures***********************/
/*                                                                    
 *     Initialize the Volume Structure                               
 */                                                                  
                                                                     
VOID InitializeVolume( starttime, endtime)             
STDTIME starttime;
STDTIME endtime;
{                                                                    
       VI = &Root_Volume;              /* Pointer to root structure */
       VI->Vol_Begin = starttime;                                    
       VI->End_Volume = endtime;                                     
       VI->Effective_Start = starttime;                              
       VI->Effective_End = endtime;                                  
       VI->Root_Abbrev = VI->Tail_Abbrev = NULL;       /* Init linked lists */ 
       VI->Root_Unit = VI->Tail_Unit = NULL;                         
       VI->Root_Format = VI->Tail_Format = NULL;                     
       VI->Root_Comment = VI->Tail_Comment = NULL;                   
       VI->Root_Station = VI->Tail_Station = NULL;                   
                                                                     
       VI->Root_Time_Entry = NULL;
       VI->Tail_Time_Entry = NULL;
       return;                                                       
}                                                                    
                                                                     
/*                                                                   
 *     Malloc a string and copy input there                          
 */                                                                  
                                                                     
UBYTE *AllocString(instr)                                     
UBYTE *instr;                                     
{                                                                    
       UBYTE *newstr;                                                
       newstr = (UBYTE *) malloc(strlen(instr)+1);     /* Get the null */      
       (VOID) strcpy(newstr, instr);           /* Copy into new buffer */      
       return(newstr);                                               
}                                                                    
                                                                     
/*                                                                   
 *     Remove spaces from keys                                       
 */                                                                  
                                                                     
#define MAXKEYLEN 50                                                 
UBYTE _searchkey[MAXKEYLEN];                                         
                                                                     
UBYTE *FixKey(inkey)                                          
UBYTE *inkey;                                          
{                                                                    
       UBYTE *oky,*iky;                                              

       if (inkey==NULL) return(NULL);                                
       for (oky=_searchkey,iky=inkey; *iky!='\0'; iky++) {           
               if (isspace(*iky)) continue;                          
		 *oky++ = *iky;
       }                                                             
       *oky++ = '\0';                                                
       return(_searchkey);                                           
}                                                                    
                                                                     
/**************Handle Abbreviation Structure and Linked Lists*************/    
/*                                                                   
 *     Search the abbreviation table for a given key                 
 */                                                                  
                                                                     
ABBREV *SearchAbbrev(inkey)                                   
UBYTE *inkey;                                   
{                                                                    
       ABBREV *looper;                                               
                                                                     
       for (looper=VI->Root_Abbrev; looper!=NULL; looper=looper->Next) {
               if (streq(looper->Abbrev_Key,FixKey(inkey))) return(looper);    
       }
       return(NULL);           /* Not Found */                       
}                                                                    
                                                                     
/*                                                                   
 *     Search the abbreviation table for a given index key           
 */                                                                  
                                                                     
ABBREV *SearchAbbrevIdx(inkey)
LONG inkey;                                            
{
       ABBREV *looper;
                     
       for (looper=VI->Root_Abbrev; looper!=NULL; looper=looper->Next) 
               if (looper->Abbrev_ID==inkey) return(looper);   
       return(NULL);           /* Not Found */                
}                                                            
                                                            
/*                                                         
 *     Malloc a new abbreviation structure and initialize it 
 */                                                        
                                                          
ABBREV *InitAbbrev()                                     
{                                                       
       ABBREV *Abbrev;                                
     
       Abbrev = (ABBREV *) malloc(sizeof (ABBREV));   
       Abbrev->Abbrev_Key = NULL;                   
       Abbrev->Abbrev_ID = 0;                      
       Abbrev->Comment = NULL;                    
       Abbrev->Use_Count = 0;                    
       Abbrev->Next = NULL;                     
                                              
       return(Abbrev);                       
}                                           
                                           
/*                                        
 *     Add a new abbreviation to the link list in alphabetical order  
 */                                                                 
                                                                   
BOOL LinkInAbbrev(inabbrev)
ABBREV *inabbrev;
{                                                                    
       int cm;                                                       
       ABBREV *looper,*last;                                         
                                                                     
       last = NULL;                                                  
       for (looper=VI->Root_Abbrev; looper!=NULL;                    
               last=looper,looper=looper->Next) {                    
               cm = strcmp(looper->Abbrev_Key,inabbrev->Abbrev_Key);   
               if (cm==0) return(FALSE);       /* Key already here! */
               if (cm<0) continue;             /* Key greater... */  
               break;                          /* Key is less */     
       }                                                             
                                                                     
       if (looper==NULL) {             /* Add to end of list */      
               if (VI->Tail_Abbrev==NULL)                            
                       VI->Root_Abbrev = VI->Tail_Abbrev = inabbrev; 
               else {                                                
                       VI->Tail_Abbrev->Next = inabbrev;            
                       VI->Tail_Abbrev = inabbrev;                   
               }                                                     
               inabbrev->Next = NULL;                                
       } else if (last==NULL) {        /* Insert to top of list */   
               inabbrev->Next = VI->Root_Abbrev;                     
               VI->Root_Abbrev = inabbrev;                           
       } else {                        /* Link into middle of list */
               last->Next = inabbrev;                                
               inabbrev->Next = looper;                              
       }                                                             
                                                                     
       return(TRUE);                                                 
}                                                                    
/*                                                                   
 *     Init, fill, and link in an abbreviation header                
 */                                                                  
                                                                     
BOOL AddAbbrev(key,comment)                            
UBYTE *key;
UBYTE *comment;
{                                                                    
       ABBREV *newabbrev;                                            
                                                                     
       newabbrev = InitAbbrev();                                     
       if (newabbrev==NULL) return(FALSE);     /* No luck */  
                                                             
       newabbrev->Abbrev_Key = AllocString(FixKey(key));    
       if (newabbrev->Abbrev_Key==NULL) return(FALSE);     
       newabbrev->Comment = AllocString(comment);                    
       if (newabbrev->Comment==NULL) return(FALSE);                  
                                                                     
       return(LinkInAbbrev(newabbrev));                              
}                                                                    
                                                                     
/*                                                                   
 *     Increment the use count of the abbreviation                   
 */                                                                  
                                                                     
VOID CountAbbrev(inabbrev)                                   
ABBREV *inabbrev;                                   
{                                                                    
       if (inabbrev==NULL) return;     /* Its ok */                  
       inabbrev->Use_Count++;                                        
}                                                                    

BOOL LinkInLocalAbbrev(inabbrev, fno) 
ABBREV *inabbrev;                                           
INT  fno;
{                                                                    
       int cm;                                                       
       ABBREV *looper,*last,*locabbrev;

       locabbrev = (ABBREV *) InitAbbrev();       
       if (locabbrev==NULL) return(FALSE);     
                                             
       locabbrev->Abbrev_Key = AllocString(inabbrev->Abbrev_Key);
       locabbrev->Abbrev_ID = inabbrev->Abbrev_ID;
       locabbrev->Comment = AllocString(inabbrev->Comment);
       locabbrev->Use_Count = inabbrev->Use_Count;
      

       if (UI[fno]->Tail_Abbrev==NULL)                              
             UI[fno]->Root_Abbrev = UI[fno]->Tail_Abbrev = locabbrev;       
       else {                                                
             UI[fno]->Tail_Abbrev->Next = locabbrev;                 
             UI[fno]->Tail_Abbrev = locabbrev;                       
       }                                                     
       locabbrev->Next = NULL;                                  
                                                                     
       return(TRUE);                                                 
}                                                                    

ABBREV *SearchLocalAbbrevIdx(fno,inkey)
INT  fno;
UWORD  inkey;                                      
{                                                                    
       ABBREV *looper;
                                                                     
       for (looper=UI[fno]->Root_Abbrev; looper!=NULL; looper=looper->Next) 
               if (looper->Abbrev_ID==inkey) return(looper);           
                                                                     
       return(NULL);           /* Not Found */                       
}                                                                    
                                                                     
/**************Handle Unit Structure and Linked Lists*************/  
                                                                    
/*                                                                   
 *     Search the unit table for a given key                         
 */                                                                  
                                                                     
UNIT *SearchUnit(inkey)                                       
UBYTE *inkey;                                       
{                                                                    
       UNIT *looper;                                                 
                                                                     
       for (looper=VI->Root_Unit; looper!=NULL; looper=looper->Next) 
               if (streq(looper->Unit_Key,FixKey(inkey))) return(looper);      
       return(NULL);           /* Not Found */                       
}                                                               
                                                                     
/*                                                                   
 *     Search the unit table for a given index key                   
 */                                                                  
                                                                     
UNIT *SearchUnitIdx(inkey)                                      
LONG inkey;                                      
{                                                                    
       UNIT *looper;                                                 
                                                                     
       for (looper=VI->Root_Unit; looper!=NULL; looper=looper->Next) 
          {
               if (looper->Unit_ID==inkey) return(looper);           
          }
                                                                     
       return(NULL);           /* Not Found */                       
}                                                                    

UNIT *SearchLocalUnitIdx(fno,inkey)
INT  fno;
UWORD  inkey;                                      
{                                                                    
       UNIT *looper;
                                                                     
       for (looper=UI[fno]->Root_Unit; looper!=NULL; looper=looper->Next) 
               if (looper->Unit_ID==inkey) return(looper);           
                                                                     
       return(NULL);           /* Not Found */                       
}                                                                    
                                                                     
/*                                                                   
 *     Malloc a new unit structure and initialize it                 
 */

UNIT *InitUnit()                                                        
{                                                                    
       UNIT *Unit;                                                   
                                                                     
       Unit = (UNIT *) malloc(sizeof (UNIT));                        
       Unit->Unit_Key = NULL;                                        
       Unit->Unit_ID = 0;                                            
       Unit->Comment = NULL;                                         
       Unit->Use_Count = 0;                                          
       Unit->Next = NULL;                                            
                                                                     
       return(Unit);                                                 
}                                                                    
                                                                     
/*                                                                   
 *     Add a new unit to the link list in alphabetical order         
 */                                                                  
                                                                     
BOOL LinkInUnit(inunit)                                        
UNIT *inunit;                                           
{                                                                    
       int cm;                                                       
       UNIT *looper,*last;                                           
                                                                     
       last = NULL;                                                  
       for (looper=VI->Root_Unit; looper!=NULL;                      
               last=looper,looper=looper->Next) {                    
               cm = strcmp(looper->Unit_Key,inunit->Unit_Key);         
               if (cm==0) return(FALSE);       /* Key already here! */
               if (cm<0) continue;             /* Key greater... */  
               break;                          /* Key is less */     
       }                                                             
                                                                     
       if (looper==NULL) {             /* Add to end of list */      
               if (VI->Tail_Unit==NULL)                              
                       VI->Root_Unit = VI->Tail_Unit = inunit;       
               else {                                                
                       VI->Tail_Unit->Next = inunit;                 
                       VI->Tail_Unit = inunit;                       
               }                                                     
               inunit->Next = NULL;                                  
       } else if (last==NULL) {        /* Insert to top of list */   
               inunit->Next = VI->Root_Unit;                         
               VI->Root_Unit = inunit;                               
       } else {                        /* Link into middle of list */
               last->Next = inunit;                                  
               inunit->Next = looper;                                
       }                                                             
      
       return(TRUE);                                                 
}                                                                    
                                                                     
/*
 *     Init, fill, and link in a unit header
 */

BOOL AddUnit(key,comment)                              
UBYTE *key;
UBYTE *comment;                                        
{                                                                    
       UNIT *newunit;                                                
                                                                     
       newunit = InitUnit();                                         
       if (newunit==NULL) return(FALSE);       /* No luck */         
       newunit->Unit_Key = AllocString(FixKey(key));                 
       if (newunit->Unit_Key==NULL) return(FALSE);                   
       newunit->Comment = AllocString(comment);                      
       if (newunit->Comment==NULL) return(FALSE);                    
       return(LinkInUnit(newunit));                                  
}                                                                    
                                                                     
/*                                                                   
 *     Increment use count of the unit                               
 */                                                                  
                                                                     
VOID CountUnit(inunit)                                         
UNIT *inunit;                                         
{                                                                    
       if (inunit==NULL) return;       /* Its ok */
       inunit->Use_Count++;                      
}                                               


/******  Store units for inputfile INP_FILENO in struct local_units ***/

                                                                     
BOOL LinkInLocalUnit(inunit, fno) 
UNIT *inunit;                                           
INT  fno;
{                                                                    
       int cm;                                                       
       UNIT *looper,*last, *locunit;
                                                                     
       locunit = (UNIT *) InitUnit();       
       if (locunit==NULL) return(FALSE);     
                                             
       locunit->Unit_Key = AllocString(inunit->Unit_Key);
       locunit->Unit_ID = inunit->Unit_ID;
       locunit->Comment = AllocString(inunit->Comment);
       locunit->Use_Count = inunit->Use_Count;
      
       if (UI[fno]->Tail_Unit==NULL)                              
             UI[fno]->Root_Unit = UI[fno]->Tail_Unit = locunit;       
       else {                                                
             UI[fno]->Tail_Unit->Next = locunit;                 
             UI[fno]->Tail_Unit = locunit;                       
       }                                                     
       locunit->Next = NULL;                                  
                                                                     
       return(TRUE);                                                 
}                                                                    


LOC_KEY_ID *InitKey(fno)
{
       LOC_KEY_ID *loc_key;

       loc_key = (LOC_KEY_ID *) malloc ( sizeof(LOC_KEY_ID ));

       loc_key->Root_Unit = loc_key->Tail_Unit = NULL;  
       loc_key->Root_Format = loc_key->Tail_Format = NULL;
       loc_key->Root_Abbrev = loc_key->Tail_Abbrev = NULL; 

       return(loc_key);
}
                                                                     

                                               
/**************Handle Comment Structure and Linked Lists*************/  
/*                                                                    
 *     Search the comment table for a given key                     
 */                                                               
                                                                 
COMMENT *SearchComment(inid)
UWORD inid;
{
       COMMENT *looper;

       for (looper=VI->Root_Comment; looper!=NULL; looper=looper->Next) 
               if (looper->Comment_ID==inid) return(looper);
       return(NULL);           /* Not Found */            
}                                                        
                                                        
/*                                                     
 *     Malloc a new comment structure and initialize it
 */   
     
COMMENT *InitComment() 
{                                                          
       COMMENT *Comment;                                   
      
       Comment = (COMMENT *) malloc(sizeof (COMMENT));
       Comment->Comment_ID = 0;                    
       Comment->Class = ' ';                      
       Comment->Text = NULL;                     
       Comment->Level = NULL;                   
       Comment->Use_Count = 0;                 
       Comment->Next = NULL;                  
                                             
       return(Comment);                     
}                                          
                                          
/*                                       
 *     Add a new commentiation to the link list in alphabetical order
 */                                                                
                                                                  
BOOL LinkInComment(incomment)                                         
COMMENT *incomment;                                         
{                                                                
       COMMENT *looper,*last;                                  
                                                              
       last = NULL;                                          
       for (looper=VI->Root_Comment; looper!=NULL;          
               last=looper,looper=looper->Next) {          
               if (looper->Comment_ID==incomment->Comment_ID) return(FALSE);   
               if (looper->Comment_ID<incomment->Comment_ID) continue;
               break;                          /* Key is less */     
       }                                                            
                                                                   
       if (looper==NULL) {             /* Add to end of list */   
               if (VI->Tail_Comment==NULL)                       
                       VI->Root_Comment = VI->Tail_Comment = incomment;
               else {                                                 
                       VI->Tail_Comment->Next = incomment;            
                       VI->Tail_Comment = incomment;                 
               }                                                    
               incomment->Next = NULL;                             
       } else if (last==NULL) {        /* Insert to top of list */ 
               incomment->Next = VI->Root_Comment;                
               VI->Root_Comment = incomment;                     
       } else {                        /* Link into middle of list */
               last->Next = incomment;                              
               incomment->Next = looper;                           
       }                                                          
      
       return(TRUE);   
}                     
                     
/*                  
 *     Init, fill and link a comment header 
 */                                       
                                         
BOOL AddComment( comnum, class,text,level)             
UWORD comnum;
UBYTE class;
UBYTE *text;
UBYTE *level;             
{                                                                    
       UNIT *levcom;                                                 
       COMMENT *newcomment;                                          
                                                                     
       newcomment = InitComment();                                   
       if (newcomment==NULL) return(FALSE);    /* No luck */         
       newcomment->Comment_ID = comnum;                              
       newcomment->Class = class;                                    
       newcomment->Text = AllocString(text);                         
       if (newcomment->Text==NULL) return(FALSE);                    
       levcom = SearchUnit(level);                
       newcomment->Level = levcom;               
       CountUnit(levcom);                      /* Increment the count */
       if (levcom!=NULL) printf("Found level %s\n",levcom->Unit_Key);  
      
       return(LinkInComment(newcomment));                            
}                                                                   
                                                                  
/*                                                                 
 *     Increment use count of the comment                       
 */                                                           
                                                             
VOID CountComment(incomment)                                          
COMMENT *incomment;                                          
{                                                           
       if (incomment==NULL) return;    /* Its ok */       
       incomment->Use_Count++; 
}                             

CITED_SOURCE *SearchCitedSource(inkey) 
UBYTE *inkey;                                             
{                                                                  
       CITED_SOURCE *looper;                                            
                                                                 
       for (looper=VI->Root_Cited_Source; looper!=NULL; looper=looper->Next){
               if (streq(looper->Cited_Source_Key,inkey)) return(looper);    
       }
       return(NULL);           /* Not Found */                       
}                                                                   
                                                                  

CITED_SOURCE *InitCitedSource() 
{                                                          
       CITED_SOURCE *source;                                   
      
       source = (CITED_SOURCE *) malloc(sizeof (CITED_SOURCE));
       source->Source_ID = 0;                      
       source->Cited_Source_Key = NULL;                      
       source->Name_Publication = NULL;                     
       source->Date_Published = NULL;                   
       source->Publisher_Name = NULL;                  
                                             
       return(source);                     
}                                          
                                          
BOOL LinkInCitedSource(insource)  
CITED_SOURCE *insource ;                                    
{                                                                
       CITED_SOURCE *looper,*last;                                  
                                                              
       last = NULL;                                          
       for (looper=VI->Root_Cited_Source; looper!=NULL;          
               last=looper,looper=looper->Next) {          
               if ((strcmp(looper->Cited_Source_Key,insource->Cited_Source_Key))== 0 ) 
                          return(FALSE);   
       }                                                            
                                                                   
       if (VI->Tail_Cited_Source==NULL)                       
              VI->Root_Cited_Source = VI->Tail_Cited_Source = insource;
       else {                                                 
              VI->Tail_Cited_Source->Next = insource;            
              VI->Tail_Cited_Source = insource;                 
       }                                                    
       insource->Next = NULL;                             
      
       return(TRUE);   
}                     

/*                  
 *     Init, fill and link a  cited source 
 */                                       
                                         
BOOL AddCitedSource ( key, name, date, publisher)             
UBYTE *key, *name, *date, *publisher;
{                                                                    
       CITED_SOURCE *new_cited_source;
                                                                     
       new_cited_source = (CITED_SOURCE *) InitCitedSource();
       if (new_cited_source==NULL) return(FALSE);    /* No luck */ 

       new_cited_source->Cited_Source_Key = AllocString(key);
       if (new_cited_source->Cited_Source_Key==NULL) return(FALSE); 
       new_cited_source->Name_Publication = AllocString(name);    
       if (new_cited_source->Name_Publication==NULL) return(FALSE);
       new_cited_source->Date_Published = AllocString(date);    
       if (new_cited_source->Date_Published==NULL) return(FALSE);
       new_cited_source->Publisher_Name = AllocString(publisher);    
       if (new_cited_source->Publisher_Name==NULL) return(FALSE);

       return(LinkInCitedSource(new_cited_source));   

}                                                                   
                                                                  
                             
/**************Handle Format Structure and Linked Lists*************/  
/*                         
 *     Search the format table for a given key   
 */                                                                  
                                                                    
FORMAT *SearchFormat(inkey)                                             
UBYTE *inkey;                                             
{                                                                  
       FORMAT *looper;                                            
                                                                 
       for (looper=VI->Root_Format; looper!=NULL; looper=looper->Next)
               if (streq(looper->Format_Key,FixKey(inkey))) return(looper);    
       return(NULL);           /* Not Found */                       
}                                                                   

FORMAT *SearchFormatName(inkey)
UBYTE *inkey;                                             
{                                                                  
       FORMAT *looper;                                            
                                                                 
       if ( MERGE_FILES ) {
            for (looper=VI->Root_Format; looper!=NULL; looper=looper->Next){
           
                    if (streq(looper->Format_Name,inkey)) {
                                return(looper);    
                    }
            }
       }
       return(NULL);           /* Not Found */                       
}                                                                   
                                                                  
/*                                                                 
 *     Search the format table for a given index key            
 */
  
FORMAT *SearchFormatIdx(inkey)   
LONG inkey;                                            
{                               
       FORMAT *looper;        
      
       for (looper=VI->Root_Format; looper!=NULL; looper=looper->Next) {
               if (looper->Format_ID==inkey) return(looper);           
       }                                                              
       return(NULL);           /* Not Found */                      
}                                                                  
                                                                  
/*                                                               
 *     Malloc a new format structure and initialize it         
 */                                                          
                                                            
FORMAT *InitFormat()                                       
{                                                         
       FORMAT *Format;                                  
       int i;        
                    
       Format = (FORMAT *) malloc(sizeof (FORMAT));   
       Format->Format_Key = NULL;                             
       Format->Format_Name = NULL;                           
       Format->Format_ID = 0;                               
       Format->Family = 0;                                 
       Format->NumKeys = 0;                               
       Format->Use_Count = 0;                            
       for (i=0; i<MAXKEYS; i++) Format->Keys[i] = NULL;
       Format->Next = NULL;                            
       return(Format);                               
}                                                   
                                                   
/*                                                
 *     Add a new format to the link list in alphabetical order 
 */                                                          
                                                            
BOOL LinkInFormat(informat)                                            
FORMAT *informat;                                          
{                                                         
       int cm;                                          
       FORMAT *looper,*last;                           
                                                      
       last = NULL;       
       for (looper=VI->Root_Format; looper!=NULL;  
               last=looper,looper=looper->Next) {   
/*
               cm = strcmp(looper->Format_Key,informat->Format_Key);
*/
               cm = strcmp(looper->Format_Name,informat->Format_Name);
               if (cm==0) return(FALSE);       /* Key already here! */ 
/*
               if (cm<0) continue; 
               break;             
*/
       }                                                            
      
       if (looper==NULL) {             /* Add to end of list */   
               if (VI->Tail_Format==NULL)                        
                       VI->Root_Format = VI->Tail_Format = informat; 
               else {                                               
                       VI->Tail_Format->Next = informat;           
                       VI->Tail_Format = informat;                
               }                                                 
               informat->Next = NULL;                           
       } else if (last==NULL) {        /* Insert to top of list */ 
               informat->Next = VI->Root_Format;                  
               VI->Root_Format = informat;                       
       } else {                        /* Link into middle of list */ 
               last->Next = informat;                                
               informat->Next = looper;                             
       }                                                           
       return(TRUE);
}

/*
 *     Init, fill, and link in a format
 */

BOOL AddFormat(key,name, family,numkeys,keylist)                
UBYTE *key;
UBYTE *name;
UWORD family;                  
UWORD numkeys;
UBYTE *keylist[MAXKEYS];                
{                                                                    
       FORMAT *newformat;                                            
       int i, Nkeys;
                                                                     
       newformat = InitFormat();                                     
       if (newformat==NULL) return(FALSE);     /* No luck */         
       newformat->Format_Key = AllocString(FixKey(key));             
       if (newformat->Format_Key==NULL) return(FALSE);               
       newformat->Format_Name = AllocString(name);                   
       if (newformat->Format_Name==NULL) return(FALSE);              
       newformat->Family = family;                                   
       newformat->NumKeys = numkeys;                                 
     
       for (i=0; i<numkeys; i++) {  
               newformat->Keys[i] = AllocString(keylist[i]);
               if (newformat->Keys[i]==NULL) return(FALSE);
       }
      
       return(LinkInFormat(newformat));
}

/*
 *     Increment the use count of the format
 */
  
VOID CountFormat(informat)                                             
FORMAT *informat;                                             
{                                                                    
       if (informat==NULL) return;     /* Its ok */                  
       informat->Use_Count++;                                        
}                                                                    

FORMAT *SearchLocalFormatIdx(fno,inkey)
INT  fno;
UWORD  inkey;                                      
{                                                                    
       FORMAT *looper;
                                                                     
       for (looper=UI[fno]->Root_Format; looper!=NULL; looper=looper->Next) 
               if (looper->Format_ID==inkey) return(looper);           
                                                                     
       return(NULL);           /* Not Found */                       
}                                                                    

BOOL LinkInLocalFormat(informat, fno) 
FORMAT *informat;                                           
INT  fno;
{                                                                    
       int cm,i;
       FORMAT *looper,*last, *looper2, *locformat;
                                                                     
       locformat = (FORMAT *) InitFormat();       
       if (locformat==NULL) return(FALSE);     
                                             
       locformat->Format_Key = AllocString(informat->Format_Key);
       locformat->Format_Name = AllocString(informat->Format_Name);
       locformat->Format_ID = informat->Format_ID;
       locformat->Family = informat->Family;
       locformat->NumKeys = informat->NumKeys;
      
       for (i=0; i<informat->NumKeys; i++)                       
               locformat->Keys[i] = AllocString(informat->Keys[i]);    
           
       if (UI[fno]->Tail_Format==NULL)                              
             UI[fno]->Root_Format = UI[fno]->Tail_Format = locformat;       
       else {                                                
             UI[fno]->Tail_Format->Next = locformat;                 
             UI[fno]->Tail_Format = locformat;                       
       }                                                     
       locformat->Next = NULL;                                  


/*
       for (looper2=UI[fno]->Root_Format; looper2!=NULL; looper2=looper2->Next) 
		printf("UI[x] format_key:  %s  %d  %s\n", looper2->Format_Key,looper2->Format_ID,looper2->Format_Name);
       for (looper2=VI->Root_Format; looper2!=NULL; looper2=looper2->Next) 
		printf("VI[] format_key:  %s  %d  %s\n", looper2->Format_Key,looper2->Format_ID,looper2->Format_Name);
*/
                                                                     
       return(TRUE);                                                 
}                                                                    
                                                                     
/**************************Station Table Support*****************************/ 
                                                                     
/*                                                                   
 *     Search the station table for the given key                    
 */                                                                  
                                                                     
STATION_ENTRY *SearchStation(inkey)                           
UBYTE *inkey;                           
{                                                                    
       STATION_ENTRY *looper;                                        
                                                                     
       for (looper=VI->Root_Station; looper!=NULL; looper=looper->Next)
               if (streq(looper->Call_Letters,FixKey(inkey))) return(looper);  
       return(NULL);           /* Not Found */                       
}                                                                    
                                                                     
/*                                                                   
 *     Malloc a new station structure and initialize it
 */
                                                                     
STATION_ENTRY *InitStation()                                         
{                                                                    
       STATION_ENTRY *Station;                                       
       int i;                                                        
                                                                     
       Station = (STATION_ENTRY *) malloc(sizeof (STATION_ENTRY));   
                                                                     
       Station->RecNum = 0;
       Station->Call_Letters = NULL;
       Station->Station_Owner = NULL;
       Station->Number_Channels = 0;
       Station->Root_Channel=Station->Tail_Channel=NULL;
       Station->Number_Comments = 0;
       Station->Root_Comment=Station->Tail_Comment=NULL; 
       Station->Update_Station = NULL;
       Station->Next = NULL;
      
       Station->Lattitude = 0;
       Station->Longitude = 0;
       Station->Elevation = 0;
       Station->Long_Order = 0;
       Station->Word_Order = 0;
       Station->Effective_Start = _ztime;
       Station->Effective_End = _ztime;
       Station->Update_Flag = ' ';
       Station->Use_Count = 0;
      
       return(Station);
}                     

/*                   
 *     Add a new station to the link list in alphabetical order  
 */

BOOL LinkInStation(instation)                                   
STATION_ENTRY *instation;                                   
{ 
       int cm;
       STATION_ENTRY *looper,*last;
       STDTIME *updated_time;
                                  
       last = NULL;              
       for (looper=VI->Root_Station; looper!=NULL;   
               last=looper,looper=looper->Next) {   
               cm = strcmp(looper->Call_Letters,instation->Call_Letters);      
               if (cm==0) {                    /* Key already here! */

               /* Update effective time ... */
		  updated_time = (STDTIME *) UpdateStartTime ( &looper->Effective_Start, 
                                  instation->Effective_Start );
		  looper->Effective_Start.year   = updated_time->year;
		  looper->Effective_Start.day    = updated_time->day;
		  looper->Effective_Start.hour   = updated_time->hour;
		  looper->Effective_Start.minute = updated_time->minute;
		  looper->Effective_Start.second = updated_time->second;
		  looper->Effective_Start.msec   = updated_time->msec;
                  
		  updated_time = (STDTIME *) UpdateEndTime ( &looper->Effective_End, 
	                          instation->Effective_End );
		  looper->Effective_End.year   = updated_time->year;
		  looper->Effective_End.day    = updated_time->day;
		  looper->Effective_End.hour   = updated_time->hour;
		  looper->Effective_End.minute = updated_time->minute;
		  looper->Effective_End.second = updated_time->second;
		  looper->Effective_End.msec   = updated_time->msec;
		  return(FALSE);  
	       }

               if (cm<0) continue;             /* Key greater... */  
               break;                          /* Key is less */    
       }   
          
       if (looper==NULL) {             /* Add to end of list */   
               if (VI->Tail_Station==NULL)  
                       VI->Root_Station = VI->Tail_Station = instation;
               else {                                                 
                       VI->Tail_Station->Next = instation;           
                       VI->Tail_Station = instation;                
               }  
               instation->Next = NULL;   
       } else if (last==NULL) {        /* Insert to top of list */
               instation->Next = VI->Root_Station;               
               VI->Root_Station = instation;                    
       } else {                        /* Link into middle of list */ 
               last->Next = instation;                               
               instation->Next = looper;                            
       }                                                           
                                                                  
       return(TRUE);                                             
}                                                               
                                                               
/**********************Load Comment Information*************************/      
/*                                                           
 *     Search the channel table for the given key          
 */                                                      
                                                        
CHANNEL_ENTRY *SearchChannel(station,loc,chan)    
STATION_ENTRY *station;
UBYTE *loc;
UBYTE *chan;    
{                                                      
       CHANNEL_ENTRY *looper; 
                                                                     
       for (looper=station->Root_Channel; looper!=NULL; looper=looper->Next) { 
               if (loc==NULL&&looper->Location!=NULL) continue;
               if (loc!=NULL&&looper->Location==NULL) continue;
               if (loc!=NULL||looper->Location!=NULL) 
                       if (!streq(looper->Location,FixKey(loc))) continue;     
               if (streq(looper->Identifier,FixKey(chan))) return(looper);     
       } 
        
       return(NULL);           /* Not Found */
}                                            
                                            
/*                                         
 *     Malloc a new channel structure and initialize it   
 */                                                     
                                                             
CHANNEL_ENTRY *InitChannel()   
{                                                      
       CHANNEL_ENTRY *Channel;                       
       int i;                                       
     
       Channel = (CHANNEL_ENTRY *) malloc(sizeof (CHANNEL_ENTRY));
       Channel->Location = NULL;                               
       Channel->Identifier = NULL;                            
       Channel->SubChannel = 0;                              
       Channel->Instrument = NULL;                          
       Channel->Optional = NULL;                           
       Channel->Signal_Response = NULL;                   
       Channel->Calibration_Input = NULL;                
       Channel->Lattitude = 0;                          
       Channel->Longitude = 0;                         
       Channel->Elevation = 0;                        
       Channel->Local_Depth = 0;                     
       Channel->Azimuth = 0;                        
       Channel->Dip = 0;                           
       Channel->Format_Type = 0;                  
       Channel->Data_Exp = 0;                    
       Channel->Sample_Rate = 0;                
       Channel->Max_Drift = 0;                 
       Channel->Channel_Flags = 0;            
       Channel->Number_Comments = 0;         
       Channel->Root_Comment = Channel->Tail_Comment = NULL;    
       Channel->Effective_Start = _ztime;                      
       Channel->Effective_End = _ztime;                       
       Channel->Update_Channel = NULL;                       
       Channel->Update_Flag = ' ';                          
       Channel->Root_Response = Channel->Tail_Response = NULL; 
       Channel->Next = NULL;                                  
       Channel->Root_Data = Channel->Tail_Data = NULL;       
       Channel->Use_Count = 0;                              
                                                           
       return(Channel);   
}                       
                         
/*                     
 *     Format nice name from channel   
 */                                  
                                    
VOID ChanText(chan,buf) 
CHANNEL_ENTRY *chan;
UBYTE *buf; 
{                                                                    
       if (chan->Location!=NULL)                                     
               sprintf(buf,"%s/%s",chan->Location,chan->Identifier); 
       else    sprintf(buf,"%s",chan->Identifier);                   
}                                                                    
                                                                     
/*                                                                   
 *     Channel name comparisons (with ZNE order)                     
 */                                                                  
                                                                     
VOID _putdes(buf, pos,search, ch)              
UBYTE *buf;
int pos;
UBYTE *search;
UBYTE ch;              
{                                                                    
       UBYTE *new;                                                   

       buf[pos] = '_';                                               
       new = (UBYTE *) strchr(search,ch);
      
       if (new==NULL) {                                              
               buf[pos] = '*';                                       
               buf[pos+1] = ch;                                      
               return;                                               
       }                                                             
       buf[pos+1] = 'A' + (new - search);                            
}                                                                    
                                                                     
/*
UBYTE *_sortkey[3] = { "HBESMLVURAX","HLGTABSMCX","ZNEABCTRI123X"};  
*/

VOID _ugname(chan,buf)                         
CHANNEL_ENTRY *chan;
UBYTE *buf;                         
{     
       UBYTE newident[20];                                           
       int i;                                                        
       UBYTE *_sortkey[3][15];  

       strcpy(_sortkey[0],"HBESMLVURAX");
       strcpy(_sortkey[1],"HLGTABSMCX");
       strcpy(_sortkey[2],"ZNEABCTRI123X");  

       for (i=0; i<8; i++) newident[i] = '\0';                       
       for (i=0; i<6; i+=2)                                          
               if (chan->Identifier[i>>1]=='\0') break;              
               else    _putdes(newident,i,_sortkey[i>>1],            
                               chan->Identifier[i>>1]);              
                                                                     
       if (chan->Location!=NULL)                                     
               sprintf(buf,"%s/%s",chan->Location,newident);         
       else    sprintf(buf,"%s",newident);                           
}                                                                    
                                                                     
int _chancomp(chan1,chan2)             
CHANNEL_ENTRY *chan1;
CHANNEL_ENTRY *chan2;             
{                                                                    
       UBYTE name1[20],name2[20],*cp;                                
       int i;                                                        
      
       if (chan1->Location==NULL&&chan2->Location!=NULL) return(-1); 
       if (chan1->Location!=NULL&&chan2->Location==NULL) return(1);  
       _ugname(chan1,name1);                                         
       _ugname(chan2,name2);                                         
      
       return(strcmp(name1,name2));                                  
}                                                                    
                                                                     
/*                                                                   
 *     Add a new channel in order of appearance                      
 */                                                                  
                                                                     
BOOL LinkInChannel(instation,inchannel)
STATION_ENTRY *instation;
CHANNEL_ENTRY *inchannel;
{                                                                    
       int cm;                                                       
      
       CHANNEL_ENTRY *looper,*last;                                  
       last = NULL;                                                  
      
/*
       for (looper=instation->Root_Channel; looper!=NULL;            
               last=looper,looper=looper->Next) {                    
               cm = _chancomp(looper,inchannel);  
               if (cm==0){
		        return(FALSE);   
	       }
               if (cm<0) continue;        
               break;                    
       }                                                             
*/



if ( !GDSN ) {

       for (looper=instation->Root_Channel; looper!=NULL;   
                                    last=looper,looper=looper->Next) {  

                  cm = strcmp(looper->Identifier,inchannel->Identifier);

              /*  Channel and subchannel already here  */

                  if ( (cm==0) && 
                      (looper->SubChannel == inchannel->SubChannel) ) {
		           	return(FALSE);   
                  }

              /*  New subchannel for channel  */

                  if ( (cm==0) && 
                       (looper->SubChannel < inchannel->SubChannel) ) {
		                continue;   
                  }
                  if (cm<0) continue;        
               
                  break;                    
       }
}
else if ( GDSN ) 
         looper = NULL;   /*  GDSN: channels stored as LPZ,
                                                       LPN,
                                                       LPE  */


      
       if (looper==NULL) {             /* Add to end of list */      
               if (instation->Tail_Channel==NULL)                    
                       instation->Root_Channel =                     
                               instation->Tail_Channel = inchannel;
               else {                                             
                       instation->Tail_Channel->Next = inchannel;
                       instation->Tail_Channel = inchannel;     
               }                                               
               inchannel->Next = NULL;            
       } else if (last==NULL) {        /* Insert to top of list */ 
               inchannel->Next = instation->Root_Channel;         
               instation->Root_Channel = inchannel;              
       } else {                        /* Link into middle of list */
               last->Next = inchannel;                              
               inchannel->Next = looper;                           
       }                                                          
    
/*
       instation->Number_Channels ++;   
*/
   
       return(TRUE);
}                  
                  
/*               
 *     Convert from ascii flag representation to bits                
 */                                                                 
                                                                     
ULONG CnvChanFlags(inflags)                                             
UBYTE *inflags;                                             
{                                                                    
       ULONG flags;                                         
       int i;                                                        
                                                                     
       /*Upcase(inflags);  */       
                                                                     
       flags = 0;                                                    
       for (i=0; i<strlen(inflags); i++)  {                           
         switch(inflags[i]) {                                        
               case 'T':       flags |= CFG_TRIGGERED;         break; 
               case 'C':       flags |= CFG_CONTINUOUS;        break;
               case 'H':       flags |= CFG_HEALTHCHAN;        break;
               case 'G':       flags |= CFG_GEODATA;           break; 
               case 'W':       flags |= CFG_ENVIRON;           break;
               case 'F':       flags |= CFG_FLAGS;             break;
               case 'S':       flags |= CFG_SYNTH;             break;
               case 'I':       flags |= CFG_CALIN;             break;
               case 'E':       flags |= CFG_EXPERIMENT;        break;
               case 'M':       flags |= CFG_MAINTENANCE;       break;
               case 'B':       flags |= CFG_BEAM;              break;
               default:                                              
            
                       fprintf(stderr,"Illegal flag %c\n",inflags[i]);
         }
       }                                                           
       return(flags);
}

/******************Deal with Response Structures**********************/ 
 
/*
 *     Init and Zero a zeros and poles response        
 */

ZEROS_POLES *InitZerosPoles( numzeros, numpoles)                  
UWORD numzeros, numpoles;                  
{                                                          
       ZEROS_POLES     *pz;                                
       int i;                                              
      
       pz = (ZEROS_POLES *) malloc(sizeof (ZEROS_POLES));  
       pz->Function_Type = ' ';                            
       pz->Cascade = 0;                                    
       pz->Response_In = NULL;                             
       pz->Response_Out = NULL;                            
       pz->A0_Normalization = 0;                           
       pz->Cite_Name = NULL;                               
       pz->Complex_Zeros = numzeros;                       
      
       if (numzeros>0) {                                   
               pz->Zero_Real_List = (VOLFLT *)             
                       malloc(sizeof(VOLFLT) * numzeros);   
                    
               pz->Zero_Imag_List = (VOLFLT *)
                       malloc(sizeof(VOLFLT) * numzeros);
               pz->Zero_Real_Errors = (VOLFLT *) 
                       malloc(sizeof(VOLFLT) * numzeros);
               pz->Zero_Imag_Errors = (VOLFLT *) 
                       malloc(sizeof(VOLFLT) * numzeros);
       }       

       for (i=0; i<numzeros; i++) { 
               pz->Zero_Real_List[i] = 0.0;    
               pz->Zero_Imag_List[i] = 0.0;   
               pz->Zero_Real_Errors[i] = 0.0;
               pz->Zero_Imag_Errors[i] = 0.0;
       }                                    
                                           
       pz->Complex_Poles = numpoles;      
       if (numpoles>0) {                 
               pz->Pole_Real_List = (VOLFLT *)
                       malloc(sizeof(VOLFLT) * numpoles); 
               pz->Pole_Imag_List = (VOLFLT *)  
                       malloc(sizeof(VOLFLT) * numpoles);
               pz->Pole_Real_Errors = (VOLFLT *)
                       malloc(sizeof(VOLFLT) * numpoles); 
               pz->Pole_Imag_Errors = (VOLFLT *) 
                       malloc(sizeof(VOLFLT) * numpoles);
       }       
              
       for (i=0; i<numpoles; i++) {
               pz->Pole_Real_List[i] = 0.0;
               pz->Pole_Imag_List[i] = 0.0;   
               pz->Pole_Real_Errors[i] = 0.0;
               pz->Pole_Imag_Errors[i] = 0.0;
       }                                    
       return(pz);                        
}      
      
/*   
 *     Init and coefficients response
 */
  
COEFFICIENTS *InitCoefficients( numnum, numden)   
UWORD numnum, numden;   
{                  
       COEFFICIENTS    *co;                                
       int i;                                              
                                                           
       co = (COEFFICIENTS *) malloc(sizeof (COEFFICIENTS));  
       co->Function_Type = ' ';                             
       co->Cascade = 0;                                    
       co->Response_In = NULL;                             
       co->Response_Out = NULL;                            
       co->Cite_Name = NULL;                               
                                                           
       co->Numerators = numnum;                            
       if (numnum>0) { 
               co->Numer_Coeff = (VOLFLT *)
                       malloc(sizeof(VOLFLT) * numnum);
               co->Numer_Error = (VOLFLT *)                    
                       malloc(sizeof(VOLFLT) * numnum);       
       }                                                      
                                                             
       for (i=0; i<numnum; i++) {                           
               co->Numer_Coeff[i] = 0.0;                   
               co->Numer_Error[i] = 0.0;                   
       }                                                   
                                                           
       co->Denominators = numden;                          
       if (numden>0) {                                     
               co->Denom_Coeff = (VOLFLT *)                
                       malloc(sizeof(VOLFLT) * numden);    
               co->Denom_Error = (VOLFLT *)
                       malloc(sizeof(VOLFLT) * numden);          
       }                                                      
      
       for (i=0; i<numden; i++) {                           
               co->Denom_Coeff[i] = 0.0;                   
               co->Denom_Error[i] = 0.0;                   
       }                                                   
       return(co);                                         
}                                                          
                                                           
/*                                                         
 *     Init and zero a decimation record                   
 */                                                        
                                                           
DECIMATION *InitDecimation()                           
{                                      
       DECIMATION *deci;     
                             
       deci = (DECIMATION *) malloc(sizeof (DECIMATION));  
       deci->Cascade = 0;                                            
       deci->Input_Rate = 0.0;                                       
       deci->Factor = 0;                                             
       deci->Offset = 0;                                             
       deci->Delay = 0.0;                                            
       deci->Correction = 0.0;      
       deci->Cite_Name = NULL;     
       return(deci);              
}                               
                               
/*                            
 *     Init and zero a response sensitivity   
 */                                         
                                           
SENSITIVITY *InitSensitivity( numcals)    
UWORD numcals;                       
{                                        
       SENSITIVITY *sens;              
       int i;                         
    
       sens = (SENSITIVITY *) malloc(sizeof (SENSITIVITY));
       sens->Cascade = 0;         
       sens->Sensitivity = 0.0;  
       sens->Frequency = 0.0;   
       sens->Num_Cals = numcals;
       sens->Cite_Name = NULL; 
                              
       if (numcals>0) {      
               sens->Cal_Value = (VOLFLT *) 
                       malloc(sizeof(VOLFLT) * numcals);     
               sens->Cal_Freq = (VOLFLT *)                 
                       malloc(sizeof(VOLFLT) * numcals);   
               sens->Cal_Time = (STDTIME *)
                       malloc(sizeof(STDTIME) * numcals);  
       }
      
       for (i=0; i<numcals; i++) {
               sens->Cal_Value[i] = 0.0;
               sens->Cal_Freq[i] = 0.0;
               sens->Cal_Time[i] = _ztime; 
       }
       return(sens);
}

/*
 *     Add the response pointer mechanism to the channel chain  
 */

RESPONSE *AddResponse(inchannel)                                
CHANNEL_ENTRY *inchannel;                                
{                                                          
       RESPONSE *rp;                                       
                                                           
       rp = (RESPONSE *) malloc(sizeof (RESPONSE));        
       rp->ptr.SENS = NULL;                                
       rp->type = ' ';                                     
       rp->Next = NULL;                                    
                                                           
       if (inchannel->Tail_Response==NULL)                 
               inchannel->Tail_Response = inchannel->Root_Response = rp;
       else    {                                           
               inchannel->Tail_Response->Next = rp;
               inchannel->Tail_Response = rp;
       }                                    
      
       return(rp);                        
}                                        
                                        
/*                                     
 *     Link a zeros and poles response to end of the chain  
 */
  
BOOL LinkInZerosPoles(inchannel,inzero)            
CHANNEL_ENTRY *inchannel;
ZEROS_POLES *inzero;            
{                                                          
       RESPONSE *rp;                                       
                                                           
       rp = AddResponse(inchannel);                        
       if (rp==NULL) return(FALSE);                        
       rp->ptr.PZ = inzero;                                
       rp->type = RESP_ZEROS_POLES;                        
                                                           
       return(TRUE);                                       
}                                                          
 
/*
 *     Link a coefficients response to end of the chain
 */                                                  
                                                    
BOOL LinkInCoefficients(inchannel,incoef)         
CHANNEL_ENTRY *inchannel;
COEFFICIENTS *incoef;         
{                                                                    
       RESPONSE *rp;                              
    
       rp = AddResponse(inchannel);
       if (rp==NULL) return(FALSE);
       rp->ptr.CO = incoef;      
       rp->type = RESP_COEFFICIENTS;
                                   
       return(TRUE);              
}                                
                                
/*                             
 *     Add a decimation response to the end of the chain 
 */
 
BOOL LinkInDecimation(inchannel,indeci)             
CHANNEL_ENTRY *inchannel;
DECIMATION *indeci;             
{                                                          
       RESPONSE *rp;
   
       rp = AddResponse(inchannel);
       if (rp==NULL) return(FALSE);
       rp->ptr.DM = indeci;      
       rp->type = RESP_DECIMATION; 
                                  
       return(TRUE);             
}   
   
/*
 *     Add a sensitivity response to the end of the chain
 */
  
BOOL LinkInSensitivity(inchannel,insens)           
CHANNEL_ENTRY *inchannel;
SENSITIVITY *insens;           
{
       RESPONSE *rp;
    
       rp = AddResponse(inchannel);
       if (rp==NULL) return(FALSE);
       rp->ptr.SENS = insens;
       rp->type = RESP_SENSITIVITY;
                                  
       return(TRUE);
}

/*********************Handle Station/Channel Comments**********************/   
/*
 *     Alloc memory for comment and zero out 
 */

COMMENT_ENTRY *InitCom()
{                                                          
       COMMENT_ENTRY *newcom;                              
                                                           
       newcom = (COMMENT_ENTRY *) malloc(sizeof (COMMENT_ENTRY));
       newcom->Start_Comment = _ztime;                    
       newcom->End_Comment = _ztime;
       newcom->Comment = NULL;     
       newcom->Comment_Level = 0; 
       newcom->Next = NULL;      
                                
       return(newcom);         
}                             
                             
/*                          
 *     Link a comment onto a station record  
 */
  
BOOL LinkInStatCom(instat,incom)                 
STATION_ENTRY *instat;
COMMENT_ENTRY *incom;                 
{  
       if (instat->Tail_Comment==NULL) 
               instat->Tail_Comment = instat->Root_Comment = incom;
       else    {
               instat->Tail_Comment->Next = incom; 
               instat->Tail_Comment = incom;
       }                                   
       return(TRUE);                     
}                                       
                                       
/*                                    
 *     Link comment onto a channel record
 */            

BOOL LinkInChanCom(inchan,incom)                 
CHANNEL_ENTRY *inchan;
COMMENT_ENTRY *incom;                 
{
       if (inchan->Tail_Comment==NULL)
               inchan->Tail_Comment = inchan->Root_Comment = incom; 
       else    {
               inchan->Tail_Comment->Next = incom;
               inchan->Tail_Comment = incom;     
       }                                        
       return(TRUE);                          
}                                            
                                            
/*                                         
 *     Init, fill, and link comment onto station record 
 */
 
BOOL AddStatCom(inroot, startime, endtime, comnum,level) 
STATION_ENTRY *inroot;
STDTIME startime, endtime;
UWORD comnum;
ULONG level;
{          
       COMMENT_ENTRY *newcom;
  
       newcom = InitCom();
       if (newcom==NULL) return(FALSE);
       newcom->Start_Comment = startime;
       newcom->End_Comment = endtime;
       newcom->Comment_Level = level;
       newcom->Comment = SearchComment(comnum); 
      
       if (newcom->Comment==NULL) {            
               printf("Couldn't find comment number %d in database\n",
                       comnum);               
               return(FALSE);                
       }                                    
       CountComment(newcom->Comment);      
       return(LinkInStatCom(inroot,newcom));      
}                                                   
                                                   
/*                                                   
 *     Init, fill, and link comment onto channel record
 */                                                    
                                                        
BOOL AddChanCom(inroot, startime, endtime, comnum, level)
CHANNEL_ENTRY *inroot;
STDTIME startime;
STDTIME endtime;
UWORD comnum;
ULONG level;
{
       COMMENT_ENTRY *newcom;
                         
       newcom = InitCom();
       if (newcom==NULL) return(FALSE);
                                       
       newcom->Start_Comment = startime;
       newcom->End_Comment = endtime;
       newcom->Comment_Level = level;
       newcom->Comment = SearchComment(comnum);
       if (newcom->Comment==NULL) {
               printf("Couldn't find comment number %d in database\n",
                       comnum);
               return(FALSE);
       }
       CountComment(newcom->Comment);
     
       return(LinkInChanCom(inroot,newcom));
}

EVENT_ENTRY *InitHypocenterInfo ( )
{
	EVENT_ENTRY	*evinfo;
	int i;

	evinfo = (EVENT_ENTRY *) malloc ( sizeof(EVENT_ENTRY) );

	evinfo->Origin_Time.year   = 0;
	evinfo->Origin_Time.day    = 0;
	evinfo->Origin_Time.hour   = 0;
	evinfo->Origin_Time.minute = 0;
	evinfo->Origin_Time.second = 0;
	evinfo->Origin_Time.msec   = 0;

        evinfo->Info_Source = (CITED_SOURCE *) InitCitedSource();
	evinfo->Lattitude = 0.0;
	evinfo->Longitude = 0.0;
	evinfo->Depth = 0.0;
	evinfo->Number_Magnitudes = 0;
	for(i=0;i<MAXMAGN;i++) {
		evinfo->Magnitude[i] = 0.0;
		evinfo->Magnitude_Type[i] = (UBYTE *) malloc(VAR11);
		evinfo->Magnitude_Source[i] = 0;
	}
	evinfo->Next = NULL;

	return (evinfo);
}


#define LOOP_OVER_TIME_ENTRIES     for(time_entry=VI->Root_Time_Entry;\
                                       time_entry!=NULL;time_entry=time_entry->Next)
#define LOOP_OVER_EVENTS     for(looper=time_entry->Root_Event;\
                                       looper!=NULL;looper=looper->Next)
LinkInEvent ( event )
EVENT_ENTRY *event;
{
        EVENT_ENTRY *looper;
        TIME_ENTRY  *time_entry;
	BOOL	    NEW_EVENT;

	NEW_EVENT = TRUE;



      LOOP_OVER_TIME_ENTRIES {
/*
printf("event day: %d  time_entry_day: %d\n",event->Origin_Time.day, time_entry->Time_Span_ID->Start_Time.day );
*/
        if ( event->Origin_Time.day != 
                    time_entry->Time_Span_ID->Start_Time.day ) continue;
/*
        if ( event->Origin_Time.day ==  1 )
               continue;
*/

	if( time_entry->Tail_Event == NULL )  {
/*
printf("event linked in on tail\n");
*/
		time_entry->Root_Event = time_entry->Tail_Event = event;
	}
	else {
	   LOOP_OVER_EVENTS {
	      if ((event->Origin_Time.hour == looper->Origin_Time.hour) &&
	          (event->Origin_Time.minute == looper->Origin_Time.minute) && 
		  (event->Origin_Time.second == looper->Origin_Time.second) ) {
                       NEW_EVENT = FALSE;
/*
			printf("event found !\n");
*/
                       break;
	      }
          }
          if ( NEW_EVENT ) {
		time_entry->Tail_Event->Next = event;
		time_entry->Tail_Event = event;
	  }
      }
  }
}

TIME_SPAN *InitTimeSpan ( )
{
    	TIME_SPAN *time_span;
 	time_span = (TIME_SPAN *) malloc ( sizeof(TIME_SPAN) );

	time_span->Flag = ' ';
	time_span->Start_Time.year = time_span->End_Time.year = 0;
	time_span->Start_Time.day = time_span->End_Time.day = 0;
	time_span->Start_Time.hour = time_span->End_Time.hour = 0;
	time_span->Start_Time.minute = time_span->End_Time.minute = 0;
	time_span->Start_Time.second = time_span->End_Time.second = 0;
	time_span->Start_Time.msec = time_span->End_Time.msec = 0;

	return(time_span);
}

TIME_ENTRY *InitTimeEntry()
{
	TIME_ENTRY *time_entry;

	time_entry = (TIME_ENTRY *) malloc(sizeof(TIME_ENTRY));

	time_entry->Time_Span_ID = NULL;
	time_entry->Root_Event = NULL;
	time_entry->Tail_Event = NULL;
	time_entry->Root_Block74 = NULL;
	time_entry->Tail_Block74 = NULL;
	time_entry->Next = NULL;

 	return(time_entry);
}

VOID UpdateTimeSpanID(time_entry,time_span)
TIME_ENTRY *time_entry;
TIME_SPAN *time_span;
{
     	LONG t1, t2, v1, v2;


        t1 = Time2InSeconds ( time_entry->Time_Span_ID->Start_Time );
        t2 = Time2InSeconds ( time_entry->Time_Span_ID->End_Time );

        v1 = Time2InSeconds ( time_span->Start_Time );
        v2 = Time2InSeconds ( time_span->End_Time );

	if ( v1 < t1 ) {
		time_entry->Time_Span_ID->Start_Time.year = 
                             time_span->Start_Time.year;
		time_entry->Time_Span_ID->Start_Time.day = 
                             time_span->Start_Time.day;
		time_entry->Time_Span_ID->Start_Time.hour = 
                             time_span->Start_Time.hour;
		time_entry->Time_Span_ID->Start_Time.minute = 
                             time_span->Start_Time.minute;
		time_entry->Time_Span_ID->Start_Time.second = 
                             time_span->Start_Time.second;
		time_entry->Time_Span_ID->Start_Time.msec = 
                             time_span->Start_Time.msec;
	}
	if ( v2 > t2 ) {
		time_entry->Time_Span_ID->End_Time.year = 
                             time_span->End_Time.year;
		time_entry->Time_Span_ID->End_Time.day = 
                             time_span->End_Time.day;
		time_entry->Time_Span_ID->End_Time.hour = 
                             time_span->End_Time.hour;
		time_entry->Time_Span_ID->End_Time.minute = 
                             time_span->End_Time.minute;
		time_entry->Time_Span_ID->End_Time.second = 
                             time_span->End_Time.second;
		time_entry->Time_Span_ID->End_Time.msec = 
                             time_span->End_Time.msec;
	}
}

LONG Time2InSeconds ( time )
STDTIME time;
{
 	LONG sec;

	sec  = (time.day * 86400);
        sec += (time.hour * 3600);
        sec += (time.minute * 60);
        sec += (time.second);
        sec += (time.msec/1000);	

	return(sec);
}

#undef   LOOP_OVER_TIME_ENTRIES
#define  LOOP_OVER_TIME_ENTRIES for(looper=VI->Root_Time_Entry;\
                                       looper!=NULL;looper=looper->Next)

LinkInTimeEntry ( time_span )
TIME_SPAN  *time_span;
{
	TIME_ENTRY  	*looper, *time_entry;
        BOOL		NEW_DAY;

	NEW_DAY = TRUE;


	if( VI->Tail_Time_Entry == NULL )  {
                time_entry = (TIME_ENTRY *) InitTimeEntry();
                time_entry->Time_Span_ID = time_span;
		VI->Root_Time_Entry = VI->Tail_Time_Entry = time_entry;
	}
	else {
	    LOOP_OVER_TIME_ENTRIES {
               if ( time_span->Start_Time.day == 
                              looper->Time_Span_ID->Start_Time.day ) {
                    UpdateTimeSpanID(looper,time_span);
                    NEW_DAY = FALSE;
                    break;
               }
            }
            if ( NEW_DAY ) {
                time_entry = (TIME_ENTRY *) InitTimeEntry();
                time_entry->Time_Span_ID = time_span;
		VI->Tail_Time_Entry->Next = time_entry;
		VI->Tail_Time_Entry = time_entry;
	    }
	}

}
	
#undef   LOOP_OVER_TIME_ENTRIES
#define  LOOP_OVER_TIME_ENTRIES     for(time_entry=VI->Root_Time_Entry;\
                                       time_entry!=NULL;time_entry=time_entry->Next)
LinkInBlock74 ( bl74 )
BLOCK74 *bl74;
{
      TIME_ENTRY *time_entry;

/* this part was included to read the Geoscope seed volume */

TIME_SPAN *ts;

/*
if(EXTRACT) {
*/

ts = (TIME_SPAN *) InitTimeSpan();

ts->Start_Time.year = bl74->Start_Time.year;
ts->Start_Time.day = bl74->Start_Time.day;
ts->Start_Time.hour = bl74->Start_Time.hour;
ts->Start_Time.minute = bl74->Start_Time.minute;
ts->Start_Time.second = bl74->Start_Time.second;
ts->Start_Time.msec = bl74->Start_Time.msec;

ts->End_Time.year = bl74->End_Time.year;
ts->End_Time.day = bl74->End_Time.day;
ts->End_Time.hour = bl74->End_Time.hour;
ts->End_Time.minute = bl74->End_Time.minute;
ts->End_Time.second = bl74->End_Time.second;
ts->End_Time.msec = bl74->End_Time.msec;
ts->Flag = 'E';
LinkInTimeEntry(ts);
/*
}
*/

/*  end of included part  */


      LOOP_OVER_TIME_ENTRIES {
            if ( bl74->Start_Time.day != 
                    time_entry->Time_Span_ID->Start_Time.day ) continue;

	    if( time_entry->Tail_Block74 == NULL ) { 
		time_entry->Root_Block74 = time_entry->Tail_Block74 = bl74;
            }
	    else {
		time_entry->Tail_Block74->Next = bl74;
		time_entry->Tail_Block74 = bl74;
            }
      }
}



BLOCK73 *InitBlock73()
{
	BLOCK73 *block73;
	UBYTE   *st, *loc, *ch;

	block73 = (BLOCK73 *) malloc(sizeof(BLOCK73));

	block73->Station_ID = (UBYTE *) malloc(6);
	block73->Location_ID = (UBYTE *) malloc(3);
	block73->Channel_ID = (UBYTE *) malloc(4);

        block73->number_data_pieces = 0;
	strcpy(block73->Station_ID,"");
	strcpy(block73->Location_ID,"  ");
	strcpy(block73->Channel_ID,"");
	block73->Start_Time = _ztime;
	block73->Start_Sequence_No = 0;
	block73->Start_Sub_Sequence_No = 0;
}


BLOCK74 *InitBlock74()
{
	BLOCK74 *block74;
	UBYTE   *st, *loc, *ch;

	block74 = (BLOCK74 *) malloc(sizeof(BLOCK74));

	block74->Station_ID = (UBYTE *) malloc(6);
	block74->Location_ID = (UBYTE *) malloc(3);
	block74->Channel_ID = (UBYTE *) malloc(4);
	strcpy(block74->Station_ID,"");
	strcpy(block74->Location_ID,"  ");
	strcpy(block74->Channel_ID,"");
	
	block74->Start_Time = _ztime;
	block74->Start_Sequence_No = 0;
	block74->Start_Sub_Sequence_No = 0;
	block74->End_Time = _ztime;
	block74->End_Sequence_No = 0;
	block74->End_Sub_Sequence_No = 0;
	block74->Accelerators = 0;
        strcpy(block74->File_Name, "");
	block74->Next = NULL;

 	return(block74);
}
