/******************************************************************************/
/****                                                                       ***/
/****    Original version modified and extended by Reinoud Sleeman.         ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/

#include "conver.h"
#include "seed.h"

#define FOREVER for(;;)
#define LOCAL static

extern STATION_ENTRY    *Current_Station;
extern CHANNEL_ENTRY   	*Current_Channel;
extern FILE 		*uf;
extern UBYTE		Input_File[];
extern STDTIME 		parsetimeX();
extern INT              LRECL_IN;
extern INT              INP_FILE_NO;
extern BOOL             MERGE_FILES, GEOSCOPE;
  
STATION_ENTRY *Top_Station=NULL;
CHANNEL_ENTRY *Top_Channel=NULL;

static STDTIME _ztime = {0,0,0,0,0,0};

#define NOISE_REC ' '  /* Blank noise records */
#define VOL_REC 'V'    /* Volume header record */
#define ABR_REC 'A'    /* Abbreviation dictionary record */
#define STA_REC 'S'    /* Station header record */
#define TIM_REC 'T'    /* Time span index */
#define DATA_REC 'D'   /* Data Record */

struct _ptable {
       int     blocktype;
       int     tapever; 
       char    vtype;  
} ptable[] = {                 
{ 5,   20,     VOL_REC },
{ 10,  20,     VOL_REC },
{ 11,  20,     VOL_REC },
{ 12,  20,     VOL_REC },
{ 30,  20,     ABR_REC },
{ 31,  20,     ABR_REC },
{ 32,  20,     ABR_REC },
{ 33,  20,     ABR_REC },
{ 34,  20,     ABR_REC },
{ 35,  20,     ABR_REC },
{ 50,  20,     STA_REC },
{ 51,  20,     STA_REC },
{ 52,  20,     STA_REC },
{ 53,  20,     STA_REC },
{ 54,  20,     STA_REC },
{ 55,  20,     STA_REC },
{ 56,  20,     STA_REC },
{ 58,  20,     STA_REC },
{ 59,  20,     STA_REC },
{ 70,  20,     TIM_REC },
{ 71,  20,     TIM_REC },
{ 72,  20,     TIM_REC },
{ 73,  20,     TIM_REC },
{ 74,  20,     TIM_REC },
{ 0, 0, 0 }};                                      

VOID srerr(name,level)
UBYTE *name;
int level;
{
       printf("Backtrace [%d] - %s\n",level,name);
}

#define DKP(e,a) if (!(a)) return(srerr(SN,e),FALSE)
                       /* Procedure to unwind on error */

LOCAL ULONG BlockExp=0,                /* Block buffer exponent */  
              
           BlockMax=0;         /* Block buffer size */             
             
LOCAL UBYTE *BlockBuffer=NULL; /* Pointer to buffer */            
LOCAL ULONG BlockCurSize=0;    /* Current Block Size */          
   
LOCAL ULONG BlockCurPos=0;     /* Current Decode Position */    

BOOL	TIME_HEADER_DONE = FALSE;
LONG    DREC;
int     SET_BL74;
    

BOOL lees_rec(buf)
UBYTE *buf;
{
	if ( (fread(buf,sizeof(char),LRECL_IN,uf)) != LRECL_IN )
		return(FALSE);
}
/*                                                             
 *     Initialize a structure for a logical record           
 */

LOGICAL_RECORD *RecordReadBuff( creexp, maxblocke, readrtn) 
UWORD 	creexp;
UWORD 	maxblocke;
BOOL 	(*readrtn)() ;
{
       LOGICAL_RECORD *New_Record;
      
       New_Record = (LOGICAL_RECORD *) malloc(sizeof(LOGICAL_RECORD)); 
       if (New_Record==NULL) return(NULL);                            
                                                                     
       if (maxblocke<12) maxblocke = 12;       /* Minimum buffer size */
       if (maxblocke>15) maxblocke = 15;       /* Maximum buffer size */
       New_Record->BlockExp = maxblocke;                               
       New_Record->BlockMax = 1<<maxblocke;                           
       New_Record->BlockBuffer = (UBYTE *) malloc(New_Record->BlockMax);
      
       if (New_Record->BlockBuffer==NULL) {                        
               free(New_Record);       /* Get out of town */      
               return(NULL);                                     
       }                                                        
                                                               
       if (creexp>15) creexp = 15;     /* Maximum logical record size */
       if (creexp<8) creexp = 8;       /* Minimum logical record size */
       New_Record->BlockSize=0;        /* Current Block Size */
       New_Record->BlockPos=0;         /* Current Decode Position */
       New_Record->RecBuff = NULL;     /* Not assigned storage yet */ 
       New_Record->RecNum = 0;       
       New_Record->NextPos = 0;       
       New_Record->RecLen = creexp;    
       New_Record->RecSize = 1<<creexp; 
       New_Record->ReadService = readrtn;
       New_Record->Continue = FALSE;
       New_Record->RecType = ' ';     
       return(New_Record);           
}

/*
 *     Get a new record from the input stream   
 */

BOOL GetARecord(inrec)
LOGICAL_RECORD *inrec;
{
       UBYTE numbuf[7];
       int i;

       FOREVER {
               inrec->BlockPos += inrec->RecSize;
               if (inrec->BlockPos>=inrec->BlockSize) {
		       lees_rec(inrec->BlockBuffer);
                       inrec->BlockPos = 0;                            
               }                                                      
                                                                     
               inrec->RecBuff = &inrec->BlockBuffer[inrec->BlockPos]; 
	       if(inrec->RecBuff[6] == 'D' ) {
    		      TIME_HEADER_DONE = TRUE;
                      for (i=0; i<6; i++) numbuf[i] = inrec->RecBuff[i];
                      numbuf[7] = '\0';                                
                      DREC  = atol(numbuf);                  
    		      return(TRUE);
	       }
               inrec->NextPos = 0;                                   
               if (inrec->RecBuff[6]==' ') continue;   /* Noise record */ 

               inrec->Continue = FALSE; 
               if (inrec->RecBuff[7]=='*') inrec->Continue = TRUE;
                                                                 
               for (i=0; i<6; i++) numbuf[i] = inrec->RecBuff[i];
               numbuf[7] = '\0';                                
                                                               
               inrec->RecNum = atol(numbuf);                  
               inrec->RecType = inrec->RecBuff[6];          
                                                           
               for (i=0; i<3; i++) numbuf[i] = inrec->RecBuff[i+8];  
               numbuf[3] = '\0';
             
               if ((atol(numbuf)==0)&&(!inrec->Continue)) continue; 
             
                       /* No blockettes here - different noise record */
                    
               return(TRUE);   /* Got it */  
       }                                    
}

#undef SN
#define SN "GetBytes"
BOOL GetBytes ( intype,  numbytes, parbuf, parpos, lastrec,  record)
UBYTE 		intype, parbuf[];
ULONG		numbytes;
INT  	 	*parpos, *lastrec;
LOGICAL_RECORD 	*record;
{
       int i;
       char numbuf[8];

       for (i=0; i<numbytes; i++) {
               if (*parpos>=record->RecSize) {
             
                       DKP(1,GetARecord(record));
	if(record->RecBuff[6] == 'D' ) {
    		TIME_HEADER_DONE = TRUE;
                for (i=0; i<6; i++) numbuf[i] = record->RecBuff[i];
                numbuf[7] = '\0';                                
                DREC  = atol(numbuf);                  
    		return(TRUE);
	}
                       if (record->RecType!=intype) {
                    
       printf("Blockette Continued from %d to %ld with wrong record type\n",    
                               *lastrec,record->RecNum);
                               return(FALSE);
                       }
                    
                       if (!record->Continue)
                    
       printf("Blockette continued from %d to %d without continue flag\n",     
      
                               *lastrec,record->RecNum); 
                       *lastrec = record->RecNum;
                       *parpos = 8;
               }
               parbuf[i] = record->RecBuff[*parpos];
               (*parpos)++;
       }
      
       parbuf[numbytes] = '\0';
       return(TRUE);
      
}
#undef SN
#define SN "ParseABlockette"
BOOL ParseABlockette(InBlock)
BLOCKETTE *InBlock;
{
       int i;
       BOOL found=FALSE;
       COMMENT *looper;


       for (i=0; ptable[i].blocktype!=0; i++) {
               if (InBlock->BlkType==ptable[i].blocktype) {
                       found=TRUE;
                       break;
               }
       }
       if (!found) {
               printf("Unknown blockette type %d seed - skipping\n",
                                                      InBlock->BlkType);
               return(TRUE);   /* Not an error to see wierd blockettes */      
       }
       if (InBlock->BlkRec!=ptable[i].vtype)   
      
               printf("Blockette %d appeared with a %c header type\n",
                       InBlock->BlkType,InBlock->BlkRec);            
                    
       InBlock->ParsePos=0;
  /* printf("blockette: %d\n",ptable[i].blocktype);   */
       if      ( ptable[i].blocktype ==  5 ) ParseStationHeader(InBlock);
       else if ( ptable[i].blocktype == 10 ) ParseNetworkHeader(InBlock);
       else if ( ptable[i].blocktype == 11 ) ParseStationIndex(InBlock);
       else if ( ptable[i].blocktype == 12 ) ParseSpanIndex(InBlock);
       else if ( ptable[i].blocktype == 30 ) ParseFormatDictionary(InBlock);
       else if ( ptable[i].blocktype == 31 ) ParseCommentDictionary(InBlock);
       else if ( ptable[i].blocktype == 32 ) ParseSourceDictionary(InBlock);
       else if ( ptable[i].blocktype == 33 ) ParseAbbrevDictionary(InBlock);
       else if ( ptable[i].blocktype == 34 ) ParseUnitsDictionary(InBlock);
       else if ( ptable[i].blocktype == 35 ) ParseBeamDictionary(InBlock);
       else if ( ptable[i].blocktype == 50 ) ParseStationID(InBlock);
       else if ( ptable[i].blocktype == 51 ) ParseStationComment(InBlock);
       else if ( ptable[i].blocktype == 52 ) ParseChannelID(InBlock);
       else if ( ptable[i].blocktype == 53 ) ParseZerosPoles(InBlock);
       else if ( ptable[i].blocktype == 54 ) ParseCoefficients(InBlock);
       else if ( ptable[i].blocktype == 55 ) ParseResponseList(InBlock);
       else if ( ptable[i].blocktype == 56 ) ParseGenericResponse(InBlock);
       else if ( ptable[i].blocktype == 58 ) ParseSensitivity(InBlock);
       else if ( ptable[i].blocktype == 59 ) ParseChannelComment(InBlock);
       else if ( ptable[i].blocktype == 70 ) ParseTimeSpanID(InBlock);
       else if ( ptable[i].blocktype == 71 ) ParseHypoCenterInfo(InBlock);
       else if ( ptable[i].blocktype == 72 ) ParseEventPhases(InBlock);
       else if ( ptable[i].blocktype == 73 ) ParseTimeSpanDataIndex(InBlock);
       else if ( ptable[i].blocktype == 74 ) ParseTimeSeriesIndex(InBlock);

       return(TRUE);                                              
}                                                                

#undef SN
#define SN "ParseBlockettes"
BOOL ParseBlockettes ( intype, record, blockette) 
UBYTE intype;
LOGICAL_RECORD *record;
BLOCKETTE *blockette; 
{
       int firstrec,lastrec, i,parpos;
       UBYTE numbuf[10];

       parpos = 0;
       if (record->RecBuff==NULL) DKP(1,GetARecord(record)); 
	if(record->RecBuff[6] == 'D' ) {
    		TIME_HEADER_DONE = TRUE;
                for (i=0; i<6; i++) numbuf[i] = record->RecBuff[i];
                numbuf[7] = '\0';                                
                DREC  = atol(numbuf);                  
    		return(TRUE);
	}
       FOREVER {
               firstrec = lastrec = record->RecNum;
               if (record->RecType!=intype) return(TRUE);      /* Done with */ 
               if (parpos==0)
                       parpos = 8;     /* Start of unblocking */
                        
               DKP(2,GetBytes(intype,3,numbuf,&parpos,&lastrec,record));
                                 blockette->BlkType = atol(numbuf); 
             
               if (blockette->BlkType==0) {    /* No, get next record */
                
                   if(blockette->BlkType==0 && TIME_HEADER_DONE ) {
                      return(TRUE);
                   }
                       DKP(3,GetARecord(record));
                       if(record->RecType == 'D' ) {
                              TIME_HEADER_DONE = TRUE;
                              for (i=0; i<6; i++) numbuf[i] = record->RecBuff[i];
                              numbuf[7] = '\0'; 
                              DREC  = atol(numbuf);                  
                              return(TRUE);
                       }
                       if (record->Continue)
                    
       printf("Blockette on %ld started but had continue flag set\n",
                               record->RecNum);
                           
                       parpos = 0;
                       continue;       /* Go around again */
               }

               DKP(4,GetBytes(intype,4,numbuf,&parpos,&lastrec,record));
                              blockette->BlkRec = intype;
             
               blockette->PackPos = atoi(numbuf) - 7;  /* Minus head */
              
               if (blockette->BufLen<blockette->PackPos)               
             
                       DKP(5,ExtendBlockette(blockette,               
                               ((blockette->PackPos-blockette->BufLen)/
                                       DefBlkExt)+1));                
                                  
               DKP(6,GetBytes(intype,blockette->PackPos,blockette->BlkBuff,    
                       &parpos,&lastrec,record));      /* Slurp in */
                         
               ParseABlockette(blockette);                          
             
               /* Cycle down for next */                          
       }
}      
                       
/*                    
 *     Unpack a fixed length string onto record 
 */                                           
                                             
#undef SN
#define SN "UnpackString"
BOOL UnpackString(InBlock,Buffer,MaxStr)
BLOCKETTE *InBlock;
UBYTE **Buffer;
ULONG MaxStr;
{
       UBYTE   *stobuff;
       int i;
      
       if (Buffer==NULL) return(FALSE);        /* What??? */ 
       stobuff = (UBYTE *) malloc(MaxStr+1);               
       *Buffer = stobuff;      /* Fix user's pointer */   
                                                         
       for (i=0; i<MaxStr; i++)                         
               *stobuff++ = InBlock->BlkBuff[(InBlock->ParsePos)+i];
                                                                 
       InBlock->ParsePos+=MaxStr;                               
       *stobuff = '\0';                                        
       stobuff = *Buffer;  
      
/*     for (i=MaxStr-1; i>=0; i--) if (stobuff[i]==' ') stobuff[i] = '\0';*/   
    
       if (stobuff[0]=='\0') { 
      
               free(stobuff); 
               *Buffer = NULL;         /* Nothing left */  
                    
       }
       return(TRUE);
}

#undef SN
#define SN "UnpackChar"
BOOL UnpackChar(InBlock,ReadChar)
BLOCKETTE *InBlock;
UBYTE *ReadChar;
{      
       *ReadChar = InBlock->BlkBuff[InBlock->ParsePos];
       InBlock->ParsePos++;    /* Skip over the character */
       return(TRUE);                                      
}                                                        
                                                        
/*                                                     
 *     Unpack a variable length string  
 *     MaxStr = 0, no maximum length   
 */
  
#undef SN
#define SN "UnpackVariable"
BOOL UnpackVariable(InBlock,Buffer, maxfld, upcase)
BLOCKETTE *InBlock;
UBYTE **Buffer;
UWORD maxfld;
BOOL upcase;
{
       UBYTE   *stobuff; 
       int     ict;     
       int     maxp;

       maxp = (int) maxfld;
       if ( maxp==0 ) 
                   maxp=1000;    /* assume varstring terminated by ~ */
         
       if (Buffer==NULL) return(FALSE);        /* What??? */   
                                                              
/*
       for (ict=0; InBlock->BlkBuff[(InBlock->ParsePos)+ict]!='~'; ict++);
*/
       for (ict=0; (InBlock->BlkBuff[(InBlock->ParsePos)+ict]!='~')&&
                   (ict<maxp) ; ict++);


       stobuff = (UBYTE *) malloc(ict+1);                             
       *Buffer = stobuff;      /* Fix user's pointer */              
                                                                    
/*
       for (;  InBlock->BlkBuff[InBlock->ParsePos]!='~';           
               InBlock->ParsePos++)                               
                       *stobuff++ = InBlock->BlkBuff[InBlock->ParsePos];
*/
       for (ict=0;  (InBlock->BlkBuff[InBlock->ParsePos]!='~')&&
                    (ict<maxp); ict++, InBlock->ParsePos++)                               
                       *stobuff++ = InBlock->BlkBuff[InBlock->ParsePos];
                                                                       
       InBlock->ParsePos++;    /* Skip over the tilde */              
       *stobuff = '\0'; 
                       
       stobuff = *Buffer;       
       if (stobuff[0]=='\0') {        
               free(stobuff);        
               *Buffer = NULL;      
       }                           
       return(TRUE);             
}                              
                              
/*                           
 *     Unpack signed or unsigned integer  
 */                                     
                                       
#undef SN
#define SN "UnpackInteger"
BOOL UnpackInteger(InBlock,InNum, FldLen, Signed)     
BLOCKETTE *InBlock;
LONG *InNum, FldLen;
BOOL Signed;
{
       UBYTE *stobuff=NULL;
       int ic;
       LONG answer;
      
       DKP(1,UnpackString(InBlock,&stobuff,FldLen)); 
      
       ic = sscanf(stobuff,"%ld",&answer);         
       free(stobuff);                            
       if (ic!=1) return(FALSE);       /* Parsing error with scanf */
       *InNum = answer;                                            
      
       return(TRUE);                                             
}                                                               
                                                               
/*                                                            
 *     Unpack signed or unsigned fixed floating point       
 */    
      
#undef SN
#define SN "UnpackFixed"
BOOL UnpackFixed ( InBlock, InFlt, FldLen, Decpls, Signed)
BLOCKETTE *InBlock;
VOLFLT *InFlt;
LONG FldLen, Decpls;
BOOL Signed;
{
       UBYTE *stobuff=NULL;
       int ic;
      
       VOLFLT answer;
      
       DKP(1,UnpackString(InBlock,&stobuff,FldLen));
       ic = sscanf(stobuff,"%lf",&answer);
       if (ic!=1) {
               printf("Error: UnpackFixed(%s,%d)\n",stobuff,ic);
               free(stobuff); 
               return(FALSE);  /* Parsing error with scanf */ 
       }                     
      
       free(stobuff);      
       *InFlt = answer;   
                         
       return(TRUE);    
}                      
                      
/*                   
 *     Unpack an exponential form  
 */                              
                                
#undef SN
#define SN "UnpackExponent"
BOOL UnpackExponent ( InBlock, InFlt, FldLen)
BLOCKETTE *InBlock;
VOLFLT *InFlt;
LONG FldLen;
{
       UBYTE *stobuff=NULL;
       int ic;
       VOLFLT answer;
      
       DKP(1,UnpackString(InBlock,&stobuff,FldLen)); 
       ic = sscanf(stobuff,"%lf",&answer);
       free(stobuff);
       if (ic!=1) return(FALSE);       /* Parsing error with scanf */
            
       *InFlt = answer;   
      
       return(TRUE);     
      
}      
/*
 *     Unpack the standard time      
 */
                                    
#undef SN                          
#define SN "UnpackTime"           
BOOL UnpackTime(InBlock,InTime)
BLOCKETTE *InBlock;
STDTIME *InTime;
{                                
       UBYTE *instring;        
       STDTIME gettime;       
                             
       DKP(1,UnpackVariable(InBlock,&instring,0,FALSE));   
                                                          
       if (instring==NULL) {                             
               *InTime = _ztime;      
               return(TRUE);         
       }                            
                                   
       if (instring[0] == '\0') { 
               gettime = _ztime; 
       } 
       else {                 
		gettime = parsetimeX (instring );
               if (gettime.year==0) {               
                       printf("Error: ParseTime(%s)\n",instring); 
                       return(FALSE);  /* Parse error */         
               }                                                
       }                                                       
      
       free(instring);
       *InTime = gettime;
                        
       return(TRUE);   
}                     
                     
#undef SN
#define SN "UnpackAbbrev"
BOOL UnpackAbbrev(InBlock,abrv)
BLOCKETTE *InBlock;
ABBREV **abrv;
{
       LONG idx;
       char nbuf[6];

       DKP(1,UnpackInteger(InBlock,&idx,3,FALSE)); 
       if (idx==0) {
               *abrv = NULL; 
               return(TRUE);
       }                   
                          
       *abrv = (ABBREV *)SearchAbbrevIdx(idx);     
       if (*abrv==NULL)                 
               printf("Could not find Abbrev index %d\n",idx); 
                                                              
       return(TRUE);                                         
}                                                           
                                                           
#undef SN
#define SN "UnpackComment"     
BOOL UnpackComment(InBlock,comm) 
BLOCKETTE *InBlock;
COMMENT_ENTRY **comm;
{                             
       LONG idx;             
       UWORD idw;
      
       DKP(1,UnpackInteger(InBlock,&idx,4,FALSE)); 
       if (idx==0) {                              
               *comm = NULL;                     
               return(TRUE);                    
       }                                       
                                              
       idw = (UWORD) idx;
       *comm = (COMMENT_ENTRY *) SearchComment(idw); 
       if (*comm==NULL)          
               printf("Could not find Comment index %ld\n",idx);
           
       return(TRUE);
}                  
                  
#undef SN 
#define SN "UnpackFormat"
BOOL UnpackFormat(InBlock,form) 
BLOCKETTE *InBlock;
FORMAT **form; 
{                       
       LONG idx;      
       char nbuf[6]; 
     
       DKP(1,UnpackInteger(InBlock,&idx,4,FALSE));
       if (idx==0) {                             
               *form = NULL;                    
               return(TRUE);                   
       }                                      
      
       if ( MERGE_FILES ) {
              *form = (FORMAT * ) SearchLocalFormatIdx(INP_FILE_NO,(UWORD)idx);
              return(TRUE);
       }

       *form = (FORMAT *) SearchFormatIdx(idx);        
       if (*form==NULL)                    
               printf("Could not find Format index %d\n",idx);
                                                             
       return(TRUE);                                        
}                                                          
                                                          
#undef SN                                                
#define SN "UnpackUnit"                                 
BOOL UnpackUnit(InBlock,unit)
BLOCKETTE *InBlock;
UNIT **unit;
{                                                      
       LONG idx;                                     
       char nbuf[6];                                
       UNIT *looper;
     
       DKP(1,UnpackInteger(InBlock,&idx,3,FALSE));   
       if (idx==0) {                                
               *unit = NULL;                       
               return(TRUE);                      
       }                                         

       if ( MERGE_FILES ) {
              *unit = (UNIT * ) SearchLocalUnitIdx(INP_FILE_NO,idx);
              return(TRUE);
       }
                                                
       *unit = (UNIT *) SearchUnitIdx(idx);
       if (*unit==NULL)          
               printf("Could not find Unit index %d\n",idx);  
                                
       return(TRUE);           
}
                              
#undef SN                    
#define SN "ParseEventPhases"
BOOL ParseEventPhases(InBlock)
BLOCKETTE *InBlock;
{                          
       printf("Events not interesting now\n");
       return(TRUE);                         
}                                          
                                          
#undef SN                                
#define SN "ParseHypoCenterInfo"        
BOOL ParseHypoCenterInfo(InBlock)
BLOCKETTE *InBlock;
{
       EVENT_ENTRY *event;
       LONG ldum;
       VOLFLT fdum;
       int i;
       CITED_SOURCE *cit_source;

       event = (EVENT_ENTRY *) InitHypocenterInfo();
       event->Info_Source = (CITED_SOURCE *) InitCitedSource();

       DKP(1,UnpackTime(InBlock,&event->Origin_Time));

/*
printf("day: %d hour: %d min: %d sec: %d.%d\n", event->Origin_Time.day,event->Origin_Time.hour, event->Origin_Time.minute, event->Origin_Time.second, event->Origin_Time.msec);
*/

       DKP(2,UnpackInteger(InBlock,&ldum,2,TRUE));
       event->Info_Source->Source_ID = (INT) ldum;
       DKP(3,UnpackFixed(InBlock,&event->Lattitude,10,6,TRUE));
       DKP(4,UnpackFixed(InBlock,&event->Longitude,11,6,TRUE));
       DKP(5,UnpackFixed(InBlock,&event->Depth,7,2,TRUE));
       DKP(6,UnpackInteger(InBlock,&ldum,2,TRUE));
       event->Number_Magnitudes = (INT) ldum;
       for ( i=0; i<event->Number_Magnitudes; i++ ) {
            DKP(8,UnpackFixed(InBlock,&fdum,5,2,TRUE));
            event->Magnitude[i] = (VOLFLT) fdum;
            DKP(7,UnpackVariable(InBlock,&event->Magnitude_Type[i],10,TRUE));
            DKP(8,UnpackInteger(InBlock,&ldum,2,TRUE));
            event->Magnitude_Source[i] = (INT) ldum;
       }
       LinkInEvent(event);

       return(TRUE);                              
      
}      
#undef SN 
#define SN "ParseTimeSpanID"  
BOOL ParseTimeSpanID(InBlock)
BLOCKETTE *InBlock;
{    
       TIME_SPAN  *time_span;

       time_span = (TIME_SPAN *) InitTimeSpan();

       DKP(1,UnpackChar(InBlock,&time_span->Flag));
       DKP(2,UnpackTime(InBlock,&time_span->Start_Time));
       DKP(3,UnpackTime(InBlock,&time_span->End_Time));

if(!EXTRACT) {
/* This is skipped when extracting one day out of the 
                                    geoscope seedfile */
       LinkInTimeEntry ( time_span );
}
       return(TRUE);
}  
  
#undef SN 
#define SN "ParseTimeSpanDataIndex"
BOOL ParseTimeSpanDataIndex(InBlock)
BLOCKETTE *InBlock;
{
       BLOCK73 	*bl73;
/*
       BLOCK74 	*bl74;
       INT      acc, sequence_no, sub_sequence_no, number_data_pieces;
       STDTIME	time;

       bl74 = (BLOCK74 *) InitBlock74();

       DKP(0,UnpackString(InBlock,&number_data_pieces,4));
       DKP(1,UnpackString(InBlock,&bl74->Station_ID,5));
       DKP(2,UnpackString(InBlock,&bl74->Location_ID,2));
       DKP(3,UnpackString(InBlock,&bl74->Channel_ID,3));
       DKP(4,UnpackTime(InBlock,&bl74->Start_Time)); 
       DKP(5,UnpackInteger(InBlock,&bl74->Start_Sequence_No,6,FALSE)); 
       DKP(6,UnpackInteger(InBlock,&bl74->Start_Sub_Sequence_No,2,FALSE)); 

       LinkInBlock74 ( bl74 );
*/
/*
       bl73 = (BLOCK73 *) InitBlock73();

       DKP(0,UnpackString(InBlock,&bl73->number_data_pieces,4));
       DKP(1,UnpackString(InBlock,&bl73->Station_ID,5));
       DKP(2,UnpackString(InBlock,&bl73->Location_ID,2));
       DKP(3,UnpackString(InBlock,&bl73->Channel_ID,3));
       DKP(4,UnpackTime(InBlock,&bl73->Start_Time)); 
       DKP(5,UnpackInteger(InBlock,&bl73->Start_Sequence_No,6,FALSE)); 
       DKP(6,UnpackInteger(InBlock,&bl73->Start_Sub_Sequence_No,2,FALSE)); 

printf("stat: %s  ch: %s  seqno: %d  nodp=%d\n",bl73->Station_ID,bl73->Channel_ID,bl73->Start_Sequence_No,bl73->number_data_pieces);
*/
       /* printf("Indexes not interesting now\n"); */
SET_BL74 = TRUE;
       return(TRUE);
}                 

#undef SN 
#define SN "ParseTimeSeriesIndex"
BOOL ParseTimeSeriesIndex(InBlock)
BLOCKETTE *InBlock;
{
       BLOCK74 	*bl74;
       INT      acc, sequence_no, sub_sequence_no;
       STDTIME	time;

       bl74 = (BLOCK74 *) InitBlock74();

       DKP(1,UnpackString(InBlock,&bl74->Station_ID,5));
       DKP(2,UnpackString(InBlock,&bl74->Location_ID,2));
       DKP(3,UnpackString(InBlock,&bl74->Channel_ID,3));
       DKP(4,UnpackTime(InBlock,&bl74->Start_Time)); 
       DKP(5,UnpackInteger(InBlock,&bl74->Start_Sequence_No,6,FALSE)); 
       DKP(6,UnpackInteger(InBlock,&bl74->Start_Sub_Sequence_No,2,FALSE)); 
       DKP(7,UnpackTime(InBlock,&bl74->End_Time)); 
       DKP(8,UnpackInteger(InBlock,&bl74->End_Sequence_No,6,FALSE)); 
       DKP(9,UnpackInteger(InBlock,&bl74->End_Sub_Sequence_No,2,FALSE)); 
       DKP(10,UnpackInteger(InBlock,&bl74->Accelerators,3,FALSE)); 
       for ( acc=0; acc < bl74->Accelerators; acc++ ) {
             DKP(7,UnpackTime(InBlock,&time)); 
             DKP(8,UnpackInteger(InBlock,&sequence_no,6,FALSE)); 
             DKP(9,UnpackInteger(InBlock,&sub_sequence_no,2,FALSE)); 
       }
       strcpy ( bl74->File_Name , Input_File) ;

       LinkInBlock74 ( bl74 );
       return(TRUE);
}                 
                 
#undef SN       
#define SN "ParseStationHeader"
BOOL ParseStationHeader(InBlock)
BLOCKETTE *InBlock;
{                              
       printf("Station tapes not yet implemented\n"); 
       return(FALSE);                                 
}

#undef SN                                            
#define SN "ParseNetworkHeader"                     
BOOL ParseNetworkHeader(InBlock)
BLOCKETTE *InBlock;
{                                                  
       LONG bsize;                               
       VOLFLT inver;            
                               
       DKP(2,UnpackFixed(InBlock,&inver,4,1,FALSE));
       DKP(1,UnpackInteger(InBlock,&bsize,2,FALSE)); 
       LRECL_IN = (int)( pow((double)2,(double)bsize));
       printf("LRECL=%d\n",LRECL_IN);
       if ( inver != (VOLFLT)2.1 && GEOSCOPE==FALSE ) {
          printf("\n\nSorry, only SEED volumes version 2.1 are supported (%lf).\n",inver);
          exit(1);
       }
       InBlock->BlkVer = inver * 10;               
       DKP(3,UnpackTime(InBlock,&VI->Vol_Begin)); 
       DKP(4,UnpackTime(InBlock,&VI->End_Volume));
                                                 
            Current_Station = NULL;                  
            Current_Channel = NULL;                 
            Top_Station = NULL;                    
            Top_Channel = NULL;                   
	    FIRST_INPUT_FILE = FALSE;
                                            
       return(TRUE);           
}                                   

#undef SN                          
#define SN "ParseStationIndex"    
BOOL ParseStationIndex(InBlock)
BLOCKETTE *InBlock;
{                                
       return(TRUE);           /* We don't really need this */
}                                                           
                                                           
#undef SN                                                 
#define SN "ParseSpanIndex"                              
BOOL ParseSpanIndex(InBlock)                                        
BLOCKETTE *InBlock;                                        
{                                                       
       return(TRUE);           /* We don't really need this */  
}                                                             
                                                             
#undef SN                                                   
#define SN "ParseFormatDictionary"                         
BOOL ParseFormatDictionary(InBlock)                                 
BLOCKETTE *InBlock;                                 
{                                                         
       UBYTE 	newkey[8];                                 
       FORMAT 	*informat;        
       int 	i;                  
       LONG 	ldum;
                              
       informat = (FORMAT *) InitFormat();       
       if (informat==NULL) return(FALSE);     
                                             
       DKP(1,UnpackVariable(InBlock,&informat->Format_Name,50,FALSE));
       DKP(2,UnpackInteger(InBlock,&ldum,4,FALSE));   
       informat->Format_ID = (UWORD) ldum;
       DKP(3,UnpackInteger(InBlock,&ldum,3,FALSE));     
       informat->Family = (UWORD) ldum;
       DKP(4,UnpackInteger(InBlock,&ldum,2,FALSE));  
       informat->NumKeys = (UWORD) ldum;
      
       for (i=0; i<informat->NumKeys; i++)                       
               DKP(5,UnpackVariable(InBlock,&informat->Keys[i],500,FALSE));    
           
       sprintf(newkey,"%04d",informat->Format_ID); 
       informat->Format_Key = (UBYTE *) AllocString(newkey);
/*
       DKP(5,LinkInFormat(informat));           
*/
       LinkInFormat(informat);           
       LinkInLocalFormat( informat,INP_FILE_NO);
                                               
       return(TRUE);                          
}                                           
                                           
#undef SN                                 
#define SN "ParseCommentDictionary"      
BOOL ParseCommentDictionary(InBlock)
BLOCKETTE *InBlock;
{
       COMMENT *incomment;
       LONG    ldum;

       incomment = (COMMENT *) InitComment();       
       if (incomment==NULL) return;        
      
       DKP(1,UnpackInteger(InBlock,&ldum,4,FALSE));
       incomment->Comment_ID = (UWORD) ldum;
       DKP(2,UnpackChar(InBlock,&incomment->Class));               
       DKP(3,UnpackVariable(InBlock,&incomment->Text,70,FALSE));  
       DKP(4,UnpackUnit(InBlock,&incomment->Level));             
       DKP(5,LinkInComment(incomment));                        
                                                              
       return(TRUE);                                         
}                                                           
                                                           
#undef SN                                                 
#define SN "ParseSourceDictionary"                       
BOOL ParseSourceDictionary(InBlock) 
BLOCKETTE *InBlock;
{                 
	UBYTE	*Name_Publication, *Date_Published, *Publisher_Name;
        UBYTE   *key, year[5];


	Name_Publication  = (UBYTE *) malloc(70);
	Date_Published  = (UBYTE *) malloc(70);
	Publisher_Name  = (UBYTE *) malloc(50);
	key  = (UBYTE *) malloc(8);

	strcpy(key,"PDEm");
        strcpy(Name_Publication,"Preliminary determination of epicenters (monthly listing).");
        sprintf(year,"%4d",VI->Vol_Begin.year);
        strcpy(Date_Published,year);
        strcpy(Publisher_Name,"USGS/NEIC");
	AddCitedSource ( key, Name_Publication, Date_Published, Publisher_Name);

       return(TRUE);                                 
}                                                   
                                                   
#undef SN                                         
#define SN "ParseAbbrevDictionary"               
BOOL ParseAbbrevDictionary(InBlock)
BLOCKETTE *InBlock;
{                                               
       UBYTE   newkey[8];                     
       ABBREV *inabbrev;                     
       LONG   ldum;
                                            
       inabbrev = (ABBREV *) InitAbbrev();            
       if (inabbrev==NULL) return(FALSE); 
                                         
       DKP(1,UnpackInteger(InBlock,&ldum,3,FALSE));
       inabbrev->Abbrev_ID = (UWORD) ldum;
       DKP(2,UnpackVariable(InBlock,&inabbrev->Comment,50,FALSE));
       sprintf(newkey,"%04d",inabbrev->Abbrev_ID);              
       inabbrev->Abbrev_Key = (UBYTE *) AllocString(newkey);
       DKP(3,LinkInAbbrev(inabbrev));                        
     
       return(TRUE);   
}                     
                     
#undef SN           
#define SN "ParseUnitsDictionary"    
BOOL ParseUnitsDictionary(InBlock)
BLOCKETTE *InBlock;
{
       UNIT *inunit;               
       LONG ldum;
UNIT *looper;
                                  
       inunit = (UNIT *) InitUnit();      
       if (inunit==NULL) return(FALSE);   
                                         
       DKP(1,UnpackInteger(InBlock,&ldum,3,FALSE));   
       inunit->Unit_ID = (UWORD) ldum;
       DKP(2,UnpackVariable(InBlock,&inunit->Unit_Key,20,TRUE));
       DKP(3,UnpackVariable(InBlock,&inunit->Comment,50,FALSE));
/*
       DKP(4,LinkInUnit(inunit));                             
*/
       LinkInUnit(inunit);                             
       for (looper=VI->Root_Unit; looper!=NULL;                      
               looper=looper->Next) {                    
       }
       LinkInLocalUnit( inunit,INP_FILE_NO);
                                                             
       return(TRUE);                                        
}                                                          
                                                          
#undef SN                                                
#define SN "ParseBeamDictionary"                        
BOOL ParseBeamDictionary(InBlock) 
BLOCKETTE *InBlock; 
{                                                      
       printf("Event type tapes not yet supported\n");
       return(FALSE);                                
}                                                   
                                                   
#undef SN                                         
#define SN "ParseStationID"                      
BOOL ParseStationID(InBlock)
BLOCKETTE *InBlock;
{                                               
       STATION_ENTRY *instation;              
       LONG  ldum;
                                             
       instation = (STATION_ENTRY *) InitStation();           
       if (instation==NULL) return(FALSE);          
                                                   
       DKP(1,UnpackString(InBlock,&instation->Call_Letters,5));
       DKP(2,UnpackFixed(InBlock,&instation->Lattitude,(long)10,(long)6,TRUE));   
       DKP(3,UnpackFixed(InBlock,&instation->Longitude,11,6,TRUE));  
       DKP(4,UnpackFixed(InBlock,&instation->Elevation,7,1,TRUE));  
       DKP(5,UnpackInteger(InBlock,&ldum,4,FALSE));      
       instation->Number_Channels = (UWORD) ldum;
       DKP(6,UnpackInteger(InBlock,&ldum,3,FALSE));      
       instation->Number_Comments = (UWORD) ldum;     
       DKP(7,UnpackVariable(InBlock,&instation->Site_Name,60,FALSE));
       DKP(8,UnpackAbbrev(InBlock,&instation->Station_Owner));
       DKP(9,UnpackInteger(InBlock,&ldum,4,FALSE));
       instation->Long_Order = (UWORD) ldum;
       DKP(10,UnpackInteger(InBlock,&ldum,2,FALSE));
       instation->Word_Order = (UWORD) ldum;
       DKP(11,UnpackTime(InBlock,&instation->Effective_Start));     
       DKP(12,UnpackTime(InBlock,&instation->Effective_End));    
       DKP(13,UnpackChar(InBlock,&instation->Update_Flag));     
                                                               
       if (Current_Station!=NULL) {                           
               if (streq(instation->Call_Letters,            
                       Current_Station->Call_Letters)) {    
                       Current_Station->Update_Station = instation;   
                       Current_Station = instation;                  
                       return(TRUE);                                
               }                                                   
       }                                                          
                                                                 
/*
       DKP(14,LinkInStation(instation));                        
*/
       LinkInStation(instation);                        

       Top_Station = instation;                               
       Current_Station = instation;                          
                                                            
       return(TRUE);                                       
}                                                        
                                                        
#undef SN                                              
#define SN "ParseStationComment"                      
BOOL ParseStationComment(InBlock) 
BLOCKETTE *InBlock;                                   
{
       COMMENT_ENTRY *incom;                        
                                                   
       if (Top_Station==NULL) {                   
               printf("Station comments appeared with no station?\n");
               return(FALSE);                                        
       }                                                            
                                                                  
       incom = (COMMENT_ENTRY *) InitCom();                                          
       if (incom==NULL) return(FALSE);                           
                                                                
       DKP(1,UnpackTime(InBlock,&incom->Start_Comment));       
       DKP(2,UnpackTime(InBlock,&incom->End_Comment));           
       DKP(3,UnpackComment(InBlock,&incom->Comment));           
       DKP(4,UnpackInteger(InBlock,&incom->Comment_Level,6,FALSE)); 
       DKP(5,LinkInStatCom(Top_Station,incom));                   
                                                                 
       return(TRUE);                                            
}                                                              
                                                              
#undef SN                                                    
#define SN "ChanSame"                                       
BOOL ChanSame(chan1,chan2)
CHANNEL_ENTRY *chan1, *chan2;
{
       if (chan1->Location==NULL&&chan2->Location!=NULL) return(FALSE);
       if (chan2->Location==NULL&&chan1->Location!=NULL) return(FALSE);
       if (chan1->Location!=NULL&&chan2->Location!=NULL)              
               if (!streq(chan1->Location,chan2->Location)) return(FALSE);
             
       if (streq(chan1->Identifier,chan2->Identifier)) return(TRUE); 
      
       return(FALSE);
}                   
                   
#undef SN         
#define SN "ParseChannelID"   
BOOL ParseChannelID(InBlock)
BLOCKETTE *InBlock;
{                            
       UBYTE *flags;       
       CHANNEL_ENTRY *inchannel;
       LONG idx;
      
       inchannel = (CHANNEL_ENTRY *) InitChannel();  
       if (inchannel==NULL) return(FALSE); 
      
       DKP(1,UnpackString(InBlock,&inchannel->Location,2));
       if (inchannel->Location[1]=='\0') inchannel->Location=NULL;
       DKP(2,UnpackString(InBlock,&inchannel->Identifier,3));

       DKP(3,UnpackInteger(InBlock,&idx,4,FALSE));
       inchannel->SubChannel = (UWORD) idx;

       DKP(4,UnpackAbbrev(InBlock,&inchannel->Instrument));        
       DKP(5,UnpackVariable(InBlock,&inchannel->Optional,30,FALSE));
       DKP(6,UnpackUnit(InBlock,&inchannel->Signal_Response));     
       DKP(7,UnpackUnit(InBlock,&inchannel->Calibration_Input));  
       DKP(8,UnpackFixed(InBlock,&inchannel->Lattitude,10,6,TRUE));
       DKP(9,UnpackFixed(InBlock,&inchannel->Longitude,11,6,TRUE)); 
       DKP(10,UnpackFixed(InBlock,&inchannel->Elevation,7,1,TRUE)); 
       DKP(11,UnpackFixed(InBlock,&inchannel->Local_Depth,5,1,FALSE));
       DKP(12,UnpackFixed(InBlock,&inchannel->Azimuth,5,1,FALSE));  
       DKP(13,UnpackFixed(InBlock,&inchannel->Dip,5,1,TRUE));      
       DKP(14,UnpackFormat(InBlock,&inchannel->Format_Type));     
       DKP(15,UnpackInteger(InBlock,&idx,2,FALSE));
       inchannel->Data_Exp = (UBYTE) idx;
       DKP(16,UnpackExponent(InBlock,&inchannel->Sample_Rate,10));
       DKP(17,UnpackExponent(InBlock,&inchannel->Max_Drift,10)); 
       DKP(18,UnpackInteger(InBlock,&idx,4,FALSE));
       inchannel->Number_Comments = (UWORD) idx;
       DKP(19,UnpackVariable(InBlock,&flags,26,TRUE));    
       inchannel->Channel_Flags = (ULONG) CnvChanFlags(flags);   
       DKP(20,UnpackTime(InBlock,&inchannel->Effective_Start));
       DKP(21,UnpackTime(InBlock,&inchannel->Effective_End)); 
       DKP(22,UnpackChar(InBlock,&inchannel->Update_Flag));
                                                          
/* in previous versions it said: if (Current_Channel=NULL)     */  
/*
       if (Current_Channel==NULL) {                       
               if (ChanSame(Current_Channel,inchannel)) 
                       Current_Channel->Update_Channel = inchannel;
                       Current_Channel = inchannel;               
                       return(TRUE);                             
       }                                                        
*/
                                                               
       DKP(23,LinkInChannel(Current_Station,inchannel));      
       Top_Channel = inchannel;                             
       Current_Channel = inchannel;                        
                                                          
       return(TRUE);                                     
}                                                       
                                                       
#undef SN                                             
#define SN "ParseZerosPoles"                         
BOOL ParseZerosPoles(InBlock) 
BLOCKETTE *InBlock; 
{                                                   
       ZEROS_POLES     *inzero;                   
       int i;                                    
       LONG idx;
                                                
       if (Current_Channel==NULL) return(FALSE);       /* No channel? */
       inzero = (ZEROS_POLES *) InitZerosPoles(0,0); 
       if (inzero==NULL) return(FALSE);                              
                                                                    
       DKP(1,UnpackChar(InBlock,&inzero->Function_Type));          
       DKP(2,UnpackInteger(InBlock,&idx,2,FALSE));    
       inzero->Cascade = (UWORD) idx;    
       DKP(3,UnpackUnit(InBlock,&inzero->Response_In));          
       DKP(4,UnpackUnit(InBlock,&inzero->Response_Out));        
       DKP(5,UnpackExponent(InBlock,&inzero->A0_Normalization,12));
       DKP(6,UnpackExponent(InBlock,&inzero->Norm_Frequency,12)); 
       DKP(7,UnpackInteger(InBlock,&idx,3,FALSE));
       inzero->Complex_Zeros = (UWORD) idx;
      
       if (inzero->Complex_Zeros>0) {                             
               inzero->Zero_Real_List = (VOLFLT *)               
                       malloc(sizeof(VOLFLT) * inzero->Complex_Zeros);
               if (inzero->Zero_Real_List==NULL) return(FALSE);      
               inzero->Zero_Imag_List = (VOLFLT *)                  
                       malloc(sizeof(VOLFLT) * inzero->Complex_Zeros);
               if (inzero->Zero_Imag_List==NULL) return(FALSE);      
               inzero->Zero_Real_Errors = (VOLFLT *)                
                       malloc(sizeof(VOLFLT) * inzero->Complex_Zeros);
               if (inzero->Zero_Real_Errors==NULL) return(FALSE);    
               inzero->Zero_Imag_Errors = (VOLFLT *)                
                       malloc(sizeof(VOLFLT) * inzero->Complex_Zeros);
               if (inzero->Zero_Imag_Errors==NULL) return(FALSE);    
       }                                                            
      
                                                                   
       for(i=0; i<inzero->Complex_Zeros; i++) {                   
               DKP(8,UnpackExponent(InBlock,&inzero->Zero_Real_List[i],12));   
               DKP(9,UnpackExponent(InBlock,&inzero->Zero_Imag_List[i],12));   
               DKP(10,UnpackExponent(InBlock,&inzero->Zero_Real_Errors[i],12));
               DKP(11,UnpackExponent(InBlock,&inzero->Zero_Imag_Errors[i],12));
       }                                                         
                                                                
       DKP(12,UnpackInteger(InBlock,&idx,3,FALSE));
       inzero->Complex_Poles = (UWORD) idx;
                                                                    
       if (inzero->Complex_Poles>0) {                              
               inzero->Pole_Real_List = (VOLFLT *)                
                       malloc(sizeof(VOLFLT) * inzero->Complex_Poles);
               if (inzero->Pole_Real_List==NULL) return(FALSE);      
               inzero->Pole_Imag_List = (VOLFLT *)                  
                       malloc(sizeof(VOLFLT) * inzero->Complex_Poles); 
               if (inzero->Pole_Imag_List==NULL) return(FALSE);       
               inzero->Pole_Real_Errors = (VOLFLT *)                 
                       malloc(sizeof(VOLFLT) * inzero->Complex_Poles);
               if (inzero->Pole_Real_Errors==NULL) return(FALSE);    
               inzero->Pole_Imag_Errors = (VOLFLT *)                
                       malloc(sizeof(VOLFLT) * inzero->Complex_Poles);
               if (inzero->Pole_Imag_Errors==NULL) return(FALSE);    
       }                                                            
                                                                   
       for(i=0; i<inzero->Complex_Poles; i++) {      
               DKP(13,UnpackExponent(InBlock,&inzero->Pole_Real_List[i],12));  
               DKP(14,UnpackExponent(InBlock,&inzero->Pole_Imag_List[i],12));  
               DKP(15,UnpackExponent(InBlock,&inzero->Pole_Real_Errors[i],12));
               DKP(16,UnpackExponent(InBlock,&inzero->Pole_Imag_Errors[i],12));
       }                                            
       DKP(17,LinkInZerosPoles(Current_Channel,inzero));
       return(TRUE);                                  
}                                                    
                                                    
#undef SN                                          
#define SN "ParseCoefficients"                    
BOOL ParseCoefficients(InBlock)       
BLOCKETTE *InBlock;       
{                                               
       COEFFICIENTS *incoef;                  
       int i;      
       LONG idx;
                  
       if (Current_Channel==NULL) return(FALSE);   
       incoef = (COEFFICIENTS *) InitCoefficients(0,0);            
       if (incoef==NULL) return(FALSE);          
                                                
       DKP(1,UnpackChar(InBlock,&incoef->Function_Type));
       DKP(2,UnpackInteger(InBlock,&idx,2,FALSE));   
       incoef->Cascade = (UWORD) idx;
       DKP(3,UnpackUnit(InBlock,&incoef->Response_In));         
       DKP(4,UnpackUnit(InBlock,&incoef->Response_Out));       
       DKP(5,UnpackInteger(InBlock,&idx,4,FALSE)); 
       incoef->Numerators = (UWORD) idx; 
                                                                 
       if (incoef->Numerators>0) {                              
               incoef->Numer_Coeff = (VOLFLT *)                
                       malloc(sizeof(VOLFLT) * incoef->Numerators);  
               if (incoef->Numer_Coeff ==NULL) return(FALSE);       
               incoef->Numer_Error = (VOLFLT *)                    
                       malloc(sizeof(VOLFLT) * incoef->Numerators);
               if (incoef->Numer_Error==NULL) return(FALSE);      
       }                                                         
                                                                
       for(i=0; i<incoef->Numerators; i++) {     
               DKP(6,UnpackExponent(InBlock,&incoef->Numer_Coeff[i],12));      
               DKP(7,UnpackExponent(InBlock,&incoef->Numer_Error[i],12));      
       }                                        
                                               
       DKP(8,UnpackInteger(InBlock,&idx,4,FALSE));   
       incoef->Denominators = (UWORD) idx;   
                                                                     
       if (incoef->Denominators>0) {                                
               incoef->Denom_Coeff = (VOLFLT *)                    
                       malloc(sizeof(VOLFLT) * incoef->Denominators);
               if (incoef->Denom_Coeff ==NULL) return(FALSE);       
               incoef->Denom_Error = (VOLFLT *)                    
                       malloc(sizeof(VOLFLT) * incoef->Denominators);
               if (incoef->Denom_Error==NULL) return(FALSE);        
       }                                                           
                                                                  
       for(i=0; i<incoef->Denominators; i++) {                   
               DKP(9,UnpackExponent(InBlock,&incoef->Denom_Coeff[i],12)); 
               DKP(10,UnpackExponent(InBlock,&incoef->Denom_Error[i],12));
       }                                                 
       DKP(11,LinkInCoefficients(Current_Channel,incoef));
       return(TRUE);
}                  
                  
#undef SN        
#define SN "ParseResponseList"    
BOOL ParseResponseList(InBlock)     
BLOCKETTE *InBlock;     
{                                             
       printf("Response list blockettes not yet supported.\n");
       return(FALSE);                                         
}                                                          
                                                            
#undef SN                                                 
#define SN "ParseGenericResponse"                        
BOOL ParseGenericResponse(InBlock)           
BLOCKETTE *InBlock;           
{                                                      
       printf("Generic Response blockettes not yet supported.\n");
       return(FALSE);                                            
}                        
                          
#undef SN              
#define SN "ParseDecimation"
BOOL ParseDecimation(InBlock)
BLOCKETTE *InBlock;
{                                      
       DECIMATION *indeci;           
       int i;                       
       LONG idx;
      
       if (Current_Channel==NULL) return(FALSE);       /* No channel */
       indeci = (DECIMATION *) InitDecimation(0);
       if (indeci==NULL) return(FALSE);
      
       DKP(1,UnpackInteger(InBlock,&idx,2,FALSE));  
       indeci->Cascade = (UWORD) idx;
       DKP(2,UnpackExponent(InBlock,&indeci->Input_Rate,10));  
       DKP(3,UnpackInteger(InBlock,&indeci->Factor,5,FALSE)); 
       DKP(4,UnpackInteger(InBlock,&indeci->Offset,5,FALSE));
       DKP(5,UnpackExponent(InBlock,&indeci->Delay,11));   
       DKP(6,UnpackExponent(InBlock,&indeci->Correction,11));
       DKP(7,LinkInDecimation(Current_Channel,indeci));    
       return(TRUE);                                     
}                                                       
                                                       
#undef SN
#define SN "ParseSensitivity"   
BOOL ParseSensitivity(InBlock)
BLOCKETTE *InBlock;
{                                       
       SENSITIVITY *insens;           
       LONG  idx;
       int i;                        
      
       if (Current_Channel==NULL) return(FALSE);       /* No channel */
       insens = (SENSITIVITY *) InitSensitivity(0); 
       if (insens==NULL) return(FALSE);                         
      
       DKP(1,UnpackInteger(InBlock,&idx,2,FALSE));  
       insens->Cascade = (UWORD) idx; 
       DKP(2,UnpackExponent(InBlock,&insens->Sensitivity,12)); 
       DKP(3,UnpackExponent(InBlock,&insens->Frequency,12));  
       DKP(4,UnpackInteger(InBlock,&idx,2,FALSE));
       insens->Num_Cals =(UWORD) idx;
                                                              
       if (insens->Num_Cals>0) {                             
               insens->Cal_Value = (VOLFLT *)               
                       malloc(sizeof(VOLFLT) * insens->Num_Cals); 
               insens->Cal_Freq = (VOLFLT *)                     
                       malloc(sizeof(VOLFLT) * insens->Num_Cals);
               insens->Cal_Time = (STDTIME *)                   
                       malloc(sizeof(STDTIME) * insens->Num_Cals); 
       }                                                          

       for (i=0; i<insens->Num_Cals; i++) {
               DKP(5,UnpackExponent(InBlock,&insens->Cal_Value[i],12));
               DKP(6,UnpackExponent(InBlock,&insens->Cal_Freq[i],12));
               DKP(7,UnpackTime(InBlock,&insens->Cal_Time[i]));      
       }                                                            
       DKP(8,LinkInSensitivity(Current_Channel,insens));          
       return(TRUE);                                            
}                                                            
                                                            
#undef SN                                                  
#define SN "ParseChannelComment"                          
BOOL ParseChannelComment(InBlock)             
BLOCKETTE *InBlock;             
{                                                       
       COMMENT_ENTRY *incom;                          
      
       if (Top_Channel==NULL) {
               printf("Channel comments appeared with no channel?\n");
               return(FALSE);                                        
       }                                                            
      
       incom = (COMMENT_ENTRY *) InitCom();
       if (incom==NULL) return(FALSE);                           
      
       DKP(1,UnpackTime(InBlock,&incom->Start_Comment));       
       DKP(2,UnpackTime(InBlock,&incom->End_Comment));        
       DKP(3,UnpackComment(InBlock,&incom->Comment));        
       DKP(4,UnpackInteger(InBlock,&incom->Comment_Level,6,FALSE));
       DKP(5,LinkInChanCom(Top_Channel,incom));                  
       return(TRUE);                                           
}                                                                              
