/******************************************************************************/
/****                                                                       ***/
/****    Original version modified and extended by Reinoud Sleeman.         ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/

#include "conver.h"
#include "seed.h"
         
extern ULONG   DefBlkLen;              /* Default blockette buffer len */      
extern ULONG   DefBlkExt;              /* Default blockette extension */
extern BOOL MERGE_FILES;
BOOL PRINT;

/* user supplied */    VOID StoreRec();

/*       
 *     Initialize a structure for a logical record
 */      
         
LOGICAL_RECORD *CreateRecordBuff( creexp,writertn) 
UWORD creexp;
BOOL (*writertn)(); 
{        
       LOGICAL_RECORD *New_Record;

       New_Record = (LOGICAL_RECORD *) malloc(sizeof(LOGICAL_RECORD));
       if (New_Record==NULL) return(NULL);                 

       if (creexp>15) creexp = 15;     /* Maximum logical record size */
       if (creexp<8) creexp = 8;       /* Minimum logical record size */
           
       New_Record->RecBuff = (UBYTE *) malloc(1<<creexp);
       if (New_Record->RecBuff==NULL) return(NULL);    /* No luck */
        
       New_Record->RecNum = 0;
       New_Record->NextPos = 0;
       New_Record->RecLen = creexp;
       New_Record->WriteService = writertn; 
       New_Record->Continue = FALSE;
       New_Record->RecType = ' ';
         
       FlushRecord(New_Record);
         
       return(New_Record);
}        
         
/*       
 *     Append data to a blockette
 */

AppendBlockette(InBlock,DataBuf, DataLen)
BLOCKETTE *InBlock;
UBYTE *DataBuf;
ULONG DataLen;
{
       ULONG   counter;

               /* See if the data will fit */
       if (InBlock->PackPos + DataLen > InBlock->BufLen) { 
               if (!ExtendBlockette(InBlock,              
                       ((InBlock->PackPos + DataLen - InBlock->BufLen) /
                               DefBlkExt) + 1)) return(FALSE);         
       }                                                              
                                                                     
       for (counter = 0; counter < DataLen; counter++)              
               InBlock->BlkBuff[InBlock->PackPos + counter] =      
                       DataBuf[counter];                          
                    
       InBlock->PackPos += DataLen; 
       return(TRUE);
      
}                  
                  
/*               
 *     Pack a fixed length string onto record
 */                                        

PackString(InBlock,InStr,MaxStr,Upcase) 
BLOCKETTE *InBlock;
UBYTE *InStr;
ULONG MaxStr;
BOOL Upcase; 
{
       UBYTE   *stobuff,txchar,dummy;
       ULONG   stoidx;
       BOOL    packresult;

       if (InStr==NULL) { 
               dummy = '\0';
               InStr = &dummy;
       }
       while (*InStr==' ') InStr++;            /* Skip leading spaces */
       stobuff = (UBYTE *) malloc(MaxStr);

       if (stobuff==NULL) { 
               printf("Couldn't allocate temp buffer in PackString\n");
               return(FALSE);
       } 
      
       for (stoidx=0; stoidx<MaxStr; stoidx++) {
               if (*InStr!='\0') {
                       txchar = *InStr; 
                       if (Upcase&&islower(txchar)) txchar = toupper(txchar);  
                       stobuff[stoidx] = txchar;
                       InStr++;
               } else  stobuff[stoidx] = ' ';          /* Space pad */
       } 
       packresult = AppendBlockette(InBlock,stobuff,MaxStr);
       free(stobuff);
      
       return(packresult);     /* Sink or swim */
}

PackChar(InBlock, InChar, Upcase)                     
BLOCKETTE *InBlock;
UBYTE InChar;
BOOL Upcase;                     
{    
       UBYTE NewStr[2];  

       NewStr[0] = InChar;
       NewStr[1] = '\0'; 
       return(PackString(InBlock,NewStr,1,Upcase));
}

/*
 *     Pack away a variable length string 
 *     MaxStr = 0, no maximum length
 */
  
PackVariable(InBlock,InStr, MaxStr, Upcase)    
BLOCKETTE *InBlock;
UBYTE *InStr;
ULONG MaxStr;
BOOL Upcase;    
{
       UBYTE   *stobuff,txchar,dummy;
       ULONG   stoidx; 
       BOOL    packresult;

       if (InStr==NULL) {
               dummy = '\0';
               InStr = &dummy;
       }                     
       while (*InStr==' ') InStr++;            /* Skip leading spaces */
       stobuff = (UBYTE *) malloc((MaxStr!=0)?MaxStr+1:strlen(InStr+1));
       stoidx = 0; 
      
       while (*InStr!='\0') {   
               if (MaxStr>0&&stoidx>=MaxStr) break;
               txchar = *InStr; 
               if (Upcase&&islower(txchar)) txchar = toupper(txchar);

               stobuff[stoidx++] = txchar; 
               InStr++;
       } 
       while (stoidx>0&&stobuff[stoidx-1]==' ') stoidx--;   /* Trail space */  
       stobuff[stoidx++] = '~';                /* Tilde terminator */  
       packresult = AppendBlockette(InBlock,stobuff,stoidx); 
       free(stobuff);

       return(packresult);     /* Sink or swim */ 
}

/*
 *     Pack signed or unsigned integer   
 */
  
PackInteger(InBlock, InNum, FldLen, Signed)        
BLOCKETTE *InBlock;
LONG InNum;
LONG FldLen;
BOOL Signed;        
{
       UBYTE   maskbuff[25];
       UBYTE   intbuff[25];
       BOOL    minus=FALSE;                    /* Signed Minus? */
       BOOL    packresult;
       LONG    start,totlen;
         
       totlen = Signed?(FldLen-1):FldLen;      /* Field without sign */
       start = Signed?1:0;                     /* Where does field start */    
       if (Signed&&InNum<0) minus = TRUE;      /* Number minus */    
       if (InNum<0) InNum = -InNum;                                   
       sprintf(maskbuff,"%%0%ldld",totlen);    /* Make final mask */
       sprintf(&intbuff[start],maskbuff,InNum);                    
       if (Signed) intbuff[0] = minus?'-':'+'; /* Place sign */  
       if (strlen(intbuff)!=FldLen) return(FALSE);     /* Number overflow */   
       packresult = AppendBlockette(InBlock,intbuff,FldLen);  
       return(packresult);                                  
}                                                          
                                                          
/*                                                       
 *     Pack signed or unsigned fixed floating point    
 */                                                  
                                                    
static VOLFLT _shfttab[] = { 1E0, 1E1, 1E2, 1E3, 1E4, 1E5, 1E6, 1E8, 1E8 };    
                                                   
PackFixed(InBlock, InFlt,FldLen,Decpls, Signed) 
BLOCKETTE *InBlock;
VOLFLT InFlt; 
LONG FldLen;
LONG Decpls;
BOOL Signed; 
{      
       UBYTE   maskbuff[25],finalbuff[30]; 
       UBYTE   intbuff[25];               
       BOOL    minus=FALSE;                    /* Signed Minus? */     
       BOOL    packresult;                                            
       LONG    start,totlen,stoidx;                                  
       LONG    InNum;                                               
                                                                   
       InFlt *= _shfttab[Decpls];                                 
       if ( InFlt > 0) 
            InNum = (LONG)(InFlt+0.5);         /* Convert and round */ 
       else InNum = (LONG)InFlt;

                                                                
       totlen = Signed?(FldLen-1):FldLen;      /* Field without sign */
       totlen --;                              /* Leave room for decimal */
       start = Signed?1:0;                     /* Where does field start */    
       if (Signed&&InNum<0) minus = TRUE;      /* Number minus */  
       if (InNum<0) InNum = -InNum;                               
                                                                 
       sprintf(maskbuff,"%%0%ldld",totlen);    /* Make final mask */ 
       sprintf(&intbuff[start],maskbuff,InNum);                     
                                                                   
       if (Signed) intbuff[0] = minus?'-':'+'; /* Place sign */   
       if (strlen(intbuff)!=FldLen-1) return(FALSE);   /* Number overflow */   
                                                                
       for (stoidx = 0; stoidx < FldLen-Decpls-1; stoidx++)    
               finalbuff[stoidx] = intbuff[stoidx];           
       finalbuff[FldLen-Decpls-1] = '.';       /* Decimal point */
       for (stoidx = 0; stoidx<Decpls; stoidx++)                 
               finalbuff[FldLen-Decpls+stoidx] =                
                       intbuff[FldLen-Decpls+stoidx-1];        
                                                              
       packresult = AppendBlockette(InBlock,finalbuff,FldLen);
       return(packresult);                                  
}                                                          
                                                          
/*                                                       
 *     Pack an exponential form                        
 */
  
PackExponent(InBlock, InFlt, FldLen)                 
BLOCKETTE *InBlock;
VOLFLT InFlt;
LONG FldLen;                 
{
       UBYTE   fltbuff[30];  
       UBYTE   fltmask[30]; 
       BOOL    packresult; 
                          
       sprintf(fltmask,"%%%ld.%ldE",FldLen,FldLen-7);   
       sprintf(fltbuff,fltmask,InFlt);                 
       packresult = AppendBlockette(InBlock,fltbuff,FldLen); 
       return(packresult);                                 
}                                                       
                                                         
/*                                                     
 *     Pack the standard time                        
 */
    
PackTime(InBlock,InTime)                               
BLOCKETTE *InBlock;
STDTIME InTime;
{ 
       UBYTE   timestr[30],msbuf[6];
       UWORD   flgsec;
       BOOL    packresult;

       flgsec = 0;
       if      (InTime.msec  !=0) flgsec = 6;
       else if (InTime.second!=0) flgsec = 5;
       else if (InTime.minute!=0) flgsec = 4;
       else if (InTime.hour  !=0) flgsec = 3;
       else if (InTime.day   !=0) flgsec = 2;
       else if (InTime.year  !=0) flgsec = 1;
                                            
       switch(flgsec) {
       case 0:        
               sprintf(timestr,"~");
               break;
       case 1:
       case 2:
               sprintf(timestr,"%04d,%03d~",InTime.year,InTime.day);
               break;
       case 3:
               sprintf(timestr,"%04d,%03d,%02d~",InTime.year,
                       InTime.day,InTime.hour);
               break;
       case 4:
               sprintf(timestr,"%04d,%03d,%02d:%02d~",InTime.year,
                       InTime.day,InTime.hour,InTime.minute);
               break; 
       case 5:
               sprintf(timestr,"%04d,%03d,%02d:%02d:%02d~",InTime.year,
                       InTime.day,InTime.hour,InTime.minute,InTime.second);
               break;
       case 6:                                       
               sprintf(msbuf,"%03d",InTime.msec);          
               if (msbuf[2]=='0') {                       
                       msbuf[2]='\0';                    
                       if (msbuf[1]=='\0')              
                               msbuf[1]='\0';          
               }                                      
               sprintf(timestr,"%04d,%03d,%02d:%02d:%02d.%s~",InTime.year,     
                       InTime.day,InTime.hour,InTime.minute,
                       InTime.second,msbuf); 
               break;                         
       }                                       
      
       packresult = AppendBlockette(InBlock,timestr,strlen(timestr)); 
       return(packresult);  
}          
                                     
/*                                         
 *     Empty out record to the write service routine
 */                                               
                                                 
FlushRecord(inrecord)
LOGICAL_RECORD *inrecord;
{
       UWORD rlen,i;
                   
       rlen = 1<<inrecord->RecLen;
       if (inrecord->NextPos==8) {     /* Don't dump out empty records */      
          
                       /* scratch off serial numbers and recycle it */
                    
               inrecord->RecBuff[6] = inrecord->RecType;
               inrecord->RecBuff[7] = inrecord->Continue?'*':' ';
               return(TRUE);
       }

       if (inrecord->NextPos>0) {
               for (i=inrecord->NextPos; i<rlen; i++)
                       inrecord->RecBuff[i] = ' ';     /* Flush out */
                StoreRec ( inrecord->RecBuff, rlen);
       }
       inrecord->RecNum++;             /* Increment ID */
       if (inrecord->RecNum>999999) inrecord->RecNum = 0;
       sprintf(inrecord->RecBuff,"%06d%c%c", 
               inrecord->RecNum,
               inrecord->RecType,
               inrecord->Continue?'*':' ');    
       inrecord->NextPos = 8;                
       return(TRUE);                       
}                                         
                                         
/*                                      
 *     Write out a "noise" record     
 */
  
NoiseRecord(inrecord)
LOGICAL_RECORD *inrecord;
{
       if (inrecord->NextPos==8) {    
               inrecord->RecBuff[8] = ' '; 
               inrecord->NextPos = 9;     
               CKP(FlushRecord(inrecord));     /* Write it out blanks */
       }                                                               
                                                                      
       CKP(FlushRecord(inrecord));     /* It had something in it */   
       return(TRUE);                                                
}                                                                  
                                                                  
/*                                                               
 *     Flush headers and setup as new header type              
 */                                                          
                                                            
NewHeader(inrecord, headtype)
LOGICAL_RECORD *inrecord;
UBYTE headtype;
{
       inrecord->RecType = headtype;                      
       inrecord->Continue = FALSE;                       
       return(FlushRecord(inrecord));                   
}      
      
/*   
 *     Jam a blockette into a record...  Keep writing records until it fits    
 */
  
RecordPack(inblock,inrecord) 
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord; 
{ 
	int i;
 
       UWORD rlen,blockavail,startover;  
      
       rlen = 1<<inrecord->RecLen;     
       if ((rlen-inrecord->NextPos)<20) {
               inrecord->Continue=TRUE; 
               FlushRecord(inrecord);  
       }                              
                                     
       sprintf(&inrecord->RecBuff[inrecord->NextPos], "%03d%04d",
               inblock->BlkType, inblock->PackPos+7);    /* Inclusive */
                                                             
       inrecord->NextPos += 7;                              
       startover = 0;                                     

       while (startover<inblock->PackPos) {             
               blockavail = rlen-inrecord->NextPos;   
               if (blockavail>=(inblock->PackPos-startover)) {
            
                       (VOID) memcpy(&inrecord->RecBuff[inrecord->NextPos],    
                                     &inblock->BlkBuff[startover],
                                     inblock->PackPos-startover); 

                       inrecord->NextPos += inblock->PackPos-startover;
                       return(TRUE);                                  
               }                                                     
                                                                    
               (VOID) memcpy(&inrecord->RecBuff[inrecord->NextPos],
                             &inblock->BlkBuff[startover],        
                             blockavail);                        
                    
                                                                
               inrecord->NextPos += blockavail;        /* Should fill up */    
               inrecord->Continue=TRUE;                        
               FlushRecord(inrecord);          /* Write that out */
               startover += blockavail;                          
       }                                                       
                                                              
       fprintf(stderr,"RecordPack should never exit here\n");
       fprintf(stderr,"Startover is %d, packpos is %d\n",   
               startover,inblock->PackPos);                
                                                          
       return(TRUE);   /* All written out */             
}                                                       
                                                       
/*                                                    
 *     Write out the proper volume header blockette 
 */                                               
               
VolumeBlockette(inblock,inrecord, station)            /*   5 or 10    */
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord;
BOOL station;
{
       SetupBlockette(inblock,
               station?SEED_STATION_HEADER:SEED_NETWORK_HEADER,SEEDVER);
       CKP(PackFixed(inblock,(VOLFLT)2.1,(LONG)4,(LONG)1,FALSE));
       CKP(PackInteger(inblock,(LONG)inrecord->RecLen,(LONG)2,FALSE));
       CKP(PackTime(inblock,VI->Vol_Begin));
       if (!station) CKP(PackTime(inblock,VI->End_Volume));
       CKP(RecordPack(inblock,inrecord));
       return(TRUE);
}                  


VOID StationHeaderIndex (inblock, inrecord )             /*  11  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
{
	STATION_ENTRY	*looper;

	int st=0;
	int sequence_no=0;          /*  dummy value */

	SetupBlockette(inblock,SEED_STATION_INDEX,SEEDVER);
	for ( looper=VI->Root_Station; looper!=NULL; looper=looper->Next )   st++;
	PackInteger(inblock,(LONG)st,(LONG)3,FALSE);
	for ( looper=VI->Root_Station; looper!=NULL; looper=looper->Next )  {
		PackString(inblock,looper->Call_Letters,(ULONG)5,TRUE);
		PackInteger(inblock,(LONG)sequence_no,(LONG)6,FALSE);
	}
	RecordPack(inblock,inrecord);
}
                  
VOID TimeSpanIndex (inblock, inrecord )                  /*  12  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
{

/*  Timespan index blockette for one timespan (day) */

	INT span=1;          /* one time span */
        INT sequence_no = 0; /* dummy value */	

	SetupBlockette(inblock,SEED_SPAN_INDEX,SEEDVER);
	PackInteger(inblock,(LONG)span,(LONG)4,FALSE);
	PackTime(inblock,VI->Vol_Begin);
	PackTime(inblock,VI->End_Volume);
	PackInteger(inblock,(LONG)sequence_no,(LONG)6,FALSE);
	RecordPack(inblock,inrecord);
}


VOID TimeSpanIndexM (inblock, inrecord )                  /*  12  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
{

/*  Timespan index blockettes for more timespans */

	INT span=0, sequence_no=0;         /* dummy value */
	TIME_ENTRY  *time_entry;

	for(time_entry=VI->Root_Time_Entry;time_entry!=NULL;time_entry=time_entry->Next) {
            if ( EXTRACT ) { 
                 if ( time_entry->Time_Span_ID->Start_Time.day != SDAY )
                        continue;
	    }
            span++;
	}

	SetupBlockette(inblock,SEED_SPAN_INDEX,SEEDVER);
	PackInteger(inblock,(LONG)span,(LONG)4,FALSE);
	for(time_entry=VI->Root_Time_Entry;time_entry!=NULL;time_entry=time_entry->Next)  {
            if ( EXTRACT ) { 
                 if ( time_entry->Time_Span_ID->Start_Time.day != SDAY )
                        continue;
	    }
	    PackTime(inblock,time_entry->Time_Span_ID->Start_Time);
	    PackTime(inblock,time_entry->Time_Span_ID->End_Time);
	    PackInteger(inblock,(LONG)sequence_no,(LONG)6,FALSE);
	}
	RecordPack(inblock,inrecord);
}


/*               
 *     Write out a abbreviation dictionary entry blockette
 */              
                
AbbrevBlockette(inabbrev,inblock,inrecord)                 /*  33  */
ABBREV *inabbrev;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord;
{ 
       SetupBlockette(inblock,SEED_ABBREV_DICT,SEEDVER); 
                                                        
       CKP(PackInteger(inblock,(LONG)inabbrev->Abbrev_ID,(LONG)3,FALSE));
       CKP(PackVariable(inblock,inabbrev->Comment,(LONG)50,FALSE)); 
       CKP(RecordPack(inblock,inrecord));                    
                                                           
       return(TRUE);                                      
}                                                        
                                                        
DumpAbbrevBlockettes(blockbuf,inrecord)
BLOCKETTE *blockbuf;
LOGICAL_RECORD *inrecord;
{
       ABBREV *looper;  
       int blockid;

       blockid = 0;
       for (looper=VI->Root_Abbrev; looper!=NULL; looper=looper->Next) {
               looper->Abbrev_ID = ++blockid;          
               CKP(AbbrevBlockette(looper,blockbuf,inrecord));
       }                                                     
       return(TRUE);                                       
}                                                         
                                                         
/*                                                      
 *     Put out the abbreviation pointer part of a blockette 
 */                                                       
                                                         
PackAbbrev(inblock,inabbrev)
BLOCKETTE *inblock;
ABBREV *inabbrev;
{
       LONG packid=0;
       ABBREV *abbrev;

       if(MERGE_FILES) {
                if ( inabbrev!=NULL) {
	              abbrev = (ABBREV *) SearchAbbrev( inabbrev->Abbrev_Key);
                      packid = (LONG) abbrev->Abbrev_ID;
                }
                return(PackInteger(inblock,packid,(LONG)3,FALSE)); 
       }

       if (inabbrev!=NULL)
               packid = (LONG)inabbrev->Abbrev_ID;
       return(PackInteger(inblock,packid,(LONG)3,FALSE)); 
}

/*
 *     Form a unit blockette
 */

UnitBlockette(inunit,inblock,inrecord)          /*   34   */
UNIT *inunit;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord;
{                                                       
       SetupBlockette(inblock,SEED_UNIT_DICT,SEEDVER); 
                                                     
       CKP(PackInteger(inblock,(LONG)inunit->Unit_ID,(LONG)3,FALSE));
       CKP(PackVariable(inblock,inunit->Unit_Key,(LONG)20,TRUE));
       CKP(PackVariable(inblock,inunit->Comment,(LONG)50,FALSE));
       CKP(RecordPack(inblock,inrecord));                
       return(TRUE);
}
  
/* 
 *     Write out all unit blockettes which are in use  
 */                                                  

DumpUnitBlockettes(blockbuf,inrecord)
BLOCKETTE *blockbuf;
LOGICAL_RECORD *inrecord;
{
       UNIT *looper;
       int blockid;

       blockid = 0;
       for (looper=VI->Root_Unit; looper!=NULL; looper=looper->Next) {
               looper->Unit_ID = ++blockid;
               CKP(UnitBlockette(looper,blockbuf,inrecord));
       }
       return(TRUE); 
}
/*   
 *     Write out the unit part of a blockette   
 */
                                               
PackUnit(inblock,inunit)                                 
BLOCKETTE *inblock;
UNIT *inunit;
{
       LONG packid=0;
       UNIT *unit;
                   
        if(MERGE_FILES) {
                if ( inunit!=NULL) {
	              unit = (UNIT *) SearchUnit( inunit->Unit_Key);
                      packid = (LONG) unit->Unit_ID;
                }
                return(PackInteger(inblock,packid,(LONG)3,FALSE)); 
        }

       if (inunit!=NULL) packid = (LONG)inunit->Unit_ID;       
       return(PackInteger(inblock,packid,(LONG)3,FALSE)); 
}                                                 
                                                 
/*                                              
 *     Form and write a comment dictionary header
 */                                            
                                              
CommentBlockette(incomment, inblock,inrecord)           /*  31  */
COMMENT *incomment;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord;
{                                                                  
       if (incomment->Comment_ID>9999) return(TRUE);   /* Internal use only */ 
       SetupBlockette(inblock,SEED_COMMENT_DICT,SEEDVER);       
       CKP(PackInteger(inblock,(LONG)incomment->Comment_ID,(LONG)4,FALSE));
       CKP(PackChar(inblock,incomment->Class,TRUE));          
       CKP(PackVariable(inblock,incomment->Text,(LONG)70,FALSE));  
       CKP(PackUnit(inblock,incomment->Level));             
       CKP(RecordPack(inblock,inrecord));                 
       return(TRUE);                                    
}                                                      
                                                     
/*                                                    
 *     Dump out all comment blockettes             
 */                                              

DumpCommentBlockettes(blockbuf,inrecord)
BLOCKETTE *blockbuf;
LOGICAL_RECORD *inrecord;
{ 
       COMMENT *looper;                         
       int blockid;                            
                                              
       blockid = 0;                          
                                            
       for (looper=VI->Root_Comment; looper!=NULL; looper=looper->Next) {      
               CKP(CommentBlockette(looper,blockbuf,inrecord)); 
       }                                                       
       return(TRUE);                                         
}                                                           
                                                           
/*                                                        
 *     Write out the comment portion of a blockette     
 */                                                   
                                                     
PackComment(inblock,incomment)
BLOCKETTE *inblock;
COMMENT *incomment;
{                                                   
       LONG packid=0;                              
                                                 
       if (incomment!=NULL)                     
               packid = (LONG)incomment->Comment_ID;   
       return(PackInteger(inblock,packid,(LONG)4,FALSE)); 
}                                                  
                                                  
CitedSourceBlockette(insource,inblock,inrecord)               /*  32  */
CITED_SOURCE *insource; 
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord;
{
       int i;

       SetupBlockette(inblock,SEED_SOURCE_DICT,SEEDVER);
       CKP(PackInteger(inblock,insource->Source_ID,(LONG)2,FALSE));
       CKP(PackVariable(inblock,(LONG)insource->Name_Publication,(LONG)70,FALSE));
       CKP(PackVariable(inblock,(LONG)insource->Date_Published,(LONG)70,FALSE));
       CKP(PackVariable(inblock,(LONG)insource->Publisher_Name,(LONG)50,FALSE));
       CKP(RecordPack(inblock,inrecord));
       return(TRUE);
      
}

DumpCitedSource(blockbuf,inrecord)        
BLOCKETTE *blockbuf;
LOGICAL_RECORD *inrecord;
{
       CITED_SOURCE *looper;
       int blockid;
      
       blockid = 0;
       for (looper=VI->Root_Cited_Source;looper!=NULL;looper=looper->Next) {
               looper->Source_ID = ++blockid;
               CKP(CitedSourceBlockette(looper,blockbuf,inrecord));
       }
       return(TRUE);
}

/*                                               
 *     Write out a format blockette            
 */
  
FormatBlockette(informat,inblock,inrecord)               /*  30  */
FORMAT *informat; 
BLOCKETTE *inblock;
LOGICAL_RECORD *inrecord;
{
       int i;

       SetupBlockette(inblock,SEED_FORMAT_DICT,SEEDVER);
       CKP(PackVariable(inblock,informat->Format_Name,(LONG)50,FALSE));
       CKP(PackInteger(inblock,(LONG)informat->Format_ID,(LONG)4,FALSE));
       CKP(PackInteger(inblock,(LONG)informat->Family,(LONG)3,FALSE));
       CKP(PackInteger(inblock,(LONG)informat->NumKeys,(LONG)2,FALSE));
       for (i=0; i<informat->NumKeys; i++)
               CKP(PackVariable(inblock,informat->Keys[i],(LONG)500,FALSE));
       CKP(RecordPack(inblock,inrecord));
       return(TRUE);
      
}
/*  
 *     Write out all format blockettes which are in use  
 */                                                    

DumpFormatBlockettes(blockbuf,inrecord)        
BLOCKETTE *blockbuf;
LOGICAL_RECORD *inrecord;
{
       FORMAT *looper;
       int blockid;
      
       blockid = 0;
       for (looper=VI->Root_Format; looper!=NULL; looper=looper->Next) {
               looper->Format_ID = ++blockid;
               CKP(FormatBlockette(looper,blockbuf,inrecord));
       }
       return(TRUE);
}

/*
 *     Write a blockette piece with the format code in it
 */
  
PackFormat(inblock,informat)                           
BLOCKETTE *inblock;
FORMAT *informat;
{                 
       LONG packid=0;      
       FORMAT *format;
      
        if(MERGE_FILES) {
                if ( informat!=NULL) {
	              format = (FORMAT *) SearchFormatName( informat->Format_Name);
                      packid = (LONG) format->Format_ID;
                }
                return(PackInteger(inblock,packid,(LONG)4,FALSE)); 
        }

       if (informat!=NULL) 
               packid = (LONG)informat->Format_ID; 
       return(PackInteger(inblock,packid,(LONG)4,FALSE));     
}

/*
 *     Write out a station blockette 
 */

PrintTime(time)
STDTIME	 time;
{
     printf("year: %d  day: %d  hr: %d  min: %d  sec: %d  msec: %d\n",
            time.year, time.day, time.hour, time.minute, time.second,
            time.msec);
}
PrintTimeP(time)
STDTIME	 *time;
{
     printf("year: %d  day: %d  hr: %d  min: %d  sec: %d  msec: %d\n",
            time->year, time->day, time->hour, time->minute, time->second,
            time->msec);
}

StationBlockette(instation,WriteBlock,WriteRec)             /*  50  */
STATION_ENTRY *instation;
BLOCKETTE *WriteBlock;
LOGICAL_RECORD *WriteRec;           
{
       CKP(NewHeader(WriteRec,'S'));
       SetupBlockette(WriteBlock,SEED_STATION_HEADER,SEEDVER);
       CKP(PackString(WriteBlock,instation->Call_Letters,(ULONG)5,TRUE));
       CKP(PackFixed(WriteBlock,(VOLFLT)instation->Lattitude,(LONG)10,(LONG)6,TRUE));
       CKP(PackFixed(WriteBlock,(VOLFLT)instation->Longitude,(LONG)11,(LONG)6,TRUE));
       CKP(PackFixed(WriteBlock,(VOLFLT)instation->Elevation,(LONG)7,(LONG)1,TRUE));
       CKP(PackInteger(WriteBlock,(LONG)instation->Number_Channels,(LONG)4,FALSE));
       CKP(PackInteger(WriteBlock,(LONG)instation->Number_Comments,(LONG)3,FALSE));
       CKP(PackVariable(WriteBlock,instation->Site_Name,(LONG)60,FALSE));
       CKP(PackAbbrev(WriteBlock,instation->Station_Owner));
       CKP(PackInteger(WriteBlock,(LONG)instation->Long_Order,(LONG)4,FALSE));
       CKP(PackInteger(WriteBlock,(LONG)instation->Word_Order,(LONG)2,FALSE));
       CKP(PackTime(WriteBlock,instation->Effective_Start));
       if (instation->Effective_End.year>3000)
               instation->Effective_End = _ztime;
       CKP(PackTime(WriteBlock,instation->Effective_End));
       CKP(PackChar(WriteBlock,instation->Update_Flag,TRUE));
       CKP(RecordPack(WriteBlock,WriteRec));
      
       return(TRUE);
}

/*
 *     Write out a channel blockette 
 */
 
ChannelBlockette(inchannel, WriteBlock, WriteRec)        /*  52  */
CHANNEL_ENTRY *inchannel;
BLOCKETTE *WriteBlock;
LOGICAL_RECORD *WriteRec;                         
{    
       UBYTE   flags[30],*flg;  
       INT     i;
                               
       SetupBlockette(WriteBlock,SEED_CHANNEL_HEADER,SEEDVER);  
       CKP(PackString(WriteBlock,inchannel->Location,(ULONG)2,TRUE)); 
       CKP(PackString(WriteBlock,inchannel->Identifier,(ULONG)3,TRUE)); 
       CKP(PackInteger(WriteBlock,(LONG)inchannel->SubChannel,(LONG)4,FALSE)); 
       CKP(PackAbbrev(WriteBlock,inchannel->Instrument));         
       CKP(PackVariable(WriteBlock,inchannel->Optional,(LONG)30,FALSE));
       CKP(PackUnit(WriteBlock,inchannel->Signal_Response));     
       CKP(PackUnit(WriteBlock,inchannel->Calibration_Input));  
       CKP(PackFixed(WriteBlock,(VOLFLT)inchannel->Lattitude,(LONG)10,(LONG)6,TRUE));
       CKP(PackFixed(WriteBlock,(VOLFLT)inchannel->Longitude,(LONG)11,(LONG)6,TRUE)); 
       CKP(PackFixed(WriteBlock,(VOLFLT)inchannel->Elevation,(LONG)7,(LONG)1,TRUE)); 
       CKP(PackFixed(WriteBlock,(VOLFLT)inchannel->Local_Depth,(LONG)5,(LONG)1,FALSE));
       CKP(PackFixed(WriteBlock,(VOLFLT)inchannel->Azimuth,(LONG)5,(LONG)1,FALSE));   
       CKP(PackFixed(WriteBlock,(VOLFLT)inchannel->Dip,(LONG)5,(LONG)1,TRUE));       
       CKP(PackFormat(WriteBlock,inchannel->Format_Type));      
       CKP(PackInteger(WriteBlock,(LONG)inchannel->Data_Exp,(LONG)2,FALSE)); 
       CKP(PackExponent(WriteBlock,(VOLFLT)inchannel->Sample_Rate,(LONG)10)); 
       CKP(PackExponent(WriteBlock,(VOLFLT)inchannel->Max_Drift,(LONG)10));   
       CKP(PackInteger(WriteBlock,(LONG)inchannel->Number_Comments,(LONG)4,FALSE));
                                                                      
       flg = flags;                                                  

       i=0;
       if (inchannel->Channel_Flags & CFG_TRIGGERED)   flags[i++] = 'T';
       if (inchannel->Channel_Flags & CFG_CONTINUOUS)  flags[i++] = 'C';
       if (inchannel->Channel_Flags & CFG_HEALTHCHAN)  flags[i++] = 'H';
       if (inchannel->Channel_Flags & CFG_GEODATA)     flags[i++] = 'G';
       if (inchannel->Channel_Flags & CFG_ENVIRON)     flags[i++] = 'W';
       if (inchannel->Channel_Flags & CFG_FLAGS)       flags[i++] = 'F';
       if (inchannel->Channel_Flags & CFG_SYNTH)       flags[i++] = 'S';
       if (inchannel->Channel_Flags & CFG_CALIN)       flags[i++] = 'I';
       if (inchannel->Channel_Flags & CFG_EXPERIMENT)  flags[i++] = 'E';
       if (inchannel->Channel_Flags & CFG_MAINTENANCE) flags[i++] = 'M';
       if (inchannel->Channel_Flags & CFG_BEAM)        flags[i++] = 'B';
                                                                    
       flags[i] = '\0';                                              
       CKP(PackVariable(WriteBlock,flags,(LONG)26,TRUE));              
       CKP(PackTime(WriteBlock,inchannel->Effective_Start));   
       if (inchannel->Effective_End.year>3000)                
               inchannel->Effective_End = _ztime;            
       CKP(PackTime(WriteBlock,inchannel->Effective_End));   
       CKP(PackChar(WriteBlock,inchannel->Update_Flag,TRUE));
       CKP(RecordPack(WriteBlock,WriteRec));               
                                                          
       return(TRUE);                                     
}                                                       
                                                       
/*                                                    
 *     Write a sensitivity blockette                
 */
  
SensBlockette(insens,inblock,inrec)                      /*  58  */
SENSITIVITY *insens; 
BLOCKETTE *inblock;
LOGICAL_RECORD *inrec;                               
{
       int i;
            
       SetupBlockette(inblock,SEED_GAIN_SENSITIVITY,SEEDVER); 
       CKP(PackInteger(inblock,(LONG)insens->Cascade,(LONG)2,FALSE));   
       CKP(PackExponent(inblock,(VOLFLT)insens->Sensitivity,(LONG)12));  
       CKP(PackExponent(inblock,(VOLFLT)insens->Frequency,(LONG)12));   
       CKP(PackInteger(inblock,(LONG)insens->Num_Cals,(LONG)2,FALSE));
                                                         
       for (i=0; i<insens->Num_Cals ; i++) {             
               CKP(PackExponent(inblock,(VOLFLT)insens->Cal_Value[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)insens->Cal_Freq[i],(LONG)12));
               CKP(PackTime(inblock,insens->Cal_Time[i]));      
       }                                                       
                                                              
       CKP(RecordPack(inblock,inrec));                       
       return(TRUE);                                       
}   
   
/*
 *     Write a decimation blockette   
 */    
      
DeciBlockette(indeci, inblock,inrec)                    /*  57  */ 
DECIMATION *indeci;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrec;                               
{    
       int i;     
                 
       SetupBlockette(inblock,SEED_DECIMATION,SEEDVER);  
       CKP(PackInteger(inblock,(LONG)indeci->Cascade,(LONG)2,FALSE));
       CKP(PackExponent(inblock,(VOLFLT)indeci->Input_Rate,(LONG)10));  
       CKP(PackInteger(inblock,(LONG)indeci->Factor,(LONG)5,FALSE)); 
       CKP(PackInteger(inblock,(LONG)indeci->Offset,(LONG)5,FALSE));
       CKP(PackExponent(inblock,(VOLFLT)indeci->Delay,(LONG)11));    
       CKP(PackExponent(inblock,(VOLFLT)indeci->Correction,(LONG)11)); 
       CKP(RecordPack(inblock,inrec));
       return(TRUE);
}                  
                  
/*               
 *     Write a zeros and poles blockette  
 */                                     
                                       
ZerosBlockette(inzero, inblock,inrec)                     /*  53  */ 
ZEROS_POLES 	*inzero;
BLOCKETTE 	*inblock;
LOGICAL_RECORD 	*inrec;                               
{
       int i; 
             
       SetupBlockette(inblock,SEED_ZEROS_POLES,SEEDVER);
       CKP(PackChar(inblock,inzero->Function_Type,TRUE));
       CKP(PackInteger(inblock,(LONG)inzero->Cascade,(LONG)2,FALSE));
       CKP(PackUnit(inblock,inzero->Response_In));      
       CKP(PackUnit(inblock,inzero->Response_Out));    
       CKP(PackExponent(inblock,(VOLFLT)inzero->A0_Normalization,(LONG)12));
       CKP(PackExponent(inblock,(VOLFLT)inzero->Norm_Frequency,(LONG)12)); 
       CKP(PackInteger(inblock,(LONG)inzero->Complex_Zeros,(LONG)3,FALSE));
       for(i=0; i<inzero->Complex_Zeros; i++) {              
               CKP(PackExponent(inblock,(VOLFLT)inzero->Zero_Real_List[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)inzero->Zero_Imag_List[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)inzero->Zero_Real_Errors[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)inzero->Zero_Imag_Errors[i],(LONG)12));
       }   
       CKP(PackInteger(inblock,(LONG)inzero->Complex_Poles,(LONG)3,FALSE));
       for(i=0; i<inzero->Complex_Poles; i++) {              
               CKP(PackExponent(inblock,(VOLFLT)inzero->Pole_Real_List[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)inzero->Pole_Imag_List[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)inzero->Pole_Real_Errors[i],(LONG)12));
               CKP(PackExponent(inblock,(VOLFLT)inzero->Pole_Imag_Errors[i],(LONG)12));
       }   
       CKP(RecordPack(inblock,inrec)); 
                                      
       return(TRUE);                 
}                                   
                                   
/*                                
 *     Write a coefficients blockette 
 */                                 
                                   
CoefBlockette(incoef, inblock,inrec)                        /*  54  */ 
COEFFICIENTS *incoef;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrec;                               
{                              
       int i;                
                            
       SetupBlockette(inblock,SEED_COEFFICIENTS,SEEDVER);    
       CKP(PackChar(inblock,incoef->Function_Type,TRUE));  
       CKP(PackInteger(inblock,(LONG)incoef->Cascade,(LONG)2,FALSE)); 
       CKP(PackUnit(inblock,incoef->Response_In));       
       CKP(PackUnit(inblock,incoef->Response_Out));     
       CKP(PackInteger(inblock,(LONG)incoef->Numerators,(LONG)4,FALSE));  
                                                             
       for(i=0; i<incoef->Numerators; i++) {                
               CKP(PackExponent(inblock,(VOLFLT)incoef->Numer_Coeff[i],(LONG)12)); 
               CKP(PackExponent(inblock,(VOLFLT)incoef->Numer_Error[i],(LONG)12));
       }                                                           
       CKP(PackInteger(inblock,(LONG)incoef->Denominators,(LONG)4,FALSE));   
                                                                
       for(i=0; i<incoef->Denominators; i++) {                 
               CKP(PackExponent(inblock,(VOLFLT)incoef->Denom_Coeff[i],(LONG)12)); 
               CKP(PackExponent(inblock,(VOLFLT)incoef->Denom_Error[i],(LONG)12));
       }                                                           
       CKP(RecordPack(inblock,inrec));                           
                                                               
       return(TRUE);                                            
}                                                             
                                                             
/*                                                          
 *     Write out whichever response is appropriate        
 */
  
RespBlockette(inresp, inblock,inrec)                               
RESPONSE *inresp;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrec;                               
{
       switch (inresp->type) {     
      
       case RESP_SENSITIVITY:     
               return(SensBlockette(inresp->ptr.SENS,inblock,inrec));
       case RESP_ZEROS_POLES:                                       
               return(ZerosBlockette(inresp->ptr.PZ,inblock,inrec));
       case RESP_COEFFICIENTS:                                     
               return(CoefBlockette(inresp->ptr.CO,inblock,inrec));
       case RESP_DECIMATION:                                      
               return(DeciBlockette(inresp->ptr.DM,inblock,inrec)); 
       case RESP_RESP_LIST:                                        
     /*  case RESP_GENERIC:   */                                  
       default:                                                  
               return(FALSE);  /* Not yet implemented */        
       }                                                       
}   
   
/*
 *     Write comment entry blockette entry out       
 */                                                
                                                  
ComBlockette(incom,inblock, inrec, station)            /*  51 or 59 */
COMMENT_ENTRY *incom;
BLOCKETTE *inblock;
LOGICAL_RECORD *inrec;
BOOL station;                                     
{  
       int i;
            
       SetupBlockette(inblock,
               station?SEED_STATION_COMMENT:SEED_CHANNEL_COMMENT,SEEDVER);     
       CKP(PackTime(inblock,incom->Start_Comment));
       CKP(PackTime(inblock,incom->End_Comment));  
       CKP(PackComment(inblock,incom->Comment));  
       CKP(PackInteger(inblock,(LONG)incom->Comment_Level,(LONG)6,FALSE));
       CKP(RecordPack(inblock,inrec));                      
       return(TRUE);                                      
}


VOID TimeSpanBlockette (inblock, inrecord, flag, time_span )  /*  70  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
UBYTE		flag;
TIME_SPAN	*time_span;
{
	SetupBlockette(inblock,SEED_TIME_SPAN_HEADER,SEEDVER);
	PackChar(inblock, flag, TRUE );
	PackTime(inblock,time_span->Start_Time);
	PackTime(inblock,time_span->End_Time);
	RecordPack(inblock,inrecord);
}


VOID HypoCenterInfo (inblock, inrecord ,evinfo)          /*  71  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
EVENT_ENTRY	*evinfo;
{
	int i;

	SetupBlockette(inblock,SEED_HYPOCENTER,SEEDVER);
	PackTime(inblock,evinfo->Origin_Time);
	PackInteger(inblock,(LONG)evinfo->Info_Source->Source_ID,(LONG)2,FALSE );
	PackFixed(inblock,(VOLFLT)evinfo->Lattitude,(LONG)10,(LONG)6,TRUE);
	PackFixed(inblock,(VOLFLT)evinfo->Longitude,(LONG)11,(LONG)6,TRUE);
	PackFixed(inblock,(VOLFLT)evinfo->Depth,(LONG)7,(LONG)2,TRUE);
	PackInteger(inblock,(LONG)evinfo->Number_Magnitudes,(LONG)2,FALSE);

	for ( i=0; i<evinfo->Number_Magnitudes; i++ ) {
		PackFixed(inblock,(VOLFLT)evinfo->Magnitude[i],(LONG)5,(LONG)2,FALSE);
		PackVariable(inblock,evinfo->Magnitude_Type[i],(LONG)5,FALSE);
             /* Suppose cited source for magnitude is the same as for 
                the event info.                                      */

		PackInteger(inblock,(LONG)evinfo->Info_Source->Source_ID,(LONG)2,FALSE);
	}
	RecordPack(inblock,inrecord);
}

VOID TimeSpanDataIndexBlockette ( inblock, inrecord, stinfo , chinfo)  /*  73  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
STATION_ENTRY	*stinfo;
CHANNEL_ENTRY	*chinfo;
{
	int ndp=1, sub=0;
	int recnum = 20;   /* dummy value ! */
	char spaces[6];

	strcpy(spaces,"     ");
	SetupBlockette(inblock,SEED_TIME_SPAN_INDEX,SEEDVER);

/* Loop over datapieces should start here  */
	PackInteger(inblock,(LONG)ndp,(LONG)4,FALSE);
	PackString(inblock,stinfo->Call_Letters,(ULONG)5,TRUE);
	PackString(inblock,chinfo->Location,(ULONG)2,TRUE);
	PackString(inblock,chinfo->Identifier,(ULONG)3,TRUE);
	PackTime(inblock,chinfo->Root_Data->Start_Time);
	PackInteger(inblock,(LONG)recnum,(LONG)6,FALSE);
	PackInteger(inblock,(LONG)sub,(LONG)2,FALSE);
/* Loop over datapieces should stop here  */

/* Trailing record */
	PackString(inblock,spaces,(ULONG)5,TRUE);
	PackString(inblock,spaces,(ULONG)2,TRUE);
	PackString(inblock,spaces,(ULONG)3,TRUE);
	PackTime(inblock,chinfo->Root_Data->Start_Time);
	PackInteger(inblock,(LONG)recnum,(LONG)6,FALSE);
	PackInteger(inblock,(LONG)sub,(LONG)2,FALSE);
	
	RecordPack(inblock,inrecord);
}


VOID TimeSeriesIndex ( inblock, inrecord, stinfo, chinfo)    /*  74  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
STATION_ENTRY	*stinfo;
CHANNEL_ENTRY	*chinfo;
{
	int sub=0;
	int recnum=20;
	int accel=0;

	SetupBlockette(inblock,SEED_TIME_SERIES_INDEX,SEEDVER);
	PackString(inblock,stinfo->Call_Letters,(ULONG)5,TRUE);
	PackString(inblock,chinfo->Location,(ULONG)2,TRUE);
	PackString(inblock,chinfo->Identifier,(ULONG)3,TRUE);
	PackTime(inblock,chinfo->Root_Data->Start_Time);
	PackInteger(inblock,(LONG)recnum,(LONG)6,FALSE);
	PackInteger(inblock,(LONG)sub,(LONG)2,FALSE);
	PackTime(inblock,chinfo->Root_Data->End_Time);
	PackInteger(inblock,(LONG)recnum,(LONG)6,FALSE);
	PackInteger(inblock,(LONG)sub,(LONG)2,FALSE);
	PackInteger(inblock,(LONG)accel,(LONG)6,FALSE);
	RecordPack(inblock,inrecord);

}


VOID TimeSeriesIndexM ( inblock, inrecord, bl74)    /*  74  */
BLOCKETTE	*inblock;
LOGICAL_RECORD	*inrecord;
BLOCK74         *bl74;
{

	SetupBlockette(inblock,SEED_TIME_SERIES_INDEX,SEEDVER);
	PackString(inblock,bl74->Station_ID,(ULONG)5,TRUE);
	PackString(inblock,bl74->Location_ID,(ULONG)2,TRUE);
	PackString(inblock,bl74->Channel_ID,(ULONG)3,TRUE);
	PackTime(inblock,bl74->Start_Time);
	PackInteger(inblock,(LONG)bl74->Updated_Start_Sequence_No,(LONG)6,FALSE);
	PackInteger(inblock,(LONG)bl74->Start_Sub_Sequence_No,(LONG)2,FALSE);
	PackTime(inblock,bl74->End_Time);
	PackInteger(inblock,(LONG)bl74->Updated_End_Sequence_No,(LONG)6,FALSE);
	PackInteger(inblock,(LONG)bl74->End_Sub_Sequence_No,(LONG)2,FALSE);
	PackInteger(inblock,(LONG)bl74->Accelerators,(LONG)6,FALSE);

	RecordPack(inblock,inrecord);

}
