/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include <stdio.h>
#include "conver.h"
#include "seed.h"
#define  FOREVER for(;;)

#define  LOOP_OVER_STATIONS  for(stinfo=VI->Root_Station;stinfo!=NULL;stinfo=stinfo->Next)
#define  LOOP_OVER_CHANNELS  for(chinfo=stinfo->Root_Channel;chinfo!=NULL;chinfo=chinfo->Next)
#define  LOOP_OVER_RESPONSES for(resp=chinfo->Root_Response;resp!=NULL;resp=resp->Next)
#define LOOP_OVER_TIME_ENTRIES   for(time_entry=VI->Root_Time_Entry;\
                                     time_entry!=NULL; time_entry=time_entry->Next)
#define LOOP_OVER_BLOCK74  for(bl74=time_entry->Root_Block74; bl74 != NULL;\
                                                  bl74=bl74->Next)

INT         INP_FILE_NO;
FILE 	    *outfile;
UBYTE	    Input_File[30];
INT         LRECL_IN, LRECL;
BYTE 	    option;
extern BOOL MERGE_FILES;
extern BOOL TIME_HEADER_DONE;   /* defined in seed_read.c */
extern FILE *uf;
extern LONG DREC;
extern int NODIR, CD_YEARS;
extern char *CD_year[];
extern int SET_BL74;

extern LOC_KEY_ID  *UI[];

parse_seedfile()
{
	STDTIME		*stdtime;

	stdtime = (STDTIME *) malloc(sizeof(STDTIME) );
	InitializeVolume ( *stdtime, *stdtime );
	EXTRACT = FALSE;

        UnpackFile();
        repair_seed();


}

StoreRec( buf, rlen)
UBYTE *buf;
UWORD rlen;
{
	int n;

	n = fwrite ( buf, sizeof(char), rlen, outfile );
	if ( n != rlen ) printf("RecBuff not written to file !\n");
}

STDTIME parsetimeX ( inbuf )
UBYTE	*inbuf;
{
	STDTIME time;
	UBYTE   buf[10];
	INT	len;


	time.year = 0;
	time.day = 0;
	time.hour = 0;
	time.minute = 0;
	time.second = 0;
	time.msec = 0;

	len = strlen(inbuf);

	switch ( len ) {
		case 4: 
	                sscanf(inbuf,"%d~",&time.year );
                        break;
		case 8: 
	                sscanf(inbuf,"%d,%d~",&time.year,&time.day );
                        break;
		case 11: 
	                sscanf(inbuf,"%d,%d,%d~", &time.year,&time.day,&time.hour );
                        break;
		case 14: 
	                sscanf(inbuf,"%d,%d,%d:%d~", &time.year,&time.day,
				&time.hour,&time.minute );
                        break;
		case 18: 
	                sscanf(inbuf,"%d,%d,%d:%d:%d~", &time.year,&time.day,
				&time.hour,&time.minute,&time.second );
                        break;
		default: 
	                sscanf(inbuf,"%d,%d,%d:%d:%d.%d~", &time.year,&time.day,
				&time.hour,&time.minute,&time.second,&time.msec );
                        break;
	}
	return(time);
}



PrintEventInfo (ev) 
EVENT_ENTRY *ev;
{
    int i;

    printf("\t origin time: ");
    PrintTime(ev->Origin_Time);
    printf("\t lattitude: %lf",ev->Lattitude);
    printf("\n\t longitude: %lf",ev->Longitude);
    printf("\n\t depth: %lf\n",ev->Depth);
    for(i=0;i<ev->Number_Magnitudes;i++) {
	printf("\t    type: %s  magnitude: %lf \n",ev->Magnitude_Type[i],ev->Magnitude[i]);
    }
    printf("\n");
}


UnpackFile()
{
	BOOL		(*readrtn)();
	INT		N, ii, lrecl;
        char            buf[31],v[3];
	UWORD		creexp, maxblocke;
	LOGICAL_RECORD	*log_rec_ptr;
	BLOCKETTE	*blockette;
	STDTIME		*stdtime;
        TIME_ENTRY      *time_entry;
	BLOCK74         *bl74;
        int p, len, year0,i;

        
        if ( (fread(buf,sizeof(char),30,uf)) != 30 ) {
              printf("File not able for reading.\n");
              exit(1);
        }
        v[0] = buf[19];
        v[1] = buf[20];
        v[2]='\0';
        sscanf(v,"%d",&len);

        if ( len <9 || len > 12 ) {
             printf("Could not find logical record size.\n");
             printf("Give log.rec size: ");
             scanf("%d",&LRECL_IN);
        }
        else LRECL_IN = (int)( pow((double)2,(double)len));

        fseek(uf,(long)0,0);
        N = LRECL_IN;

	creexp = 1;
	while ( N/2 != 1 ) {
		N /= 2;
		creexp++;
	}
        maxblocke = creexp;

	FIRST_INPUT_FILE = TRUE;

        ii=0;

        log_rec_ptr = (LOGICAL_RECORD *) RecordReadBuff (creexp, maxblocke,readrtn);
        blockette = (BLOCKETTE *) CreateBlockette();

        INP_FILE_NO = ii;
        UI[ii] = (LOC_KEY_ID *) InitKey(ii);

        do {
	        ParseBlockettes ( log_rec_ptr->RecType, log_rec_ptr, blockette ) ;
	        printf("RecNum for %c %ld\n",log_rec_ptr->RecType, log_rec_ptr->RecNum);
        } while ( !TIME_HEADER_DONE );

        /*printf("RecNum for 'D': %ld\n",log_rec_ptr->RecNum);*/


        LRECL = LRECL_IN;

        if ( SET_BL74 == TRUE ) {
              create_set_74 ( log_rec_ptr->RecNum );
        }

        free(log_rec_ptr);
        free(blockette);
        TIME_HEADER_DONE = FALSE;       /*   Next file  !  */

        /* This part is needed for the case there is no directory structure
           on the CDROM (or the input is one seed file ), e.g. Geoscope.
         */

        if ( NODIR == TRUE ) {
             year0 = 0;
             CD_YEARS = 0;
             i = 0;

             LOOP_OVER_TIME_ENTRIES  { 
                  if ( time_entry->Time_Span_ID->Start_Time.year != year0 ) {
                             CD_year[i] = (char *)malloc(5);
                             sprintf(CD_year[i],"%4d",time_entry->Time_Span_ID->Start_Time.year);
printf("CDYEAR: %s\n",CD_year[i]);
                             CD_YEARS++;
                             i++;
                             year0 = time_entry->Time_Span_ID->Start_Time.year;
                  }

/*
                  printf("time_entry: (%d) %d\n",time_entry->Time_Span_ID->Start_Time.year,time_entry->Time_Span_ID->Start_Time.day);

                  LOOP_OVER_BLOCK74   {
                     printf("show74 stat (%5s) (%3s) (%d) start_time: (%d)(%d) %d:%d:%d.%d  (%d) end_time: (%d)(%d) %d:%d:%d.%d\n",
bl74->Station_ID,bl74->Channel_ID,bl74->Start_Sequence_No,bl74->Start_Time.year,bl74->Start_Time.day,bl74->Start_Time.hour,bl74->Start_Time.minute,bl74->Start_Time.second,bl74->Start_Time.msec,bl74->End_Sequence_No,bl74->End_Time.year,bl74->End_Time.day,bl74->End_Time.hour,bl74->End_Time.minute,bl74->End_Time.second,bl74->End_Time.msec);
            
		 }
*/
             }
         }
}


STDTIME *UpdateStartTime ( Time, timevar )
STDTIME *Time, timevar;
{

    if ( Time->year > timevar.year) {
         Time = &timevar;
	 return(Time);
    }
    else if ( Time->year < timevar.year ) {
         return(Time);
    }
    else if ( Time->year == timevar.year ) {
         if ( Time->day > timevar.day) {
               Time = &timevar;
               return(Time);
         }
         else if (Time->day < timevar.day ) {
               return(Time);
	 }
	 else if ( Time->day == timevar.day ) {
              if ( Time->hour > timevar.hour ) {
                    Time = &timevar;
	            return(Time);
              }
              else if ( Time->hour < timevar.hour ) {
                    return(Time);
	      }
              else if ( Time->hour == timevar.hour ) {
                   if ( Time->minute > timevar.minute ) {
                         Time = &timevar;
                         return(Time);
                   }
                   else if ( Time->minute < timevar.minute ) {
                        return(Time);
                   }
                   else if ( Time->minute == timevar.minute ) {
                        if ( Time->second > timevar.second ) {
                              Time = &timevar;
		              return(Time);
                        }
                        else if ( Time->second < timevar.second ) {
                              return(Time);
                        }
                        else if ( Time->second == timevar.second ) {
                             if ( Time->msec > timevar.msec ) {
                                   Time = &timevar;
                                   return(Time);
                             }
                             else if ( Time->msec < timevar.msec ) {
                                   return(Time);
                             }
                             else return(Time);
                        }
                   }
              }
         }
    }
}


STDTIME *UpdateEndTime ( Time, timevar )
STDTIME *Time, timevar;
{
    if ( Time->year < timevar.year) {
         Time = &timevar;
	 return(Time);
    }
    else if ( Time->year > timevar.year ) {
         return(Time);
    }
    else if ( Time->year == timevar.year ) {
         if ( Time->day < timevar.day) {
               Time = &timevar;
               return(Time);
         }
         else if (Time->day > timevar.day ) {
               return(Time);
	 }
	 else if ( Time->day == timevar.day ) {
              if ( Time->hour < timevar.hour ) {
                    Time = &timevar;
	            return(Time);
              }
              else if ( Time->hour > timevar.hour ) {
                    return(Time);
	      }
              else if ( Time->hour == timevar.hour ) {
                   if ( Time->minute < timevar.minute ) {
                         Time = &timevar;
                         return(Time);
                   }
                   else if ( Time->minute > timevar.minute ) {
                        return(Time);
                   }
                   else if ( Time->minute == timevar.minute ) {
                        if ( Time->second < timevar.second ) {
                              Time = &timevar;
		              return(Time);
                        }
                        else if ( Time->second > timevar.second ) {
                              return(Time);
                        }
                        else if ( Time->second == timevar.second ) {
                             if ( Time->msec < timevar.msec ) {
                                   Time = &timevar;
                                   return(Time);
                             }
                             else if ( Time->msec > timevar.msec ) {
                                   return(Time);
                             }
                             else return(Time);
                        }
                   }
              }
         }
    }
}


create_set_74 ( drec )
long int drec;
{

        int             N, ii, Nfiles, lrecl,p, recno, lrecno,n;
        int             num_samples_old, nbl74;
        int             last_year, last_day, last_hour, last_minute;
        int             last_seconds, last_frac_sec, NEWTR;
        UWORD           creexp, maxblocke;
        LOGICAL_RECORD  *log_rec_ptr;
        BLOCKETTE       *blockette;
        STDTIME         *stdtime;
        TIME_ENTRY      *time_entry;
        BLOCK74         *abl74[300], *nextbl74, *lastbl74, *bl74;
        ABBREV          *looper;
        float           Ts;
        double          tis, tis_old, sr;
        char            clrecno[7], rectype, last_rectype;
        char            buf[4096], stid[6], chid[4], locid[4];
        char            stid_old[6], chid_old[4];


	struct s_id {
                char station[5];            /* Station id                     */
                char location[2];           /* Location id                    */
                char channel[3];            /* Channel id                     */
                char res[2];                /* reserved */
        };

	struct s_time {
               unsigned short int year;     /* year (ie 1987)                 */
               unsigned short int day;      /* day of the year (Jan 1, is 1)  */
               char hour;                   /* hour of the day (0-23)         */
               char minute;                 /* minute of day (0-59)           */
               char seconds;                /* seconds of day (0-59,60 for    */
                                            /*                 leap seconds)  */
               char unused;                 /* get on word boundry            */
               unsigned short int frac_sec; /* .0001 seconds (0-9999)         */
	};

	struct s_data {
        unsigned short int num_samples;    /* Number of samples in record    */
        short int sample_rate;             /* Sample rate factor             */
        short int sample_rate_multiplier;  /* Sample rate multiplier factor  */
        /* this gives an bus alignment error */
        /* long int flags;      */         /* all the flags together         */
        unsigned char  f1;
        unsigned char  f2;
        unsigned char  f3;
        unsigned char  f4;
        /* this gives an bus alignment error */
        /* long int num_frac_sec_corr; */  /* Number of .0001 sec time       */
        short int t1;
        short int t2;
        unsigned short int begin_data;     /* Byte number of beginning of    */
                                           /*   correction (first byte of    */
                                           /*   rec is 0)                    */
        unsigned short int begin_blockette;/* Byte number of beginning of    */
                                           /*   first blockette (0 if none)  */

	};

	struct s_id     *did;
	struct s_time   *dtime;
	struct s_data   *ddata;
 

        NEWTR=FALSE;

        LOOP_OVER_TIME_ENTRIES {
                  printf("remove bl74 for time entry: %d\n",time_entry->Time_Span_ID->Start_Time.day);

                  bl74 = time_entry->Root_Block74;
                  while(bl74 != NULL ) {
                             nextbl74 = bl74->Next;
                             free(bl74);
                             bl74 = nextbl74;
	          }
		  time_entry->Root_Block74 = time_entry->Tail_Block74 = 
                                                          (BLOCK74 *) NULL;
        }

	strcpy(stid_old,"   ");
	strcpy(chid_old,"   ");
        nbl74=1;
        num_samples_old = 0;
        tis_old = 0;

        fseek(uf,(long)(((int) drec  -1)*LRECL_IN), 0);

        while ((n=fread(buf,sizeof(*buf),LRECL,uf)) > 0 ) {

	      n=sscanf(buf,"%6c%c",clrecno,&rectype);
              lrecno= atoi(clrecno);
              if(rectype!='D') {
                         if( last_rectype == 'D') NEWTR =TRUE;
                         else continue;
              }
              last_rectype = rectype;

              did =   (struct s_id *) (buf+8);
              dtime = (struct s_time *) (buf+12+8);
              ddata = (struct s_data *) (buf+22+8);
 
	      sscanf(did,"%5c%2c%3c", stid,locid,chid);
                     
              tis = (double) dtime->day * 86400 +
                    (double) dtime->hour * 3600 +
                    (double) dtime->minute * 60 +
                    (double) dtime->seconds +
                    (double) dtime->frac_sec/10000;

              if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier > 0)
                       sr = ddata->sample_rate * ddata->sample_rate_multiplier;
              if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier < 0)
                       sr = (-1) * ddata->sample_rate / ddata->sample_rate_multiplier;
              if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier > 0)
                       sr = (-1) * ddata->sample_rate_multiplier/ (float)(ddata->sample_rate);
              if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier < 0)
                       sr = 1./( (float)(ddata->sample_rate)/ ( (float)ddata->sample_rate_multiplier) );

             /* printf("(%5s,%3s) recno:%d %d:%d:%02d:%02d.%04d (ns:%d) sr=%f tis=%lf\n", 
                          stid,chid,lrecno,dtime->day,dtime->hour,dtime->minute,
                            dtime->seconds,dtime->frac_sec, ddata->num_samples,sr,tis);
*/

              Ts = ((float)num_samples_old) / sr;

/*
              if ( fabs(tis_old + Ts - tis )> 0.01 ) NEWTR = TRUE;
*/
              if ( fabs(tis_old + Ts - tis )> 1 ) NEWTR = TRUE;


             /* check here for new station, channel or trigger */

              if ( strncmp(stid,stid_old,strlen(stid)) != 0 || 
                   strncmp(chid,chid_old,strlen(chid)) != 0 ||
                   NEWTR == TRUE ) {
                       printf("time correction and clock error not included !!!\n");

	               printf("(NEWTR) stid=(%5s) chid=(%3s)  recno = %d  %d:%d:%02d:%02d.%04d (%d)  \n",
                                stid,chid,lrecno,dtime->day,dtime->hour,dtime->minute,
                                dtime->seconds,dtime->frac_sec, ddata->num_samples);
                       /* printf("tis_old = %lf Ts = %f  tis=  %lf nos_old=%d D=%lf\n",
                                  tis_old,Ts,tis,num_samples_old,(tis-tis_old)); */


		       NEWTR = FALSE;

                       /* first store end time and end sequence number */

                       if(nbl74>1) {
                                 lastbl74->End_Time.year = last_year;
                                 lastbl74->End_Time.day = last_day;
                                 lastbl74->End_Time.hour = last_hour;
                                 lastbl74->End_Time.minute = last_minute;
                                 lastbl74->End_Time.second = last_seconds;
                                 lastbl74->End_Time.msec = last_frac_sec;
                 
	                         lastbl74->End_Sequence_No = lrecno-1;
	                         lastbl74->End_Sub_Sequence_No = 1;
	                         lastbl74->Updated_End_Sequence_No = lrecno-1;

                                 /* break out of loop after update of last bl74 */
                                 if(strncmp(stid,"    ",4)==0) {
                                        printf("^^^^^^^^^^^^stid=(%s)\n");break;
                                 }
                                 if(rectype==' ' && last_rectype == 'D') break;
                       }

                       /* then create a new bl 74 */

                       abl74[nbl74] = (BLOCK74 *) InitBlock74();
               
                       strncpy(abl74[nbl74]->Station_ID,stid,5);
                       abl74[nbl74]->Station_ID[5] = '\0';
	               strncpy(abl74[nbl74]->Location_ID,locid,2);
                       abl74[nbl74]->Location_ID[5] = '\2';
                       strncpy(abl74[nbl74]->Channel_ID,chid,3);
                       abl74[nbl74]->Channel_ID[3] = '\0';

                       abl74[nbl74]->Start_Time.year = dtime->year;
                       abl74[nbl74]->Start_Time.day = dtime->day;
                       abl74[nbl74]->Start_Time.hour = dtime->hour;
                       abl74[nbl74]->Start_Time.minute = dtime->minute;
                       abl74[nbl74]->Start_Time.second = dtime->seconds;
                       abl74[nbl74]->Start_Time.msec = dtime->frac_sec;

	               abl74[nbl74]->Start_Sequence_No = lrecno;
	               abl74[nbl74]->Start_Sub_Sequence_No = 1;
	               abl74[nbl74]->Updated_Start_Sequence_No = lrecno;

               
		       abl74[nbl74]->Accelerators = 0;
	               strcpy(abl74[nbl74]->File_Name,Input_File);
		       abl74[nbl74]->Next = NULL;

                       LOOP_OVER_TIME_ENTRIES {
/*
printf("LINKIN bl74[nbl74] (%5s) (%3s) (%d) start_time: %d:%02d:%02d.%04d  (%d) end_time: %d:%02d:%02d.%04d\n",
abl74[nbl74]->Station_ID,abl74[nbl74]->Channel_ID,abl74[nbl74]->Start_Sequence_No,abl74[nbl74]->Start_Time.hour,abl74[nbl74]->Start_Time.minute,abl74[nbl74]->Start_Time.second,abl74[nbl74]->Start_Time.msec,abl74[nbl74]->End_Sequence_No,abl74[nbl74]->End_Time.hour,abl74[nbl74]->End_Time.minute,abl74[nbl74]->End_Time.second,abl74[nbl74]->End_Time.msec);
*/
                                    LinkInBlock74(abl74[nbl74]);
/*
printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
show_bl74() ;
printf("++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
*/
                                    lastbl74 = (BLOCK74 *) abl74[nbl74];
                      }
                      nbl74++;

                      if(nbl74>299) {
                           printf("Too many bl 74.\n");
                           exit(1);
                      }
            }



                      /* Save values of last record */

		      tis_old = tis;
                      num_samples_old = ddata->num_samples;

                      strcpy ( stid_old, stid );
                      strcpy ( chid_old, chid );

/* Update this time with number of samples in last block */
/*  dtime += Ts */
                      tis = (double) dtime->day * 86400 +
                            (double) dtime->hour * 3600 +
                            (double) dtime->minute * 60 +
                            (double) dtime->seconds +
                            (double) dtime->frac_sec/10000;

                      if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier > 0)
                          sr = ddata->sample_rate * ddata->sample_rate_multiplier;
                      if (ddata->sample_rate > 0 && ddata->sample_rate_multiplier < 0)
                          sr = (-1) * ddata->sample_rate / ddata->sample_rate_multiplier;
                      if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier > 0)
                          sr = (-1) * ddata->sample_rate_multiplier/ (float)(ddata->sample_rate);
                      if (ddata->sample_rate < 0 && ddata->sample_rate_multiplier < 0)
                          sr = 1./( (float)(ddata->sample_rate)/ ( (float)ddata->sample_rate_multiplier) );

                      Ts = (float)((float)(ddata->num_samples-1)) / sr;
/* printf("tis = %lf  Ts = %lf\n",tis,(double)Ts); */
                      tis += (double)Ts;
/* printf("tis = %lf\n",tis); */

                      dtime->day = 0; 
                      dtime->hour = 0; 
                      dtime->minute = 0; 
                      dtime->seconds = 0; 
                      dtime->frac_sec = 0;

	             while ( tis > 86400 ) {
		             dtime->day += 1;
		             tis -= 86400;
	             }
	             while ( tis > 3600 ) {
		             dtime->hour += 1;
		             tis -= 3600;
	             }
	             while ( tis > 60 ) {
		             dtime->minute += 1;
		             tis -= 60;
	             }

                     dtime->seconds = (int)floor(tis);
                     dtime->frac_sec = (int)(fmod(tis,(double)1)*10000 +0.0001);

		      last_year = dtime->year;
		      last_day = dtime->day;
		      last_hour = dtime->hour;
		      last_minute = dtime->minute;
		      last_seconds = dtime->seconds;
		      last_frac_sec = dtime->frac_sec;
/*
               printf("lasttime  %d:%d:%d:%02d.%02d (%04d)\n",
                         dtime->day,dtime->hour,dtime->minute,
                         dtime->seconds,dtime->frac_sec, ddata->num_samples);
*/

    }
}
