/******************************************************************************/
/****                                                                       ***/
/****       Written by:       Reinoud Sleeman                               ***/
/****                                                                       ***/
/****                                                                       ***/
/****       Orfeus Data Center                                              ***/
/****                                                                       ***/
/****       Budapestlaan 4                                                  ***/
/****       3584 CD  Utrecht                                                ***/
/****       The Netherlands                                                 ***/
/****                                                                       ***/
/****                                                                       ***/
/****       21/02/92           Version 1.9            CDLOOK                ***/
/****                                                                       ***/
/******************************************************************************/
#include "user.h"
#include "sac.h"
#include "icon_bitmap1"

#define fmax(a,b) if(b>a) a=b
#define fmin(a,b) if(b<a) a=b

extern int                NDP, GDSN, SEED;
extern int                DTYP;
extern int                *xdata[];
extern float              *ydata[];
extern int                NCH;
extern int                nst_selected;
extern int                stbuf[];
extern int                pch;
extern STATION_STRUCT     *stinfo[];
extern int                cursor_shape;
extern char               Orientation, BandCode;
extern float              upper_freq, lower_freq, corner_freq;
extern evinfo             event[];
extern int                selevt;
extern OLsunic1_base_window_objects *bfr;



int            screen;
unsigned long  bg, fg;

int                   X0, Y0, chi, SELS, X1, Y1, CHI, XT1, XT2;
int                   MULTIPLEXED;
float                 min_y[MAX_STATIONS][6], max_y[MAX_STATIONS][6];
float                 val, AMAX[4], SCALEY[4], xev_conv, PLS;
extern STDTIME2       pltime;
extern double         TPb, TPe;


char strlst();

void write_sac ( st, ch, tb, np, p1)
int st, ch, np, p1;
double tb;
{
	FILE *outfile;
	char outfile_name[100], channel[10];	
	struct sac sach;		
	int i, j, hsec, hmsec, ch2;
	char character[8+1], stcode[6];
	int ascii;	
	int binary;
	double ort;

	/* select ASCII or binary output (binary chosen here) */
	ascii = FALSE;
	binary = !ascii;

/*                 +=======================================+                 */
/*=================|          Build the SAC header         |=================*/
/*                 +=======================================+                 */

	/* initialize to all nulls */
	sach = sac_null;


	if (np == 0) {
		fprintf(stderr,"Output Data Contains No Samples\n");
		return;
	}

        switch ( DTYP ) {
                 case 1:
			strcpy ( channel, "S");
                        break;
                 case 2:
			strcpy ( channel, "L");
                        break;
                 case 3:
			strcpy ( channel, "I");
                        break;
                 case 4:
			strcpy ( channel, "H");
                        break;
                 case 5:
			strcpy ( channel, "B");
                        break;
                 case 6:
			strcpy ( channel, "M");
                        break;
                 case 7:
			strcpy ( channel, "L");
                        break;
                 case 8:
			strcpy ( channel, "V");
                        break;
                 case 9:
			strcpy ( channel, "U");
                        break;

                 default:
			strcpy ( channel, "__");
        }

        if ( ch == 0 ) strcat ( channel,"HZ");
        if ( ch == 1 ) strcat ( channel,"HN");
        if ( ch == 2 ) strcat ( channel,"HE");

	/* give values to SAC-required variables */
        if(MULTIPLEXED) ch2 = 0;
        else ch2 = ch;

        if(GDSN) sach.delta = stinfo[st]->data_entry[DTYP]->dt;
        if(SEED) sach.delta = stinfo[st]->seed_entry[DTYP][ch2]->dt;

        ort = (double) event[selevt].time.hour * 3600 +
              (double) event[selevt].time.minute * 60 +
              (double) event[selevt].time.second +
              (double) event[selevt].time.msec/1000;

        while(tb>86400) tb -= 86400; 

	sach.b = (float)(tb-ort);
	sach.e = (float)((double) np - 1) * ((double) sach.delta);
        sach.e += sach.b;

	sach.npts = np;
	sach.iftype = ITIME;			/* known a priori */
	sach.leven = TRUE;			/* known a priori */

	/* record start time */
	sach.nzyear = event[selevt].time.year;
        sach.nzjday = julian_day (event[selevt].month, event[selevt].day_of_month);

	sach.nzhour = event[selevt].time.hour;
	sach.nzmin  = event[selevt].time.minute;
        sach.nzsec =  event[selevt].time.second;
        sach.nzmsec = event[selevt].time.msec;

	sach.iztype = IO;      /* EVENT ORIGIN TIME */
              
	/* station and component names */
	sprintf (character, "%-8.8s", stinfo[st]->station_code);
	strncpy (sach.kstnm, character, 8);
	sprintf (character, "%-8.8s", channel);
	strncpy (sach.kcmpnm, character, 8);

	/* event info */
        sach.evla = event[selevt].lat;
        sach.evlo = event[selevt].lon;
        sach.evdp = event[selevt].dep;

	/* give values available from SEED tables to SAC optional variables */
	sach.stla = stinfo[st]->flat;
	sach.stlo = stinfo[st]->flon;
	sach.stel = stinfo[st]->felev;
	sach.stdp = 0;

	/* Sac dip, azimuth definition is not equal to Seed definition */
        if ( channel[2] == 'Z' ) {
                  sach.cmpaz = 0.0;
                  sach.cmpinc = 0.0;
        }
        if ( channel[2] == 'N' ) {
                  sach.cmpaz = 0.0;
                  sach.cmpinc = 90.0;
        }
        if ( channel[2] == 'E' ) {
                  sach.cmpaz = 90.0;
                  sach.cmpinc = 90.0;
        }


	/* mysterious LLL-set values */
	sach.internal1 = 2.0;
	sach.internal4 = 6;
	sach.internal5 = 0;
	sach.internal6 = 0;
	sach.unused27 = FALSE;

	/* not sure if these are needed, but they might be. Values might be wrong */
	sach.lpspol = FALSE;
	sach.lcalda = TRUE;

/*                 +=======================================+                 */
/*=================|  build name for and open output file  |=================*/
/*                 +=======================================+                 */

	/* 1988.023.15.34.08.2800.ANMO.SPZ is the filename for a seismogram from
	   year 1988, Julian day 23, 15:34:08.2800 UT, recorded at station ANMO
	   from component SPZ. */

        j=0;
        for(i=0;i<strlen(stinfo[st]->station_code);i++) {
                if(stinfo[st]->station_code[i] == ' ') continue;
                stcode[j++] = stinfo[st]->station_code[i];
        }
        stcode[j] = '\0';

        hsec = (int)floor((double)pltime.second);
        hmsec = (int)(fmod((double)pltime.second,(double)1)*1000);

	sprintf (outfile_name, "%04d.%03d.%02d.%02d.%02d.%03d.%s.%s",
		pltime.year,
		pltime.day,
		pltime.hour,
		pltime.minute,
		hsec,
                hmsec,
                stcode,
		channel);

	if ((outfile = fopen (outfile_name, "w")) == NULL)
	{
		fprintf (stderr, "\tWARNING (output_data):  ");
		fprintf (stderr, "Output file %s is not available for writing.\n",
			outfile_name);
		fprintf (stderr, "\tExecution continuing.\n");
		return;
	}


/*                 +=======================================+                 */
/*=================|       write a SAC binary file         |=================*/
/*                 +=======================================+                 */

	if (binary)
	{
		/* describe the file being written */
		fprintf (stderr, "Writing %s, %s, %5d samples (binary),",
			stinfo[st]->station_code,
			channel,
			np);
		fprintf (stderr, " starting %04d,%03d %02d:%02d:%02d:%03d UT\n",
			pltime.year,
			pltime.day,
			pltime.hour,
			pltime.minute,
			hsec,
                        hmsec);

		/* write the SAC header */

		if (fwrite (&sach, sizeof (struct sac), 1, outfile) != 1)
		{
			fprintf (stderr, "WARNING (write_sac):  ");
			fprintf (stderr, "failed to write SAC header to %s.\n",
				outfile_name);
			fprintf (stderr, "\tExecution continuing.\n");
		}

		/* write the SAC data */

                for ( i=p1; i< (p1+np) ; i++ ) {
		  fwrite (&ydata[ch][i], sizeof (float), 1, outfile) ;
                }
	}

	fclose (outfile);



}

void 
sac_loop (wtb,wte )
double wtb, wte;                /*  wtb, wte :  time window boundaries  */
{
	STDTIME           *sec_to_time( );

	double 	          XP1, XP2, x_offset, XPb, XPe;
	float             c[3], d[3], dt;
	int               p1,   tr, Ntr, loop_counter;
        int               stc, S;
	int               ch,  np, pt;
	int               av, r1, npt, sp1[3],snp[3];
	char              loc_orientation;
	
	void              draw_plot_window (), decode_data();
	int               data_type_avail_for_station();

        xv_set ( bfr->base_window, 
                 FRAME_SHOW_FOOTER, TRUE,
                 FRAME_LEFT_FOOTER, " ",
                 FRAME_RIGHT_FOOTER, "Writing SAC files .....",
                 NULL);


        XPb = 1e15;
        XPe = -1e15;


        /* Loop over selected station(s):
         *   -  determine which elements in xdata and ydata are in interval
         *      wtb, wte.  XP1 and XP2 are time boundaries of actual plot.
         *   -  determine max. epicentral distance.          
         *   -  read the data, decode them and determine max. and min. value
         */

        for ( stc = 0; stc < nst_selected; stc++ ) { 

	      XP1 = wtb;
	      XP2 = wte;

              if ( GDSN ) {

                  /* See if selected station has selected data type. */

	          if ( data_type_avail_for_station(stbuf[stc]) == FALSE ) 
                        continue;
                  if(pch>0 && stinfo[stbuf[stc]]->number_channels[DTYP] == 1) 
                        continue;

                  find_data_points ( stc, wtb, wte, &p1, &np, &XP1, &XP2 ,(int)0 );
                  if(np < 0 ) continue;

                  if ( XP1 < XPb ) XPb = XP1;
                  if ( XP2 > XPe ) XPe = XP2;

	      }

	      if ( SEED ) {

                 /* Check fotr multiplexed data */
                 MULTIPLEXED = check_for_multiplex ( stbuf[stc], BandCode );

                 if(Orientation == 'A' ) loop_counter = 3;
                 else                    loop_counter = 1;
      
                 while ( loop_counter >= 1 ) {

                     pt = channel_order ( loop_counter, &loc_orientation );

                     /* Do loop only once for multiplexed data */
                     if(MULTIPLEXED == TRUE ) {
                          pt=0;
                          loop_counter=0;
                     }

                     if ( data_type_avail_for_station(stbuf[stc],pt) == FALSE ) {
                                loop_counter--;
                                continue;
                     }

                     find_data_points ( stc, wtb, wte, &sp1[pt], &snp[pt], &XP1, &XP2, pt );

                     if ( XP1 < XPb ) XPb = XP1;
                     if ( XP2 > XPe ) XPe = XP2;
                     loop_counter--;
                 }
              }

        }


	/* Loop over selected stations to write the data */

        for ( stc=0; stc<nst_selected; stc++) {

	      XP1 = wtb;
	      XP2 = wte;
              S=stbuf[stc];

              if ( GDSN ) {

	           if ( data_type_avail_for_station(stbuf[stc]) == FALSE ) 
                                    continue;
                   if(pch>0 && stinfo[stbuf[stc]]->number_channels[DTYP] == 1) 
                                    continue;

                   find_data_points ( stc, wtb, wte, &p1, &np, &XP1, &XP2 ,(int)0);
                   if(np < 0 ) continue;

	           read_data (stbuf[stc]);
                   decode_data (stbuf[stc]);

               }

               if ( SEED ) {

                 if(Orientation == 'A' ) loop_counter = 3;
                 else                    loop_counter = 1;

                 /* Check fotr multiplexed data */
                 MULTIPLEXED = check_for_multiplex ( stbuf[stc], BandCode );
      
                 while ( loop_counter >= 1 ) {

                      pt = channel_order ( loop_counter, &loc_orientation );

                      if(MULTIPLEXED==TRUE) {
                          pt=0;
                          loop_counter=0;
                      }

                      if ( data_type_avail_for_station(stbuf[stc],pt) == FALSE ) {
                                loop_counter--;
                                continue;
                      }

                      find_data_points ( stc, wtb, wte, &sp1[pt], &snp[pt], &XP1, &XP2, pt );

                      loop_counter--;
                 }

                 if ( read_seed_data ( stbuf[stc] ) == FALSE ) continue;
                 av = decode_seed_data ( stbuf[stc] );
                 if ( av == FALSE ) continue;

               }

	       for ( ch=0; ch < NCH; ch++) {

                    chi = (NCH==1) ? pch:ch;

                    if(SEED) {
                          if(MULTIPLEXED==TRUE) {
                                    chi=0;
                                    p1=sp1[0];
                                    np=snp[0];
                          }
                          else {
                                    p1=sp1[chi];
                                    np=snp[chi];
                          }
                          if(np<0) continue;
                          
                    }

                    if(GDSN) {
		         dt  = stinfo[S]->data_entry[DTYP]->dt;
		         Ntr = stinfo[S]->data_entry[DTYP]->Ntriggers;
                    }
                    if ( SEED ){
		         dt  = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->dt;
		         Ntr = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->Ntriggers;
                    }

                    if ( BP_FILTER == TRUE ) {

                         /* Check nyquist frequency */

/*
                         nyquist ( dt, &upper_freq, stc );
                         nyquist ( dt, &lower_freq, stc );
*/

                         bp_filter_coeff ( upper_freq, lower_freq, c, d, dt );
                         bp_filter_data ( ydata[chi]+p1, np, c, d);
                    }
                    if ( HP_FILTER == TRUE ) {

/*
                         nyquist ( dt, &corner_freq, stc );
*/

                         hp_filter_coeff ( corner_freq, c, d, dt );
                         hp_filter_data ( ydata[chi]+p1, np, c, d);
                    }

                    chi = (NCH==1) ? pch:ch;
                    get_point_limits ( ydata[chi]+p1, np, &min_y[stc][chi], &max_y[stc][chi]);

                    if ( min_y[stc][chi] == 0 &&  max_y[stc][chi] == 0 ) {
                         max_y[stc][chi] = 1;
                         min_y[stc][chi] = -1;
                    }

		    /* NP = (int)((XPe-XPb)/dt) -1; */


		    for(tr=1;tr<=Ntr;tr++) {

if(GDSN) {
			if(stinfo[S]->data_entry[DTYP]->trigger[tr]->bool == FALSE ) 
                                                                    continue;
			x_offset = (double) ((( stinfo[S]->data_entry[DTYP]->trigger[tr]->t1)));

			r1 = stinfo[S]->data_entry[DTYP]->trigger[tr]->R1;
			npt = stinfo[S]->data_entry[DTYP]->trigger[tr]->npoints;
}
if(SEED) {
                        if(MULTIPLEXED) chi=0;
			if(stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->bool == FALSE ) 
                                                                    continue;
			x_offset = (double) ((( stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->t1)));

			r1 = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->R1;
			npt = stinfo[stbuf[stc]]->seed_entry[DTYP][chi]->trigger[tr]->npoints;
}

                    chi = (NCH==1) ? pch:ch;
			 write_sac (  S, chi, x_offset, npt,  r1 ); 
		   }
 
             } /* end of loop over stations */


	}

}


