/*
 * This file is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify this file without charge, but are not authorized to
 * license or distribute it to anyone else except as part of a product
 * or program developed by the user.
 * 
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * This file is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS FILE
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even
 * if Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#ifndef lint
static char     sccsid[] = "@(#)guide.c	1.28 90/07/09 Copyright 1989 Sun Microsystems";
#endif

#include "guide.h"

/*
 * Table of attribute name strings indexed by attribute.  The entries in this
 * table must match G_ATTRS.
 */
static char    *attr_string[] = {
	":background-color",
	":button-type",
	":choices",
	":choice-label-types",
	":choice-colors",
	":columns",
	":constant-width",
	":done-handler",
	":drawing-model",
	":events",
	":event-handler",
	":foreground-color",
	":height",
	":help",
	":horizontal-scrollbar",
	":icon-file",
	":icon-mask-file",
	":label",
	":label-type",
	":label-bold",
	":layout-type",
	":mapped",
	":max-value",
	":menu-handler",
	":menu-item-colors",
	":menu-item-defaults",
	":menu-item-handlers",
	":menu-item-labels",
	":menu-item-label-types",
	":menu-item-menus",
	":menu",
	":menu-title",
	":menu-type",
	":min-value",
	":multiple-selections",
	":name",
	":notify-handler",
	":orientation",
	":owner",
	":pinnable",
	":pinned",
	":read-only",
	":repaint-proc",
	":resizable",
	":rows",
	":scrollable-height",
	":scrollable-width",
	":selection-required",
	":setting-type",
	":show-border",
	":show-endboxes",
	":show-footer",
	":show-range",
	":show-value",
	":slider-width",
	":stored-length",
	":text-type",
	":ticks",
	":type",
	":user-data",
	":value-length",
	":value-x",
	":value-y",
	":vertical-scrollbar",
	":width",
	":x",
	":y",
	0,
};

/*
 * Table of button name strings indexed by button type.  The entries in this
 * table must match G_BUTTON_TYPES.
 */
static char    *button_type_string[] = {
	":normal",
	":abbreviated",
	0,
};

/*
 * Table of drawing model strings indexed by drawing model.  The entries in
 * this table must match G_DRAWING_MODELS.
 */
static char    *drawing_model_string[] = {
	":xview",
	":xwindows",
	":postscript",
	0,
};

/*
 * Table of event name strings indexed by event type.  The entries in this
 * table must match G_EVENT_TYPES.
 */
static char    *event_type_string[] = {
	":keyboard",
	":keyboard-left",
	":keyboard-right",
	":keyboard-top",
	":mouse",
	":mouse-drag",
	":mouse-enter",
	":mouse-exit",
	":mouse-move",
	0,
};

/*
 * Table of label name strings indexed by layout type.  The entries in this
 * table must match G_LABEL_TYPES.
 */
static char    *label_type_string[] = {
	":string",
	":glyph",
	0,
};

/*
 * Table of menu name strings indexed by menu type.  The entries in this
 * table must match G_MENU_TYPES.
 */
static char    *menu_type_string[] = {
	":command",
	":exclusive",
	":nonexclusive",
	0,
};

/*
 * Table of layout name strings indexed by attribute.  The entries in this
 * table must match G_LAYOUT_TYPE.
 */
static char    *layout_type_string[] = {
	":horizontal",
	":vertical",
	0,
};

/*
 * Setting types.
 */
static char    *setting_type_string[] = {
	":exclusive",
	":nonexclusive",
	":check",
	":stack",
	0,
};

/*
 * Text types.
 */
static char    *text_type_string[] = {
	":alphanumeric",
	":numeric",
	0,
};

/*
 * Table of object type name strings indexed by object type.  The entries in
 * this table must match G_TYPES.
 */
static char    *type_string[] = {
	":base-window",
	":button",
	":canvas-pane",
	":control-area",
	":gauge",
	":menu",
	":message",
	":popup-window",
	":scrolling-list",
	":setting",
	":slider",
	":term-pane",
	":text-field",
	":text-pane",
	0,
};

/*
 * Internal procedure declarations.
 */
int             lookup();

/*
 * Return a list of object attributes for the specified object type.
 */
G_ATTRS        *
G_object_attrs(type)
	G_TYPES         type;
{
	switch (type) {
	case G_BASE_WINDOW: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_WIDTH,
			G_HEIGHT,
			G_BACKGROUND_COLOR,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_MAPPED,
			G_SHOW_FOOTER,
			G_RESIZABLE,
			G_ICON,
			G_ICON_MASK,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}
		
	case G_GAUGE: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_VALUE_X,
			G_VALUE_Y,
			G_SLIDER_WIDTH,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_LAYOUT_TYPE,
			G_ORIENTATION,
			G_SHOW_RANGE,
			G_MIN_VALUE,
			G_MAX_VALUE,
			G_TICKS,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}

	case G_MESSAGE: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_LABEL_BOLD,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}

	case G_BUTTON: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_CONSTANT_WIDTH,
			G_BUTTON_TYPE,
			G_WIDTH,
			G_HEIGHT,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_MENU_NAME,
			G_NOTIFY_HANDLER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}
		
	case G_CANVAS_PANE: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_BACKGROUND_COLOR,
			G_FOREGROUND_COLOR,
			G_MENU_NAME,
			G_HSCROLL,
			G_SCROLLABLE_WIDTH,
			G_VSCROLL,
			G_SCROLLABLE_HEIGHT,
			G_REPAINT_PROC,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_DRAWING_MODEL,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}

	case G_CONTROL_AREA: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_BACKGROUND_COLOR,
			G_FOREGROUND_COLOR,
			G_SHOW_BORDER,
			G_MENU_NAME,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		return attrs;
	}

	case G_MENU: {
		static G_ATTRS	attrs[] = {
			G_TYPE,
			G_NAME,
			G_HELP,
			G_COLUMNS,
			G_LABEL,
			G_LABEL_TYPE,
			G_MENU_TYPE,
			G_MENU_HANDLER,
			G_MENU_TITLE,
			G_MENU_ITEM_LABELS,
			G_MENU_ITEM_LABEL_TYPES,
			G_MENU_ITEM_DEFAULTS,
			G_MENU_ITEM_HANDLERS,
			G_MENU_ITEM_MENUS,
			G_MENU_ITEM_COLORS,
			G_PINNABLE,		/* must follow G_MENU_ITEM_* */
			G_USER_DATA,
			-1,
		};
		return attrs;
	}

	case G_SCROLLING_LIST: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_LAYOUT_TYPE,
			G_ROWS,
			G_READ_ONLY,
			G_MULTIPLE_SELECTIONS,
			G_SELECTION_REQUIRED,
			G_MENU_NAME,
			G_NOTIFY_HANDLER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}

	case G_TERM_PANE: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_BACKGROUND_COLOR,
			G_FOREGROUND_COLOR,
			G_SHOW_BORDER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		return attrs;
	}

	case G_TEXT_PANE: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_BACKGROUND_COLOR,
			G_FOREGROUND_COLOR,
			G_SHOW_BORDER,
			G_READ_ONLY,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		return attrs;
	}

	case G_POPUP_WINDOW: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_WIDTH,
			G_HEIGHT,
			G_BACKGROUND_COLOR,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_MAPPED,
			G_SHOW_FOOTER,
			G_RESIZABLE,
			G_PINNED,
			G_DONE_HANDLER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}

	case G_SETTING: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_VALUE_X,
			G_VALUE_Y,
			G_LAYOUT_TYPE,
			G_FOREGROUND_COLOR,
			G_SETTING_TYPE,
			G_ROWS,
			G_COLUMNS,
			G_LABEL,
			G_LABEL_TYPE,
			G_NOTIFY_HANDLER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_CHOICES,
			G_CHOICE_LABEL_TYPES,
			G_CHOICE_COLORS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}
		
	case G_SLIDER: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_VALUE_X,
			G_VALUE_Y,
			G_SLIDER_WIDTH,
			G_FOREGROUND_COLOR,
			G_LABEL,
			G_LABEL_TYPE,
			G_LAYOUT_TYPE,
			G_ORIENTATION,
			G_SHOW_ENDBOXES,
			G_SHOW_RANGE,
			G_SHOW_VALUE,
			G_MIN_VALUE,
			G_MAX_VALUE,
			G_TICKS,
			G_NOTIFY_HANDLER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}

	case G_TEXT_FIELD: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_X,
			G_Y,
			G_WIDTH,
			G_HEIGHT,
			G_FOREGROUND_COLOR,
			G_TEXT_TYPE,
			G_LABEL,
			G_LABEL_TYPE,
			G_VALUE_X,
			G_VALUE_Y,
			G_LAYOUT_TYPE,
			G_VALUE_LENGTH,
			G_STORED_LENGTH,
			G_MAX_VALUE,
			G_MIN_VALUE,
			G_READ_ONLY,
			G_NOTIFY_HANDLER,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}
		
	default: {
		static G_ATTRS  attrs[] = {
			G_TYPE,
			G_NAME,
			G_OWNER,
			G_HELP,
			G_EVENT_HANDLER,
			G_EVENTS,
			G_USER_DATA,
			-1,
		};
		
		return attrs;
	}
	}
}

/*
 * Change an attribute into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_attr_to_string(attr)
	G_ATTRS         attr;
{
	return attr_string[attr];
}

/*
 * Change a string into an attribute.  Returns the attribute number if
 * successful, otherwise -1.
 */
G_ATTRS
G_string_to_attr(s)
	char           *s;
{
	return lookup(s, attr_string);
}

/*
 * Change a button type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_button_type_to_string(val)
	G_BUTTON_TYPES   val;
{
	return button_type_string[val];
}

/*
 * Change a string into a button type.
 */
G_BUTTON_TYPES
G_string_to_button_type(s)
	char           *s;
{
	return lookup(s, button_type_string);
}

/*
 * Change a drawing model into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_drawing_model_to_string(drawing_model)
	G_DRAWING_MODELS	drawing_model;
{
	return drawing_model_string[drawing_model];
}

/*
 * Change a string into an attribute.  Returns the attribute number if
 * successful, otherwise -1.
 */
G_DRAWING_MODELS
G_string_to_drawing_model(s)
	char           *s;
{
	return lookup(s, drawing_model_string);
}

/*
 * Change an event type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_event_type_to_string(event)
	G_EVENT_TYPES	event;
{
	return event_type_string[event];
}

/*
 * Change a string into an attribute.  Returns the attribute number if
 * successful, otherwise -1.
 */
G_EVENT_TYPES
G_string_to_event_type(s)
	char           *s;
{
	return lookup(s, event_type_string);
}

/*
 * Change a label type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_label_type_to_string(val)
	G_LABEL_TYPES   val;
{
	return label_type_string[val];
}

/*
 * Change a string into a label type.
 */
G_LABEL_TYPES
G_string_to_label_type(s)
	char           *s;
{
	return lookup(s, label_type_string);
}

/*
 * Change a layout type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_layout_type_to_string(val)
	G_LAYOUT_TYPES  val;
{
	return layout_type_string[val];
}

/*
 * Change a string into a layout type.
 */
G_LAYOUT_TYPES
G_string_to_layout_type(s)
	char           *s;
{
	return lookup(s, layout_type_string);
}

/*
 * Change a menu type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_menu_type_to_string(val)
	G_MENU_TYPES   val;
{
	return menu_type_string[val];
}

/*
 * Change a string into a menu type.
 */
G_MENU_TYPES
G_string_to_menu_type(s)
	char           *s;
{
	return lookup(s, menu_type_string);
}

/*
 * Change a setting type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_setting_type_to_string(val)
	G_SETTING_TYPES val;
{
	return setting_type_string[val];
}

/*
 * Change a string into a setting type.
 */
G_SETTING_TYPES
G_string_to_setting_type(s)
	char           *s;
{
	return lookup(s, setting_type_string);
}

/*
 * Change a setting type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_text_type_to_string(val)
	G_TEXT_TYPES val;
{
	return text_type_string[val];
}

/*
 * Change a string into a text type.
 */
G_TEXT_TYPES
G_string_to_text_type(s)
	char           *s;
{
	return lookup(s, text_type_string);
}

/*
 * Change a type into a string.  Returns a string.
 * 
 * REMIND: error checking.
 */
char           *
G_type_to_string(type)
	G_TYPES         type;
{
	return type_string[type];
}

/*
 * Change a string into a type.  Returns the type number if successful,
 * otherwise -1.
 */
G_TYPES
G_string_to_type(s)
	char           *s;
{
	return lookup(s, type_string);
}

/*
 * Search for a string in the specified table.  Returns the table index if
 * successful, otherwise -1.
 */
static int
lookup(s, tab)
	char           *s;
	char          **tab;
{
	int             i;

	for (i = 0; tab[i]; i++)
		if (strcmp(s, tab[i]) == 0)
			return i;
	return -1;
}
