/***    Module:    DISPDRIV
  
DISPDRIV contains the device dependent display routines for
the Color Graphics Adapter (CGA), the Enhanced Graphics Adapter (EGA),
and the Professional Graphics Adapter (PGA).  The global variables
that define the device are allocated and initialized here.
  
***/
#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <memory.h>
#include <malloc.h>
#include "menu.h"          /* included for integers FALSE and TRUE  */
#include "sonic.h"
  
#define MAXDISPNS  640     /* The maximum size of all of the displays */
#define MAXDISPNL  480
  
  
  
extern unsigned char getmem (int, int);
extern int putmem (int, int, int);
extern int REFRESH_FREE;  /* defined in opencd.c  */
  
#if defined(NOASM)
/*  int PGAsend (unsigned char *,int ); */
int WritePixelEGA (int ,int ,int );
int DisplayLineEGA (unsigned char *,int ,int ,int );
int ClearDisplayEGA (int );
#else
/* extern int PGAsend (unsigned char *,int );*/
extern int WritePixelEGA (int ,int ,int );
extern int DisplayLineEGA (unsigned char *,int ,int ,int );
extern int ClearDisplayEGA (int );
#endif
  
  
/* int PGAreceive (unsigned char *,int *);
*/
  
  
int    DisplayDevice, dispnl, dispns, numDN, numshades, OneScreen;
/* DisplayDevice indicates the type of display (CGA=0, EGA=1, PGA=2).
dispnl is the number of lines in the display.
dispns is the number of samples in the display.
numDN  is the number of unique pixel values (e.g. 4 bits -> numDN=16).
numshades is the number of shades each of red, green, and blue
(e.g.  numshades=16 gives 16**3 = 4096 possible colors).
OneScreem is true if there is only one screen, i.e. there is not
image screen and a text screen.
*/
  
  
struct  Color {
    unsigned char  r, g, b;
} DefaultPalette[16], PaletteTable[16];
  
/* This is the structure for the color palette.
The current palette is stored in PaletteTable.
For PGA extend arraysize to 256
*/
  
  
  
/*Refresh buffer stuff */
int RefreshLines;
unsigned int  segment;
unsigned char far *RefreshBuf[MAXDISPNL];
  
  
  
/* #if defined(NOASM)
void PGAsend (command, len)
int  len;
unsigned char  command[];
{
  
unsigned char   i, OutWritePtr, OutReadPtr, numbytes;
  
do {
OutWritePtr = getmem (0x0C600,0x0300);
OutReadPtr = getmem (0x0C600,0x0301);
numbytes = OutReadPtr - OutWritePtr - 1;
} while (numbytes < len);
for (i = 0; i < len; i++)
putmem (0x0C600, OutWritePtr++, command[i]);
  
putmem (0x0C600, 0x0300, OutWritePtr);
}
#endif
*/
  
  
/*
void PGAreceive (answer, p_len)
int   *p_len;
unsigned char  answer[];
{
unsigned char   i, InWritePtr, InReadPtr, numbytes;
  
do {
InReadPtr  = getmem (0x0C600,0x0303);
InWritePtr = getmem (0x0C600,0x0302);
numbytes = InWritePtr - InReadPtr;
} while (numbytes == 0);
for (i = 0; i < numbytes; i++)
answer[i] = getmem (0x0C610,InReadPtr++);
  
putmem (0x0C600, 0x0303, InReadPtr);
*p_len = numbytes;
}
*/
  
  
  
void DisplayOn()
  
/***  DisplayOn figures out which display device is on the system and
then initializes for that device.  The device is turned on.
The global variables that define the display characteristics
and the default color palette are initialized.  The refresh
buffer is zeroed.
***/
{
    int   i;
    unsigned char   cmd[7];
    unsigned char  buffer[MAXDISPNS];
    union  REGS  inreg, outreg;
    struct SREGS segregs;
    struct Color DP[16];
  
    DisplayDevice = 0;
  
    /* Test for Professional Graphics Adapter (PGA) */
    if (DisplayDevice == 0) {
        putmem (0x0C000, 0x63DB, 0x0AD);
        if (getmem (0x0C000, 0x63DB) == 0x0AD)
            DisplayDevice = 2;
    }
  
    /* Test for Enhanced Graphics Adapter (EGA) */
    if (DisplayDevice == 0) {
        inreg.x.ax = 0x1200;
        inreg.x.bx = 0x0FF10;
        inreg.x.cx = 0x000F;
        int86 (0x10, &inreg, &outreg);
        if ( (outreg.h.cl < 0x0C) && (outreg.h.bh <= 1) && (outreg.h.bl <= 3) )
            DisplayDevice = 1;
    }
  
  
  
  
    /* Initialize for the appropriate device */
    switch (DisplayDevice) {
        case 0 :
            dispnl = 200;
            dispns = 640;
            numDN = 2;
            numshades = 2;
            OneScreen = 1;
            inreg.x.ax = 0x0006;  /* turn on CGA */
            int86 (0x10, &inreg, &outreg);
            DefaultPalette[0].r = 0;
            DefaultPalette[1].r = 255;
            DefaultPalette[0].g = 0;
            DefaultPalette[1].g = 255;
            DefaultPalette[0].b = 0;
            DefaultPalette[1].b = 255;
            break;
  
        case 1 :
            dispnl = 350;
            dispns = 640;
            numDN = 16;
            numshades = 4;
            OneScreen = 1;
            inreg.x.ax = 0x0010;  /* turn on EGA */
            int86 (0x10, &inreg, &outreg);
            inreg.x.ax = 0x0500;  /* select page */
            int86 (0x10, &inreg, &outreg);
  
            DP[0].r  = 0;              DP[0].g  = 0;              DP[0].b  = 0;
            DP[1].r  = 64;             DP[1].g  = 0;              DP[1].b  = 0;
            DP[2].r  = 128;            DP[2].g  = 0;              DP[2].b  = 0;
            DP[3].r  = 192;            DP[3].g  = 0;              DP[3].b  = 0;
            DP[4].r  = 192;            DP[4].g  = 64;             DP[4].b  = 0;
            DP[5].r  = 192;            DP[5].g  = 128;            DP[5].b  = 0;
            DP[6].r  = 192;            DP[6].g  = 192;            DP[6].b  = 0;
            DP[7].r  = 128;            DP[7].g  = 192;            DP[7].b  = 0;
            DP[8].r  = 0;              DP[8].g  = 192;            DP[8].b  = 0;
            DP[9].r  = 0;              DP[9].g  = 192;            DP[9].b  = 128;
            DP[10].r = 0;              DP[10].g = 128;            DP[10].b = 128;
            DP[11].r = 0;              DP[11].g = 128;            DP[11].b = 192;
            DP[12].r = 0;              DP[12].g = 0;              DP[12].b = 192;
            DP[13].r = 128;            DP[13].g = 0;              DP[13].b = 192;
            DP[14].r = 192;            DP[14].g = 0;              DP[14].b = 192;
            DP[15].r = 192;            DP[15].g = 192;            DP[15].b = 192;
  
            for (i = 0; i <= 15; i++)
            {
                DefaultPalette[i].r = DP[i].r;
                DefaultPalette[i].g = DP[i].g;
                DefaultPalette[i].b = DP[i].b;
            }
            break;
  
            /*  Due to memory problems full PGA support has been omitted  */
  
        case 2:
            dispnl = 480;
            dispns = 640;
            /*  change to 256  */
            numDN = 16;
            numshades = 16;
            OneScreen = 1;
  
            cmd[0] = 0x43; cmd[1] = 0x58; cmd[2] = 0x20;
            cmd[3] = 0x0D0; cmd[4] = 0x00;
            cmd[5] = 0x0F; cmd[6] = 0x00;
            PGAsend (cmd, 7);      /* 'CA DISPLA 0  CLEARS 0 '  */
  
            /* full PGA: change 15 to 255     */
  
            for (i = 0; i <= 15; i++) {
                DefaultPalette[i].r = i;
                DefaultPalette[i].g = i;
                DefaultPalette[i].b = i;
            }
            break;
    }
  
    segread (&segregs);
    segment = segregs.ds;
  
    i = 0;
    if ( REFRESH_FREE ) {
        while (( (RefreshBuf[i] = _fmalloc(dispns)) != NULL) && (i < dispnl) )
            i++;
        REFRESH_FREE = FALSE;
    }
    else {
        printf ("WARNING:  dispns not free; this can cause allocation errors !!\n");
        pause(10);
    }
  
    RefreshLines = i;
    memset (buffer, 0, dispns);
    for (i = 0;  i < RefreshLines;  i++) {
        movedata (segment, (unsigned int) buffer,
        FP_SEG(RefreshBuf[i]), FP_OFF(RefreshBuf[i]), dispns);
    }
}
  
  
void DisplayOff()
  
/***  DisplayOff turns off the display device returning the screen
to text mode.
***/
{
    union  REGS  inreg, outreg;
    int    i;
  
  
    switch (DisplayDevice) {
        case 0 :
            inreg.x.ax = 0x0002;
            int86 (0x10, &inreg, &outreg);
            break;
  
        case 1 :
            inreg.x.ax = 0x0002;
            int86 (0x10, &inreg, &outreg);
            break;
  
            /* case 2:
            cmd[0] = 0x0D0;   cmd[1] = 1;
            PGAsend (cmd, 2);
            break;
            */
    }
    OneScreen = 0;
  
    if ( ! REFRESH_FREE ) {
        for (i = 0;  i < RefreshLines;  i++)
            _ffree ( RefreshBuf[i]);
        REFRESH_FREE = TRUE;
    }
}
  
  
void WritePalette (coltab)
  
/* full PGA support: extend 16 to 256  */
  
struct Color  coltab[16];
{
    int   i;
    unsigned char  red, green, blue;
    union  REGS  inreg, outreg;
  
  
    for (i = 0; i < numDN; i++) {
        PaletteTable[i].r = coltab[i].r;
        PaletteTable[i].g = coltab[i].g;
        PaletteTable[i].b = coltab[i].b;
    }
  
    switch (DisplayDevice) {
        case 1 :
            for (i = 0; i <= 15; i++) {
                red = (coltab[i].r & 0x80) >> 5 | (coltab[i].r & 0x40) >> 1;
                green = (coltab[i].g & 0x80) >> 6 | (coltab[i].g & 0x40) >> 2;
                blue = (coltab[i].b & 0x80) >> 7 | (coltab[i].b & 0x40) >> 3;
                inreg.x.ax = 0x1000;
                inreg.x.bx = ((red | green | blue) << 8) | i;
                int86 (0x10, &inreg, &outreg);
            }
            return;
  
            /* case 2:
  
            for (i = 0; i <= 15; i++) {
            cmd[0] = 0x0EE;
            cmd[1] = i;
            cmd[2] = coltab[i].r >> 4;
            cmd[3] = coltab[i].g >> 4;
            cmd[4] = coltab[i].b >> 4;
            PGAsend (cmd, 5);
            }
            */
            return;
    }
}
  
  
/*
#if defined(NOASM)
void WritePixelEGA (line, sample, DN)
int   line, sample, DN;
{
int   address, y, x;
  
outp (0x3CE, 5);
outp (0x3CF, 2);
y = line - 1;
x = sample - 1;
address = (y << 6) + (y << 4) + (x >> 3);
outp (0x3CE, 8);
outp (0x3CF, 0x80 >> (x & 7));
getmem (0x0A000, address);
putmem (0x0A000, address, DN);
outp (0x3CE, 5);
outp (0x3CF, 0);
outp (0x3CE, 8);
outp (0x3CF, 0xFF);
}
#endif
*/
  
void WritePixel (line, sample, DN)
  
/***  WritePixel writes a pixel on the display screen.
Parameter   type       description
line     integer     The line coordinate of the pixel
sample   integer     The sample coordinate of the pixel
DN       integer     The DN value of the pixel
***/
  
int    line, sample, DN;
{
    int           address, bit, mask;
  
    switch (DisplayDevice) {
        case 0 :
            line--;
            sample--;
            bit = 7 - (sample & 7);
            DN = (DN & 1) << bit;
            mask = ~(1 << bit);
  
            if ((line & 1) == 0)
                address = 0x8000 + 40*line + (sample >> 3);
            else
                address = 0x0A000 + 40*(line-1) + (sample >> 3);
            putmem (0x0B000, address, (getmem(0xB000,address) & mask) | DN);
            return;
  
        case 1 :
            WritePixelEGA (line, sample, DN);
            return;
  
            /* case 2:
            x = sample - 1;
            y = 480 - line;
            cmd[0] = 0x0D9;
            cmd[1] = y;  cmd[2] = y >> 8;
            cmd[3] = x;  cmd[4] = x >> 8;
            cmd[5] = x;  cmd[6] = x >> 8;
            cmd[7] = 0;       cmd[8] = DN;
            PGAsend (cmd, 9);
            return;
            */
    }
}
  
  
void ReadPixel (line, sample, p_DN)
  
/***  ReadPixel read a pixel value from the display screen.
Parameter   type       description
line     integer     The line coordinate of the pixel
sample   integer     The sample coordinate of the pixel
p_DN     int ptr     The DN value of the pixel is returned
***/
  
int   line, sample, *p_DN;
{
    int   address, y, x, mask, bit;
  
    /* PGA: change 16 to 256 */
  
    switch (DisplayDevice) {
        case 0 :
            line--;
            sample--;
            bit = 7 - (sample & 7);
  
            if ((line & 1) == 0)
                address = 0x8000 + 40*line + (sample >> 3);
            else
                address = 0x0A000 + 40*(line-1) + (sample >> 3);
  
            *p_DN = (getmem(0x0B000,address) & (1 << bit)) >> bit;
            return;
  
        case 1 :
            y = line - 1;
            x = sample - 1;
            address = (y << 6) + (y << 4) + (x >> 3);
            mask = 0x80 >> (x & 7);
            *p_DN = 0;
            for (bit = 3;  bit >= 0;  bit--) {
                outp (0x3CE, 4);
                outp (0x3CF, bit);
                if ( (getmem (0x0A000,address) & mask) > 0 )
                    *p_DN = (*p_DN << 1) | 1;
                else
                    *p_DN = *p_DN << 1;
            }
            return;
  
            /*  case 2:
            x = sample - 1;     y = 480 - line;
            cmd[0] = 0x0D8;
            cmd[1] = y;  cmd[2] = y >> 8;
            cmd[3] = x;  cmd[4] = x >> 8;
            cmd[5] = x;  cmd[6] = x >> 8;
            PGAsend (cmd, 7);
            for (x = 0; x < 1000; x++) ;
            PGAreceive (cmd, &len);
            *p_DN = cmd[8];
            return;
            */
    }
}
  
/*
  
#if defined(NOASM)
void DisplayLineEGA (buffer, line, samp, ns)
  
int    line, samp, ns;
unsigned char  buffer[];
{
int   address, i;
unsigned char   mask;
  
outp (0x3CE, 5);
outp (0x3CF, 2);
address = 80*(line-1) + ((samp-1) >> 3);
mask = 0x80 >> ((samp-1) & 7);
  
for (i = 0; i < ns; i++) {
outp (0x3CE, 8);
outp (0x3CF, mask);
getmem (0x0A000, address);
putmem (0x0A000, address, buffer[i]);
mask = mask >> 1;
if (mask == 0) {
address++;
mask = 0x80;
}
}
  
outp (0x3CE, 5);
outp (0x3CF, 0);
outp (0x3CE, 8);
outp (0x3CF, 0x0FF);
}
#endif
*/
  
void DisplayLine (buffer, line, sample, ns)
  
/***  DisplayLine writes a line of pixels on the display screen.
Parameter   type       description
buffer   char ptr    The array of pixel values
line     integer     The line coordinate of the first pixel
sample   integer     The sample coordinate of the first pixel
ns       integer     The number of pixels to display
***/
  
int    line, sample, ns;
unsigned char  buffer[];
{
    int            i, address, mask, bit;
  
    switch (DisplayDevice) {
        case 0 :
            line--;
            sample--;
            if ((line & 1) == 0)
                address = 0x8000 + 40*line + (sample >> 3);
            else
                address = 0x0A000 + 40*(line-1) + (sample >> 3);
            bit = 7 - (sample & 7);
            mask = getmem(0x0B000,address) & ~((1 << (bit+1)) - 1);
            for (i = 0;  i < ns;  i++) {
                mask |= (buffer[i] & 1) << bit;
                bit--;
                if (bit < 0) {
                    putmem(0x0B000,address, mask);
                    address++;
                    bit = 7;
                    mask = 0;
                }
            }
            putmem(0x0B000,address,
            mask | (getmem(0x0B000,address) & ((1 << (bit+1)) - 1)) );
            return;
  
        case 1 :
            DisplayLineEGA (buffer, line, sample, ns);
            return;
  
            /*  case 2:
            y = 480 - line;  x1 = sample - 1;  x2 = x1 + ns - 1;
            cmd[0] = 0x0D9;
            cmd[1] = y;   cmd[2] = y >> 8;
            cmd[3] = x1;  cmd[4] = x1 >> 8;
            cmd[5] = x2;  cmd[6] = x2 >> 8;
            PGAsend (cmd, 7);
            s = 0;
            while (s < ns) {
            len = ns - s;   if (len > 128)  len = 128;
            cmd[0] = 127 + len;
            PGAsend (cmd, 1);
            PGAsend (&buffer[s], len);
            s += 128;
            }
            return;
            */
  
    }
}
  
  
#if defined(NOASM)
void ClearDisplayEGA (DN)
  
int     DN;
{
    int  address;
  
    outp (0x3CE, 5);
    outp (0x3CF, 2);
    outp (0x3CE, 8);
    outp (0x3CF, 0xFF);
  
    for (address = 0; address < 80*350;  address++) {
        getmem (0x0A000,address);
        putmem (0x0A000, address, DN);
    }
    outp (0x3CE, 5);
    outp (0x3CF, 0);
}
#endif
  
void ClearDisplay (DN)
  
/***  ClearDisplay sets the whole display to a particular value.
Parameter   type       description
DN       integer     The DN value to set (usually 0)
***/
  
int     DN;
{
    int  address, junk;
  
    switch (DisplayDevice) {
        case 0 :
            junk = 0x0FF*(DN & 1);
            for (address = 0; address < 8000;  address++) {
                putmem(0x0B800,address, junk);
                putmem(0x0BA00,address, junk);
            }
            return;
        case 1 :
            ClearDisplayEGA (DN);
            return;
            /*
            case 2:
            cmd[0] = 0x0F;  cmd[1] = DN;
            PGAsend (cmd, 2);
            return;
            */
    }
}
