/***************************************************************************/
/**                                                                       **/
/**                Module: MKPLOT - Plot the seismogram.                  **/
/**                                                                       **/
/***************************************************************************/
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "menu.h"
#include "struct.h"
#include "sonic.h"
  
#define  abs_max(a,b)  ( (fabs(a)) > (fabs(b)) ? (fabs(a)) : (fabs(b)) )
#define  fmax(a,b)      ( (a) > (b) ? (a) : (b) )
#define  fmin(a,b)      ( (a) < (b) ? (a) : (b) )
  
#define LINESIZE 81
#define TORAD    3.14159265359 / 180
  
extern    INT     nst;
extern  FLOAT     *plotbuffer;
extern    BYTE    *menu1[];                /*  Defined in module menu  */
extern    INT     plchoice;
extern    INT     TextLine;                /*  Defined in module disputil  */
extern    INT     TextSample;
extern    INT     DisplayDevice;           /*  Defined in module dispdriv  */
extern    INT     dispnl;
extern    INT     dispns;
extern    INT     PLOTBUF_FREE;
extern  INT     SHORT_PERIOD;
extern  INT     LONG_PERIOD;
extern  INT     INTERM_PERIOD;
extern  INT     comp;
extern  INT     numblocks;
extern  INT     DISPLAY_FLAG;
  
FILE      *id3, *id4, *id5;
int       nbl, nfirst, delpb, delpe, pos[MAXSTAT], nrec1, nrec2;
float     maxm[3], dsecb, dsece, plsec, maxvx, minvy, maxvy, dx,  angle_var;
float   upper_frq, lower_frq;
BOOL    BPFILTER, BPFILTPAR;
BOOL    HPFILTER, HPFILTPAR;
  
  
void CrtPlot  ( crp, rot, stnum, nchan, t0, t1, stinfo, evinfo )
BYTE    *stinfo, *evinfo;
INT     crp, rot, stnum, nchan;
FLOAT   *t0, *t1;
{
    INT   i, j, k, cmpmax, tmppos[MAXSTAT], rp;
    FLOAT dist, distmin, distmax, angle, tplbeg, tplend;
  
    switch ( plchoice ) {
  
        case 1:
  
            /*  Determine number of blocks to plot and maximum for time window  */
            rp = ReadPrm (crp, rot, stnum, t0, t1, &angle, &cmpmax);
  
            /*  Check if data are available  */
            if ( nbl == 0 ) {
                InitDisplay ();
                blankstring (menu1[0]);
                strcpy (menu1[0], " NO DATA FOR THIS DATA TYPE ");
                DrawText (menu1[0], (dispnl/10), (dispns/64), (dispnl/35),0,15);
                strcpy (menu1[0], " SELECT CONTINUE            ");
                DrawText (menu1[0], (dispnl/5), (dispns/64), (dispnl/35), 0,15);
                return;
            }
  
            /*  Turn on the display device and set color for background  */
            if ( !DISPLAY_FLAG )  InitDisplay ();
            /*       if ( cpl != 2 ) */   ClearDisplay(12);
  
            /*  Draw the title of the plot  */
            DrawTitle ( evinfo, stinfo, *t0 );
  
            /*  Draw the axes of the plot  */
            DrawAxes ( nchan, distmin, distmax, rot, angle, cmpmax );
  
            /*  Plot the data  */
            PlotRec ( stnum, nchan, dist, rot, angle );
  
            break;
  
        case 2:
  
            strset (stinfo, ' ');
            distmin = 180;
            distmax = 0;
            j = 0;
            nrec1 = nst;
  
            /*  Check if selected datatype is available. Make a second selection  */
            for ( i=0; i < nst; i++ )  {
  
                stnum = st[i].nums;
                if ( SHORT_PERIOD )    numblocks = st[stnum].len[0];
                if ( LONG_PERIOD  )    numblocks = st[stnum].len[1];
                if ( INTERM_PERIOD )   numblocks = st[stnum].len[2];
  
                if ( numblocks == 0 )          /*  Parameter nrec1 is number   */
                    nrec1 -= 1 ;              /*  of stations which recorded  */
                else                           /*  selected datatype           */
                    tmppos[j++] = st[i].nums;
            }
            if ( nrec1 == 0 ) {
                InitDisplay ();
                blankstring (menu1[0]);
                strcpy (menu1[0], " NO DATA FOR THIS DATA TYPE ");
                DrawText (menu1[0], (dispnl/10), (dispns/64), (dispnl/35),0,15);
                strcpy (menu1[0], " SELECT CONTINUE            ");
                DrawText (menu1[0], (dispnl/5), (dispns/64), (dispnl/35), 0,15);
                return;
            }
  
            /*  Determine distmin and distmax  */
            if ( nrec1 == 1 ) {
                distmin = st[ (tmppos[0]) ].epi;
                distmax = distmin ;
            }
            else {
                for ( i =0; i < nrec1; i++ ) {
                    distmin = fmin ( st[ (tmppos[i]) ].epi , distmin );
                    distmax = fmax ( st[ (tmppos[i]) ].epi , distmax );
                }
            }
  
            /*  Store values of t0 and t1 in tplbeg and tplend  */
            tplbeg = *t0;
            tplend = *t1;
            nrec2 = nrec1;
            k = 0;
  
            /*  Loop over the stations which recorded the requested datatype  */
            for ( i=0; i < nrec1; i++ ) {
  
                stnum = tmppos[i];
                dist = st[stnum].epi;
                if ( SHORT_PERIOD )        numblocks = st[stnum].len[0] -3;
                else if ( LONG_PERIOD )    numblocks = st[stnum].len[1] -3;
                else if ( INTERM_PERIOD )  numblocks = st[stnum].len[2] -3;
  
                /*  Reset values for t0 and t1  */
                *t0 = tplbeg;
                *t1 = tplend;
  
                /*  Determine number of blocks to plot and maximum  */
                rp = ReadPrm (crp, rot, stnum, t0, t1, &angle, &cmpmax );
  
                /*  Check if the requested component is available  */
                if ( ! ( nbl == 0 ) ) {
  
                    /*  Store station position in global parameter pos  */
                    pos[k] = tmppos[i];
  
                    /*  Turn on the display device and set color for background  */
                    if ( k == 0 &&  !DISPLAY_FLAG )   InitDisplay ();
                    if ( k == 0 )              ClearDisplay(12);
  
                    /*  Draw the title of the plot  */
                    if ( k == 0 ) DrawTitle (evinfo, stinfo, tplbeg);
  
                    /*  Draw the axes of the plot  */
                    if ( k == 0 ) DrawAxes ( nchan, distmin, distmax, rot,
                        angle, cmpmax );
                    /*  Plot the data  */
                    PlotRec ( stnum, nchan, dist, rot, angle );
                    k++;
                }
                else                /*  Parameter nrec2 is number of stations  */
                    /*  which actually have data for the data  */
                    nrec2--;         /*  type and time window                   */
  
            }  /*  End of for loop over the stations  */
  
            if ( nrec2 == 0 )   {
                InitDisplay ();
                blankstring (menu1[0]);
                strcpy (menu1[0], " NO DATA FOR THIS DATA TYPE - COMPONENT COMBINATION ");
                DrawText (menu1[0], (dispnl/10), (dispns/64), (dispnl/35),0,15);
                blankstring (menu1[0]);
                strcpy (menu1[0], " MAYBE YOUR OFFSET IS TOO SMALL.  - SELECT CONTINUE ");
                DrawText (menu1[0], (dispnl/5), (dispns/64), (dispnl/35), 0,15);
                return;
            }
  
            /*  Reset values for t0 and t1  */
            *t0 = tplbeg;
            *t1 = tplend;
  
    }  /*  End of switch ( plchoice )  */
}
  
void Zoom ( evtime, stnum, btime, etime )
int   stnum;
float evtime, *btime, *etime;
{
    int   left, right, bottom, up, line, sample, lsize, ssize;
    int   p_line, p_sample, color;
    long int  temp;
    float     t0, tw, dt, time1, time2, x1, x2, tdif, tcr;
  
    t0 = *btime;
    dt = 1 / hdr.rate;
    tcr = st[stnum].tcor;
  
    left = 1;
    right = dispns;
    if ( plchoice == 1 ) {
        bottom = dispnl*11/14;
        up = dispnl*3/14;
    }
    else {
        bottom = dispnl*29/35;
        up = dispnl*2/35;
    }
  
    p_line = dispnl / 2;
    p_sample = dispns / 2;
    color = 3;
  
    /*  First time border of time window  */
    do {
        MoveCursor ( &p_line, &p_sample, left, right, bottom, up,
        evtime, t0, tcr, maxvx, maxvy, dx, &x1 );
        DrawLine ( up, p_sample, bottom, p_sample, color );
  
        time1 = x1 * dx;
        tdif = fmod (time1, dt);
        time1 -= tdif;
  
        /*  Second time border of time window  */
        do {
            MoveCursor ( &p_line, &p_sample, left, right, bottom, up,
            evtime, t0, tcr, maxvx, maxvy, dx, &x2 );
  
            /*  Lines cannot be choosen at the same sample position  */
            if ( x2 != x1 )
                DrawLine (up, p_sample, bottom, p_sample, color);
  
        } while ( x2 == x1 );
  
        time2 = x2 * dx;
        tdif = fmod (time2, dt);
        time2 -= tdif;
  
        /*  Check if the time window is stil big enough to contain samples  */
  
        tw = fabs (time2 - time1);
        if ( tw < (5*dt) ) {
            color = 0;
            line = 6 * dispnl / 7;
            sample = 1;
            lsize = dispnl / 7 + 1;
            ssize = dispns;
            DrawBox (line, sample, lsize, ssize, color);
  
            blankstring (menu1[0]);
            strcpy (menu1[0],"     Time window too small ");
            DrawText (menu1[0], (51*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
            strcpy (menu1[0],"     Enter two new values ");
            DrawText (menu1[0], (53*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
            color = 3;
        }
    } while ( tw < (5*dt) );
  
    /*  Remove 'zoom and cursor menu'  */
    color = 12;
    temp = (long) 305 * dispnl / 350;
    line = (short) temp;
    sample = 1;
    temp = (long) 45 * dispnl / 350 + 1;
    lsize = (short) temp;
    ssize = dispns;
    DrawBox (line, sample, lsize, ssize, color);
  
    if ( time2 > time1 )    {
        *btime = t0 + time1;
        *etime = t0 + time2;
    }
    else {
        *btime = t0 + time2;
        *etime = t0 + time1;
    }
}
  
void Curs ( evtime, stnum, t0 )
int   stnum;
float evtime, t0;
{
    int   left, right, bottom, up;
    int   p_line, p_sample;
    float     x, tcr;
  
    p_line = dispnl / 2;
    p_sample = dispns / 2;
    left = 1;
    right = dispns;
    bottom = dispnl*11/14;
    up = dispnl*3/14;
    tcr = st[stnum].tcor;
    MoveCursor ( &p_line, &p_sample, left, right, bottom, up,
    evtime, t0, tcr, maxvx, maxvy, dx, &x);
}
  
void WrtHd ( stnum, nchan, t0, t1 )
int   stnum, nchan ;
float *t0, *t1;
{
    FILE      *id;
    char      fname[15], ch, stinfo[81], stname[50];
    char      dum1[LINESIZE], dum2[LINESIZE];
    int       i, i0, j, k, k2, l, m, yr, dofy, hour, minute;
    int       nsamp;
    long  int     npoints, ntot1, ntot2;
    float     fmaxm[3], ntot, tplbeg, tplend;
    double    time, sec;
    char  *prog_name = "WRTHD";
  
    clearscr();
    screen_setup();
  
    blankstring (menu1[0]);
    strcpy (menu1[0], " WRITE DATA TO HARD DISK ");
    textdisp (menu1, 0, 1, 79, 1);
  
    blankstring (menu1[0]);
    strcpy (menu1[0], " FILENAME(S) : ");
    textdisp (menu1, 0, 5, 79, 5);
  
    set_colour (3, 0, 18, 79, 20);
  
    blankstring (menu1[0]);
    strcpy (menu1[0], " WARNING: only poles and zeroes as they appear in the second data log, are");
    textdisp (menu1, 0, 18, 79, 18);

    blankstring (menu1[0]);
    strcpy (menu1[0], "          written to hard disk (so there is no check if all conjugated poles");
    textdisp (menu1, 0, 19, 79, 19);

    blankstring (menu1[0]);
    strcpy (menu1[0], "          and zeroes are present ).");
    textdisp (menu1, 0, 20, 79, 20);
  
    set_colour (3, 0, 22, 79, 24);
    blankstring (menu1[0]);
    textdisp (menu1,  0, 23, 79, 23);
    strcpy (menu1[0], "SCANNING THE DATA BLOCKS");
    textdisp (menu1, 25, 23, 79, 23);
  
    /*  Store values of t0 and t1 in tplbeg and tplend  */
    tplbeg = *t0;
    tplend = *t1;
  
    /*  Time plot: determine normalization constant, poles and zeroes. */
    if ( plchoice == 1 ) {
        rddtl ( stnum, nchan );
        repair ( stnum, nchan );
    }
  
    /*  Do-loop over the different seismograms in a delta vs. time plot  */
    l = 0;
    do {
        if ( plchoice == 2 )  {
            stnum = pos[l];
  
            /*  Read station log to determine stlat, stlon and stelev  */
            rdstl ( stnum, stinfo, stname, dum1, dum2 );
  
            nchan = 1;
  
            /*  Read second data log to determine a0, poles and zeroes  */
            rddtl ( stnum, nchan );
  
            /*  Check for possible errors  */
            repair ( stnum, nchan );
  
            if ( SHORT_PERIOD )       numblocks = st[stnum].len[0] - 3;
            else if ( LONG_PERIOD )   numblocks = st[stnum].len[1] - 3;
            else if ( INTERM_PERIOD ) numblocks = st[stnum].len[2] - 3;
  
            /*  Set t0 and t1 to original values  */
            *t0 = tplbeg;
            *t1 = tplend;
  
            /*  Determine the parameters to write data to disk  */
            ReadPrm (0, 0, stnum, t0, t1, 0, 0);
        }
  
        /*  Determine parameters year ,dofy  */
        yr = hdr.year;
        dofy = hdr.day;
  
        /*  Determine parameters: hour, min, sec and msec from t0  */
        time = *t0 ;
        if ( time > 86400 ) {
            time -= 86400;
            dofy += 1;
        }
        hour = time / 3600;
        minute  = (int) (fmod (time, 3600) / 60);
        sec  = fmod ((fmod (time, 3600)), 60);
  
        m = 0;
  
        /*  Do loop over the number of components (nchan) to open the file(s)  */
        /*  and write the header(s) to the file(s).                            */
        do {
            if ( nchan == 3 ) comp = m;
  
            /*  Open file on hard disk  */
            OpenHdFile ( stnum, fname);
  
            /*  Print filename on screen  */
            blankstring (menu1[0]);
            strcpy (menu1[0], fname);
            textdisp (menu1, 16, 5+m+l, 79, 5+m+l);
  
            WrHeader (stnum, yr, dofy, hour, minute, sec );
            m++;
  
        } while ( m < nchan );  /*  End of do loop over the components  */
  
        /*  Move file handle to position of first datablock */
        movedblck (stnum, nfirst );
  
        /*  Determine values ntot and initialize ch, k and fmax  */
        ntot  = floor ( (*t1 -*t0) * hdr.rate + 1.1 );
        ntot1 = (long) ntot;
        ch = '\n';
        k  = 1;
        for ( i=0; i < 3; i++ )     fmaxm[i] = 1;
  
        /*  Loop over the data blocks */
        for ( j=0; j < nbl; j++ )  {
  
            rddatablock ( &nsamp );
            decode ( fmaxm );
            m = 0;
  
            /*  Do loop over the components to write data to hard disk  */
            do {
                if ( nchan == 3 ) {
                    comp = m;
                    if ( m == 0 ) {
                        ntot2 = ntot1;
                        k2 = k;
                    }
                    else {
                        ntot1 = ntot2;
                        k = k2;
                    }
                }
  
                /*  File streamer id depends on the component value  */
                if ( comp == 0 )      id = id3;
                else if ( comp == 1 ) id = id4;
                else if ( comp == 2 ) id = id5;
                i0 = comp * 330;
  
                /*  Loop over the data samples  */
                for ( i=0; i < nsamp; i++ ) {
  
                    /*  Seconds from first and last block not to write to disk  */
                    if ( j == 0 || j == nbl-1 ) {
                        if ( delpb > 0 || delpe > 0 ) {
                            if ( ( i < delpb && j==0 ) ||
                                ( i >= nsamp - delpe && j == nbl-1 ) )
                                continue;  /* that is: next data sample */
                        }
                    }
  
                    if ( k == 1 || ((k-1) % 512 ) == 0 ) {
                        if ( ntot1 >= 512 )   npoints = 512;
                        else                  npoints = ntot1 +1;
  
                        if ( fprintf (id, "\n%3ld\n", npoints) != 5 )
                            error_and_exit (prog_name,"Error 1 for writing data to disk.");
  
                        ntot1 -= 512;
                        k = 1;
                    }
  
                    if ( fprintf (id, "%12.3E ", *(plotbuffer+i0+i)) != 13 )
                        error_and_exit (prog_name,"Error 2 for writing data to disk.");
  
                    if ( ( k % 6 ) == 0 ) {
                        if ( fprintf (id, "%c", ch) != 1 )
                            error_and_exit (prog_name,"Error 3 for writing data to disk.");
                    }
                    k++;
  
                }  /*  End of for-loop over the data samples  */
                m++;
  
            } while ( m < nchan );  /* End of do loop over the components  */
  
            if (!PLOTBUF_FREE ) {
                free ( plotbuffer ); /* allocated in decode ! */
                PLOTBUF_FREE = TRUE;
            }
        }  /*  End of for-loop over the data blocks  */
  
        /*  Close file(s) on hard disk  */
  
        if ( nchan == 1 ) {
            if ( comp == 0 )      fclose (id3);
            else if ( comp == 1 ) fclose (id4);
            else if ( comp == 2 ) fclose (id5);
        }
        else {
            fclose (id3);
            fclose (id4);
            fclose (id5);
        }
        l++;
    } while ( l < nrec2 );  /*  End of do-loop over the stations  */
  
    /*  Reset values for t0 and t1  */
    *t0 = tplbeg;
    *t1 = tplend;
}
  
int PlotMenu ( nchan , cro, chan_avail, wr_all)
int nchan, cro, chan_avail;
char wr_all[];
{
    char  answer[20], plmenu[81], extra[10];
    int   i, color, line, sample, lsize, ssize;
    int choice;
    long  int temp;
  
    strset (answer, ' ');
    strset (extra, ' ');
    color = 0;
    temp = (long) 305 * dispnl / 350;
    line = (short) temp;
    sample = 1;
    temp = (long) 45 * dispnl / 350 + 1;
    lsize = (short) temp;
    ssize = dispns;
    DrawBox (line, sample, lsize, ssize, color);
  
    strcpy (plmenu, " PRevious    ZOom     REplay    HP-filter   BP-filter");
    DrawText (plmenu, (51*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
  
    strcpy (plmenu, " COntinue    WRite");
    if ( plchoice == 1 ) {
        if ( nchan == 1 ) strcat (plmenu, "    CUrsor ");
        else if ( nchan == 3 ) {
            if ( cro == 0 ) strcat (plmenu, "    ROtate ");
            else if ( cro == 1 ) strcat (plmenu, "    ROtate  more ");
        }
    }
    DrawText (plmenu, (53*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
  
    TextLine = dispnl;
    TextSample = (dispns/128);
    TypeText (" Command : ");
    AcceptText ( answer );
  
    for (i=0; i < 2; i++) answer[i] = toupper(answer[i]);
  
    if ( answer[0] == 'Z' && answer[1] == 'O' ) {
        color = 0;
        DrawBox (line, sample, lsize, ssize, color);
        strcpy (plmenu, " NumLock must be turned on    . sets boundary /");
        strcat (plmenu, " exits cursor mode ");
        DrawText (plmenu, (51*dispnl/56), (dispns/128), (dispnl/35)-1, 0, 15);
        strcpy (plmenu, " - step decrease    + step increase ");
        strcat (plmenu, "   5 cursor position ");
        DrawText (plmenu, (53*dispnl/56), (dispns/128), (dispnl/35)-1, 0, 15);
        strcpy (plmenu, " 4 cursor left    6 cursor right    8 cursor up ");
        strcat (plmenu, "   2 cursor down ");
        DrawText (plmenu, (55*dispnl/56), (dispns/128), (dispnl/35)-1, 0, 15);
        choice = 1;
    }
    else if ( answer[0] == 'R' && answer[1] == 'E' ) {
        color = 12;
        DrawBox (line, sample, lsize, ssize, color);
        choice = 2;
    }
    else if ( answer[0] == 'C' && answer[1] == 'U') {
        color = 0;
        DrawBox (line, sample, lsize, ssize, color);
        strcpy (plmenu, " NumLock must be turned on    . sets boundary /");
        strcat (plmenu, " exits cursor mode ");
        DrawText (plmenu, (51*dispnl/56), (dispns/128), (dispnl/35)-1, 0, 15);
        strcpy (plmenu, " - step decrease    + step increase ");
        strcat (plmenu, "   5 cursor position ");
        DrawText (plmenu, (53*dispnl/56), (dispns/128), (dispnl/35)-1, 0, 15);
        strcpy (plmenu, " 4 cursor left    6 cursor right    8 cursor up ");
        strcat (plmenu, "   2 cursor down ");
        DrawText (plmenu, (55*dispnl/56), (dispns/128), (dispnl/35)-1, 0, 15);
        choice = 3;
    }
    else if ( answer[0] == 'P' && answer[1] == 'R' ) {
        color = 12;
        DrawBox (line, sample, lsize, ssize, color);
        choice = 4;
    }
    else if ( answer[0] == 'R' && answer[1] == 'O' ) {
        if (cro == 0 ) {
            color = 12;
            DrawBox (line, sample, lsize, ssize, color);
            choice = 5;
        }
        else if ( cro == 1 ) {
            TextLine = dispnl;
            TextSample = (dispns/128);
            TypeText ( " Give angle for extra rotation: ");
            AcceptText( extra);
            sscanf(extra,"%f",&angle_var);
            DrawBox (line, sample, lsize, ssize, color);
            choice = 8;
        }
    }
    else if ( answer[0] == 'W' && answer[1] == 'R' ) {
        if ( plchoice == 1  && chan_avail == 3 && comp != 3 ) {
            TextLine = dispnl;
            TextSample = (dispns/128);
            TypeText ( " Write all components to hard disc ? (y/n) : ");
            AcceptText( extra);
            sscanf(extra,"%s",wr_all);
        }
        DisplayOff();
        choice =  6;
    }
    else if ( answer[0] == 'C' && answer[1] == 'O' ) {
        DisplayOff();
        choice = 7;
    }
    else if ( answer[0] == 'B' && answer[1] == 'P' ) {
        DrawBox (line, sample, lsize, ssize, color);
        strcpy (plmenu, "Bandpass filtering on RAW data. Give upper and");
        DrawText (plmenu, (51*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
        strcpy (plmenu, "lower corner frequency: ");
        DrawText (plmenu, (53*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
        TextLine = dispnl;
        TextSample = (dispns/128);
        TypeText("  ");
        AcceptText( extra);
        sscanf(extra,"%f %f",&upper_frq, &lower_frq);
        BPFILTPAR = TRUE;   /* new filter frequencies !! */
        choice = 9;
    }
    else if ( answer[0] == 'H' && answer[1] == 'P' ) {
        DrawBox (line, sample, lsize, ssize, color);
        strcpy (plmenu, "Highpass filtering on raw data. Give corner frequency:");
        DrawText (plmenu, (51*dispnl/56), (dispns/128), (dispnl/35), 0, 15);
        TextLine = dispnl;
        TextSample = (dispns/128);
        TypeText("  ");
        AcceptText( extra);
        sscanf(extra,"%f",&upper_frq);
        HPFILTPAR = TRUE;   /* new filter frequencies !! */
        choice =  10;
    }
    return(choice);
}
  
INT ReadPrm ( count, rot, stnum, begintime, endtime, angle, cmpmax )
INT    count;         /*  counter for this function  */
INT    rot;           /*  rotation parameter  */
INT    stnum;         /*  station number  */
INT    *cmpmax;       /*  Component which contains largest maximum  */
float  *begintime;    /*  begin time of the plot  */
float  *endtime;      /*  end time of the plot  */
float  *angle;        /*  angle of rotation  */
{
    INT        i, j, k, nsmp, blcks, numday, LAST_BLOCK, rp=0;
    static int nbl1, nfirst1;
    float      ffmax[3], hdbeg, hdend, tend, rmax, prod;
  
    /*
    *  crp = 0 : for the selected station and datatype this function determines
    *            the plotparameters (for the first time).
    *  crp = 1 : for the same station and datatype the program enters this
    *            function for the second time or more so one can use nbl and
    *            nfirst which are already known (used with zoom).
    *  crp = 2 : same as case 1 but now one has to use the values which were
    *            determined for crp = 0 (used with replay and previous).
    *  crp = 3 : this function can be skipped (used with replay after writing
    *            to disk).
    *
    */
  
    if ( count == 3 ) return rp;
  
    /*  Initialization of parameters k, fmax and max  */
    k = 0;
    for ( i=0; i < 3; i++ ) {
        ffmax[i] = 1;
        maxm[i]  = 0;
    }
  
    /*  Initialization of the plot parameters  */
    if ( count == 0 )  {                        /*  First time to plot  */
        blcks = numblocks;
        nfirst = 0;
    }
    else if ( count == 1 )   {                       /*  Zoom  */
        blcks = nbl;
    }
  
    else if ( count == 2 && plchoice == 1 ) {        /*  Replay or previous  */
        blcks = nbl1;
        nfirst = nfirst1;
    }
  
    dsecb = 0;
    delpb = 0;
    dsece = 0;
    delpe = 0;
    nbl   = 0;                         /*  *nbl is set to zero again !  */
    LAST_BLOCK = FALSE;
  
    /*       Print menu1 on the screen  */
    set_colour (3, 0, 22, 79, 24);
  
    blankstring (menu1[0]);
    textdisp (menu1, 0, 23, 79, 23);
    strcpy (menu1[0], "SCANNING THE DATA BLOCKS");
    textdisp (menu1, 25, 23, 79, 23);
  
    /*  Move file handle to position of first datablock  */
    movedblck ( stnum, nfirst );
    if ( rot == 1 )          *angle = st[stnum].back + 180;
    else if ( rot == 2 )     *angle += angle_var;
/*printf("\n begin = %f", *begintime);*/
/*printf("\n end   = %f", *endtime);*/
/*pause(5);*/
  
    /*  Loop over the data blocks  */
    for ( i=0; i < blcks; i++ ) {
  
        rddatablock( &nsmp);
        if ( i%5 == 0 ) {
            strcpy(menu1[0],"WORKING ");
            textdisp(menu1,72,23,79,23);
        }
  
        /*  Check if component comp is available  */
        if ( plchoice == 2 ) {
            if ( comp >= hdr.nchan ) return rp;
        }
  
        /*  Determine begintime and endtime of datablock  */
        if ( i == 0 ) numday = hdr.day;
        hdbeg = (float) hdr.hour*3600+hdr.minute*60+hdr.sec+hdr.mss/1000 ;
        hdend = hdbeg + (float) (nsmp-1) / hdr.rate ;
/*printf("\n i = %d numday = %d day = %d",i, numday, hdr.day);*/
/*printf("\n begin = %f end = %f", hdbeg, hdend);*/
/*printf("\n rate = %f samps = %d", hdr.rate, nsmp);*/
/*printf("\n   time   =   %d   %d   %d   %d",hdr.hour,hdr.minute,hdr.sec,hdr.mss);*/
/*pause(5);*/
  
        /*  Apply time correction */
        hdbeg += st[stnum].tcor;
        hdend += st[stnum].tcor;
  
        if ( hdr.day != numday ) {
            hdbeg += 86400;
            hdend += 86400;
        }
  
        /*  Check if header time is element of specified time interval  */
  
        if ( !( hdend <= *begintime || (hdbeg - *endtime) > 1.0E-06  ) ) {
            nbl ++;
  
            if ( nbl == 1 )  {       /*  nfirst is number of first block  */
                nfirst = nfirst +i;                  /*  of time interval  */
                dsecb = *begintime - hdbeg;
  
                if ( *begintime <= hdbeg )  delpb = 0;
                else  {
                    prod = dsecb * hdr.rate;
                    /* Prevent rounding errors ! */
                    delpb = ( fmod(prod,1.0) >= 0.5) ? ceil(prod) : floor(prod);
                }
            }
  
            /*  Determine for last data block *dsece and set stopvalue  */
  
            if ( hdbeg <= *endtime && hdend > *endtime ) {
                dsece = hdend - *endtime;
  
                if ( *endtime >= hdend )    delpe = 0;
                else  {
                    prod = dsece * hdr.rate;
                    /* Prevent rounding errors ! */
                    delpe = (fmod(prod,1.0) >= 0.5) ? ceil(prod):floor(prod);
                }
  
                LAST_BLOCK = TRUE;
            }
  
            /*  Store hdend in parameter tend - last hdend which belongs to  */
            /*  time interval  */
            tend = hdend;
  
            /*  Decode data to fill plotbuffer  */
            decode ( ffmax );
  
            /*  Rotate the N/S and E/W components  */
            if ( rot == 1  || rot == 2 )
                Rotate (plotbuffer+330, plotbuffer+660, nsmp, *angle, 0, 0 );
  
            if ( BPFILTER ) {
                if ((upper_frq/hdr.rate>=0.5) ||(lower_frq/hdr.rate>=0.5)) {
                    printf("Out of Nyquist interval, NO FILTERING !\n");
                    pause(10);
                    BPFILTER = FALSE;
                }
                else
                    BP_Filter ( plotbuffer, plotbuffer+330, plotbuffer+660, nsmp, nbl-1);
            }
            if ( HPFILTER ) {
                if ( upper_frq / hdr.rate >= 0.5 ) {
                    printf("Out of Nyquist interval, NO FILTERING !\n");
                    pause(10);
                    HPFILTER = FALSE;
                }
                else
                    HP_Filter ( plotbuffer, plotbuffer+330, plotbuffer+660, nsmp, nbl-1);
            }
  
            if ( i%5 ==3 ) {
                strcpy(menu1[0],"        ");
                textdisp(menu1,72,23,79,23);
            }
  
            /*  Determine maximum in plotbuffers  */
            for ( j=0; j < nsmp; j++ ) {
                if ( ( nbl == 1 && dsecb > 0 ) || dsece > 0 ) {
                    if ( ( j < delpb && nbl == 1 ) || ( j > nsmp - delpe ) )
                        continue;
                    /* that is: next data sample */
                }
  
                /* Maximum is determined  */
                if ( hdr.nchan == 1 ) {
                    maxm[comp] = abs_max ( maxm[comp], *(plotbuffer+j) );
                }
                else  {
                    maxm[0] = abs_max ( maxm[0], *(plotbuffer+j) );
                    maxm[1] = abs_max ( maxm[1], *(plotbuffer+330+j) );
                    maxm[2] = abs_max ( maxm[2], *(plotbuffer+660+j) );
                }
  
            }   /*  End for loop over data samples  */
  
        } /* End of if statement */
  
        if (!PLOTBUF_FREE ) {
            free ( plotbuffer ); /* allocated in decode ! */
            PLOTBUF_FREE = TRUE;
        }
  
        if ( LAST_BLOCK ) break;  /* break out of loop over data blocks */
  
        if ( ( i == (blcks-1) && dsece == 0 ) || hdbeg > *endtime ) {
            dsece = tend - *endtime;
            break;          /* break out of loop over data blocks */
        }
  
    }  /*  End of loop over data blocks  */
  
  
    /*  Begintime and endtime of data record and number of seconds to plot  */
    /*  (plsec) are determined  */
    /*  No gaps before beginning and after end of the plot of one seismogram  */
    /*  Gaps are allowed in case of a epicentral distance vs. time plot  */
  
    if ( plchoice == 1 )  {
        if ( dsecb < 0 )  {
            *begintime -= dsecb;
            dsecb = 0;
            delpb = 0;
        }
        if ( dsece < 0 ) {
            *endtime += dsece;
            dsece = 0;
            delpe = 0;
        }
        plsec = *endtime - *begintime;
    }
    else  if ( plchoice == 2 ) {
        plsec = *endtime - *begintime;
        if ( dsecb<0 ) *begintime -= dsecb;
        if ( dsece<0 ) *endtime += dsece;
    }
  
    /*  Store found values for nbl and nfirst - used with replay  */
    if ( count == 0 && plchoice == 1 ) {
        nbl1 = nbl;
        nfirst1 = nfirst;
    }
  
    /*  Check for maximum equal zero  */
    for ( i=0; i < 3; i++ ) if ( maxm[i] == 0 ) maxm[i] = 1;
  
    /*  In case of rotation all maxima are set to the largest maximum.  */
    /*  Parameter *cmpmax is determined - the component with largest maximum.  */
    if ( rot == 1  || rot == 2   ) {
        rmax = 0;
        for ( i=0; i < 3; i++ ) {
            if ( maxm[i] > rmax ) {
                rmax = maxm[i];
                *cmpmax  = i;
            }
        }
        for ( i=0; i < 3; i++ )   maxm[i] = rmax;
    }
    return rp;
}
  
void PlotRec ( stnum,  nchan, dist, rot, angle )
int   stnum;        /*  station number  */
int   nchan;        /*  number of channels to plot  */
int   rot;          /*  Rotation parameter  */
float dist;         /*  Epicentral distance  */
float angle;        /*  Rotation angle  */
{
    int   i, j, k, k2, m, x1, x2, x3, x4;
    int   line[3], ylast[3], color, nsamp, numday;
    int   hght;
    float ffmax[3], xlast[3], ntot, xf, dt, prod;
    float hdbeg, hdend, st0, st2, scale, DX, dp;
  
    /*  Initialize some parameters  */
  
    st0 = 0;
    st2 = 0;
    for ( i=0; i < 3; i++ ) {
        xlast[i] = 0;
        ylast[i] = 0;
    }
    color = 15;
  
    /*  Determine total points to plot and sample interval  */
    dt = 1 / hdr.rate;
    ntot = floor (plsec * hdr.rate + 1.1);
    DX = (float) (dispns-1) / (ntot-1);
  
    /*  Determine line position of the seismogram  */
    LinePos ( nchan, dist, minvy, maxvy, line );
  
    /*  Determine the scale factor  */
    if ( plchoice == 1 && nchan == 1 ) scale = dispnl*2/7-1;
    else if ( plchoice == 1 && nchan == 3 ) scale = dispnl*2/21-1;
    else if ( plchoice == 2 ) scale = (dispnl*2)/(7*nrec1);
  
    for ( i=0; i < 3; i++ ) ffmax[i] = maxm[i] / scale;
  
    /*  Determine the offset in the distance vs. time plot  */
    if ( plchoice == 2 ) {
  
        if ( dsecb <= 0 ) {
            prod = fabs(dsecb) * hdr.rate;
            st0 = ( fmod(prod,1.0) >= 0.5) ? ceil(prod) : floor(prod);
            dp = fmod ((fabs (dsecb)), 1) * hdr.rate;
        }
        else {
            st0 = fmod (dsecb, 1) * hdr.rate;
            dp = st0;
            st0=0;
        }
    }
  
  
    /*  Move file handle to position of first datablock to plot  */
    movedblck ( stnum, nfirst );
  
    /*  Loop over the data blocks */
    for ( j=0; j < nbl; j++ )  {
  
        rddatablock (&nsamp);
  
        /*  Decode the binary data  */
        decode ( ffmax );
  
        /*  Rotate the N/S and E/W components  */
        if ( rot == 1  || rot == 2)
            Rotate ( plotbuffer+330, plotbuffer+660, nsamp, angle, 0, 0 );
  
        /*  Filter all components  */
        if ( BPFILTER ) {
            BP_Filter ( plotbuffer, plotbuffer+330, plotbuffer+660, nsamp, j);
        }
        if ( HPFILTER ) {
            HP_Filter ( plotbuffer, plotbuffer+330, plotbuffer+660, nsamp, j);
        }
  
        /*  Set value for k - for first block 0 and all others 1  */
        k = (j==0) ? 0 : 1;
  
        /*  Determination of st - time interval to skip between two blocks  */
        if ( j == 0 ) numday = hdr.day;
  
        hdbeg = (float) hdr.hour*3600+hdr.minute*60+hdr.sec+hdr.mss/1000 ;
        if ( hdr.day != numday ) hdbeg += 86400;
  
        if ( j > 0 ) {
            st0 = 0;
            if ( (hdbeg - (hdend+dt)) > dt/2 )
                st2 = (hdbeg - (hdend+dt)) * hdr.rate;
            else st2 = 0;
        }
        hdend = hdbeg + (float) (hdr.nsamp -1) / hdr.nchan / hdr.rate;
        m = 0;
  
        /*  Do loop over the components  */
  
        do {
            if ( nchan == 3 ) comp = m;
  
            /*  For second and third plot k has to be reset  */
            if ( m == 0 ) k2 = k;
            else           k = k2;
  
            /*  Loop over the data samples  */
            for ( i=0; i < nsamp-1; i++ ) {
  
                /* is this OK ? */
                /*  Seconds from first and last block not to plot  */
                if ( j == 0 && delpb > 0 && i < delpb ) continue;
                if ( j == nbl-1 &&  delpe > 0 && i >= nsamp-1-delpe ) continue;
  
                /* Last sample previous block is connected to first sample  */
                if ( j > 0 && i == 0 && st2 == 0 ) {
                    x1 = ylast[m];
                    x2 = (int) xlast[m] + 1;
                    x3 = line[m] - (int) *(plotbuffer+comp*330);
                    xf = xlast[m] + DX;
                    x4 = (int) xf + 1;
                    DrawLine (x1, x2, x3, x4, color);
                }
  
                /* Line and sample values are determined  */
                x1 = line[m] - (int) *(plotbuffer+comp*330+i);
                xf = (float) (k+st0+st2) * DX + xlast[m];
                x2 = (int) xf + 1;
                x3 = line[m] - (int) *(plotbuffer+comp*330+i+1);
                xf = xf + DX;
                x4 = (int) xf + 1;
                DrawLine (x1, x2, x3, x4, color);
                k++;
  
                /*  Last line and sample number are stored  */
                if ( i == nsamp-2 ) {
                    ylast[m] = x3;
                    xlast[m] = xf;
                }
  
            }  /*  End do-loop data samples  */
            m++;
  
        } while ( m < nchan );  /*  End do loop components  */
  
        if (!PLOTBUF_FREE ) {
            free ( plotbuffer );
            PLOTBUF_FREE = TRUE;
        }
  
    }  /*  End do-loop data blocks  */
  
    /*  Draw stcode next to seismogram for epicentral dist. vs. time plot  */
    if ( plchoice == 2 ) {
        hght = dispnl/50;
        DrawBox (line[0]-(hght/2)-2, dispns-(5*hght)-4, hght+4, 5*hght, 12);
        DrawText (st[stnum].codes, line[0]+(hght/2), dispns-(5*hght), hght, 0, 15);
    }
}
  
void LinePos ( nchan, dist, minn, maxn, line )
int nchan, line[];
float dist, minn, maxn;
{
    int m;
    float displine;
  
    if ( plchoice == 1 ) {
        if ( nchan == 1 ) line[0] = dispnl/2;
        else {
            for ( m=0; m < 3; m++ ) line[m] = dispnl*13/42+ m*4*dispnl/21;
        }
    }
    else {
        displine = (float) ( dispnl*2/35 + (dist - maxn)/(minn - maxn) *
        (dispnl*27/35) );
        line[0] = (int) displine;
    }
}
  
void DrawTitle ( evinfo, stinfo, t0 )
BYTE   *evinfo, *stinfo;
float  t0;
{
    BYTE   text[140], sttime[12];
    BYTE   freq[10];
    INT    color = 15, hour, minute;
    float  sec;
  
    if ( t0 > 86400 )     t0 -= 86400;
  
    hour = t0 / 3600;
    minute  = (int) (fmod (t0, 3600) / 60);
    sec  = fmod ((fmod (t0, 3600)), 60);
    sprintf (sttime, "%02d:%02d:%05.2f", hour, minute, sec);
  
    if ( plchoice == 1 ) {
        strcpy (text, "DATE              EVENT TIME         LAT          LON");
        strcat (text, "       Depth    mb     Ms ");
        DrawText (text, 3*dispnl/70+1, dispns/64, dispnl/66+1, 0, color);
        DrawText (evinfo, 3*dispnl/35, dispns/64, dispnl/44+1 , 0, color);
        strcpy (text, "CODE       LAT         LON          D       BAZI     ");
        strcat (text, "      START TIME ");
        if ( BPFILTER ) strcat (text,"   BAND PASS FILTERED ");
        else if ( HPFILTER ) strcat (text,"   HIGH PASS FILTERED ");
        else strcat ( text,"   RAW DATA ");
        DrawText (text, 9*dispnl/70+1, dispns/64, dispnl/66+1, 0, color);
        strcpy ( text, stinfo ); 

        /*  Store datatype  */
        if ( SHORT_PERIOD )         strcat (text, "  SP  ");
        else if ( INTERM_PERIOD )   strcat (text, "  IP  ");
        else if ( LONG_PERIOD )     strcat (text, "  LP  ");

        strcat ( text, sttime );
        if ( BPFILTER ) {
            sprintf(freq,"%6.2f",lower_frq);
            strcat ( text,freq);
            sprintf(freq,"%6.2f",upper_frq);
            strcat ( text,freq);
        }
        else if ( HPFILTER ) {
            sprintf(freq,"%6.2f",upper_frq);
            strcat ( text,freq);
        }
        else strcat ( text,"   NOT FILTERED");
        DrawText (text, 6*dispnl/35, dispns/64, dispnl/44+1, 0, color);
    }
    else {
        strcpy (text, "DATE            EVENT TIME      LAT        LON");
        strcat (text, "      Depth   mb    Ms    START TIME ");
        if ( BPFILTER ) strcat (text,"   BAND PASS FILTERED ");
        else if ( HPFILTER ) strcat (text,"   HIGH PASS FILTERED ");
        else strcat ( text,"   RAW DATA");
        DrawText (text, dispnl/35-3, dispns/64, dispnl/66+1, 0, color);
        strcpy (text, evinfo);
        strcpy (text+61, sttime);
        if ( BPFILTER ) {
            sprintf(freq,"%6.2f",lower_frq);
            strcat ( text,freq);
            sprintf(freq,"%6.2f",upper_frq);
            strcat ( text,freq);
        }
        else if ( HPFILTER ) {
            sprintf(freq,"%6.2f",upper_frq);
            strcat ( text,freq);
        }
        else strcat ( text,"   NOT FILTERED");
        DrawText (text, dispnl/19, dispns/64, dispnl/66+2, 0, color);
    }
}
  
void DrawAxes ( nchan, distmin, distmax, rot, angle, cmpmax )
int   nchan, rot, cmpmax;
float distmin, distmax, angle;
{
    char  titlex[15], cmp[10];
    int   m, left, right, bottom, up;
    int   color=15, hght, len;
    float delvx, delvy, dif;
  
    /*  Determination of parameters for x-axis  */
    /*  MILLISECONDS */
    if ( plsec <= 2 ) {
        maxvx = plsec*1000;
        dx = .001;
        delvx = 500;
        strcpy (titlex, "Time (msec)");
    }
    /*  SECONDS */
    else if ( plsec <= 120 ) {
        maxvx = plsec;
        dx = 1;
        strcpy (titlex, "Time (sec)");
        if (plsec <= 4) delvx = 0.5;
        else if (plsec <=30) delvx = 2;
        else if (plsec <= 60) delvx = 5;
        else delvx = 10;
    }
    /*  MINUTES */
    else if ( plsec <= 7200 ) {
        maxvx = plsec / 60;
        dx = 60;
        strcpy (titlex, "Time (min)");
        if (plsec <= 900) delvx = 1;
        else if (plsec <= 1800) delvx = 5;
        else if (plsec <= 3600 )delvx = 10;
        else if (plsec <= 5400) delvx = 15;
        else delvx = 30;
    }
    /*  HOURS */
    else  {
        maxvx = plsec / 3600;
        dx = 3600;
        delvx = 1;
        strcpy (titlex, "Time (hour)");
    }
  
    /*  Determination of parameters for y-axis  */
    if ( plchoice == 1 && nchan == 1 ) {
        minvy = -maxm[comp];
        maxvy = maxm[comp];
        delvy = maxm[comp] / 5;
    }
  
    if ( plchoice == 2 ) {
  
        dif = distmax - distmin;
        if ( dif <= 0.5 ) {
            minvy = distmin - 0.1;
            maxvy = distmax + 0.1;
        }
        else  if ( dif < 2.5 ) {
            minvy = distmin - 1.5;
            maxvy = distmax + 1.5;
        }
        else if ( dif < 5.0 ) {
            minvy = distmin - 2.5;
            maxvy = distmax + 2.5;
        }
        else {
            minvy = distmin - 4;
            maxvy = distmax + 4;
        }
  
        if ( minvy < 0 ) minvy = 0;
        dif = maxvy - minvy;
        if ( dif <= 1.0 )        delvy = 0.02;
        else if ( dif <= 2.0 )   delvy = .1;
        else if ( dif <= 5.0 )   delvy = .5;
        else if ( dif <= 15.0 )  delvy = 2.0;
        else if ( dif <= 30.0 )  delvy = 5.0;
        else if ( dif <= 90.0 )  delvy = 10.0;
        else                     delvy = 20.0;
  
    }
  
    /*  Determination of the positions of the axes  */
    left = 1;
    right = dispns;
  
    if ( plchoice == 1 && nchan == 1 ) {
        bottom = dispnl*11/14;
        up = dispnl*3/14;
    }
    else if ( plchoice == 2 ) {
        bottom = dispnl*29/35;
        up = dispnl*2/35;
    }
  
    /*  Loop over the components  */
    for (m=0; m < nchan; m++) {
        if ( nchan == 3 ) comp = m;
        if ( plchoice == 1 && nchan == 3 ) {
            bottom = dispnl*3/14 + (m+1)*4*dispnl/21;
            up = dispnl*3/14 + m*4*dispnl/21;
  
            if ( rot == 0 ) {
                minvy = -maxm[comp];
                maxvy = maxm[comp];
                delvy = maxm[comp] / 2;
            }
            else {
                minvy = -maxm[cmpmax];
                maxvy = maxm[cmpmax];
                delvy = maxm[cmpmax] / 2;
            }
        }
  
        /*  Draw the axes  */
        DrawX (left, right, bottom, delvx, color, 1, 1);
        DrawY (bottom, up, left, delvy, color, 1, 1);
        DrawX (left, right, up, delvx, color, 0, -1);
        DrawY (bottom, up, right, delvy, color, 0, -1);
  
        /*  Draw component information in the upper right corner  */
        if ( plchoice == 1 ) {
            if ( comp == 0 ) strcpy (cmp, " Z ");
            if ( rot == 0 ) {
                if ( comp == 1 ) strcpy (cmp, "N/S");
                else if ( comp == 2 ) strcpy (cmp, "E/W");
            }
            else {
                if ( comp == 1 )  {
                    sprintf (cmp, "%.1f", fmod ((angle-180), 360));
                    strcat (cmp, " R");
                }
                else if ( comp == 2 ) {
                    sprintf (cmp, "%.1f", fmod ((angle-90), 360));
                    strcat (cmp, " T");
                }
            }
            hght = dispnl/50;
            len = strlen (cmp)+1;
            DrawBox (up+(hght/2)-2,dispns-((len+1)*hght)-2,hght+4,len*hght,12);
            DrawText (cmp, up+3*hght/2, dispns-((len+1)*hght), hght, 0, color);
        }
    }
  
    if( plchoice == 2)  {
        strcpy(cmp," ");
        if( SHORT_PERIOD  )      strcpy(cmp,"SP ");
        else if( LONG_PERIOD  )  strcpy(cmp,"LP ");
        else if( INTERM_PERIOD ) strcpy(cmp,"IP ");
        if(comp == 0)       strcat(cmp,"Z  ");
        else if(comp == 1)  strcat(cmp,"N/S");
        else if(comp == 2)  strcat(cmp,"E/W");
        else strcat(cmp,"???");
  
        hght = dispnl/50;
        len = strlen (cmp) +1;
        DrawBox (up+(hght/2)-2,dispns-((len+1)*hght)-2,hght+4,len*hght,12);
        DrawText (cmp, up+3*hght/2, dispns-((len+1)*hght), hght, 0, color);
    }
  
    /*  Draw the text for the axes  */
    DrawText (titlex, (bottom+dispnl/32), (dispns/2), (dispnl/44+1),0,color);
}
  
void DrawX (left, right, line, delv, color, prtext, tic)
int left, right, line, color, prtext, tic;
float delv;
{
    char  text[6];
    int   i, posn, ntics;
    float nseg, delx, val;
  
    /*  Draw the bottom line  */
    DrawLine (line, left, line, right, color);
  
    /*  Draw the tics  */
    nseg = maxvx / delv;
    delx = (float) (right - left) / nseg;
    ntics = (int) nseg + 1;
  
    for (i=0; i < ntics; i++) {
        posn = left + (int) i*delx;
        DrawLine (line, posn, line - (tic*4), posn, color);
        if ( prtext == 1 && i != 0 ) {
            val = i * delv;
            if ( delv >= 1 )
                sprintf(text, "%d", (int) val);
            if ( delv < 1 && delv >= .1 )
                sprintf(text, "%.1f", val);
            DrawText (text, line-6, posn+1, 5, 0, color);
        }
    }
    ntics = nseg;
    for (i=0; i <= ntics; i++) {
        posn = left + (int) delx/2 + (int) i*delx;
        DrawLine (line, posn, line - (tic*2), posn, color);
    }
}
  
void DrawY (bottom, up, sample, delv, color, prtext, tic)
int   bottom, up, sample, color, prtext, tic;
float delv;
{
    char  text[20];
    int   i, ipos, ntics;
    float posn, off1, off2, nl1, nl2, val, maxn, minn, ftics;
  
    /*  Draw the left line  */
    DrawLine (bottom, sample, up, sample, color);
  
    /*  Draw the tics  */
    off1 = fmod (minvy, delv);
    if ( off1 < - 0.001 ) {
        minn = minvy - off1;
        nl1 = (float) bottom + off1 / (maxvy-minvy) * (float) (bottom-up);
    }
    else if ( off1 > 0.001 ) {
        minn = minvy + delv - off1;
        nl1 = (float)bottom-(delv-off1)/(maxvy-minvy)*(float)(bottom-up);
    }
    else {
        minn = minvy;
        nl1 = (float) bottom;
    }
    off2 = fmod (maxvy, delv);
    if ( off2 < - 0.001 ) {
        maxn = maxvy - delv - off2;
        nl2 = (float) up + (delv+off2) / (maxvy-minvy) * (float) (bottom-up);
    }
    else if ( off2 > 0.001 ) {
        maxn = maxvy - off2;
        nl2 = (float) up + off2 / (maxvy-minvy) * (float) (bottom-up);
    }
    else {
        maxn = maxvy;
        nl2 = (float) up;
    }
    ftics = (maxn-minn) / delv + 1;
    ntics = (int) ftics;
  
    for ( i=0; i < ntics; i++ ) {
        posn = nl1 - ((float) i) / (ntics-1) * (nl1-nl2);
        ipos = (int) posn;
        DrawLine (ipos, sample, ipos, sample+(tic*4), color);
        if ( prtext == 1 ) {
            if ( ! ( i == 0 && off1 == 0 ) )  {
                if ( ! ( i == (ntics-1) && (fabs(off2)) < (delv/3) ) ) {
                    val = minn + (float) i * delv;
                    if ( delv <= .1 )          sprintf (text, "%.2f", val);
                    else if ( delv  <= 1.0)    sprintf (text, "%.1f", val);
                    else if ( delv  < 1000.)   sprintf (text, "%.0f", val);
                    else                       sprintf (text, "%.2E", val);
                    DrawText ( text, ipos-1 , sample+5, 5, 0, color);
                }
            }
        }
    }
    for ( i=0; i < ntics-1; i++ ) {
        posn = nl1 - ((float) i + 0.5 ) / (ntics-1) * (nl1-nl2);
        ipos = (int) posn;
        DrawLine (ipos, sample, ipos, sample+(tic*2), color);
    }
}
  
void OpenHdFile ( stnum, fname )
char *fname;
int  stnum;
{
    FILE *idp;
    char *prog_name = "OpenHdFile";
  
    /*  Compose the filename  */
  
    strcpy (fname, st[stnum].codes);
    if ( SHORT_PERIOD )       strcat (fname, "SP");
    else if ( LONG_PERIOD )   strcat (fname, "LP");
    else                      strcat (fname, "IP");
    if ( comp == 0 )       strcat ( fname, "Z" );
    else if ( comp == 1 )       strcat ( fname, "N" );
    else if ( comp == 2 )       strcat ( fname, "E" );
    else printf("error in openhdfile\n");
    strcat ( fname, ".DAT");
  
    if ( (idp = fopen (fname, "w")) == NULL ) {
        sprintf (menu1[0],"Couldn't open file %s on hard disk.", fname);
        error_and_exit ( prog_name, menu1[0]);
    }
  
    if ( comp == 0 )      id3 = idp;
    else if ( comp == 1 ) id4 = idp;
    else                  id5 = idp;
}
  
void WrHeader ( stnum, yr, dofy, hour, minute, sec )
int   yr, dofy, hour, minute, stnum;
double sec;
{
    FILE  *id;
    char c, ch, chan[5];
    int i;
    char *prog_name = "WRHEADER";
  
    c  = ' ';
    ch = '\n';
    if ( comp == 0 ) id = id3;
    if ( comp == 1 ) id = id4;
    if ( comp == 2 ) id = id5;
  
    /*  Write header parameters to the output file  */
  
    if ( fprintf (id, "%-4s", st[stnum].codes) != 4 )
        error_and_exit (prog_name, "Error 1 for writing header to disk ");
  
    if ( fprintf (id, "%c", c) != 1 )
        error_and_exit (prog_name, "Error 2 for writing header to disk ");
  
    /*  Compose the instrument code  */
    if ( SHORT_PERIOD )       strcpy ( chan, "SP" );
    else if ( LONG_PERIOD )   strcpy ( chan, "LP" );
    else if ( INTERM_PERIOD ) strcpy ( chan, "IP" );
  
    if ( comp == 0 )      strcat ( chan,"Z ");
    else if ( comp == 1 ) strcat ( chan,"N ");
    else if ( comp == 2 ) strcat ( chan,"E ");
  
    for ( i=0; i<4; i++ ) {
        if ( fprintf (id, "%c", chan[i]) == NULL )
            error_and_exit (prog_name, "Error 3 for writing header to disk ");
    }
  
    if ( fprintf (id, " %9.4f", st[stnum].lat) == NULL )
        error_and_exit (prog_name, "Error 4 for writing header to disk ");
  
    if ( fprintf (id, " %9.4f", st[stnum].lon) != 10 )
        error_and_exit (prog_name, "Error 5 for writing header to disk ");
  
    if ( fprintf (id, " %7.1f", st[stnum].eleva) != 8 )
        error_and_exit (prog_name, "Error 6 for writing header to disk ");
  
    if ( fprintf (id, " %4d", yr) != 5 )
        error_and_exit (prog_name, "Error 7 for writing header to disk ");
  
    if ( fprintf (id, " %3d", dofy) != 4 )
        error_and_exit (prog_name, "Error 8 for writing header to disk ");
  
    if ( fprintf (id, " %2d", hour) != 3 )
        error_and_exit (prog_name, "Error 9 for writing header to disk ");
  
    if ( fprintf (id, " %2d", minute) != 3 )
        error_and_exit (prog_name, "Error 10 for writing header to disk ");
  
    if ( fprintf (id, " %5.2f", sec) != 6 )
        error_and_exit (prog_name, "Error 11 for writing header to disk ");
  
    if ( fprintf (id, " %5.2f", hdr.rate) != 6 )
        error_and_exit (prog_name, "Error 12 for writing header to disk ");
  
    if ( fprintf (id, " %12.3E", calib[comp].a0 ) != 13 )
        error_and_exit (prog_name, "Error 13 for writing header to disk ");
  
    if ( fprintf (id, " %12.3E", calib[comp].cal_ds) != 13 )
        error_and_exit (prog_name, "Error 14 for writing header to disk ");
  
    if ( fprintf (id, " %12.3E", calib[comp].cal_dsave) != 13 )
        error_and_exit (prog_name, "Error 15 for writing header to disk ");
  
    if ( fprintf (id, "\n%2d\n", calib[comp].np) != 4 )
        error_and_exit (prog_name, "Error 16 for writing header to disk ");
  
  
    for ( i=0; i < calib[comp].np; i++ )  {
        if ( fprintf (id,"%12.3E %12.3E ", calib[comp].poles[i].x, calib[comp].poles[i].y) == NULL )
            error_and_exit (prog_name, "Error 17 for writing header to disk ");
  
        if ( ((i+1) % 3) == 0 && i != calib[comp].np -1 )  {
            if ( fprintf (id, "%c", ch) != 1 )
                error_and_exit (prog_name, "Error 18 for writing header to disk ");
        }
    }
  
    if ( fprintf (id, "\n%2d\n", calib[comp].nz) != 4 )
        error_and_exit (prog_name, "Error 19 for writing header to disk ");
  
  
    for ( i=0; i < calib[comp].nz; i++ )  {
        if ( fprintf (id,"%12.3E %12.3E ", calib[comp].zeroes[i].x, calib[comp].zeroes[i].y) != 26 )
            error_and_exit (prog_name, "Error 20 for writing header to disk ");
  
        if ( ((i+1) % 3) == 0 && i != calib[comp].nz -1 ) {
            if ( fprintf (id, "%c", ch) != 1 )
                error_and_exit (prog_name, "Error 21 for writing header to disk ");
        }
    }
}
  
void Rotate ( si1, si2, ns, angle, lnpi, lnpo )
int   ns, lnpi, lnpo;
float si1[], si2[], angle;
{
    int i;
    float cosa, sina, con11, con12, con21, con22, tsi1, tsi2;
  
    cosa = cos (TORAD * angle);
    sina = sin (TORAD * angle);
    con11 = cosa;
    con12 = sina;
    con21 = -sina;
    con22 = cosa;
    if ( lnpi == 1 ) {
        con12 = -con12;
        con22 = -con22;
    }
    if ( lnpo == 1 ) {
        con21 = -con21;
        con22 = -con22;
    }
    for ( i=0; i < ns; i++ ) {
        tsi1 = con11 * si1[i] + con12 * si2[i];
        tsi2 = con21 * si1[i] + con22 * si2[i];
        si1[i] = tsi1;
        si2[i] = tsi2;
    }
}
  
  
void HP_Filter (si0, si1, si2, ns, i)
int   ns, i;
float si0[], si1[], si2[];
{
    static float c[3], d[3];
    int  j;
    float *y;
    static float x1,y01;
    static float x11,y11;
    static float x12,y12;
    char *prog_name = "HP_FILTER";
  
    if((y = (float *) malloc ( 990*sizeof(float) )) == NULL )
        error_and_exit(prog_name, "Malloc unsuccessful, Try 1 ");
  
    if( i == 0) {
        hp_filter_coef(c, d);
        y[0] = c[0] * si0[0];
    }
    if ( i > 0 ) {
        y[0] = c[0] * si0[0] + c[1] * x1 + d[1] * y01;
    }
    for(j=1; j<ns; j++) {
        y[j] = c[0] * si0[j] + c[1] * si0[j-1] + d[1] * y[j-1];
        y01 = y[j];
        x1 = si0[j];
    }
    for(j=0;j<ns;j++) si0[j] = y[j];
    free(y);
    if(hdr.nchan==1)return;
  
    if((y = (float *) malloc ( 990*sizeof(float) )) == NULL )
        error_and_exit(prog_name, "Malloc unsuccessful, Try 2 ");
  
    if( i == 0) {
        hp_filter_coef(c, d);
        y[0] = c[0] * si0[0];
    }
    else if ( i > 0 ) {
        y[0] = c[0] * si0[0] + c[1] * x11 + d[1] * y11;
    }
    for(j=2; j<ns; j++) {
        y[j] = c[0] * si0[j] + c[1] * si0[j-1] + d[1] * y[j-1];
        y11 = y[j];
        x11 = si0[j];
    }
    for(j=0;j<ns;j++) si1[j] = y[j];
    free(y);
  
    if((y = (float *) malloc ( 990*sizeof(float) )) == NULL )
        error_and_exit(prog_name, "Malloc unsuccessful, Try 3 ");
  
    if( i == 0) {
        hp_filter_coef(c, d);
        y[0] = c[0] * si0[0];
    }
    else if ( i > 0 ) {
        y[0] = c[0] * si0[0] + c[1] * x12 + d[1] * y12;
    }
    for(j=2; j<ns; j++) {
        y[j] = c[0] * si0[j] + c[1] * si0[j-1] + d[1] * y[j-1];
        y12 = y[j];
        x12 = si0[j];
    }
    for(j=0;j<ns;j++) si2[j] = y[j];
    free(y);
}
  
void BP_Filter (si0, si1, si2, ns, i)
int   ns, i;
float si0[], si1[], si2[];
{
    static float c[3], d[3];
    int  j;
    float dum;
    float *y;
    static float x1,x2,y01,y2;
    static float x11,x21,y11,y21;
    static float x12,x22,y12,y22;
    char *prog_name = "BP_Filter";
  
    if((y = (float *) malloc ( 990*sizeof(float) )) == NULL )
        error_and_exit(prog_name, "Malloc unsuccessful, Try 1 ");
    if( i == 0) {
        bp_filter_coef(c, d);
        y[0] = 0.0;
        y[1] = c[1] * si0[0] + d[1] * y[0];
    }
    if ( i > 0 ) {
        dum = c[1] * x1 + c[2] * x2;
        y[0] = dum + d[1] * y01 + d[2] * y2;
        dum = c[1] * si0[0] + c[2] * x1;
        y[1] = dum + d[1] * y[0] + d[2] * y01;
    }
  
    for(j=2; j<ns; j++) {
        dum = c[1] * si0[j-1] + c[2] * si0[j-2];
        y[j] = dum + d[1] * y[j-1] + d[2] * y[j-2];
        y01 = y[j];
        y2 = y[j-1];
        x1 = si0[j];
        x2 = si0[j-1];
    }
    for(j=0;j<ns;j++) si0[j] = y[j];
    free(y);
    if(hdr.nchan==1) return;
  
    if((y = (float *) malloc ( 990*sizeof(float) )) == NULL )
        error_and_exit(prog_name, "Malloc unsuccessful, Try 2 ");
    if( i == 0) {
        bp_filter_coef(c, d);
        y[0] = 0.0;
        y[1] = c[1] * si1[0] + d[1] * y[0];
    }
    if ( i > 0 ) {
        dum = c[1] * x11 + c[2] * x21;
        y[0] = dum + d[1] * y11 + d[2] * y21;
        dum = c[1] * si1[0] + c[2] * x11;
        y[1] = dum + d[1] * y[0] + d[2] * y11;
    }
    for(j=2; j<ns; j++) {
        dum = c[1] * si1[j-1] + c[2] * si1[j-2];
        y[j] = dum + d[1] * y[j-1] + d[2] * y[j-2];
        y11 = y[j];
        y21 = y[j-1];
        x11 = si1[j];
        x21 = si1[j-1];
    }
    for(j=0;j<ns;j++) si1[j] = y[j];
    free(y);
  
    if((y = (float *) malloc ( 990*sizeof(float) )) == NULL )
        error_and_exit(prog_name, "Malloc unsuccessful, Try 3 ");
    if( i == 0) {
        bp_filter_coef(c, d);
        y[0] = 0.0;
        y[1] = c[1] * si2[0] + d[1] * y[0];
    }
    if ( i > 0 ) {
        dum = c[1] * x12 + c[2] * x22;
        y[0] = dum + d[1] * y12 + d[2] * y22;
        dum = c[1] * si2[0] + c[2] * x12;
        y[1] = dum + d[1] * y[0] + d[2] * y12;
    }
    for(j=2; j<ns; j++) {
        dum = c[1] * si2[j-1] + c[2] * si2[j-2];
        y[j] = dum + d[1] * y[j-1] + d[2] * y[j-2];
        y12 = y[j];
        y22 = y[j-1];
        x12 = si2[j];
        x22 = si2[j-1];
    }
    for(j=0;j<ns;j++) si2[j] = y[j];
    free(y);
}
  
/*  compute bandpass filter coefficients in the time domain. */
  
void bp_filter_coef( a, b)
float a[], b[];
{
    float dt, wc1, wc2, what1, what2, wh;
  
    if ( BPFILTPAR ) {
        wc1 = upper_frq;
        wc2 = lower_frq;
    }
    else printf("Filter parameters not known. \n");
  
    dt= 1./hdr.rate;
  
    /*
    what1=2.0*tan(3.14159265*wc1*dt);
    what2=2.0*tan(3.14159265*wc2*dt);
    */
    what1=tan(3.14159265*wc1*dt);
    what2=tan(3.14159265*wc2*dt);
    wh= (1+what1)*(1+what2);
    a[0]=0.0;
    a[1]= -what2/wh;
    a[2]= -a[1];
    b[0]=1.0;
    b[1]= ((1+what1)*(1-what2)+(1-what1)*(1+what2))/wh;
    b[2]= (-1)*(1-what1)*(1-what2)/wh;
  
}
  
void hp_filter_coef( a, b)
float a[], b[];
{
    float dt, wc1, what;
  
    if ( HPFILTPAR ) {
        wc1 = upper_frq;
    }
    else {
        printf("Filter parameters not known. \n");
        pause(10);
    }
  
    dt= 1./hdr.rate;
  
    what=tan(3.14159265*wc1*dt);
    a[0]=1./(1+what);
    a[1]= -a[0];
    b[0]=0.0;
    b[1]=(1-what)/(1+what);
}
  
void user1_time ( begintime, endtime, evnum )
float  *begintime, *endtime;
int    evnum;
{
    int   i;
    float length, deflt;
  
    set_colour (3,0,22,79,24);
    blankstring (menu1[0]);
    if(evnum==1 || evnum==999)
        sprintf(menu1[0]," ENTER LENGTH OF SIGNAL TO BE PLOTTED (default: %8.2f seconds):    ",
        (*endtime- *begintime));
    else        
        sprintf(menu1[0]," ENTER LENGTH OF SIGNAL FROM ORIGIN TIME (default: %8.2f seconds): ",
        (*endtime- *begintime));  
    textdisp(menu1, 0, 23, 79, 23);

    set_curs(23,71);
    deflt = 0.;
    myfloat(deflt,&length);
    if (length > 0.)  *endtime = *begintime + length;
}
  
void user2_time ( t0, t1, evtime )
float *t0, *t1, evtime ;
{
    float deflt, length, offs;
    int   i;
  
    set_colour (3,0,22,79,24);
    blankstring (menu1[0]);
    sprintf(menu1[0]," ENTER OFFSET FROM ORIGIN TIME (default: %8.2f) ",(*t0-evtime));
    textdisp(menu1, 0, 22, 79, 22);

    blankstring (menu1[0]);
    sprintf(menu1[0]," ENTER LENGTH OF SIGNAL TO BE PLOTTED (default: %8.2f) ",(*t1-*t0));
    textdisp(menu1, 0, 23, 79, 23);

    set_curs(22,62); 
    deflt = *t0-evtime;
    myfloat(deflt,&offs);
  
    set_curs(23,62);
    deflt = *t1 - *t0;
    myfloat(deflt,&length);
  
    *t0 = evtime + offs;
    *t1 = *t0 + length;

    blankstring (menu1[0]);
    textdisp(menu1,0,22,79,22);
    textdisp(menu1,0,23,79,23); 
    textdisp(menu1,0,24,79,24);
}


