/***   Module: OPENCD2 - Open Compact Disc (version for SONIC2).
---------------------------------------------------------
  
OPENCD2 contains the functions:
  
title        : To plot the program titles on the screen.
contents     : To determine the directories on CD-ROM and
select the year or month of interest.
selparam     : To get the event and/or station parameters.
getfilenames : determines the names of the file paths for CD-ROM.
openfile     : opens a specified file name on CD-ROM.
  
programmmed  : march 1989.
  
version 1.81
version 1.93  - september 1990
  
***/
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <conio.h>
#include <time.h>
#include "menu.h"
#include "sonic.h"
  
#define LINESIZE 81
#define MAXFILE  30
#define PATHL    50
  
extern char *menu1[];             /*  Defined in module Menu  */
extern char *mnu2[];
extern char *mnu3[];
extern int DisplayDevice;         /*  Defined in module Dispdriv  */
extern int dispnl;
extern int dispns;
  
FILE *id1;                        /*  File identity for CD-ROM I/O  */
char filepath[MAXFILE][PATHL];    /*  Strings for the path names of files  */
static char drpath[50];           /*  Drive letter for CD-ROM  */
char year[6][5];                  /*  Strings for determined years  */
char mon1[12][4];                 /*  Strings for determined months  */
char mon2[12][4];                 /*     "     "      "        "     */
char mon3[12][4];                 /*     "     "      "        "     */
char mon4[12][4];                 /*     "     "      "        "     */
char mon5[12][4];                 /*     "     "      "        "     */
char mon6[12][4];                 /*     "     "      "        "     */
char mmon[18][50];                /*  Strings for menu  */
  
char mon[12][4] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
"JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};
  
char *mesg[] = {
    "                       WELCOME TO                           ",
    "                                                            ",
    "               SONIC II  1.97  ( AUGUST 1991 )              ",
    "                                                            ",
    "    THIS  PROGRAM SELECTS  SEISMIC HEADER  DATA STORED      ",
    "    ON  CD-ROM AND WRITES A RESULT FILE  TO HARD DISK.      ",
    "   IT IS WRITTEN FOR IBM AND COMPATIBLE PERSONAL COMPUTERS  ",
    "   WITH THE ENHANCED GRAPHICS ADAPTER, THE COLOR GRAPHICS   ",
    "   ADAPTER  OR  THE PROFESSIONAL GRAPHICS ADAPTER.          ",
    "                                                            ",
    "   THIS PROGRAM IS WRITTEN FOR USERS OF THE EVENT CD-ROM    ",
    "                        CREATED BY                          ",
    "                   THE ORFEUS DATA CENTER                   ",
    "                            AND                             ",
    "         THE NATIONAL EARTHQUAKE INFORMATION CENTER         ",
    "                                                            ",
    "  YOUR SYSTEM MUST BE EQUIPPED WITH A CGA, EGA, VGA OR PGA. ",
    "      READ THE INFO.DOC FILE FOR THE LATEST REVISIONS.      ",
""};
  
  
char *menuA[] = {
    " SEARCH OF :        ",
    "                    ",
    " EVENTS             ",
    " EVENTS AND STATIONS",
    "                    ",
    " SCAN OF   :        ",
    "                    ",
    " MONTH              ",
    " YEAR               ",
    " WHOLE CD-ROM       ",
    "                    ",
    " CONTINUE           ",
    " QUIT               ",
""};
  
  
char *param[] = {
    " EVENT PARAMETERS :                                         ",
    "                                                            ",
    " 1. min latitude  :           2. max latitude  :            ",
    " 3. min longitude :           4. max longitude :            ",
    " 5. min depth     :           6. max depth     :            ",
    " 7. min magnitude :           8. max magnitude :            ",
    "                                                            ",
    " STATION PARAMETERS :                                       ",
    "                                                            ",
    " 9. min latitude  :          10. max latitude  :            ",
    "11. min longitude :          12. max longitude :            ",
    "13. min delta     :          14. max delta     :            ",
    "15. min azimuth   :          16. max azimuth   :            ",
""};
  
int  nmain;             /*  Number of main directories on CD-ROM  */
int  nsub[6];           /*  Number of subdirectories for each main dir  */
int  cd_disc;
  
float evlat1, evlat2, evlon1, evlon2, dep1, dep2, magn1, magn2;
float stlat1, stlat2, stlon1, stlon2, del1, del2, azis1, azis2;
  
int REFRESH_FREE;
  
void title ()
{
    char text[LINESIZE], ch;
    int  i, color, line, sample, lsize, ssize;
  
    /*  Plot the first screen (graphics mode)  */
    REFRESH_FREE = TRUE;
    InitDisplay();
  
    if ( DisplayDevice == 0 )
        ClearDisplay(0);
    else {
        clearscr();
        ClearDisplay(15);
    }
  
    /*  Set Font Type equal to 5  */
    Font (5);
  
    sample = dispns/10;
    lsize = dispnl/6+1;
    ssize = dispns/11+5;
 
    /*  Draw the colored boxes  */
    for ( i=0; i < 5; i++ ) {
        if ( DisplayDevice == 0 ) color = 1;
        else {
            if ( i == 0 || i == 3 ) color = 9;
            else if ( i == 1 || i == 4 ) color = 11;
            else if ( i == 2 ) color = 15;
        }
  
        line = dispnl / 12 + i * dispnl / 6;
  
        DrawBox (line, sample, lsize, ssize, color);
    }
  
    if ( DisplayDevice == 0 ) color = 0;
    else   color = 16;
  
    /*  Draw the text  */
    strcpy(text,"S");
    for ( i=0; i < 3; i++ ) {
        sample = dispns/10+dispns/128+i;
        DrawText (text, (dispnl/4-dispnl/70), sample, (dispnl/7), 0, color);
    }
    strcpy(text,"O");
    for ( i=0; i < 3; i++ ) {
        sample = dispns/10+dispns/128+i;
        DrawText (text, (dispnl*5/12-dispnl/70), sample, (dispnl/7), 0, color);
    }
    strcpy(text,"N");
    for ( i=0; i < 3; i++ ) {
        sample = dispns/10+dispns/128+i;
        DrawText (text, (dispnl*7/12-dispnl/70), sample, (dispnl/7), 0, color);
    }
    strcpy(text,"I");
    for ( i=0; i < 3; i++ ) {
        sample = dispns/10+dispns/32+i;
        DrawText (text, (dispnl*3/4-dispnl/70), sample, (dispnl/7), 0, color);
    }
    strcpy(text,"C");
    for ( i=0; i < 3; i++ ) {
        sample = dispns/10+dispns/128+i;
        DrawText (text, (dispnl*11/12-dispnl/70), sample, (dispnl/7), 0, color);
    }
  
    if ( DisplayDevice == 0 ) color = 1;
    strcpy (text,"eismic Software");
    DrawText(text, (dispnl/4-dispnl/70), (dispns/5), (dispnl/10), 0, color);
    strcpy (text,"RFEUS");
    DrawText(text, (dispnl*5/12-dispnl/70), (dispns/5), (dispnl/10), 0, color);
    strcpy (text,"EIC");
    DrawText(text, (dispnl*7/12-dispnl/70), (dispns/5), (dispnl/10), 0, color);
    strcpy (text,"nternational");
    DrawText(text, (dispnl*3/4-dispnl/70), (dispns/5), (dispnl/10), 0, color);
    strcpy (text,"ooperation");
    DrawText(text, (dispnl*11/12-dispnl/70), (dispns/5), (dispnl/10), 0,color);
  
    strcpy (text,"by");
    DrawText(text, (dispnl/4+dispnl/25), (dispns/2), (dispnl/25), 0, color);
    strcpy (text,"and");
    DrawText(text, (dispnl*5/12+dispnl/25), (dispns/2), (dispnl/25), 0, color);
    strcpy (text,"through");
    DrawText(text, (dispnl*7/12+dispnl/25), (dispns/3),(dispnl/25), 0, color);
  
    /*  Set Font Type equal to zero again  */
    Font (0);
  
    /*  Pause before display is turned off  */
    pause(5);
    DisplayOff();
  
    /*  Print the second sreen (text mode)  */
    clearscr();
    set_colour(1, 0, 0, 79, 24 );
  
    if ( DisplayDevice == 0 ) color = BLACK << 4 | WHITE;
    else   color = BLUE << 4 | CYAN;
    menucol (5, 2, 74, 21, color);
  
    set_colour (2, 10, 5, 69, 5);
  
    set_colour (3, 9, 19, 68, 20);
    textdisp (mesg, 9, 3, 67, 20);
  
    /*  Choose your cd-rom drive letter  */
    set_colour (2, 5, 23, 74, 23); 

    blankstring(menu1[0]);
    strcpy (menu1[0]," ENTER CD-ROM DRIVE [L]: ");
    textdisp (menu1, 5, 23, 79, 23);
    set_curs (23, 30);
        
    mystring(50,drpath);
    i = strlen(drpath);
    if ( i==0 ) strcpy ( drpath,"L:");
    else if ( i==1 ) strcat ( drpath,":");
}
  
int contents2 ( count, cdinfo, ncol, stinfo )
char *cdinfo, *stinfo;
int  count, *ncol;
{
    char stmon[5], ch;
    char *menuB[14], months[12][4], path[50], tempmonth[5];
    int  i, j, k, row, col, fmon, color2, valmonth;
    int  lry0, lry[6], ipos[6], inpt;
    int k1, k2;
    struct find_t cdrom;
    char *prog_name = "CONTENTS";
  
    if ( count == 0 ) {
  
        /*  Determine volume ID.  */
  
        do {
            strcpy (path, drpath);
            strcat (path,"\\*");
  
            if ( _dos_findfirst ( path, _A_SUBDIR, &cdrom ) != 0 ) {
                set_colour ( 2, 5, 23, 74, 23 );
                set_colour ( 3, 5, 23, 35, 23);
                
                blankstring(menu1[0]);
                strcpy (menu1[0], "  DRIVE LETTER IS NOT CORRECT. ENTER NEW LETTER  (Q to QUIT): ");
                textdisp (menu1, 5, 23, strlen(menu1[0])+14, 23);
                set_curs (23, 71);
                                         
                mystring(50,drpath);
                i = strlen(drpath);
                if ( i==0 ) strcpy ( drpath,"L:");
                else if ( i==1 ) strcat ( drpath,":");
                if ( drpath[0] == 'q' ||  drpath[0] == 'Q' ) return(ESC);
            }
        } while ( _dos_findfirst ( path, _A_SUBDIR, &cdrom ) != 0 ) ;
  
        /*  Determine disc type  */
  
        if ( _dos_findfirst ( path, _A_SUBDIR, &cdrom ) != 0 )
            error_and_exit (prog_name, "Error reading volume. ");
        if ( strncmp ( cdrom.name, "ORFEUS", 6 ) == 0 ) cd_disc = 5;
        else  cd_disc = 0;
  
        j = 0;
  
        /*  Determine the main directories ( YEARS ).  */
  
        if ( _dos_findfirst ( path, _A_SUBDIR, &cdrom ) != 0 )
            error_and_exit (prog_name, "Error reading year directory. ");
  
        do {
            if (strlen(cdrom.name) == 4) {
                strcpy (tempmonth, cdrom.name);
                valmonth = atoi(tempmonth);
                if (valmonth == 0) continue;
                strcpy ( year[j++], cdrom.name) ;
              }
        }  while ( _dos_findnext ( &cdrom ) == 0 ) ;
  
        nmain = j; /*  mdir is the number of main directories found  */
        if (nmain == 0) error_and_exit(prog_name, "No year directories found. Please check for the correct CD-ROM.");
        if ( nmain == 1 && (strcmp(year[0],"1988")==0) ) cd_disc =5;
  
        for (i=0; i < 14; i++) strcpy (mmon[i], "");
  
        /* Loop to find the subdirectories: MONTHS  */
        for (i=0; i < nmain; i++)  {
            strcat (mmon[0], year[i]);
            strcat (mmon[0], "    ");
            strcat (mmon[1], "        ");
  
            k = 0;
            fmon = 11;
  
            strcpy ( path, drpath);
            strcat ( path, "\\" );
            strcat ( path, year[i] );
            strcat ( path, "\\*" );
  
            if ( _dos_findfirst ( path, _A_SUBDIR, &cdrom ) != 0 )
                error_and_exit (prog_name, "Error reading month directories. " );
            do {
                if (strlen(cdrom.name) != 3) continue;
                for ( j=0; j<12; j++ ) {
/*                    if ( strncmp ( cdrom.name,"VLP",3 ) == 0 ) break; */
                    if ( strncmp ( cdrom.name, mon[j], 3 ) == 0 ) {
                        if ( j < fmon )  fmon = j;
                        k++;
                        strcpy (months[j], cdrom.name);
                        break;
                    }
                }
            } while ( _dos_findnext (  &cdrom ) == 0 );
            if (k == 0) error_and_exit(prog_name, "No month directories found. Please check for the correct CD-ROM");
  
            nsub[i] = k;
  
            /*  Rearrange array months if first month is not january  */
            if ( fmon > 0 ) {
                for (j=0; j < nsub[i]; j++)
                    strcpy (months[j], months[j + fmon]);
            }
  
            /*  If total number of months is smaller than twelve set months  */
            /*  equal to spaces  */
            if ( nsub[i] < 12 ) {
                for (j=nsub[i]; j < 12; j++)
                    strcpy (months[j], "   ");
            }
  
            /*  Store the found months in strings for menu and memory  */
            for (j=0; j < 12; j++) {
                if ( i == 0 ) strcpy (mmon[j+2], months[j]);
                else    strcat (mmon[j+2], months[j]);
  
                strcat (mmon[j+2],"     ");
                if ( i == 0 ) strcpy (mon1[j], months[j]);
                else if ( i == 1 ) strcpy (mon2[j], months[j]);
                else if ( i == 2 ) strcpy (mon3[j], months[j]);
                else if ( i == 3 ) strcpy (mon4[j], months[j]);
                else if ( i == 4 ) strcpy (mon5[j], months[j]);
                else if ( i == 5 ) strcpy (mon6[j], months[j]);
            }
  
        }  /*  End of do-loop over sub directories  */
  
    }  /*  End of if ( count == 0 )  */
  
    clearscr();
    screen_setup();
  
    /*  Menu A is displayed  */
    if ( DisplayDevice != 0) {
       color2 = BLUE << 4 | LTCYAN;
       menucol (0, 5, 14, 5, color2);
       menucol (0, 10, 14, 10, color2);
    }
  
    if ( count == 0 ) lry0 = 14;
    else lry0 = 17;
  
    textdisp (menuA, 5, 5, 24, lry0);
  
    /*  Menu B is displayed  */
    for (i=0; i < 14; i++) menuB[i] = &mmon[i][0];
  
    textdisp (menuB, 32, 5, (strlen(mmon[0])+31), 18);
  
    if ( count == 1 ) {
        blankstring (menu1[0]);
        strcpy (menu1[0], " SELECTION");
        textdisp (menu1, 0, 1, 79, 1);
  
        textdisp (mnu2, 0, 23, strlen(mnu2[0])-1, 23);
  
        row = 16;
        col = 0;
        inpt = menucurs6 (&row, &col, 4,15,15,18,
                                      4,15,15,18,
                                      4,15,15,18,
                                      4,15,15,18,
                                      4,15,15,18,
                                      4,15,15,18);
  
        if ( inpt == ESC || row == 17 ) return(ESC);
    }
  
    /*  Cursor selection for first menu */
    do {
        blankstring (menu1[0]);
        strcpy (menu1[0]," SELECTION");
        textdisp (menu1, 0, 1, 79, 1);
  
        textdisp (mnu2, 0, 23, strlen(mnu3[0])-1, 23);
  
        row = 7;
        col = 0;
        inpt = menucurs6 (&row, &col, 4,6,26,9,
                                      4,6,26,9,
                                      4,6,26,9,
                                      4,6,26,9,
                                      4,6,26,9,
                                      4,6,26,9);
  
        if ( inpt == ESC ) return (ESC);
  
        if ( row == 7 ) *stinfo = 'n';
        else if ( row == 8 ) *stinfo = 'y';
  
        /*  Cursor selection for second menu */
p0:     blankstring (menu1[0]);
        strcpy (menu1[0], " SELECTION");
        textdisp (menu1, 0, 1, 79, 1);
  
        textdisp (mnu3, 0, 23, strlen(mnu3[0])-1, 23);
  
        row = 12;
        col = 0;
        inpt = menucurs6 (&row, &col, 4,11,19,15,
                                      4,11,19,15,
                                      4,11,19,15,
                                      4,11,19,15,
                                      4,11,19,15,
                                      4,11,19,15);
  
        if ( inpt == ESC ) re_vid (row, 4, row, 19, row);
  
    } while ( inpt == ESC );
  
    /*  Cursor selection from menu gives year and month  */
    if ( row == 12 ) {
        blankstring (menu1[0]);
        strcpy (menu1[0], " SELECT MONTH");
        textdisp (menu1, 0, 1, 79, 1);
  
        row = 7;
        col = 0;
        for (i=0; i<nmain; i++) {
            lry[i] = nsub[i]+7;
            ipos[i] = i*8 + 30;
        }
        if (nmain < 6) {
            for (i=nmain; i<6; i++) {
                lry[i] = lry[i-1];
                ipos[i] = ipos[i-1];
            }
        }
        inpt = menucurs6 (&row, &col,ipos[0],6,ipos[0]+6,lry[0],
                                     ipos[1],6,ipos[1]+6,lry[1],
                                     ipos[2],6,ipos[2]+6,lry[2],
                                     ipos[3],6,ipos[3]+6,lry[3],
                                     ipos[4],6,ipos[4]+6,lry[4],
                                     ipos[5],6,ipos[5]+6,lry[5]);
  
        /*  The page bottom is displayed  */
        textdisp (mnu3, 0, 23, strlen(mnu3[0])-1, 23);
  
        if ( inpt == ESC ) {
            i = col*8 + 30;
            re_vid (row, i, row, i+6, row);
  
            goto p0;
        }
        else {
            strcpy (cdinfo, year[col]);
            row -= 5;
  
            i = col*8;
            strncpy(stmon,mmon[row]+i,3);
            stmon[3] = '\0';
            strcat(cdinfo,stmon);
  
            *ncol = col;
        }
  
    } /*  End of if ( row == 12 )  */
  
    else if ( row == 13 ) {
  
        blankstring (menu1[0]);
        strcpy (menu1[0], " SELECT YEAR");
        textdisp (menu1, 0, 1, 79, 1);
  
        row = 5;
        col = 0;
  
        for (i=0; i<nmain; i++) {
            ipos[i] = i*8 + 30;
        }
        if (nmain < 6) {
            for (i=nmain; i<6; i++) {
                ipos[i] = ipos[i-1];
            }
        }
        inpt = menucurs6 (&row, &col,ipos[0],4,ipos[0]+7,6,
                                     ipos[1],4,ipos[1]+7,6,
                                     ipos[2],4,ipos[2]+7,6,
                                     ipos[3],4,ipos[3]+7,6,
                                     ipos[4],4,ipos[4]+7,6,
                                     ipos[5],4,ipos[5]+7,6);
  
        /*  The page bottom is displayed  */
        textdisp (mnu3, 0, 23, strlen(mnu3[0])-1, 23);
  
        if ( inpt == ESC ) {
            k1 = 30 + col*8;
            k2 = k1 + 7;
            re_vid ( row, k1, row, k2, row);
            goto p0;
        }
        else {
            strcpy (cdinfo, year[col]);
            *ncol = col;
        }
  
    }  /*  End of if ( row == 13 )  */
  
    else if ( row == 14 ) {
        strcpy (cdinfo, "ALL");
        *ncol = 0;
    }
    return (1);
}
  
void selparam ( count, stinfo )
int  count;
char stinfo;
{
    char *fmenu[13], str1[3], str2[87];
    int i, answ, lry;
    float deflt;
  
    /*  Default settings for parameters retreive all events and stations  */
  
    if ( count == 0 ) {
        evlat1=-90; evlat2=90; evlon1=-180; evlon2=180; dep1=0; dep2=800;
        magn1=4.5; magn2=8.5;
        stlat1=-90; stlat2=90; stlon1=-180; stlon2=180; del1=0; del2=180;
        azis1=0; azis2=360;
    }
  
    strcpy (str1, "  ");
    strcpy (str2, "       ");
  
    /*  Store the default parameters in strinds param  */
    sprintf (param[2]+20,  "%7.2f", evlat1);
    sprintf (param[2]+49,  "%7.2f", evlat2);
    sprintf (param[3]+20,  "%7.2f", evlon1);
    sprintf (param[3]+49,  "%7.2f", evlon2);
    sprintf (param[4]+20,  "%7.2f", dep1);
    sprintf (param[4]+49,  "%7.2f", dep2);
    sprintf (param[5]+20,  "%7.2f", magn1);
    sprintf (param[5]+49,  "%7.2f", magn2);
    sprintf (param[9]+20,  "%7.2f", stlat1);
    sprintf (param[9]+49,  "%7.2f", stlat2);
    sprintf (param[10]+20, "%7.2f", stlon1);
    sprintf (param[10]+49, "%7.2f", stlon2);
    sprintf (param[11]+20, "%7.2f", del1);
    sprintf (param[11]+49, "%7.2f", del2);
    sprintf (param[12]+20, "%7.2f", azis1);
    sprintf (param[12]+49, "%7.2f", azis2);
    
    clearscr();
    screen_setup();
 
    blankstring(menu1[0]);
    if ( stinfo == 'y' ) strcpy (menu1[0]," EVENT AND STATION PARAMETERS");
    else strcpy (menu1[0]," EVENT PARAMETERS");
    textdisp (menu1, 0, 1, 79, 1);
  
    for (i=0; i < 13; i++) fmenu[i] = &param[i][0];
  
    if ( stinfo == 'y' ) lry = 17;
    else lry = 11;
                             
    blankstring(menu1[0]);
    strcpy (menu1[0],"  ENTER THE NUMBER OF THE PARAMETER TO CHANGE  (0 IF READY) : ");
  
    do {
        do {
            screen_setup();
            textdisp (fmenu, 2, 5, 61, lry);
            textdisp (menu1, 0, 23, 79, 23);
 
            set_curs (23, 63);
            mystring (3,str1);
            if (strlen(str1) > 0) sscanf(str1,"%d",&answ);
            else answ = 0;
            set_curs (23, 63);
  
        } while ( ( stinfo == 'y' && ( answ < 0 || answ > 16 ) ) ||
        ( stinfo == 'n' && ( answ < 0 || answ > 8 ) ) );
  
        switch ( answ ) {
            case 1:
                set_curs (7, 22);
                printf("%s", str2); 
                set_curs (7, 22);
                deflt = evlat1;
                myfloat(deflt,&evlat1);
                sprintf (param[2]+20,  "%7.2f", evlat1);
                break;
            case 2:
                set_curs (7, 51);
                printf("%s", str2);
                set_curs (7, 51);
                deflt = evlat2;
                myfloat(deflt,&evlat2);
                sprintf (param[2]+49,  "%7.2f", evlat2);
                break;
            case 3:
                set_curs (8, 22);
                printf("%s", str2);
                set_curs (8, 22);
                deflt = evlon1;
                myfloat(deflt,&evlon1);
                sprintf (param[3]+20,  "%7.2f", evlon1);
                break;
            case 4:
                set_curs (8, 51);
                printf("%s", str2);
                set_curs (8, 51);
                deflt = evlon2;
                myfloat(deflt,&evlon2);
                sprintf (param[3]+49,  "%7.2f", evlon2);
                break;
            case 5:
                set_curs (9, 22);
                printf("%s", str2);
                set_curs (9, 22);
                deflt = dep1;
                myfloat(deflt,&dep1);
                sprintf (param[4]+20,  "%7.2f", dep1);
                break;
            case 6:
                set_curs (9, 51);
                printf("%s", str2);
                set_curs (9, 51);
                deflt = dep2;
                myfloat(deflt,&dep2);
                sprintf (param[4]+49,  "%7.2f", dep2);
                break;
            case 7:
                set_curs (10, 22);
                printf("%s", str2);
                set_curs (10, 22);
                deflt = magn1;
                myfloat(deflt,&magn1);
                sprintf (param[5]+20,  "%7.2f", magn1);
                break;
            case 8:
                set_curs (10, 51);
                printf("%s", str2);
                set_curs (10, 51);
                deflt = magn2;
                myfloat(deflt,&magn2);
                sprintf (param[5]+49,  "%7.2f", magn2);
                break;
            case 9:
                set_curs (14, 22);
                printf("%s", str2);
                set_curs (14, 22);
                deflt = stlat1;
                myfloat(deflt,&stlat1);
                sprintf (param[9]+20,  "%7.2f", stlat1);
                break;
            case 10:
                set_curs (14, 51);
                printf("%s", str2);
                set_curs (14, 51);
                deflt = stlat2;
                myfloat(deflt,&stlat2);
                sprintf (param[9]+49,  "%7.2f", stlat2);
                break;
            case 11:
                set_curs (15, 22);
                printf("%s", str2);
                set_curs (15, 22);
                deflt = stlon1;
                myfloat(deflt,&stlon1);
                sprintf (param[10]+49, "%7.2f", stlon2);
                break;
            case 12:
                set_curs (15, 51);
                printf("%s", str2);
                set_curs (15, 51);
                deflt = stlon2;
                myfloat(deflt,&stlon2);
                sprintf (param[10]+49, "%7.2f", stlon2);
                break;
            case 13:
                set_curs (16, 22);
                printf("%s", str2);
                set_curs (16, 22);
                deflt = del1;
                myfloat(deflt,&del1);
                sprintf (param[11]+20, "%7.2f", del1);
                break;
            case 14:
                set_curs (16, 51);
                printf("%s", str2);
                set_curs (16, 51);
                deflt = del2;
                myfloat(deflt,&del2);
                sprintf (param[11]+49, "%7.2f", del2);
                break;
            case 15:
                set_curs (17, 22);
                printf("%s", str2);
                set_curs (17, 22);
                deflt = azis1;
                myfloat(deflt,&azis1);
                sprintf (param[12]+20, "%7.2f", azis1);
                break;
            case 16:
                set_curs (17, 51);
                printf("%s", str2);
                set_curs (17, 51);
                deflt = azis2;
                myfloat(deflt,&azis2);
                sprintf (param[12]+49, "%7.2f", azis2);
                break;
        }
    } while ( answ != 0 );
}
  
void getfilenames ( mdir, subdir, nfiles )
char mdir[],subdir[];
int *nfiles;
{
    char path[50];
    int num=0;
    struct find_t cdrom;
    char *prog_name = "GETFILENAMES";
  
    /*  DOS-command. Result to file out  */
    strcpy (path,drpath);
    strcat (path, "\\");
    strcat (path, mdir);
    strcat (path, "\\");
    strcat (path, subdir);
    strcat (path, "\\*.EVT");
  
    if ( _dos_findfirst ( path, _A_RDONLY, &cdrom ) != 0 )
        error_and_exit (prog_name,"Error reading eventfiles. ");
  
    do {
        if ( strncmp ( cdrom.name, ".", 1 ) != 0 ) {
            strcpy (filepath[num], drpath);
            strcat (filepath[num], "\\");
            strcat (filepath[num], mdir);
            strcat (filepath[num], "\\");
            strcat (filepath[num], subdir);
            strcat (filepath[num], "\\");
            strcat (filepath[num], cdrom.name);
            strcat (filepath[num], '\0');
  
            num++;
        }
    } while ( _dos_findnext ( &cdrom ) == 0 );
  
    *nfiles = num;
}
  
void openfile2 ( num )
int num;
{
        /*  Open file on CD-ROM (High level I/O)  */
    if(( id1=fopen(filepath[num],"rb")) == NULL) {
        sprintf(menu1[0],"Could not open file %s",filepath[num]);
        error_and_exit ("openfile2", menu1[0]);
    }
}
void pause (mz)
int mz;
{
    time_t time1, time2;
    
    time(&time1);
    do {
        time(&time2);
        if (time2-time1 > mz) break;
    } while (1);
}
