/**********************************************************************/
/**               Module: RDLOG - Read Log.                               **/
/***************************************************************************/
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "menu.h"
#include "struct.h"
#include "sonic.h"
  
#define LINESIZE     81
#define NEVT         18      /*  Maximum number of events for 1 menu page  */
#define PI           3.14159265359
#define HALFPI        .5*PI
#define TWOPI         (2.0*PI)
#define DEG           (180./PI)        /* Degrees/Radian */
#define RAD           (PI/180.)        /* Radians/Degree */
#define EQTRAD        6378.163
#define POLRAD        6356.177
#define FLATTENING    (EQTRAD-POLRAD)/EQTRAD
  
extern    FILE    *id1;               /*  Defined in module opencd  */
extern    BOOL    ORF2;
extern    BYTE    *menu1[];           /*  Defined in module menu  */
extern    BYTE    *mnu3[];
extern    INT     plchoice;
extern  INT     SHORT_PERIOD;
extern  INT     LONG_PERIOD;
extern  INT     comp;
  
float  st0, ct0, phi0;
  
char   mstat[18][81];            /*  Menu strings for station selection  */
char   cdev[NEVT][81];           /*  Strings for event information  */
int    cdver;                    /*  CD-ROM volume version  */
int    nst;                      /*  Total number of selected stations  */
  
float rrpoles[] = {-0.788E+05, -0.500E+05, -0.990E+04, -0.672E+04, -0.263E+03,
    -0.530E+03, -0.625E-01, -0.997    , -0.126E+04, -0.628E+03,
    -0.283E+01, -0.283E+01, -0.283E+01, -0.283E+01, -0.862E+02,
-0.626E+02, -0.263E+03, -0.997    , -0.862E+02, -0.626E+02 };
  
float irpoles[] = { 0.0      ,  0.0      ,  0.0      ,  0.0      ,  0.407E+03,
    0.0      ,  0.0      ,  0.765    ,  0.0      ,  0.0      ,
    0.0      ,  0.0      ,  0.0      ,  0.0      ,  0.258E+02,
0.791E+01, -0.407E+03, -0.765    , -0.258E+02, -0.791E+02 };
  
float rrzeroes[] = {-0.401E+03, -0.101E+04, 0, 0, 0, 0, 0, 0, 0};
  
  
void rdcdid ( totstat )
int *totstat;
{
    int   i, maxi;
    char    *buffer;
  
    buff_all(&buffer);
    cdver=0;
    cdread(0,buffer);
  
    /*  Read the first line of the buffer and fill totstat and cdver  */
    sscanf (buffer+12, "%d", totstat);
    sscanf (buffer+77, "%d", &cdver);
  
    /*  For cd-version 1 the bits are shifted  */
    if ( cdver != 2 ) {
        cdver = 1;
        swbyte (buffer);
  
        sscanf (buffer+12, "%d", totstat);
        sscanf (buffer+77, "%d", &cdver);
    }
  
    if ( *totstat > MAXSTAT )  *totstat = MAXSTAT;
  
    /*  Read nstat lines and fill arrays statpos and st.len  */
    maxi = ( *totstat<= 24 ) ? ( *totstat ) : 24 ;
  
    for (i=0; i < maxi; i++) {
        sscanf (buffer+(i+1)*80, "%d %d %d %d", &st[i].pos, &st[i].len[0],
        &st[i].len[1], &st[i].len[2]);
  
        if ( *totstat > 24 )
            sscanf (buffer+(i+1)*80,"%*d %*d %*d %*d %d %d %d %d", &st[i+24].pos,
            &st[i+24].len[0], &st[i+24].len[1], &st[i+24].len[2]);
    }
    free(buffer);
}
  
int rdevl ( count, totevt, evnum, evinfo, evday, evtime, evtime2, evlat, evlon )
char  *evinfo;                       /*  Event information  */
int   count;                         /*  Counter  */
int   *totevt;                       /*  Total of events per day  */
int   *evnum;                        /*  Selected event number  */
int   *evday;                        /*  Day of event  */
float *evtime, *evlat, *evlon;       /*  Event time, latitude and longitude  */
float *evtime2;                      /*  Next event on the same day  */
{
    char    line[LINESIZE], *info[NEVT], word[8];
    int     i, row, col, inpt, lry, evhour, evmin, evsec, imax;
    int     recno;
    char *buffer;
    float  evmsec;
  
    /*
    *  cev = 0: the date has been changed and the strings for the event menu
    *           have to be filled.
    *  cev = 1: the date is unchanged so the event menu can be printed.
    *
    */
    if ( count == 0 ) {
  
        /*  Clear contents of strings  */
        for (i=0; i < NEVT; i++) strset (cdev[i], ' ');
  
        *evtime = *evtime2 = 0;
  
        buff_all(&buffer); 
        cdread(0,buffer);
        sscanf(buffer+30,"%d", &recno);

        /*  Move file handle to EVENT-LOG  */
        cdread(recno,buffer);
  
        /*  Determine date event in day of year  */
        sscanf (buffer+(2*80), "%*s %d", evday);
  
        i = 0;
  
        /*  Store several lines of the buffer starting with line 7  */
        /*  First line read is:  DATE   TIME   ...  */
        do {
            strncpy ( line, &buffer[(i+6)*80], 80 );
            line[80]='\0';
            sscanf (line, "%s", word);
  
            if ( strlen (word) != 0 )  {
                strcpy (cdev[i], line);
                i++;
            }
  
        } while ( ( strlen (word) != 0 ) && i < NEVT-1 );
  
        *totevt = i-1;
        free(buffer);
  
    }    /* End of count = 0 */
  
    if ( ORF2 ) {               /* ORFEUS 88 1  */
        if (  *evday == 27  ) {
            strcpy( cdev[1],"     1988 01 27  19:16:45.4    57.888   -32.653   10.0  5.1  5.1 ");
        }
    }
  
    /*  Event information is displayed  */
    clearscr();
    screen_setup();

    blankstring (menu1[0]);
    strcpy (menu1[0], " EVENT INFORMATION  ");
    textdisp (menu1, 0, 1, 79, 1);

    blankstring (menu1[0]);
    strcpy (menu1[0], " SELECT EVENT ");
    textdisp (menu1, 0, 2, 79, 2);

    lry = *totevt + 3;
    imax = *totevt + 1;
    if ( *totevt > 1 ) {
        strcpy (cdev[*totevt+1], "                   COMPLETE     DATA    RECORD                               ");
        lry++;
        imax++;
    }
  
    for ( i=0; i < imax; i++) info[i] = &cdev[i][0];
    textdisp (info, 0, 3, strlen(cdev[0])-1, lry);
  
    textdisp (mnu3, 0, 23, 79, 23);
    row = 4;
    col = 0;
    lry++;
    inpt = menucurs6 (&row, &col, 3,3,65,lry,
    3,3,65,lry,
    3,3,65,lry,
    3,3,65,lry,
    3,3,65,lry,
    3,3,65,lry);
  
    if ( inpt == ESC ) {
        clearscr();
        return ESC;
    }
  
    *evnum = row - 3;
    i = row - 3;               /*  i is being used as the event actually selected */
    if ( *evnum > *totevt ) {  /*  Complete data record selected means  */
        *evnum = 999;          /*  that evtime is the first event time  */
        i = 1;                 /*  thus i set equal to 1.             */
    }
  
    sscanf (info[i]+16, "%d:%d:%d.%f", &evhour, &evmin, &evsec, &evmsec);
    *evtime = (float) evhour*3600 + evmin*60 + evsec + evmsec/100;
    sscanf (info[i]+28, "%f %f", evlat, evlon);
  
    if ( ORF2 ) { /* ORFEUS 88 1  */
        if ( *evday == 27  ) (*evlon) *= -1;
    }
  
    if ( *totevt >= 2 && *evnum < *totevt ) {
        sscanf (info[i+1]+16, "%d:%d:%d.%f", &evhour, &evmin, &evsec, &evmsec);
        *evtime2 = (float) evhour*3600 + evmin*60 + evsec + evmsec/100;
    }
  
    /*  Selected event information is stored  */
    strcpy (evinfo, info[i]+5);

    if ( ORF2 ) { /* ORFEUS 88 1  */
        if (  *evday == 27  )
            strcpy( evinfo,"1988 01 27  19:16:45.4    57.888   -32.653   10.0  5.1  5.1 ");
    }
    refpt ( ggtogc(*evlat*RAD), *evlon*RAD );
    return(1);
}
  
int selst ( count, totevt, evday, evtime, totstat, stnum, distmax )
int   count, totevt, evday, totstat, *stnum;
float  evtime, *distmax;
{
    char   *menuC[18], lat[10], lon[10], strd[10], elev[15];
    char   ch;
    int    i, k, row, col, row1, col1, row2, col2, inpt;
    int    recno, csel, temp, p;
    char *buffer;
  
    /*
    *  cse = 0: first time for the selected event thus the station menu strings
    *           have to be filled.
    *  cse = 1: the station menu strings are filled but filled again if there
    *           are more than one event.
    *  cse = 2: the station menu strings are filled so the station menu is
    *           printed.
    *
    */
  
    if ( count == 0 || ( count == 1 && totevt > 1 ) ) {
        set_colour ( 3, 0, 22, 79, 24);
        blankstring (menu1[0]);
        textdisp (menu1, 0, 23, 79, 23);
 
  
        /*  Clear contents of mstat  */
        for ( i=0; i < 18; i++) strset (mstat[i],' ');
        buff_all(&buffer);
  
        /*  Read for all station logs stlat and stlon. Determine the distance  */
  
        for (i=0; i < totstat; i++) {
  
            /*  Determination of position of station log  */
            recno = st[i].pos - 1;
            cdread(recno,buffer);
  
            /*  Read the 5th line of the buffer  */
            /*  Fill the parameters stcode, stinst, lat and long */
            sscanf (buffer+(4*80), "%*d %s %s %s %s %s", st[i].codes, st[i].inst, lat, lon, elev);
  
            /*  Determine the time correction  */
            timecor (buffer, evday, evtime, i);
  
            /*  Error occured on ORFEUS_881 CD_ROM concerning Echery station
            in Indexfile.                                                  */
            if ( strncmp ( st[i].codes, "ECH1", 4) == 0 ) {
                st[i].len[1] = st[i].len[0];
                st[i].len[0] = 0;
            }
  
            /*  Write station code to terminal screen  */
            strcpy (menu1[0], "SCANNING  STATION  LOG:  ");
            strcat (menu1[0], st[i].codes);
            textdisp (menu1, 22, 23, 79, 23);
  
            /*  Determine floating point values stlat and stlon  */
            ch = lat[strlen(lat)-1];
            lat[ strlen (lat)-1 ] = '\0';
            sscanf (lat, "%f", &st[i].lat);
            if ( ch == 'S' ) st[i].lat = -st[i].lat;
  
            ch = lon[ strlen (lon)-1 ];
            lat[ strlen (lon)-1 ] = '\0';
            sscanf (lon, "%f", &st[i].lon);
            if ( ch == 'W' ) st[i].lon = -st[i].lon;
  
            ch = elev[ strlen (elev)-1 ];
            if(ch == '-') {
                sscanf (buffer+(4*80), "%*d %*s %*s %*s %*s %*s %s", elev);
                elev[ strlen (elev)-1 ] = '\0';
                sscanf (elev, "%f", &st[i].eleva);
                st[i].eleva= -(st[i].eleva);
            }
  
            if(ch == 'M' || ch == 'm') {
                elev[ strlen (elev)-1 ] = '\0';
                sscanf (elev, "%f", &st[i].eleva);
            }
  
            /*  Determine the distance to the selected event  */
            delaz ( ggtogc(st[i].lat*RAD), st[i].lon*RAD, &st[i].epi, &st[i].back );
            st[i].epi *= DEG;
            st[i].back *= DEG;
  
        }  /*  End of do-loop over stations  */
  
        free(buffer);
  
        /*  Close file on CD-ROM and sort on distance  */
        fclose (id1);
  
        qsort( &st[0].epi,totstat,(sizeof(struct stat)),compare);
  
        for ( i=0 ; i<totstat; i++) {
            sprintf (strd,"%7.2f", st[i].epi);
  
            /*  Store parameters stcode, stinst and del in mstat  */
            if ( i < 16 ) {      /* Column 1 */
                k = i+2;
                p = 0;
            }
            else if (i < 32) {   /* Column 2 */
                k = i+2-16;
                p = 1;
            }
            else {               /* Column 3 */
                k = i+2-32;
                p = 2;
            }
            strcpy (mstat[k]+26*p, st[i].codes);
            strcat (mstat[k], "   ");
            strcpy (mstat[k]+26*p+6, st[i].inst);
            strcat (mstat[k], "       ");
            strcpy (mstat[k]+26*p+14,strd);
            strcat (mstat[k], "      ");
  
        } /*  End of while loop  */
  
        /*  Strings mstat are prepared for printing  */
        temp = totstat % 16;
        if ( temp != 0 ) {
            for ( i=2+temp; i < 18; i++ )
                strcat (mstat[i], "                          ");
        }
        openfile();
  
        /*  Store the menu title in mstat[0]  */
        strcpy (mstat[0], "CODE  INST     DIST       ");
        if ( totstat > 16 ) strcat (mstat[0], "CODE  INST     DIST       ");
        if ( totstat > 32 ) strcat (mstat[0], "CODE  INST     DIST       ");
  
    }  /*  End of if (  )  */
  
    /*  Menu1 is displayed  */
    clearscr();
    screen_setup();

    blankstring (menu1[0]);
    strcpy (menu1[0]," STATIONS AVAILABLE");  
    textdisp (menu1, 0, 1, 79, 1);
  
    /*  MenuC is displayed  */
    for (i=0; i < 18; i++) menuC[i] = &mstat[i][0];
    textdisp (menuC, 2, 4, strlen(menuC[0])+1, 21);
  
    /*  Menu2 is displayed  */
  
    /*  Cursor selection from menu gives station name and number  */
    if ( plchoice == 1 ) {
        blankstring (menu1[0]);
        strcpy (menu1[0], " SELECT ONE STATION");
        textdisp (menu1, 0, 2, 79, 2);

        textdisp (mnu3, 0, 23, strlen(mnu3[0])-1, 23);
        row = 6;
        col = 0;
        csel = 0;
        inpt = selec(csel, totstat, &col, &row);
        if ( inpt == ESC ) {
            clearscr();
            return ESC;
        }
  
        stparam (col, row, stnum);
        nst = 1;
    }
    else {
        blankstring (menu1[0]);
        strcpy (menu1[0], " SELECT TWO STATIONS FOR DISTANCE RANGE");
        textdisp (menu1, 0, 2, 79, 2);

        blankstring (menu1[0]);
        strcpy (menu1[0], "  SELECT FIRST CHOICE THEN PRESS ENTER  (ESC RETURNS TO PREVIOUS MENU) ");
        textdisp (menu1, 0, 23, 79, 23);
  
        col = 0;
        row = 6;
        csel = 1;
        inpt = selec (csel, totstat, &col, &row);
        if ( inpt == ESC ) {
            clearscr();
            return ESC;
        }
  
        /*  Store column and row of the first selection  */
        col1 = col;
        row1 = row;
  
        blankstring (menu1[0]);
        strcpy (menu1[0], "  SELECT SECOND CHOICE THEN PRESS ENTER  (ESC RETURNS TO PREVIOUS MENU) ");
        textdisp (menu1, 0, 23, 79, 23);
  
        csel = 0;
        inpt = selec (csel, totstat, &col, &row);
        if ( inpt == ESC ) {
            clearscr();
            return ESC;
        }
  
        /*  Check the second selection  */
        while ( row == row1 && col == col1 ) {
            blankstring (menu1[0]);
            strcpy (menu1[0], "  SECOND CHOICE MUST BE DIFFERENT FROM FIRST CHOICE ! ");
            textdisp (menu1, 0, 23, 79, 23);
  
            inpt = selec (csel, totstat, &col, &row);
            if ( inpt == ESC ) {
                clearscr();
                return ESC;
            }
        }  /*  End of while loop  */
  
        row2 = row;
        col2 = col;
  
        /*  If the nearest station has been selected last rearrange contents  */
        if ( (col2 < col1) || (col2 == col1 && row2 < row1) ) {
            col2 = col1;
            row2 = row1;
            col1 = col;
            row1 = row;
        }
  
        /*  Determine number of selected stations  */
        nst = (col2-col1) * 16 + row2 - row1 + 1;
  
        /*  Determine position on CDROM for selected stations  */
        for ( i=0; i < nst; i++ ) {
            row = row1+i;
            col = col1;
            if ( row > 21 ) {
                row -= 16;
                col++;
            }
            stparam (col, row, &st[i].nums);
            if ( i == (nst-1) ) *distmax = st[ st[i].nums ].epi;
  
        }  /*  End of loop over the selected stations  */
  
        /*  Order on distance changed into order on CDROM  */
        qsort ( &st[0], nst, sizeof(struct stat ), compare2 );
  
    }  /*  End if ( plchoice == 2 )  */
  
    return inpt;
}
  
int compare(el1,el2)
float *el1,*el2;
  
{
    if( (*el1) < (*el2) ) return (-1);
    else if( (*el1) == (*el2) ) return (0);
    else return (1);
}
  
int compare2 ( tp1, tp2 )
struct stat  *tp1, *tp2;
{
    if ( tp1->nums < tp2->nums ) return(-1);
    else if ( tp1->nums == tp2->nums ) return(0);
    else return(1);
}
  
int rdstl ( stnum, stinfo, stname, comment1, comment2 )
char  *stinfo, stname[], comment1[], comment2[];
int   stnum;
{
    char  dum[81];
    int   i, j, recno;
    char  *buffers, *find;
  
    buff_all(&buffers);
    recno = st[stnum].pos-1;
    cdread(recno,buffers);

    /*  Retrieve file name from 1st line of buffer */
    /*  A colon (:) marks the beginning of the string */
    /*  Two spaces marks the end of the string */
    strncpy(stname,(strstr(buffers,":")+1),49);
    if ((find = strstr(stname,"  ")) != NULL) *find = '\0';
    else stname[50] = '\0';
  
    /*  Store station information  */
  
    strcpy ( stinfo, st[stnum].codes );
    if ( strlen (st[stnum].codes) == 3 ) strcat (stinfo, " ");
    sprintf (dum, " %8.3f", st[stnum].lat);
    strcat ( stinfo, dum );
    sprintf (dum, " %8.3f", st[stnum].lon);
    strcat ( stinfo, dum );
    sprintf (dum, " %6.2f", st[stnum].epi);
    strcat ( stinfo, dum );
    sprintf (dum, " %5.1f", st[stnum].back);
    strcat ( stinfo, dum );
  
    /*  Read the 24th or  25th line of the buffer  ( comments).   */
  
    if ( plchoice == 1 )  {
        j = 22;
        for (i=21; i < 23; i++) {
            sscanf((buffers+i*80),"%s",comment1);
            if (strncmp("comment",comment1,7) == 0) j = i+1;
            else if (strncmp("COMMENT",comment1,7) == 0) j = i+1;
        }
        strncpy(comment1,buffers+j*80,80) ;
        strncpy(comment2,buffers+(j+1)*80,80);
        comment1[80] = '\0';
        comment2[80] = '\0';
    }
    free(buffers);
    return(1);
}
  
void rddtl ( stnum, nchan )
int    stnum, nchan;
{
    char      line[LINESIZE], word1[5], word2[25], ch;
    int       i, imin, imax, j, k, l, m, nline, nl, nchar, ncol, recno;
    float     temp1, temp2, s[8];
    char    *buffer;
  
    /*  Initilization of some values  */
    for ( j=0; j < 3; j++ ) {
        calib[j].a0 = 0;
        calib[j].cal_dsave = 0;
        calib[j].cal_ds = 0;
        calib[j].cal_freq = 0;
        calib[j].np = 0;
        calib[j].nz = 0;
  
        for ( i=0; i < 30; i++ ) {
            calib[j].poles[i].x = 0;
            calib[j].poles[i].y = 0;
        }
  
        for ( i=0; i < 30; i++ ) {
            calib[j].zeroes[i].x = 0;
            calib[j].zeroes[i].y = 0;
        }
    }
  
    /*  Determination of position of first data log  */
    if ( SHORT_PERIOD ) recno = st[stnum].pos;
    else if ( LONG_PERIOD ) recno = st[stnum].pos+st[stnum].len[0];
    else recno = st[stnum].pos+st[stnum].len[0]+ st[stnum].len[1];
  
    buff_all(&buffer);
    cdread(recno,buffer);
  
    /*  Look for cal values  */
    nl=0;
    do {
  
        strncpy ( line, &buffer[nl*80], 80 );
        line[80] = '\0';
        sscanf (line, "%s", word2);
        nl++;
  
    } while ( ! ( word2[0] == '1') );
    nl -= 1;
  
    /* determine ave cal (cal_dsave) and measured cal (cal_ds) from
    datalog1. Takes only first value for each channel.
    Can easily be changed for future applications.
    */
  
    for( j=0; j<3; j++) {
  
        for(i=0;i<8;i++) s[i]=0.;
        strncpy ( line, &buffer[nl*80], 80 );
        line[80] = '\0';
  
        m= sscanf(line, "%E %E %E %E %E %E %E", &s[0], &s[1], &s[2], &s[3], &s[4], &s[5], &s[6]);
  
        switch(m) {
            case 4: /* ORFEUS_NARS */
                calib[j].cal_ds= s[1];
                calib[j].cal_dsave= s[2];
                calib[j].cal_freq= s[3];
                break;
            case 5: /* NEIC */
                calib[j].cal_ds= 0.0;
                calib[j].cal_dsave= s[3];
                calib[j].cal_freq= s[4];
                break;
            case 6: /* ORFEUS_NARS and NEIC (NARS data logs) */
                calib[j].cal_ds= s[3];
                calib[j].cal_dsave= s[4];
                calib[j].cal_freq= s[5];
                break;
            case 7: /* NEIC */
                calib[j].cal_ds= s[4];
                calib[j].cal_dsave= s[5];
                calib[j].cal_freq= s[6];
                break;
            default:
                if(s[0] <= .9) break;
                else {
                    printf("No proper cal. values\n");
                    pause(5);
                    calib[j].cal_dsave= 0.;
                    calib[j].cal_freq= 0.;
                    calib[j].cal_ds= 0.;
                    break;
                }
        }
        nl+=2;
    }
  
    recno=-1;
    cdread(recno,buffer);
  
    /*  Look for A0 in the buffer and determine the line number nl  */
    nl = 0;
  
    do {
        strncpy ( line, &buffer[nl*80], 80 );
        line[80] = '\0';
        sscanf (line, "%s", word2);
        nl++;
    } while ( ! ( word2[0] == 'A' && (word2[1] == '0' || word2[1] == 'O') ) );
  
    nl -= 1;
  
    /*  Determination of the position of A0 in the line  */
    nchar = 0;
    do {
        nchar++;
    } while ( ! ( *(buffer+nl*80+nchar) == 'A' ) );
  
    /*  Determine number of columns in datalog 2  */
    ncol = 0;
    strset (word2, ' ');
  
    do {
        j = 0;
        imin = nchar + 6 + ncol*23;
        imax = imin + 19;
        for ( i=imin; i < imax; i++ ) word2[j++] = *(buffer+(nl+1)*80+i);
        word2[j] = '\0';
        ncol++;
    } while ( (strncmp (word2, "          ", 10)) != 0 && ncol < 4 );
  
    ncol -= 1;
  
    /*  If all constants are the same they are not repeated in datalog 2.  */
    /*  Set all values to values in the first column  */
    if ( ncol == 1 ) comp = 0;
  
    /*  Do-loop over the three components  */
    m = 0;
  
    do {
        if ( nchan == 3 ) comp = m;
  
        /*  Determination of normalization constant A0  */
        j = 0;
        imin = nchar + 6 + comp * 23;
        imax = imin + 19;
        for ( i=imin; i < imax; i++ ) word2[j++] = *(buffer+nl*80+i);
        word2[j] = '\0';
        sscanf (word2, "%E", &calib[m].a0);
  
        /*  Determination of the poles  */
        strset (word1, ' ');
        strset (word2, ' ');
        nline = nl+1;
        l = 0;
  
        do {
            temp1 = temp2 = 0;
            strset (word1, ' ');
            strset (word2, ' ');
  
            strncpy ( line, &buffer[nline*80], 80 );
            line[80] = '\0';
            sscanf (line, "%s", word1);
  
            if ( word1[0] == 'P' ) {
                if ( m == 0 )       sscanf ( line, "%*s  %s", word2 );
                else if ( m == 1 )  sscanf ( line, "%*s  %*s  %s", word2 );
                else if ( m == 2 )  sscanf ( line, "%*s  %*s  %*s  %s", word2 );
  
                if(!( word2[0] == '+' || word2[0] == '-' || word2[0] == '0' || word2[0] == '.')) {
                    for ( i=0; i<19; i++ ) word2[i] = word2[i+1];
                }
  
                word2[9] = ' ' ;
                word2[19] = '\0' ;
  
                if ( word2[0] == '+' || word2[0] == '-' )
                    sscanf (word2, "%E %E", &temp1, &temp2);
                else sscanf (word2, "%E", &temp2);
  
                /*  Store the values in the complex array poles  */
                calib[m].poles[l].x = (double) temp1;
                calib[m].poles[l].y = (double) temp2;
  
                if ( temp1 != 0 || temp2 != 0 ) l++;
                nline++;
  
            }  /*  End if  */
  
        } while ( word1[0] == 'P' );  /*  End do-loop over poles  */
  
        calib[m].np = l;
  
        /*  Determination of the zeroes  */
        strset (word1, ' ');
        strset (word2, ' ');
        l = 0;
  
        do {
            temp1 = temp2 = 0;
            strncpy ( line, &buffer[nline*80], 80 );
            line[80] = '\0';
            sscanf (line, "%s", word1);
  
            if ( word1[0] == 'Z' ) {
                j = 0;
                imin = nchar + 6 + comp * 23;
                imax = imin + 19;
                for ( i=imin; i < imax; i++ ) word2[j++] = line[i];
                word2[9] = ' ';   /*  Substitute ',' by space  ' '  */
                word2[j] = '\0';
                sscanf (word2, "%1s", &ch);
  
                if ( ch != 'S' ) {
                    if ( word2[0] == '+' || word2[0] == '-' )
                        sscanf (word2, "%E %E", &temp1, &temp2);
                    else sscanf (word2, "%E", &temp2);
  
                    /*  Store the values in the complex array zeroes  */
                    calib[m].zeroes[l].x = (double) temp1;
                    calib[m].zeroes[l].y = (double) temp2;
  
                    if ( temp1 != 0 || temp2 != 0 )
                        l++;
                }
                else {
                    sscanf (word2, "%*1s %*c %*c %d", &k);
                    l += k;
                }
                nline++;
  
            }  /*  End if  */
        } while ( word1[0] == 'Z' );  /*  End of do-loop over zeroes  */
  
        calib[m].nz = l;
        m++;
  
    } while ( nchan == 3 && m < nchan && m < ncol );  /*  End do-loop  */
  
    /*  If all constants are the same they are not repeated in datalog 2.  */
    /*  Set all values to values in the first colom  */
    if ( ncol == 1) {
        for ( j=1; j < 3; j++ ) {
            calib[j].a0 = calib[0].a0;
            calib[j].np = calib[0].np;
            calib[j].nz = calib[0].nz;
  
            for ( i=0; i < calib[0].np; i++ ) {
                calib[j].poles[i].x = calib[0].poles[i].x;
                calib[j].poles[i].y = calib[0].poles[i].y;
            }
  
            for ( i=0; i < calib[0].nz; i++ ) {
                calib[j].zeroes[i].x = calib[0].zeroes[i].x;
                calib[j].zeroes[i].y = calib[0].zeroes[i].y;
            }
        }  /* end of for loop */
    } /* end if */
  
    free(buffer);
}
  
void repair ( stnum,  nchan )
int    stnum, nchan;
{
    int i, m;
  
    /*  Check vertical component for stations RSNT, RSSD, RSNY and RSON  */
    if ( ( strncmp (st[stnum].codes, "RSNT", 4) == 0 ) ||
        ( strncmp (st[stnum].codes, "RSSD", 4) == 0 ) ||
        ( strncmp (st[stnum].codes, "RSNY", 4) == 0 ) ||
    ( strncmp (st[stnum].codes, "RSON", 4) == 0 ) )   {
        if ( SHORT_PERIOD && ( comp == 0 || nchan == 3 ) ) {
            if ( ( calib[0].poles[2].x +10100 ) < 1.0E-06 ) {
                calib[0].a0 = -calib[0].a0;
                calib[0].np = 20;
                for ( i=0; i < 20; i++ ) {
                    calib[0].poles[i].x = rrpoles[i];
                    calib[0].poles[i].y = irpoles[i];
                }
                if ( plchoice == 1 )  {
                    blankstring (menu1[0]);
                    strcpy (menu1[0], "VERTICAL SP POLES REPAIRED");
                    textdisp (menu1, 42, 9, 79, 9);
                }
            }
        }
    }
  
    /*  Check horizontal components for stations RSNY and RSON  */
    if ( ( strncmp (st[stnum].codes, "RSNY", 4) == 0 ) ||
    ( strncmp (st[stnum].codes, "RSON", 4) == 0 ) ) {
        if ( SHORT_PERIOD && ( comp == 1 || comp == 2 || nchan == 3 ) ){
            if ( calib[1].nz == 8 ) {
                calib[1].a0 = .249E+18;
                calib[2].a0 = .249E+18;
                calib[1].nz = 9;
                calib[2].nz = 9;
                for ( m=1; m < 3; m++ ) {
                    for ( i=0; i < 9; i++ ) {
                        calib[m].zeroes[i].x = rrzeroes[i];
                        calib[m].zeroes[i].y = 0;
                    }
                }
                if ( plchoice == 1 ) { 
                    blankstring (menu1[0]);
                    strcpy (menu1[0], "HORIZONTAL SP ZEROES REPAIRED");
                    textdisp (menu1, 42, 11, 79, 11);
                }
            }
        }
    }
}
  
void display_comm (comment1,comment2,stnum)
char comment1[], comment2[];
int stnum;
{
  
    /*  Display time correction and comments in stationlog  */
  
    char correction[10];
  
    if ( plchoice == 1 ) {
        blankstring (menu1[0]);
        strcpy (menu1[0], "TIME CORRECTION:  ");
        sprintf (correction, "%.3f", st[stnum].tcor);
        strcat (menu1[0], correction);
        textdisp (menu1, 42, 5, strlen(menu1[0])+41, 5);

        blankstring (menu1[0]);
        strcpy (menu1[0], "COMMENTS IN STATION LOG: ");
        textdisp (menu1, 0, 19, 79, 19);

        blankstring (menu1[0]);
        strcpy (menu1[0], comment1);
        textdisp (menu1, 0, 20, 79, 20);

        blankstring (menu1[0]);
        strcpy (menu1[0], comment2);
        textdisp (menu1, 0, 21, 79, 21);
    }
}
  
void timecor ( buffer, evday, evtime, stnum)
char *buffer;
int       evday, stnum;
float     evtime;
{
    char  line[LINESIZE], word1[15];
    int   i, j, k, ncor, nl, eq, day[6], time[6], evhour, hour, dif1, dif2;
    float temp[6], hr;
  
    /*  Initialization of some parameters  */
    for ( i=0; i < 6; i++ ) {
        temp[i] = 0;
        day[i]  = 0;
        time[i] = 0;
    }
  
    /*  Read the 12th or 13th line of the buffer to find Time Correction */
    j = 11;
    for (i=11; i < 13; i++) {
        sscanf((buffer+i*80),"%s",line);
        if ((k=strncmp("TIME",line,4))== 0) j = i;
        else if ((k=strncmp("time",line,4)) == 0) j = i;
    }
    k = j;
    strncpy(line,buffer+k*80,80);
    line[80] = '\0';
  
  
    /*  Fill parameter ncor ( number of time corrections in stationlog )  */
    sscanf (line, "%*s %*s %d", &ncor);
    if ( (ncor < 0) || (ncor > 6) )  ncor = 0;
  
    /*  Return if there are no time corrections  */
    if ( ncor == 0 ) return;
  
    for ( i=0; i < ncor; i++ ) {
  
        /*  Read line from the buffer - store time corrections in array temp  */
        if ( i <= 2 ) nl = i;
        else nl = i-3;
  
        /* The value k is determined above for the line */
        strncpy(line,(buffer+(k+2+nl)*80),80);
        line[80] = '\0';
  
        if ( i <= 2 ) sscanf (line,"%*s %*s %*s %s", word1);
        else          sscanf (line,"%*s %*s %*s %*s %*s %*s %*s %s", word1);
  
        if ( (strlen(word1)) != 0 ) {
            if ( i <=2 )
                sscanf (line, "%*s %d %d %f", &day[i], &time[i], &temp[i]);
            else
                sscanf (line, "%*s %*s %*s %*s %*s %d %d %f", &day[i], &time[i], &temp[i]);
        }
  
    }  /*  End do loop over time corrections  */
  
    /*  Return if there is only one time correction  */
    st[stnum].tcor = temp[0];
    if ( ncor == 1 ) return;
  
    /*  Check if all time corrections are equal  */
    i = 0;
    do {
        if ( (fabs (temp[i] - temp[i+1])) < 1.0E-06 ) eq = 0;
        else eq = 1;
        i++;
    } while ( i < (ncor-1) && eq == 0 );
  
    /*  Return if all time corrections are eqaul  */
    st[stnum].tcor  = temp[0];
    if ( eq == 0 ) return;
  
    /*  Select the time correction closest to evtime  */
    hr = evtime / 3600;
    evhour = (int) hr;
    k = 0;
    dif1 = 24;
  
    for ( i=0; i < ncor; i++ ) {
        if ( day[i] == evday ) {
            hour = time[i] / 100;
            dif2 = abs ( evhour - hour );
            if ( dif2 < dif1 ) {
                dif1 = dif2;
                k = i;
            }
        }
    }
    st[stnum].tcor = temp[k];
}
  
int selec ( rev, totstat, col, row )
int rev, totstat, *col, *row;
{
    int inpt, lry, i;
  
    if ( totstat <= 16 )  {      /* column 1 */
        lry = totstat+6;
        inpt = menucurs6 (row, col, 0,5,24,lry,
        0,5,24,lry,
        0,5,24,lry,
        0,5,24,lry,
        0,5,24,lry,
        0,5,24,lry);
    }
    else if ( totstat <= 32) {  /* column 2 */
        lry = totstat+6-16;
        inpt = menucurs6 (row, col, 0,5,24,22,
        26,5,50,lry,
        26,5,50,lry,
        26,5,50,lry,
        26,5,50,lry,
        26,5,50,lry);
    }
    else {                                 /* column 3 */
        lry = totstat+6-32;
        inpt = menucurs6 (row, col, 0,5,24,22,
        26,5,50,22,
        52,5,76,lry,
        52,5,76,lry,
        52,5,76,lry,
        52,5,76,lry);
    }
    if ( rev == 1 ) {
        i = *col * 26;
        re_vid (*row, i, *row, i+24, *row );
    }
  
    return inpt;
}
  
void stparam ( col, row, stnum)
int col, row, *stnum;
{
    *stnum = row+(col*16)-6;
}
  
/*  geocen - performs calculations relating to
postitions, distances, and azimuths*/
  
/* stores the coordinates (geocentric latitude and */
/* longitude) of a refernce point on the earth's surface */
  
void refpt(lat, lon)
float lat, lon;
{
    st0 = cos(lat);
    ct0 = sin(lat);
    phi0 = lon;
}
  
/* calculates the geocentric distance, and */
/* azimuth from the reference point to a given */
/* secondary point, and the back azimuth from */
/* the secondary point to the reference point */
  
void delaz(lat, lon, delta, az1)
float lat, lon, *delta, *az1;
{
    float ct1, st1, sdlon, cdlon, cdelt, sdelt,az;
  
    ct1 = sin(lat);
    st1 = cos(lat);
    sdlon = sin(lon - phi0);
    cdlon = cos(lon - phi0);
    cdelt = st0*st1*cdlon + ct0*ct1;
    cvrtop (st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, &az);
    *delta = atan2(sdelt, cdelt);
    cvrtop (st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, &az);
    *az1 = az;
    if (*az1 < 0) *az1 +=  TWOPI;
}
  
/* calculates the geocentric coordinates of a */
/* secondary point, given its geocentric distance */
/* and azimuth from the reference point */
  
void back (delta, az0, lat, lon)
float delta, az0, *lat, *lon;
{
    float sdelt, cdelt, cz0, ct1;
    float st1, dlon;
  
    sdelt = sin(delta);
    cdelt = cos(delta);
    cz0 = cos(az0);
    ct1 = st0*sdelt*cz0 + ct0*cdelt;
    cvrtop (st0*cdelt-ct0*sdelt*cz0,sdelt*sin(az0), &st1, &dlon);
    *lat = atan2(ct1, st1);
    *lon = phi0 + dlon ;
    if (fabs(*lon) > PI) *lon = *lon - sign(TWOPI, *lon);
}
  
/* cvrtop - convert from rectangular to polar coordinates */
void cvrtop(x, y, r, theta)
float  x, y; /* (input)*/
float  *r, *theta;    /* (output - may overlay x, y) */
{
    float rad;
  
    rad = myhypot(x, y);
    *theta = atan2(y, x);
    *r = rad;
}
  
/* cv_lat - convert between geographic and geocentric latitude */
#define c1    ((1.-FLATTENING)*(1.-FLATTENING))
#define c2    (HALFPI*(1./c1 -1.))
  
/*  converts from geographic to geocentric latitude */
  
float ggtogc (lat)
float lat;
{
    float gg;
  
    if ((HALFPI-fabs(lat)) < .02)
        gg = lat/c1 -sign(c2, lat);
    else
        gg = atan(c1 * tan(lat));
    return gg;
}
  
/*  converts from geocentric to geographic latitude */
float gctogg(lat)
float lat;
{
    float gc;
  
    if ((HALFPI - fabs(lat)) < .02) gc = c1*(lat + sign(c2, lat));
    else gc = atan(tan(lat)/c1);
    return gc;
}
  
/* myhypot - euclidian distance, accurately and avoiding overflow */
float myhypot(a, b)
float a, b;
{
    float l, s, t;
    float euc;
  
    l = fabs(a);
    s = fabs(b);
    if (s > l) {
        t = s;
        s = l;
        l = t;
    }
    if (l == 0.0) euc = 0.0;
    else {
        s = s/l;
        euc = l*sqrt(1.0 + s*s);
    }
    return euc;
}
  
float sign (a, b)
float a, b;
{
    if (b > 0) return a;
    else return -a;
}
