/***   Module: RDLOG2 - Read Log (version for SONIC2).
-----------------------------------------------
  
Rdlog2 contains the functions:
  
openhdfile : opens a file on hard disk.
rdcdid     : reads the CD-ROM identity block with stationlog
positions and data record lengths.
rdevl      : reads the event log.
rdstl      : reads the station log.
  
wrtoscr    : writes event date to screen.
swbyte     : switches the bytes for cd-version 1.
dist       : determines distance and azimuth between event and station.
sign       : returns a if b > 0 , returns -a if b < 0.
  
programmed : nov 1988.
revised    : march 1989
  
***/
  
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "menu.h"
#include "sonic.h"
  
#define BUFSIZE  2000
#define LINESIZE 81
#define MAXSTAT  72
#define PATHL    25
#define NEVT     25
#define PI            3.14159265
#define HALFPI        .5*PI
#define TWOPI         (2.0*PI)
#define DEG           (180./PI)        /* Degrees/Radian */
#define RAD           (PI/180.)        /* Radians/Degree */
#define EQTRAD        6378.163
#define POLRAD        6356.177
#define FLATTENING    (EQTRAD-POLRAD)/EQTRAD
  
extern FILE  *id1;                          /*  Defined in module opencd2  */
extern char  filepath[][PATHL];
extern char  mon[][4];
extern float evlat1, evlat2, evlon1, evlon2, dep1, dep2, magn1, magn2;
extern float stlat1, stlat2, stlon1, stlon2, del1, del2, azis1, azis2;
  
extern char *menu1[];                       /*  Defined in module menu  */
extern int DisplayDevice;                   /*  Defined in module dispdriv  */
extern int cd_disc;
  
FILE  *id3;
char  buffer[BUFSIZE];
char  evtline[NEVT][81];
int   cdver, totstat, statpos[MAXSTAT], datalen[MAXSTAT][3], nmatch=0;
float evlat, evlon, dep, magn;
float stlat, stlon, del, azis;
float st0, ct0, phi0;
  
void openhdfile ( cdinfo, fname )
char *cdinfo, fname[];
{
    char c;
    int  i;
  
    clearscr();
    screen_setup();
  
    /*   Print the menu top text  */
    blankstring (menu1[0]);
    strcpy (menu1[0]," FILE NAME: ");
    textdisp (menu1, 0, 1, 79, 1);
  
    /*  Print the menu bottom text  */
    blankstring (menu1[0]);
    strcpy (menu1[0]," ENTER NEW FILE NAME AND/OR PRESS ENTER");
    textdisp (menu1, 0, 23, 79, 23);
  
    /*  Print the menu text  */
    strcpy (fname, cdinfo);
    strcat (fname, ".CAT");
  
    blankstring (menu1[0]);
    strcpy (menu1[0], " THE DEFAULT FILE NAME IS:  ");
    strcat (menu1[0], fname);
    textdisp (menu1, 0, 5, 79, 5);
  
    /*  Read the '\n' character in the inputque  */
    fflush ( stdin );
    set_curs (5, 28);
    scanf("%c",&c);
  
    if (!( c == '\n' )) {
        scanf ("%s", fname);
        for (i = strlen (fname)+1; i > 0; i--)   fname[i] = fname[i-1];
        fname[0] = c;
    }
  
    if((id3=fopen(fname,"w"))==NULL) {
        sprintf(menu1[0],"Could not open file %s.",fname);
        error_and_exit ("openhdfile", menu1[0]);
    }
}
  
void rdcdid2 ( num )
int num;
{
    char line[LINESIZE];
    int i, j, maxi;
    unsigned int n;
    char *prog_name = "RDCDIC";
  
    /*  Read first block of file  */
    if ( n = fread (buffer, sizeof(*buffer), BUFSIZE, id1) == NULL)
        error_and_exit (prog_name,"Could not read file.");
  
    /*  Read the first line of the buffer and fill nstat and cdver  */
    for (i=0;i < 80; i++) line[i] = buffer[i];
  
    sscanf (line+12, "%d", &totstat);
    sscanf (line+77, "%d", &cdver);
  
    if ( cdver != 2 ) {
        cdver = 1;
        swbyte (buffer);
        for (i=0;i < 80; i++) line[i] = buffer[i];
  
        sscanf (line+12, "%d", &totstat);
        sscanf (line+77, "%d", &cdver);
    }
  
    /*  Reparation for a file on cd-rom volume 4  */
    if ( cd_disc != 5 ) {
        if ( strncmp (filepath[num], "l:\\1984\\MAR\\8407900.EVT", 23) == 0 )
            totstat = 49;
    }
  
    if ( totstat > 72 )
        error_and_exit (prog_name,"Too many stations for SONIC.");
  
    /*  Read nstat lines and fill arrays statpos and datalen  */
    if ( totstat <= 24 ) maxi = totstat;
    else maxi = 24;
  
    for (i=0; i < maxi; i++) {
        for (j=0; j < 80; j++)
            line[j] = buffer[(i+1)*80+j];
        sscanf (line, "%d %d %d %d", &statpos[i],
        &datalen[i][0], &datalen[i][1], &datalen[i][2]);
        if ( totstat > 24 )
            sscanf (line, "%*d %*d %*d %*d %d %d %d %d", &statpos[i+24],
            &datalen[i+24][0], &datalen[i+24][1], &datalen[i+24][2]);
        if ( totstat > 48 )
            sscanf (line, "%*d %*d %*d %*d %*d %*d %*d %*d %d %d %d %d",
            &statpos[i+48], &datalen[i+48][0],
            &datalen[i+48][1], &datalen[i+48][2]);
    }
}
  
void rdevl2 ( nevt )
int *nevt;
{
    char line[LINESIZE], word[6];
    int i, j, k;
  
    /*  Move file handle to Event Log  */
    cdread(0,buffer);
    sscanf(buffer+30,"%d",&i);
    cdread(i,buffer);
  
    /*  Read all event lines of the buffer starting with line 8  */
    *nevt = 0;
    k = 0;
    strcpy (word, "  ");
  
    do {
        for (j=0; j < 80; j++) line[j] = buffer[j + (k+7) * 80];
        line[80] = '\0';
  
        sscanf (line, "%s", word);
  
        if ( strlen (word) == 4 ) {
            sscanf (line+28,"%f %f %f %f", &evlat, &evlon, &dep, &magn);
  
            /*  Check if event parameters are within the boundaies  */
            if ( evlat >= evlat1 && evlat <= evlat2 && evlon >= evlon1 &&
                evlon <= evlon2 && dep >= dep1 && dep <= dep2 &&
            magn >= magn1 && magn <= magn2 ) {
                strcpy (evtline[*nevt], line);
                (*nevt)++;
                nmatch++;
            }
        }
  
        /*  Write event date to screen  */
        wrtoscr ( k, line );
        k++;
    } while ( strlen (word) == 4 );
}
  
void rdstl2 ( nevt, stnum )
int nevt, stnum;
{
    char stline[LINESIZE], code[10], lat[10], lon[10], c[4], ch;
    int i, k, stblk;
    float azie;
  
    /*  Determination of position of STATION-LOG  */
    stblk = statpos[stnum] - 1;
    cdread(stblk,buffer);
  
    /*  Read the 5th line of the buffer and determine lat and lon  */
    for (i=0; i < 80; i++) stline[i] = buffer[4*80 + i];
    stline[80] = '\0';
  
    sscanf (stline, "%*d %s %*s %s %s", code, lat, lon);
  
    /*  Determine floating point errors for stlat and stlon  */
    ch = lat[strlen(lat)-1];
    lat[strlen(lat)-1] = '\0';
    sscanf (lat, "%f", &stlat);
    if ( ch == 'S' ) stlat = -stlat;
  
    ch = lon[strlen(lon)-1];
    lat[strlen(lon)-1] = '\0';
    sscanf (lon, "%f", &stlon);
    if ( ch == 'W' ) stlon = -stlon;
  
    /*  Determine event latitude and longitude  */
    sscanf (evtline[nevt]+28, "%f %f", &evlat, &evlon);
    refpt(ggtogc(evlat*RAD),evlon*RAD);
  
    /*  Determine the distance to the selected event  */
    delaz2 (ggtogc(stlat*RAD), stlon*RAD, &del, &azie, &azis);
    del *= DEG;
    azie *= DEG;
    azis *= DEG;
  
    if ( stnum == 0 ) k=0;
  
    if ( stlat >= stlat1 && stlat <= stlat2 && stlon >= stlon1 &&
        stlon <= stlon2 && del >= del1 && del <= del2 &&
    azis >= azis1 && azis <= azis2 ) {
        k++;
  
        /*  Write station code to screen  */
        set_colour ( 3, 0, 7, 79, 8);
        blankstring (menu1[0]);
        strcpy (menu1[0]," STATION: ");
        strcat (menu1[0], code);
        textdisp (menu1, 0, 7, 79, 7);
  
        blankstring (menu1[0]);
        strcpy (menu1[0], " STATIONS FOUND PER EVENT: ");
        sprintf (c, "%d ", k);
        strcat (menu1[0], c);
        textdisp (menu1, 25, 7, 79, 7);
  
        fprintf (id3, "%s\n", stline);
    }
}
  
void wrtoscr ( num, evtl )
int  num;
char *evtl;
{
    char yr[5], dy[3];
    int  mn;
  
    /*  Print text on screen  */
    set_colour ( 3, 0, 4, 79, 5);
    if ( num == 0 ) {
        blankstring (menu1[0]);
        strcpy (menu1[0]," DATE :                   TOTAL EVENTS FOUND :      ");
        sscanf (evtl, "%s %d %s", yr, &mn, dy);
        sprintf (menu1[0]+8, "%s %s %s", dy, mon[mn-1], yr);
    }
    sprintf (menu1[0]+47, "%d", nmatch);
    textdisp (menu1, 0, 4, 51, 4);
}
  
void swbyte ( temp )
char temp[];
{
    int i, tt;
  
    for (i=0; i < 2000; i=i+4) {     /*  CD-VERSION 1 for ascii blocks  */
        tt = temp[i];
        temp[i] = temp[i+3];
        temp[i+3] = tt;
        tt = temp[i+1];
        temp[i+1] = temp[i+2];
        temp[i+2] = tt;
    }
}
  
/*  geocen - performs calculations relating to
postitions, distances, and azimuths*/
  
/* stores the coordinates (geocentric latitude and */
/* longitude) of a refernce point on the earth's surface */
  
void refpt(lat, lon)
float lat, lon;
{
    st0 = cos(lat);
    ct0 = sin(lat);
    phi0 = lon;
}
  
/* calculates the geocentric distance, and */
/* azimuth from the reference point to a given */
/* secondary point, and the back azimuth from */
/* the secondary point to the reference point */
  
void delaz2(lat, lon, delta, az0, az1)
float lat, lon, *delta, *az0, *az1;
{
    float ct1, st1, sdlon, cdlon, cdelt, sdelt,az;
  
    ct1 = sin(lat);
    st1 = cos(lat);
    sdlon = sin(lon - phi0);
    cdlon = cos(lon - phi0);
    cdelt = st0*st1*cdlon + ct0*ct1;
    cvrtop (st0*ct1-st1*ct0*cdlon, st1*sdlon, &sdelt, &az);
    *az0 = az;
    *delta = atan2(sdelt, cdelt);
    cvrtop (st1*ct0-st0*ct1*cdlon, -sdlon*st0, &sdelt, &az);
    *az1 = az;
    if (*az0 < 0) *az0 +=  TWOPI;
    if (*az1 < 0) *az1 +=  TWOPI;
}
  
/* calculates the geocentric coordinates of a */
/* secondary point, given its geocentric distance */
/* and azimuth from the reference point */
  
void back (delta, az0, lat, lon)
float delta, az0, *lat, *lon;
{
    float sdelt, cdelt, cz0, ct1;
    float st1, dlon;
  
    sdelt = sin(delta);
    cdelt = cos(delta);
    cz0 = cos(az0);
    ct1 = st0*sdelt*cz0 + ct0*cdelt;
    cvrtop (st0*cdelt-ct0*sdelt*cz0,sdelt*sin(az0), &st1, &dlon);
    *lat = atan2(ct1, st1);
    *lon = phi0 + dlon ;
    if (fabs(*lon) > PI) *lon = *lon - sign(TWOPI, *lon);
}
  
/* cvrtop - convert from rectangular to polar coordinates */
void cvrtop(x, y, r, theta)
float  x, y; /* (input)*/
float  *r, *theta;    /* (output - may overlay x, y) */
{
    float rad;
  
    rad = myhypot(x, y);
    *theta = atan2(y, x);
    *r = rad;
}
  
/* cv_lat - convert between geographic and geocentric latitude */
#define c1    ((1.-FLATTENING)*(1.-FLATTENING))
#define c2    (HALFPI*(1./c1 -1.))
  
/*  converts from geographic to geocentric latitude */
  
float ggtogc (lat)
float lat;
{
    float gg;
  
    if ((HALFPI-fabs(lat)) < .02) gg = lat/c1 -sign(c2, lat);
    else gg = atan(c1 * tan(lat));
    return gg;
}
  
/*  converts from geocentric to geographic latitude */
float gctogg(lat)
float lat;
{
    float gc;
  
    if ((HALFPI - fabs(lat)) < .02) gc = c1*(lat + sign(c2, lat));
    else gc = atan(tan(lat)/c1);
    return gc;
}
  
/* myhypot - euclidian distance, accurately and avoiding overflow */
float myhypot(a, b)
float a, b;
{
    float l, s, t;
    float euc;
  
    l = fabs(a);
    s = fabs(b);
    if (s > l) {
        t = s;
        s = l;
        l = t;
    }
    if (l == 0.0) euc = 0.0;
    else {
        s = s/l;
        euc = l*sqrt(1.0 + s*s);
    }
    return euc;
}
  
float sign (a, b)
float a, b;
{
    if (b > 0) return a;
    else return -a;
}
  
void cdread (recno, buf)
int recno;
char *buf;
{
    int n;
    char *prog_name = "CDREAD";
  
    if (recno >= 0) cdpos(recno);
    if (n = fread( buf,sizeof( *buf),BUFSIZE,id1) == NULL)
        error_and_exit (prog_name,"Could not read data log.");
    if (cdver == 1) swbyte( buf);
}
  
void cdpos (recno)
int recno;
{
    long int offset;
    int position;
    char *prog_name = "CDPOS";
  
    offset = (long) (recno)*2000;
    position = fseek(id1,offset,SEEK_SET);
    if (position) error_and_exit (prog_name," Seeking error.");
}
