/*****************************************************************************/
/**                                                                         **/
/**   Program: SONIC1 - Interactive Seismogram Display Program.             **/
/**                                                                         **/
/**            18/07/89  -  1.6                                             **/
/**            25/07/89  -  1.7                                             **/
/**            18/08/89  -  1.71                                            **/
/**            04/12/89  -  1.72                                            **/
/**            05/02/90  -  1.81                                            **/
/**            07/31/90  -  1.91                                            **/
/**            14/09/90  -  1.92                                            **/
/**            03/10/90  -  1.93                                            **/
/**                                                                         **/
/*****************************************************************************/
  
#include <stdio.h>
#include <process.h>
#include "menu.h"
#include "sonic.h"
  
#define LINESIZE 81

extern FILE  *id1;                          /*  Defined in module opencd  */
extern INT   plchoice;                      /*  Defined in module menu  */
extern BOOL  BPFILTER, HPFILTER;
INT PLOTBUF_FREE;
INT DISPLAY_FLAG = FALSE;
INT DATA_FLAG = FALSE;
  
main ()
{
    BYTE  evinfo[120], stinfo[120], stname[50];
    BYTE  comm1[LINESIZE], comm2[LINESIZE], wr_all[3];
    INT   totevt, evnum, totstat, stnum, nchan=0;
    INT   evday, chan_avail;
    INT   cgf=0, cev=0, cst=0, crp=0, cplm=0, rot=0, cro=0;
    INT   row, inpt=0, outpt=0, stop=0;
    FLOAT evtime, evtime2, evlat, evlon, distmax;
    FLOAT begintime, endtime, t0, t1, t2, t3;
  
    /*  Display title pages and determine the contents of the CD-ROM  */
  
    PLOTBUF_FREE = TRUE;
    BPFILTER = FALSE;
    HPFILTER = FALSE;
  
    title ();
  
    /*  Select one event-file and open this file  */
  
    /*  LOOP 1  */
    if (contents() != ESC) do {
        if (getfile ( cgf ) == ESC) break;
        openfile ();
  
        /*  Read first block of file and determine number of stations  */
        rdcdid ( &totstat );
  
        /*  Read event log and determine event parameters  */
  
        /*  LOOP 2  */
        while ( ( inpt = rdevl ( cev, &totevt, &evnum, evinfo, &evday, &evtime,
        &evtime2, &evlat, &evlon ) ) != ESC ) {
  
            /*  Make choice of plot to create  */
  
            /*  LOOP 3  */
            while ( ( inpt = prpmenu ( evinfo ) ) != ESC ) {
                cplm = 0;
  
                switch ( plchoice ) {
  
                    /*********************************************/
                    case 1:    /*  Creation of the amplitude vs. time plot  */
                        /*********************************************/
  
                        /*  Select station, read station log and station parameters  */
  
                        /*  LOOP A1  */
                        while ( ( inpt = selst ( cst, totevt,  evday,
                        evtime, totstat, &stnum, &distmax ) ) != ESC ) {
  
                            /*  LOOP B1 */
                            while ( ( inpt = rdstl ( stnum, stinfo, stname, comm1,
                            comm2) ) != ESC ) {
  
                                /*  Select data type, component and determine number of blocks */
  
                                /*  LOOP C1 */
                                while ( ( inpt = seldata ( stnum, stname, &nchan,
                                &chan_avail ) ) != ESC ) {
  
                                    /*  Display comments in stationlog and time correction */
                                    display_comm ( comm1, comm2, stnum);
  
                                    /*  Determine begintime end endtime of seismogram  */
                                    rdtimes ( stnum, totevt, evnum, evday, evtime,
                                    evtime2, &begintime, &endtime );
  
                                    /*  Give user possibility to change the time axis ... */
  
                                    if ( endtime > begintime )
                                        user1_time ( &begintime, &endtime, evnum );
  
                                    /*  Display the seismogram  */
                                    CrtPlot ( crp, rot, stnum, nchan, &begintime, &endtime,
                                    stinfo, evinfo );
  
                                    t0 = begintime;
                                    t2 = begintime;
                                    t1 = endtime;
                                    t3 = endtime;
  
                                    /*  Display the plot menu  */
  
                                    /*  LOOP D1  */
                                    do {
  
                                        outpt = PlotMenu ( nchan , cro, chan_avail, wr_all );
                                        DISPLAY_FLAG = TRUE;
                                        switch ( outpt ) {
  
                                            /*  Enlarge a specified part of the seismogram  */
                                            case 1:
                                                t2 = t0;
                                                t3 = t1;
                                                Zoom ( evtime, stnum, &t0, &t1 );
                                                crp = 1;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                break;
  
                                                /*  Set original values for time window  */
                                            case 2:
                                                t0 = begintime;
                                                t1 = endtime;
                                                crp = 2;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                break;
  
                                                /*  Cursor for determination of amplitude and time  */
                                            case 3:
                                                Curs ( evtime, stnum, t0 );
                                                break;
  
                                                /*  Set previous values for time window  */
                                            case 4:
                                                t0 = t2;
                                                t1 = t3;
                                                crp = 2;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                break;
  
                                                /*  Rotate the N/S and E/W components  */
                                            case 5:
                                                rot = 1;
                                                crp = 1;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                cro = 1;
                                                break;
  
                                                /*  Write figure in plot to file on hard disk  */
                                            case 6:
                                                if ( wr_all[0] == 'y' ) nchan = 3;
                                                WrtHd ( stnum, nchan, &t0, &t1 );
                                                crp = 3;
                                                DISPLAY_FLAG = FALSE;
  
                                                /*  Print the main menu on the screen ( text mode )  */
                                            case 7:
                                                cplm = 1;
                                                stop = 1;
                                                prmmenu ( &row );
                                                break;
  
                                                /*  Rotate radial and transverse components  */
                                            case 8:
                                                rot = 2;
                                                crp = 1;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                break;
                                                /* Band pass filter all components */
                                            case 9:
                                                crp = 1;
                                                BPFILTER = TRUE;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                /* if filter on filtered data  BPFILTER should be left TRUE ! */
                                                BPFILTER = FALSE;  /* filter on raw data ! */
  
                                                break;
                                                /* High pass filter all components */
                                            case 10:
                                                crp = 1;
                                                HPFILTER = TRUE;
                                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                                stinfo, evinfo );
                                                /* if filter on filtered data  HPFILTER should be left TRUE ! */
                                                HPFILTER = FALSE;  /* filter on raw data ! */
  
                                                break;
  
                                        }  /*  End of switch ( outpt )  */
  
                                    } while ( stop == 0 );  /*  END LOOP D1  */
  
                                    rot = 0;
                                    stop = 0;
                                    cro = 0;
  
                                    /*  Menu selection determines where the program continues  */
  
                                    if ( cplm == 1 && row != 5 ) break;
                                    DATA_FLAG = FALSE;
                                    crp = 0;
                                    DISPLAY_FLAG = FALSE;
  
                                }                 /*  END LOOP C1  */
                                break;
  
                            }      /* END LOOP B1 */
  
                            if ( row == 5 ) cplm = 0;
                            if ( cplm == 1 && row != 6 ) break;
                            cst = 2;
                            DATA_FLAG = FALSE;
                            crp = 0;
                            DISPLAY_FLAG = FALSE;
  
                        }  /*  END LOOP A1  */
  
                        break;
  
                        /**************************************/
                    case 2:      /*  Epicentral distance vs. time plot */
                        /**************************************/
  
                        /*  Select minimum and maximum distance of interest  */
  
                        /*  LOOP A2  */
                        while ( ( inpt = selst ( cst, totevt, evday,
                        evtime, totstat, &stnum, &distmax ) ) != ESC ) {
  
                            /*  Select data type, component, number of blocks  */
  
                            /*  LOOP B2  */
                            while ( ( inpt = seldata ( stnum, stname, &nchan,
                             &chan_avail ) ) != ESC ) {
  
                                /*  Determine the begin and end time  */
                                if (( evnum > totevt )  || ( totevt == 1 ) )
                                    settime2 ( &t0, &t1, evtime );
                                else settime ( evtime, distmax, &t0, &t1 );
  
                                /*  Give user possibility to change the time axis ... */
                                user2_time ( &t0, &t1, evtime );
  
                                /*  Display the epicentral distance vs. time plot  */
                                CrtPlot ( crp, rot, stnum, nchan, &t0, &t1, stinfo,
                                evinfo );
  
                                /*  Display the plot menu  */
  
                                /*  LOOP C2  */
                                do {
  
                                    outpt = PlotMenu ( nchan , cro, chan_avail, wr_all );
                                    DISPLAY_FLAG = TRUE;
  
                                    switch ( outpt ) {
  
                                        /*  Enlarge a specified part of the seismogram  */
                                        case 1:
                                            t2 = t0;
                                            t3 = t1;
                                            Zoom ( evtime, stnum, &t0, &t1 );
                                            crp = 0;
                                            CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                            stinfo, evinfo );
                                            break;
  
                                            /*  Set original values for time window  */
                                        case 2:
                                            t0 = evtime;
                                            t1 = t0;
                                            crp = 0;
                                            CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                            stinfo, evinfo );
                                            break;
  
                                            /*  Set previous values for time window  */
                                        case 4:
                                            t0 = t2;
                                            t1 = t3;
                                            crp = 0;
                                            CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                            stinfo, evinfo );
                                            break;
  
                                            /*  Write figure in plot to file on hard disk  */
                                        case 6:
                                            WrtHd ( stnum, nchan, &t0, &t1 );
  
                                            crp = 0;
                                            DISPLAY_FLAG = FALSE;
  
                                            /*  Print the main menu on the screen ( text mode )  */
                                        case 7:
                                            cplm = 1;
                                            stop = 1;
                                            prmmenu ( &row );
                                            break;
  
                                            /* Band pass filter all components */
                                        case 9:
                                            crp = 0;
                                            BPFILTER = TRUE;
                                            CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                            stinfo, evinfo );
                                            /* if filter on filtered data  BPFILTER should be left TRUE ! */
                                            BPFILTER = FALSE;  /* filter on raw data ! */
  
                                            break;
  
                                            /* High pass filter all components */
                                        case 10:
                                            crp = 0;
                                            HPFILTER = TRUE;
                                            CrtPlot ( crp, rot, stnum, nchan, &t0, &t1,
                                            stinfo, evinfo );
                                            /* if filter on filtered data  HPFILTER should be left TRUE ! */
                                            HPFILTER = FALSE;  /* filter on raw data ! */
  
                                            break;
  
  
                                    }  /*  End of switch ( outpt )  */
  
                                } while ( stop == 0 );  /*  END LOOP  C2  */
  
                                stop = 0;
  
                                /*  Menu selection determines where the program continues  */
  
                                if ( cplm == 1 && row != 5 ) break;
                                DATA_FLAG = TRUE;
                                crp = 0;
                                DISPLAY_FLAG = FALSE;
  
                            }   /*  END LOOP  B2  */
  
                            if ( row == 5 ) cplm = 0;
                            if ( cplm == 1 && row != 6 ) break;
                            cst = 2;
                            DATA_FLAG = FALSE;
                            crp = 0;
                            DISPLAY_FLAG = FALSE;
  
                        }           /*  END LOOP A2  */
  
                        break;   /*  End of case 2 of switch ( plchoice )  */
  
                }  /*  END OF SWITCH   */
  
                /*  Set counters to right values to continue  */
                cst = 2;
                DATA_FLAG = FALSE;
                crp = 0;
                DISPLAY_FLAG = FALSE;
                if ( row == 6 ) cplm = 0;
                if ( cplm == 1 && row != 7 ) break;
  
            }  /*  END LOOP 3  */
  
            /*  Set counters to right values to continue  */
            cev = 1;
            cst = 1;
            DATA_FLAG = FALSE;
            crp = 0;
            DISPLAY_FLAG = FALSE;
            if ( row == 7 ) cplm = 0;
            if ( cplm == 1 && row != 8 ) break;
  
        }  /*  END LOOP 2  */
  
        /*  Set the counter cgf to the right value  */
        if ( row == 8 ) cplm = 0;
        if ( cplm == 0 ) cgf = 1;
        else {
            if ( row == 9 ) cgf = 1;
            if ( row == 10 ) cgf = 0;
        }
  
        /*  Set all other counters to zero  */
        cev = 0; cst = 0;
        crp = 0; cplm = 0;
        DATA_FLAG = FALSE;
        DISPLAY_FLAG = FALSE;
  
        /*  Close the file on cd-rom  */
        fclose (id1);
  
    }  while ( row == 9 || row == 10 || inpt == ESC ); /*  END LOOP 1  */
 
    DisplayOff();
    system("CLS");
    exit(1);
}
