      PROGRAM LAMGEO
c
C     LAMGEO transforms coordinates (i.e. lambert into geographic).
C     Roy Freeman scavanged it from existing routines TRCH and TRCOORD
C     written by Bernd Lahmeyer. The transformation routines called by
c     TRC (previously TRCOORD) were written by E. Klingele and I. Marson.
c     The language in the interactive part and the code reflects
c     this international parenthood.
c     NOTE: This program is NOT an example of good programming style
c     and it was not intended for export. Use it with discretion, please.
c     It was coded for VAX/VMS fortran 66 and only partially updated to
c     f77. I have run it on the SUN with minor changes.
c     Most subroutines are now running with real*16 which is really only
c     necessary in "ELLEB" and "EBELL". The latter is deactivated due
c     to errors in some of the constants. Real*16 eats memory space
c     and CPU. If I was sure I was getting maximal
c     accuracy with only real*8 in the crucial Lambert transformations,
c     I would return it to double precision/real*8
c
C     This version of LAMGEO reads has input formats
c     to transfer EGT gravity data in Lambert back to geographic
c     coordinates. Last update: 24 july 1992 rf
c
c
c
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER*30 fileI,fileO
      REAL*4 g,z
      real*8 x,y
      CHARACTER*10 SUBTR
      DIMENSION IPAR(10),PAR(10)
      NPARM=10
C
      write (6,*) '  '
      write (6,*) 'Welcome to Lamgeo. You must explicitly declare input
     + format in program code!'
      write (6,*) '  '
C
   30 WRITE(6,*)' '
      WRITE(6,*)' ENTER THE DATA FILE NAME'
      WRITE(6,*)' '
      READ(5,'(A)',ERR=30)FILEI
      OPEN(1,FILE=FILEI,FORM='FORMATTED',STATUS='OLD',READONLY)
   31 WRITE(6,*)' '
      WRITE(6,*)' ENTER THE OUTPUT FILE NAME'
      WRITE(6,*)' '
      READ(5,'(A)',ERR=31) FILEO
      OPEN(2,FILE=FILEO,STATUS='NEW')
c
c     Open list output file for parameter records
c
      open(15, file='LAMGEO.PAR',status='new')
C
C     ask for transformation
c
      call trch(IPAR,PAR,NPARM,SUBTR)
c
c     main data loop
      WRITE(6,*)' '
      WRITE(6,*)' WAIT  :  THE PROGRAM IS WORKING'
      WRITE(6,*)' '
      n=0
 48   continue
      n=n+1
c
C     read the input file
c
crf      read(1,*,ERR=90)x,y,g,z
c      the above line has been commented out, but it usually works
c      if there are four values. This is not the case with the 
c      EGT gravity data where the height is sometimes missing. So...
c       now you have to put in explicitly the format for input data.
C   
      read (1,1000,err=90)x,y,g,z
c
c     for gravsouth.dat
 1000 format(4E15.7)
c     for gravsouth2.dat, etc. use
c 1000 format(4E17.8)
c
      call trc(x,y,x2,y2,ipar,par,nparm,subtr)
c
c     write the old and transformed data on output file
c       x2 and y2 = the transformed coordinates
c
crf      write(2,1004)x,y,g,z,x2,y2
      write(2,1005)n,x2,y2,g,z,x,y
c
c     loop around again
c
      GOTO 48
c
C     end of loop
c
c 1004  format(1x,4(F12.3,1x),2(F12.8,1x))
1005  format(1x,i5,1x,2(f12.8,1x),f7.2,1x,f8.3,1x,2(f13.4,1x))
c
c      goto 101
c
c     error messages
c
  90   write(6,*)' End of file, input error, or format change'
c      goto 101
C
 101  CLOSE (1)
      CLOSE(2)
c
      write(6,*) 'Number of points transformed = ',n
c
c
C     PROGRAM END
      end
C

      SUBROUTINE TRC (XX1,YY1,XX2,YY2,IPAR,PAR,NPARM,SUBTR)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit real*16 (a-h,o-z)
      real*4 XX1,YY1,XX2,YY2,PAR
C
C     subroutine to call transformation subroutines
c     originally assembled from exisiting routines (E. Klingele
c     and I Marson) by Bernd Lahmeyer for the EGT in 1988
c     The language and use of capital letters identifies the authors
c
c     This version corrected (subroutine "ELLEB") and updated
c     by Roy Freeman (rf) and Emile Klingele in July 1992.
c     We also made the change real*8 to real*16 (necessary [on the VAX] for
c     ELLEB and EBELL and a good idea (?) for all the Lambert calculations)
c     However, double precision is adequate for all the other routines.
C
C     (X1,Y2) ==> (X2,Y2)
C
C
      CHARACTER *(*) SUBTR
      DIMENSION IPAR(NPARM),PAR(NPARM)
C
      COMMON/FAKTOR/RHO
C
      COMMON /LAMB1/ PAR1,PAR2,PAR3,PAR4,IELL
      COMMON /LAMB2/ PAR5,PAR6,PAR7,PAR8
C
      RHO  = 180.0/(4.0*ATAN(qext(1.0)))
      RAD  = 206264.8062
C
crf      X1=DBLE(XX1)
crf      Y1=DBLE(YY1)
c
      X1=QEXT(xx1)
      Y1=QEXT(yy1)
C
      IF (SUBTR.EQ.'LBRH'    ) GOTO 100
      IF (SUBTR.EQ.'RHLB'    ) GOTO 200
      IF (SUBTR.EQ.'LAMBERT' ) GOTO 300
      IF (SUBTR.EQ.'LBEN'    ) GOTO 400
      IF (SUBTR.EQ.'ENLB'    ) GOTO 500
      IF (SUBTR.EQ.'COORD(1)') GOTO 600
      IF (SUBTR.EQ.'COORD(2)') GOTO 700
      IF (SUBTR.EQ.'LAMBINV' ) GOTO 800
      IF (SUBTR.EQ.'ELLEB'   ) GOTO 900
      IF (SUBTR.EQ.'EBELL'   ) GOTO 1000
      IF (SUBTR.EQ.'OHNE'    ) GOTO 1100
      STOP 'FEHLER IN TRC: WERT VON SUBTR FALSCH'
C
100   CALL LBRH(IPAR(1),IPAR(3),IPAR(2),X1,Y1,KZ,X2,Y2)
      GOTO 9999
C
200   CALL RHLB(IPAR(1),IPAR(3),IPAR(2),X1,Y1,ILH,X2,Y2)
      GOTO 9999
C
300   IF (IPAR(1).LE.-1) THEN
       PAR1=DBLE(PAR(1))
       PAR2=DBLE(PAR(2))
       PAR3=DBLE(PAR(3))
       PAR4=DBLE(PAR(4))
       IELL=IPAR(2)
      ENDIF
      IF (IPAR(1).EQ.-2) THEN
       PAR5=DBLE(PAR(5))
       PAR6=DBLE(PAR(6))
       PAR7=DBLE(PAR(7))
       PAR8=DBLE(PAR(8))
      ENDIF
      CALL LMBERT (IPAR(1),X1,Y1,X2,Y2)
      GOTO 9999
C
400   CALL LBEN(IPAR(1),IPAR(3),IPAR(2),X1,Y1,KZ,X2,Y2)
      GOTO 9999
C
500   CALL ENLB(IPAR(1),IPAR(3),IPAR(2),X1,Y1,ILH,X2,Y2)
      GOTO 9999
C
600   LATG=Y1
      LATP=(Y1-LATG)*60
      LATS=((Y1-LATG)*60-LATP)*60
      LONG=X1
      LONP=(X1-LONG)*60
      LONS=((X1-LONG)*60-LONP)*60+.5
      RLAT=((LATG*3600.)+(LATP*60.)+LATS)/RAD
      RLON=((LONG*3600.)+(LONP*60.)+LONS)/RAD
      CALL COORD (IPAR(1),RLAT,RLON,X2,Y2,IPAR(2),IPAR(3))
      GOTO 9999
C
700   CALL COORD (IPAR(1),RLAT,RLON,X1,Y1,IPAR(2),IPAR(3))
      RLAT=RLAT*RAD+0.5
      RLON=RLON*RAD+0.5
      IL1=RLAT
      IL2=RLON
      CALL CONW (IL1,LAT)
      CALL CONW (IL2,LON)
      LATG=LAT/10000
      LATP=(LAT-(LATG*10000))/100
      LATS=(LAT-(LATG*10000)-(LATP*100))
      LONG=LON/10000
      LONP=(LON-(LONG*10000))/100
      LONS=(LON-(LONG*10000)-(LONP*100))
      X2=LONG+(LONP/60.+LONS/3600.)
      Y2=LATG+(LATP/60.+LATS/3600.)
      GOTO 9999
C
800   IF (IPAR(1).EQ.-1) THEN
       PAR1=DBLE(PAR(1))
       PAR2=DBLE(PAR(2))
       PAR3=DBLE(PAR(3))
       PAR4=DBLE(PAR(4))
       IELL=IPAR(2)
       CALL LMBERT (IPAR(1),X1,Y1,X2,Y2)
       GOTO 9999
      ENDIF
      CALL LMBINV (X1,Y1,X2,Y2)
      GOTO 9999
C
900   X1= X1*3600.-26782.5
      Y1= Y1*3600.-169028.66
      CALL ELLEB (X1,Y1,X2,Y2)
      X2=X2+600000.
      Y2=Y2+200000.
      GOTO 9999
C
1000  X1=X1-600000.
      Y1=Y1-200000.
c      
      write (6,*) 'Subroutine EBELL has been deactivated due to errors'
crf      CALL EBELL (X1,Y1,X2,Y2)
crf      X2= (X2 + 26782.5D00)/3600.
crf      Y2= (Y2 + 169028.66D00)/3600.
      GOTO 9999
C
1100  X2=X1
      Y2=Y1
      GOTO 9999
C
9999  CONTINUE
      XX2=SNGL(X2)
      YY2=SNGL(Y2)
      RETURN
      END
      SUBROUTINE LBRH(IENT,INS,ILH,L,B,KZ,R,H)
crf      IMPLICIT DOUBLE PRECISION  (A-H,L-Z)
      implicit real*16 (a-h,l-z)
      COMMON/FAKTOR/RHO
      DL1=L-REAL(ILH)
      DL2=DL1*DL1
      DL3=DL2*DL1
      DL4=DL2*DL2
      DL5=DL3*DL2
      CB1=COS(B/RHO)
      CB2=CB1*CB1
      CB3=CB2*CB1
      CB4=CB2*CB2
      CB5=CB3*CB2
      RHO1=RHO
      RHO2=RHO1*RHO1
      RHO3=RHO2*RHO1
      RHO4=RHO2*RHO2
      RHO5=RHO3*RHO2
      T1=ATAN(B/RHO)
      T2=T1*T1
      T4=T2*T2
      ETA2=0.006719219*CB2
      N=6398786.849/SQRT(1.0+ETA2)
      NT1=N*T1
      CALL BOGEN(IENT,B,NULLX)
      ZWOX=NT1*CB2/(2.0*RHO2)
      VIERX=NT1*CB4*(5.0-T2+9.0*ETA2)/(24.0*RHO4)
      EINSY=N*CB1/RHO1
      DREIY=N*CB3*(1.0-T2+ETA2)/(6.0*RHO3)
      FUENFY=N*CB5*(5.0-18.0*T2+T4)/(120.0*RHO5)
      X=NULLX+ZWOX*DL2+VIERX*DL4
      Y=EINSY*DL1+DREIY*DL3+FUENFY*DL5
      KZ=ILH/3
      R=REAL(KZ)*1.0E06+5.0E05+Y
      H=X
      RETURN
      END
      SUBROUTINE LBEN(IENT,INS,ILH,L,B,IZONE,E,N)
crf      IMPLICIT DOUBLE PRECISION   (A-H,L-Z)
      implicit real*16 (a-h,l-z)
      COMMON/FAKTOR/RHO
      DL1=L-REAL(ILH)
      DL2=DL1*DL1
      DL3=DL2*DL1
      DL4=DL2*DL2
      DL5=DL3*DL2
      DL6=DL3*DL3
      CB1=COS(B/RHO)
      CB2=CB1*CB1
      CB3=CB2*CB1
      CB4=CB2*CB2
      CB5=CB3*CB2
      CB6=CB3*CB3
      RHO1=RHO
      RHO2=RHO1*RHO1
      RHO3=RHO2*RHO1
      RHO4=RHO2*RHO2
      RHO5=RHO3*RHO2
      RHO6=RHO3*RHO3
      T1=ATAN(B/RHO)
      T2=T1*T1
      T4=T2*T2
      ETA2=0.006768170*CB2
      N1=6399936.608/SQRT(1.0+ETA2)
      MHNT1=N1*T1*0.9996
      MHN=0.9996*N1
      CALL BOGEN(IENT,B,NULLX)
      NULLX=0.9996*NULLX
      ZWOX=MHNT1*CB2/(2.0*RHO2)
      VIERX=MHNT1*CB4*(5.0-T2+9.0*ETA2)/(24.0*RHO4)
      SECHSX=MHNT1*CB6*(61.0-58.0*T2+T4)/(720.0*RHO6)
      EINSY=MHN*CB1/RHO1
      DREIY=MHN*CB3*(1.0-T2+ETA2)/(6.0*RHO3)
      FUENFY=MHN*CB5*(5.0-18.0*T2+T4+ETA2*(14.0-58.0*T2))/(120.0*RHO5)
      X=NULLX+ZWOX*DL2+VIERX*DL4+SECHSX*DL6
      Y=EINSY*DL1+DREIY*DL3+FUENFY*DL5
      IZONE=(ILH+183)/6
      CORR=0.0
      N=X
      IF(N.LT.0.0) N=N+1.0E07
      E=Y+5.0E05
      RETURN
      END
      SUBROUTINE RHLB(IENT,INS,KZ,R,H,ILH,L,B)
crf      IMPLICIT DOUBLE PRECISION  (A-H,L-Z)
      implicit real*16 (a-h,l-z)
      COMMON/FAKTOR/RHO
      Y1=R-REAL(KZ)*1.0E06-5.0E05
      Y2=Y1*Y1
      Y3=Y2*Y1
      Y4=Y2*Y2
      Y5=Y3*Y2
      Y6=Y3*Y3
      X=H
      ILH=KZ*3
      CALL BREITE(IENT,X,BF)
      CBF=COS(BF/RHO)
      ETA2=0.006719219*CBF*CBF
      N1=6398786.849/SQRT(1.0+ETA2)
      N2=N1*N1
      N3=N2*N1
      N4=N2*N2
      N5=N3*N2
      N6=N3*N3
      T1=ATAN(BF/RHO)
      T2=T1*T1
      T4=T2*T2
      RHOT1=RHO*T1
      NULLB=BF
      ZWOB=-RHOT1*(1.0+ETA2)/(2.0*N2)
      VIERB=RHOT1*(5.0+3.0*T2+6.0*ETA2*(1.0-T2))/(24.0*N4)
      SECHSB=-RHOT1*(61.0+90.0*T2+45.0*T4)/(720.0*N6)
      EINSL=RHO/(N1*CBF)
      DREIL=-RHO*(1.0+2.0*T2+ETA2)/(6.0*N3*CBF)
      FUENFL=RHO*(5.0+28.0*T2+24.0*T4)/(120.0*N5*CBF)
      L=REAL(ILH)+EINSL*Y1+DREIL*Y3+FUENFL*Y5
      B=NULLB+ZWOB*Y2+VIERB*Y4+SECHSB*Y6
      IF(INS.LT.0) B=-B
      RETURN
      END
      SUBROUTINE ENLB(IENT,INS,IZONE,E,N,ILH,L,B)
crf      IMPLICIT DOUBLE PRECISION  (A-H,L-Z)
      implicit real*16 (a-h,l-z)
      COMMON/FAKTOR/RHO
      Y1=E-5.0E05
      Y2=Y1*Y1
      Y3=Y2*Y1
      Y4=Y2*Y2
      Y5=Y3*Y2
      Y6=Y3*Y3
      X=N
      IF(INS.LT.0) X=X-1.0E07
      ILH=(IZONE-30)*6-3
      XMH=X/0.9996
      CALL BREITE(IENT,XMH,BF)
      CBF=COS(BF/RHO)
      ETA2=0.006768170*CBF*CBF
      N=6399936.608/SQRT(1.0+ETA2)
      MHN1=0.9996*N
      MHN2=MHN1*MHN1
      MHN3=MHN2*MHN1
      MHN4=MHN2*MHN2
      MHN5=MHN3*MHN2
      MHN6=MHN3*MHN3
      T1=ATAN(BF/RHO)
      T2=T1*T1
      T4=T2*T2
      RHOT1=RHO*T1
      NULLB=BF
      ZWOB=-RHOT1*(1.0+ETA2)/(2.0*MHN2)
      VIERB=RHOT1*(5.0+3.0*T2+6.0*ETA2*(1.0-T2))/(24.0*MHN4)
      SECHSB=-RHOT1*(61.0+90.0*T2+45.0*T4)/(720.0*MHN6)
      EINSL=RHO/(MHN1*CBF)
      DREIL=-RHO*(1.0+2.0*T2+ETA2)/(6.0*MHN3*CBF)
      FUENFL=RHO*(5.0+28.0*T2+24.0*T4)/(120.0*MHN5*CBF)
      L=REAL(ILH)+EINSL*Y1+DREIL*Y3+FUENFL*Y5
      B=NULLB+ZWOB*Y2+VIERB*Y4+SECHSB*Y6
C     IF(INS.LT.0) B=-B-90.0
      RETURN
      END
      SUBROUTINE BOGEN(IENT,B,G)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit real*16 (a-h,o-z)
crf      DOUBLE PRECISION E0,E2,E4,E6,G,B,RHO
      real *16 E0,E2,E4,E6,G,B,RHO
      COMMON/FAKTOR/RHO
      GO TO (10,20),IENT
C     BERECHNUNG DER MERIDIANBOGENLAENGE AUS DER BREITE AUF
C     DEM BESSELELLIPSOID:
  10  E0=111120.619608
      E2=-15988.6383
      E4=    16.7300
      E6=-    0.0218
      G=E0*B+E2*SIN(B/RHO*2.0)+E4*SIN(B/RHO*4.0)+E6*SIN(B/RHO*6.0)
      RETURN
C     BERECHNUNG DER MERIDIANBOGENLAENGE AUS DER BREITE AUF DEM
C     HAYFORDELLIPSOID:
  20  E0=111136.536655
      E2=-16107.0347
      E4=    16.9763
      E6=-    0.0223
      G=E0*B+E2*SIN(B/RHO*2.0)+E4*SIN(B/RHO*4.0)+E6*SIN(B/RHO*6.0)
      RETURN
      END
      SUBROUTINE BREITE(IENT,G,B)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
crf      DOUBLE PRECISION E0,F2,F4,F6,SIGMA,RHO,G,B
      IMPLICIT real*16 (A-H,O-Z)
      real *16 E0,F2,F4,F6,SIGMA,RHO,G,B
      COMMON/FAKTOR/RHO
      IB=IABS(IENT)
      GO TO (10,20),IB
C     BERECHNUNG DER GEOGRAPHISCHEN BREITE AUS DER MERIDIANBOGENLAENGE
C     DES LOTFUSZPUNKTES AUF DEM BESSELELLIPSOID:
  10  E0=111120.619608
      F2=0.143885358
      F4=0.000210790
      F6=0.000000423
      SIGMA=G/E0
      B=SIGMA+F2*SIN(2.0*SIGMA/RHO)+F4*SIN(4.0*SIGMA/RHO)+
     1F6*SIN(6.0*SIGMA/RHO)
      RETURN
C     BERECHNUNG DER GEOGRAPHISCHEN BREITE AUS DER MERIDIANBOGENLAENGE
C     DES LOTFUSZPUNKTES AUF DEM HAYFORDELLIPSOID:
  20  E0=111136.536655
      F2=0.144930071
      F4=0.000213851
      F6=0.000000432
      SIGMA=G/E0
      B=SIGMA+F2*SIN(2.0*SIGMA/RHO)+F4*SIN(4.0*SIGMA/RHO)+
     1F6*SIN(6.0*SIGMA/RHO)
      RETURN
      END
      SUBROUTINE LMBERT (IMODE,LONG,LAT,X,Y)
      implicit real*16 (A-H,O-Z)
C
C
C*********************************************************************
C
C
C     PROGRAM LAMBERT              VERSION  2      E.KLINGELE  1987
C                                  VERSION         B.LAHMEYER  1988
C
C     THIS PROGRAM PERFORM TRANSFORMATIONS OF GEOGRAPHIC COORDINATES
C     TO CARTESIANS COORDINATES IN LAMBERT COMFORMAL CONICAL PROJECTION
C     WITH TWO STARDARD PARALLELS (MODE 1)
C
C     IT IS ALSO POSSIBLE TO TRANSFORM DIRECTELY THE COORDINATES FROM
C     A PROJECTION TO AN OTHER PROJECTION
C     (SYSTEM WITH DIFFERENT STANDARD PARALELELS)
C
C*********************************************************************
C
C
C                   A T T E N T I O N
C                   -----------------
C
C
C*********************************************************************
C
C INPUT:  IMODE            = 1  MODE1, = 2 MODE2
C                          =-1  MODE1, =-2 MODE2 (INITIALIZATION)
C
C         LAT,LONG      LATTITUDE AND LONGITUDE OF POINTS TO BE
C                       TRANSFORMED    (IMODE=1)
C
C                       X- AND Y-COORDINATES TO BE TRANSFORMED
C                       (IMODE=2)
C
C OUTPUT: X(I)  ,Y(I)   X-AND Y-COORDINATES OF TRANSFORMED POINTS
C                       (IMODE=1 AND IMODE=2)
C
C
C
C COMMON BLOCK LAMB1: CPAR1,CPAR2,CMERIC,CPARO,IELL
C
C  THE STANDARD PARALLELS, THE CENTRAL MERIDIEN AND THE ORIGIN PARLLEL
C  OF THE FIRST PROJECTION
C  IELL.NE.0 : HAYFORD ELLIPSOD
C
C COMMON BLOCK LAMB2: CPAR12,CPAR22,CMERIC2,CPARO2
C
C  THE STANDARD PARALLELS, THE CENTRAL MERIDIEN AND THE ORIGIN PARALLEL
C  OF THE SECOND PROJECTION (ONLY IMODE=2)
C
C
C
      COMMON /LAMB1/ CPAR1,CPAR2,CMERIC,CPARO,IELL
      COMMON /LAMB2/ CPAR12,CPAR22,CMERC2,CPARO2
C
crf      DOUBLE PRECISION MERIC,MERIDC,MERIC2,N1,N2,LAT,LONG,LATT,LONGT
      real*16 MERIC,MERIDC,MERIC2,N1,N2,LAT,LONG,LATT,LONGT
C
      DATA PI/3.141592654/
C
C
C A1,USF1 : NORMAL VERSION
C A2,USF2 : HAYFORD ELLIPSOID (FOR EGT OPTION)
C
      DATA A1,USF1/6378.137,298.257/
      DATA A2,USF2/6378.388,297.000/
C
C
C INITIALIZATION MODE 1
C
      IF (IMODE.LT.0) THEN
       A   = A1
       USF = USF1
        IF (IELL.NE.0) THEN
         A   = A2
         USF = USF2
        ENDIF

       PI4=PI/4.
       F=1./USF
       DRAD=PI/180.
C
      IF (CPARO.GE.0.) THEN
       PARAO=CPARO*DRAD
       PARA1=CPAR1*DRAD
       PARA2=CPAR2*DRAD
      ELSE
       PARAO=-CPARO*DRAD
       PARA1=-CPAR2*DRAD
       PARA2=-CPAR1*DRAD
      ENDIF
       MERIC=CMERIC*DRAD
       SINOO=SIN(PARAO)
       SIN1=SIN(PARA1)
       SIN2=SIN(PARA2)
       E2=2.*F-F*F
       E=SQRT(E2)
       E05=E/2.
C
C     COMPUTE THE TRANSFORMATION PARAMETER
C
       DEN1=SQRT(1.-E2*SIN1*SIN1)
       N1=A/DEN1
       DEN2=SQRT(1.-E2*SIN2*SIN2)
       N2=A/DEN2
       ANUM=LOG(N1*COS(PARA1))-LOG(N2*COS(PARA2))
       Z21=Z2(E,E05,PARA1,SIN1)
       Z22=Z2(E,E05,PARA2,SIN2)
       SINO=ANUM/(LOG(Z21)-LOG(Z22))
       TG=N1*COS(PARA1)/(SINO*(Z21**SINO))
       TD=N2*COS(PARA2)/(SINO*(Z22**SINO))
       C=(TG+TD)/2.
       RO=C*(Z2(E,E05,PARAO,SINOO)**SINO)
C
C INITIALIZATION MODE 2
C
       IF(IMODE.EQ.-2) THEN
         MERIC2=CMERC2*DRAD
        IF (CPARO.GE.0.) THEN
          PARO2=CPARO2*DRAD
          PAR12=CPAR12*DRAD
          PAR22=CPAR22*DRAD
         ELSE
          PARO2=-CPARO2*DRAD
          PAR12=-CPAR22*DRAD
          PAR22=-CPAR12*DRAD
         ENDIF
         SINOO2=SIN(PARO2)
         SIN1=SIN(PAR12)
         SIN2=SIN(PAR22)
C
C     COMPUTE THE PARAMETER OF THE SECOND TRANSFORMATION
C
         DEN1=SQRT(1.-E2*SIN1*SIN1)
         N1=A/DEN1
         DEN2=SQRT(1.-E2*SIN2*SIN2)
         N2=A/DEN2
         ANUM=LOG(N1*COS(PAR12))-LOG(N2*COS(PAR22))
         Z21=Z2(E,E05,PAR12,SIN1)
         Z22=Z2(E,E05,PAR22,SIN2)
         SINO2=ANUM/(LOG(Z21)-LOG(Z22))
         TG=N1*COS(PAR12)/(SINO2*(Z21**SINO2))
         TD=N2*COS(PAR22)/(SINO2*(Z22**SINO2))
         C2=(TG+TD)/2.
         RO2=C2*(Z2(E,E05,PARO2,SINOO2)**SINO2)
         RO12=RO*RO
         AL1=1./SINO
         AL2=1./SINO2
         AL21=SINO2/SINO
         SHIFT=SINO2*(MERIC-MERIC2)
       ENDIF
       RETURN
      ENDIF
C
C END INITIALIZATION
C
      IF (IMODE.EQ.2) GOTO 2
C
C     TRANSFORM THE COORDINATES FROM GEOGRAPHIC TO LAMBERT
C
1     LATT=LAT*DRAD
      IF (CPARO.LT.0.) LATT=-LAT*DRAD
      LONGT=LONG*DRAD
      DLON=LONGT-MERIC
      SINT=SIN(LATT)
      RM=ABS(C*(Z2(E,E05,LATT,SINT)**SINO))
      ARG=DLON*SINO
      X=RM*SIN(ARG)*1000.
      Y=RO*1000.-RM*COS(ARG)*1000.
      IF (CPARO.LT.0.) Y=-Y
      RETURN
C
C     MODE 2 : TRANSFORM FROM LAMBERT NO. 1 TO LAMBERT NO. 2
C
2     X1=LONG/1000.
      Y1=LAT/1000.
      IF (CPARO.LT.0.) Y1=-LAT
      R1=SQRT(X1*X1+RO12-2.*RO*Y1+Y1*Y1)
      T=(R1/C)**AL1
      R2=T**SINO2
      R2=R2*C2
      ROY=RO-Y1
      X2=R2*SIN(SHIFT+AL21*ASIN(X1/R1))
      IF(ROY.GT.R1) ROY=1.
      SISI=SIGN(QEXT(1.),X1)
      ARGC=SISI*SHIFT+AL21*ACOS(ROY/R1)
      Y2=RO2-R2*COS(ARGC)
      X=X2*1000.
      Y=Y2*1000.
      IF (CPARO.LT.0.) Y=-Y2
      RETURN
      END
C
C
crf      DOUBLE PRECISION FUNCTION Z2(E,E05,TETHA,SINV)
      REAL*16 FUNCTION Z2(E,E05,TETHA,SINV)
      IMPLICIT real*16 (A-H,O-Z)
C
C
C     THIS FUNCTION COMPUTE THE RATIO
C     ((1+E*SINV)/(1-E*SINV))**E/2
C     WHERE E IS THE MAJOR ECCENTRICITY OF THE ELLIPSOID
C     AND V A PARALLEL VALUE
C
C
      DATA PI4/0.7853981634/
      PHI=PI4-TETHA/2.
      T1=TAN(PHI)
      ANUM=1.+E*SINV
      DENOM=1.-E*SINV
      T2=(ANUM/DENOM)**E05
      Z2=T1*T2
      RETURN
      END
      SUBROUTINE COORD (N,XLAT,YLON,X,Y,NF,MG)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
      IF (N.NE.1) GOTO 2
      IF (MG.EQ.1) YLON=YLON+0.2173342162
      IF (NF.EQ.1) DLON=YLON-0.1570796327
      IF (NF.EQ.2) DLON=YLON-0.2617993878
      CALL GIPI (XLAT,DLON,X,Y1)
      Y=Y1+1500000.
      IF (NF.EQ.2) Y=Y1+2520000.
      RETURN
2     CONTINUE
      NF=Y/1.E6
      IF (NF.EQ.1) Y1=Y-1500000.
      IF (NF.EQ.2) Y1=Y-2520000.
      CALL PIGI (X,Y1,XLAT,DLON)
      IF (NF.EQ.1) YLON=DLON+0.1570796327
      IF (NF.EQ.2) YLON=DLON+0.2617993878
      IF (MG.EQ.1) YLON=YLON-0.2173342162
      RETURN
      END
      SUBROUTINE GIPI (RLAT,DLON,X,Y1)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
      ALFA(N)=DLON**N/REAL(N)
      SIN1=SIN(RLAT)
      COS1=COS(RLAT)
      CALL NRHO (RLAT,EN,RHO)
      CHI=.9996*COS1*EN
      CALL ARCMER (1,X1,RLAT)
      COS2=COS1*COS1
      COS4=COS2*COS2
      COS6=COS4*COS4
      X=.9996*X1+CHI*SIN1*(ALFA(2)-(1./6.-COS2-COS4/99.)*ALFA(4)+(1./120
     *.-COS2/2.+COS4)*ALFA(6))
      Y1=CHI*(ALFA(1)-(.5-COS2-COS4/296.)*ALFA(3)+
     *(1./24.-COS2/1.2+COS4/1.017+COS6/50.)*ALFA(5))
      RETURN
      END
      SUBROUTINE PIGI (X,Y1,RLAT,DLON)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
C     CONVERTE DA PIANE A GEOGRAFICHE
      BETA(N)=YY**N/REAL(N)
      X1=X/.9996
      CALL ARCMER (2,X1,RLAT1)
      COS1=COS(RLAT1)
      COS2=COS1*COS1
      COS4=COS2*COS2
      CALL NRHO (RLAT1,EN,RHO)
      YY=Y1/EN/COS1/.9996
      RLAT=RLAT1-SIN(RLAT1)*COS1*((1.+COS2/147.75)*BETA(2)-(1./2.+COS2/3
     *.062+COS4/74.3)*BETA(4)+(3./8.+COS4/8.)*BETA(6))
      DLON=BETA(1)-(1.-COS2/2.+COS4/295.5)*BETA(3)+(1.-COS2/1.2+COS4/22.
     *8)*BETA(5)-(1.-COS2/.86+COS4/3.9)*BETA(7)
      RETURN
      END
      SUBROUTINE NRHO (RLAT,EN,RHO)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
C
      DATA AA/6378388./,E2/0.0067226700223/
C
      DD=1.-E2*SIN(RLAT)**2
      EN=AA/SQRT(DD)
      RHO=EN*(1.-E2)/DD
      RETURN
      END
      SUBROUTINE ARCMER (N,X,RLAT)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
C
C     SE N=1 EFFETTUA LA CONVERSIONE DIRETTA
C     SE N=2 EFFETTUA LA CONVERSIONE INVERSA
C
      NR=0
      A1=6367654.5001
      A2=16107.0347
      A3=16.9762
      A4=0.0223
      IF (N.NE.1) GOTO 2
1     CONTINUE
      X=A1*RLAT-A2*SIN(2*RLAT)+A3*SIN(4*RLAT)-A4*SIN(6*RLAT)
      RETURN
2     CONTINUE
      RLAT=X/A1
10    SRX=A1*RLAT-A2*SIN(2*RLAT)+A3*SIN(4*RLAT)-A4*SIN(6*RLAT)
      DX=X-SRX
      RLAT=RLAT+DX/A1
      IF ((X-SRX).LE.0.0005) RETURN
      NR=NR+1
      IF (NR.NE.50) GOTO 10
      WRITE (6,100)
100   FORMAT (1X,'TROPE ITERAZIONE IN ARCMER')
      STOP
      END
      SUBROUTINE CONW (II,LON)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
C
C TRASFORMA DA SECONDI IN GRADI, PRIMI E SECONDI
C
      ISW=0
      IF (II.LT.0) ISW=1
      II=IABS(II)
      IG=II/3600
      IP=(II-IG*3600)/60
      IS=II-IG*3600-IP*60
      LON=IG*10000+IP*100+IS
      IF (ISW.EQ.1) LON=-LON
      RETURN
      END
      SUBROUTINE LMBINV (X,Y,LON,LAT)
crf      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT real*16 (A-H,O-Z)
C
C U.P. ZUR RUECKTRANSFORMATION VON RECTWINKLIGEN KOORDINATE IN LAMBERT
C TRANSFORMATION IN LAENGE UND BREITE. DIE ROUTINE LAMBERT MUSS VORHER
C INITIALISIERT WERDEN.
C
C EINGABE
C
C X    Y     KOORDINATEN IN LAMBERT TRANSFORMATION
C
C AUSGABE
C
C LON LAT    BERECHNETE GEOGRAFISCHE KOORDINATEN
C
C
      COMMON /LAMB1/ PAR1,PAR2,PAR3,PAR4,IELL
C
      real*16 LAT,LON
C
      DATA PREC/10./,NIT/100/
C
      FAKT1=4.*ATAN(1.)/180.
      FAKT2=40000000./360.
C
      X1=0.
      Y1=0.
      LAT=PAR4
      LON=PAR3
      DO 10 I=1,NIT
      LAT=LAT+(Y-Y1)/FAKT2
      LON=LON+(X-X1)/(FAKT2*COS(LAT*FAKT1))
      CALL LMBERT (1,LON,LAT,X1,Y1)
      IF (ABS(X1-X).LT.PREC .AND. ABS(Y1-Y).LT.PREC) RETURN
10    CONTINUE
      END
      SUBROUTINE ELLEB (L1,B1,X1,Y1)
C
C
C     ROUTINE DE TRANSFORMATION DES COORDONNES GEOGRAPHIQUES L,B
C     (ELLIPSOIDALES) EN COORDONNEES CARTESIENNES SUISSES
C
C     LES PARAMETRES SONT LES MEMES QUE POUR EBELL
C
C
      IMPLICIT real*16 (A-H,O-Z)
      real*16 L,MY,IW
      real*8 X1,Y1,L1,B1
      dimension D(8),E(8),F(8),RW(8),IW(8)
C
      L=QEXT(L1)
      B=QEXT(B1)
C
      D(1)= 0.683825462627Q0
      D(2)= -3.91798328045Q-8
      D(3)= 1.4965410352Q-15
      D(4)= -8.039471422Q-23
      D(5)= 7.0021390Q-30
      D(6)= -5.586904Q-37
      D(7)= 4.0402Q-44
      D(8)= -3.06Q-51
      E(1)= 2.3635916074715Q-2
      E(2)= 0.0Q0
      E(3)= 4.527219881Q-17
      E(4)= -3.89081120Q-24
      E(5)= 2.3407700Q-31
      E(6)= -1.59674Q-38
      E(7)= 1.287Q-45
      E(8)= 0.0Q0
      F(1)= 4.515344386039Q+1
      F(2)= 1.17912305209Q-4
      F(3)= 5.8474201864Q-10
      F(4)= 2.73386187Q-15
      F(5)= 1.4308547Q-20
      F(6)= 7.66562Q-26
      F(7)= 4.2445Q-31
      F(8)= 2.40Q-36
      P=30.91849390613Q0*L
      Q=B*F(8)
      I=8
    1 I=I-1
      Q=B*(Q+F(I))
      IF(I.GT.1) GO TO 1
      RW(1)=Q
      IW(1)=P
      DO 2 I=2,8
      RW(I)=Q*RW(I-1)-P*IW(I-1)
      IW(I)=P*RW(I-1)+Q*IW(I-1)
    2 CONTINUE
      X=D(8)*RW(8)
      Y=D(8)*IW(8)
      MY=0.Q0
      I=8
    3 I=I-1
      X=X+D(I)*RW(I)
      Y=Y+D(I)*IW(I)
      MY=MY+E(I)*IW(I)
      IF(I.GT.1) GO TO 3
C
      X1=SNGLQ(Y)
      Y1=SNGLQ(X)
C
      RETURN
      END
       SUBROUTINE EBELL (X1,Y1,L1,B1)
C
C
C     ROUTINE DE TRANSFORMATION DES COORDONEES CARTESIENNES SUISSES (Y,X)
C     EN COORDONNEES GEOGRAPHIQUES L,B (ELLIPSOIDALES)
C
C     L ORIGINE DE CES COORDONNEES SE TROUVE A BERNE
C
C     Y ET X SONT EN METRES , L ET B SONT EN SECONDES SEXAGESIMALES
C     ET SONT DONNES PAR RAPPORT AU POINT D ORIGINE
C
C     (YO=0. , XO=0. , LO=7 26 22.50 , BO=46 57 08.66)
C
C     MY DONNE LA CONVERGENCE DES MERIDIENS EN SECONDES
C
C     L AXE DES Y CORESPOND A LA DIRECTION OUEST-EST ALORS QUE L AXE
C     X CORRESPOND A LA DIRECTION SUD-NORD
C
C
C**************************************************************************
C
C
C                    ATTENTION
C                    ---------
C
C     LES COORDONNEES CARTESIENNES DES CARTES TOPOGRAPHIQUES
C     ONT POUR ORIGINE YO=600000 ET XO=200000
C
crf     IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit real*16 (a-h,o-z)
      real*16 X1,Y1,L1,B1
      real*16 A(8),BB(8),C(8),RZ(8),IZ(8),MY,L
C
      X=QEXT(Y1)
      Y=QEXT(X1)
C D->Q
crf   There are proably errors in the exponents in the following
crf   list. The program has been deactiviated. Contact R. Freeman (rf)
crf   or E. Klingele (Institute fuer Geophysik, ETH-Zuerich)
c
      A(1)= 1.4623614572021Q0
      A(2)= 1.225255821052Q-7
      A(3)= 1.3687923002Q-14
      A(4)= 1.971224191Q-21
      A(5)= 2.97898051Q-28
      A(6)= 4.650273Q-35
      A(7)= 7.48203Q-38
      A(8)= 1.229Q-38
      BB(1)=3.4564252673326Q-2
      BB(2)=2.89600437564Q-9
      BB(3)=4.651046030Q-16
      BB(4)=6.43850954Q-23
      BB(5)=9.600412Q-30
      BB(6)=1.50512Q-36
      BB(7)=2.422Q-38
      BB(8)=0.0Q0
      C(1)= 2.2146704979846Q-2
      C(2)= -1.280815253730Q-9
      C(3)= 7.4775676024Q-18
      C(4)= 4.691943327Q-24
      C(5)= -3.6550101Q-31
      C(6)= 3.71615Q-38
      C(7)= 1.6901Q-38
      C(8)= 1.96Q-38
      RZ(1)=X
      IZ(1)=Y
      DO 1 I=2,8
      RZ(I)=X*RZ(I-1)-Y*IZ(I-1)
      IZ(I)=Y*RZ(I-1)+X*IZ(I-1)
    1 CONTINUE
      Q=A(8)*RZ(8)
      P=A(8)*IZ(8)
      MY=0.Q0
      I=8
    2 CONTINUE
      I=I-1
      Q=Q+A(I)*RZ(I)
      P=P+A(I)*IZ(I)
      MY=MY+BB(I)*IZ(I)
      IF(I.GT.1) GO TO 2
      L=3.2343101932327Q-2*P
      B=Q*C(8)
      I=8
    3 I=I-1
      B=Q*(B+C(I))
      IF(I.GT.1) GO TO 3
C
      L1=SNGLQ(L)
      B1=SNGLQ(B)
C
      RETURN
      END
      SUBROUTINE TRCH (IPAR,PAR,NPARM,SUBTR)
C
C U.P. ZUM FESLEGEN EINER TRANSFORMATION, GEGEBENENFALLS WIRD DIE
C TRANSFORMATION INITIALISIERT.
C
      DIMENSION IPAR(NPARM),PAR(NPARM)
      CHARACTER*(*) SUBTR
      CHARACTER*1 T
C
      WRITE (15,*) 'Applied TRANSFORMATION:'
C
1     WRITE (6,'(/A,/)') ' Choose desired TRANSFORMATION:'
      WRITE (6,*)
      WRITE (6,*) '1   GEOGRAPHIC       - GAUSS-KRUEGER'
      WRITE (6,*) '2   GAUSS-KRUEGER    - GEOGRAPHIC '
      WRITE (6,*)
      WRITE (6,*) '3   GEOGRAPHIC       - LAMBERT'
      WRITE (6,*) '4   LAMBERT          - GEOGRAPHIC'
      WRITE (6,*) '5   LAMBERT 1        - LAMBERT 2'
      WRITE (6,*)
      WRITE (6,*) '6   GEOGRAPHIC       - UTM'
      WRITE (6,*) '7   UTM              - GEOGRAPHIC'
      WRITE (6,*)
      WRITE (6,*) '8   GEOGRAPHIC       - GAUSS-BOAGA'
      WRITE (6,*) '9   GAUSS-BOAGA      - GEOGRAPHIC'
      WRITE (6,*)
      WRITE (6,*) '10  GEOGRAPHIC          - SWISS NATIONAL NET'
      WRITE (6,*) '11  SWISS NATIONAL NET  - GEOGRAPHIC'
      WRITE (6,*)
      WRITE (6,*) '12  NO TRANSFORMATION'
      WRITE (6,*)
C
      READ (5,*,ERR=1) N
      GOTO (100,200,300,900,400,500,600,700,800,950,960,970) N
      GOTO 1
C
C GEOGRAPHISCH - GAUSS-KRUEGER
C
100   CONTINUE
      IPAR(1)=1
      SUBTR='LBRH'
101   WRITE (6,*) 'BEZUGSMERIDIAN:'
      READ (5,*,ERR=101) IPAR(2)
      WRITE (15,102) IPAR(2)
102   FORMAT (' GEOGRAPHISCH - GAUSS-KRUEGER',/
     *        ' BEZUGSMERIDIAN :',I5)
      GOTO 1000
C
C GAUSS-KRUEGER - GEOGRAPHISCH
C
200   CONTINUE
      IPAR(1)=-1
      SUBTR='RHLB'
201   WRITE (6,*) 'BEZUGSMERIDIAN (KENNZIFFER):'
      READ (5,*,ERR=201) IPAR(2)
203   WRITE (6,*) 'NORDHALBKUGEL ODER SUEDHALBKUGEL (N/S):'
      READ (5,'(A)',ERR=203) T
      IF (T.NE.'N' .AND. T.NE.'S') GOTO 203
      IPAR(3)=1
      IF (T.EQ.'S') IPAR(3)=-1
      WRITE (15,202) IPAR(2)
202   FORMAT (' GAUSS-KRUEGER - GEOGRAPHISCH',/
     *       ,' BEZUGSMERIDIAN (KENNZIFFER) :',I5)
      IF (T.EQ.'N') THEN
       WRITE (15,*) ' NORDHALBKUGEL'
      ELSE
       WRITE (15,*) ' SUEDHALBKUGEL'
      ENDIF
      GOTO 1000
C
C GEOGRAPHISCH - LAMBERT
C
300   CONTINUE
      IPAR(1)=1
      SUBTR='LAMBERT'
301   write (6,*) ' '
      WRITE (6,*)'Enter PARA1,PARA2,MERI0,PARA0 (EGT=41 65 10 50):'
      READ (5,*,ERR=301) PAR(1),PAR(2),PAR(3),PAR(4)
      IF (PAR(1).GE.PAR(2)) GOTO 301
303   write (6,*) ' '
      Write (6,*) ' Do you want to use the standard'
      WRITE (6,*) 'EGT parameters (Earth radius, etc.) (Y/N) ? '
      READ (5,'(A)',ERR=303) T
      IF (T.EQ.'Y' .or. T  .eq.'y') THEN
       IPAR(2)=1
      ELSE IF (T.EQ.'N'  .or. T .eq. 'n') THEN
       IPAR(2)=0
      ELSE IF (T.NE.'N' .AND. T .NE. 'Y' .and. T .ne. 'n' 
     +         .and. T .ne. 'y') THEN
       GOTO 303
      ENDIF
      WRITE (15,302) PAR(1),PAR(2),PAR(3),PAR(4)
302   FORMAT (' GEOGRAPHISCH - LAMBERT',/
     *       ,' PARALLEL 1:',F10.3,/
     *       ,' PARALLEL 2:',F10.3,/
     *       ,' MERIDIAN 0:',F10.3,/
     *       ,' PARALLEL 0:',F10.3,/)
      IF (IPAR(2).NE.0) WRITE (15,'(A)') ' EGT parameters'
C INITIALIZATION
      IPAR(1)=-IPAR(1)
      CALL TRC (DU,DU,DU,DU,IPAR,PAR,NPARM,SUBTR)
      IPAR(1)=-IPAR(1)
      GOTO 1000
C
C LAMBERT - GEOGRAPHISCH
C
900   CONTINUE
      IPAR(1)=1
      SUBTR='LAMBINV'
901   WRITE (6,*)'Enter PARA1,PARA2,MERI0,PARA0 (EGT= 41 65 10 50):'
      READ (5,*,ERR=901) PAR(1),PAR(2),PAR(3),PAR(4)
      IF (PAR(1).GE.PAR(2)) GOTO 901
903   write (6,*) ' '
      Write (6,*) ' Do you want to use the standard'
      WRITE (6,*) 'EGT parameters (Earth radius, etc.) (Y/N) ? '
      READ (5,'(A)',ERR=903) T
      IF (T.EQ.'Y'  .or.  T  .eq.  'y') THEN
       IPAR(2)=1
      ELSE IF (T.EQ.'N'  .or.  T .eq. 'n') THEN
       IPAR(2)=0
      ELSE IF (T.NE.'N' .AND. T.NE.'Y'  .and.  T .ne. 'y'
     +          .and.  T .ne. 'n') THEN
       GOTO 903
      ENDIF
      WRITE (15,902) PAR(1),PAR(2),PAR(3),PAR(4)
902   FORMAT (' LAMBERT - GEOGRAPHISCH',/
     *       ,' PARALLEL 1:',F10.3,/
     *       ,' PARALLEL 2:',F10.3,/
     *       ,' MERIDIAN 0:',F10.3,/
     *       ,' PARALLEL 0:',F10.3,/)
      IF (IPAR(2).NE.0) WRITE (15,'(A)') ' EGT parameters'
C INITIALIZATION
      IPAR(1)=-IPAR(1)
      CALL TRC (DU,DU,DU,DU,IPAR,PAR,NPARM,SUBTR)
      IPAR(1)=-IPAR(1)
      GOTO 1000
C
C LAMBERT 1 - LAMBERT 2
C
400   CONTINUE
      IPAR(1)=2
      SUBTR='LAMBERT'
401   WRITE (6,*) 'LAMBERT 1  PARA1,PARA2,MERI0,PARA0 (EGT= 41 65
     + 10 50):'
      READ (5,*,ERR=401) PAR(1),PAR(2),PAR(3),PAR(4)
      IF (PAR(1).GE.PAR(2)) GOTO 401
402   WRITE (6,*) 'LAMBERT 2  PARA1,PARA2,MERI0,PARA0 (EGT= 41 65
     + 10 50):'
      READ (5,*,ERR=402) PAR(5),PAR(6),PAR(7),PAR(8)
      IF (PAR(5).GE.PAR(6)) GOTO 402
403   write (6,*) ' '
      Write (6,*) ' Do you want to use the standard'
      WRITE (6,*) 'EGT parameters Earth radius, etc.) (Y/N) ? '
      READ (5,'(A)',ERR=403) T
      IF (T.EQ.'Y' .or. T .eq. 'y') THEN
       IPAR(2)=1
      ELSE IF (T.EQ.'N'  .or. T .eq. 'n') THEN
       IPAR(2)=0
      ELSE IF (T.NE.'N' .AND. T.NE.'Y'  .and.  T .ne. 'n'
     +         .and. T  .ne. 'y') THEN
       GOTO 403
      ENDIF
      WRITE (15,412) PAR(1),PAR(2),PAR(3),PAR(4)
     *              ,PAR(5),PAR(6),PAR(7),PAR(8)
412   FORMAT (' LAMBERT 1 - LAMBERT 2',//
     *       ,' LAMBERT 1  PARALLEL 1:',F10.3,/
     *       ,'            PARALLEL 2:',F10.3,/
     *       ,'            MERIDIAN 0:',F10.3,/
     *       ,'            PARALLEL 0:',F10.3,//
     *       ,' LAMBERT 2  PARALLEL 1:',F10.3,/
     *       ,'            PARALLEL 2:',F10.3,/
     *       ,'            MERIDIAN 0:',F10.3,/
     *       ,'            PARALLEL 0:',F10.3,/)
      IF (IPAR(2).NE.0) WRITE (15,'(A)') ' EGT parameters'
C INITIALIZATION
      IPAR(1)=-IPAR(1)
      CALL TRC (DU,DU,DU,DU,IPAR,PAR,NPARM,SUBTR)
      IPAR(1)=-IPAR(1)
      GOTO 1000
C
C GEOGRAPHISCH - UTM
C
500   CONTINUE
      IPAR(1)=2
      SUBTR='LBEN'
501   WRITE (6,*) 'BEZUGSMERIDIAN:'
      READ (5,*,ERR=501) IPAR(2)
      WRITE (15,502) IPAR(2)
502   FORMAT (' GEOGRAPHISCH - UTM',/
     *        ' BEZUGSMERIDIAN :',I5)
      GOTO 1000
C
C UTM - GEOGRAPHISCH
C
600   CONTINUE
      IPAR(1)=-2
      SUBTR='ENLB'
601   WRITE (6,*) 'BEZUGSMERIDIAN (KENNZIFFER):'
      READ (5,*,ERR=601) IPAR(2)
603   WRITE (6,*) 'NORDHALBKUGEL ODER SUEDHALBKUGEL (N/S):'
      READ (5,'(A)',ERR=603) T
      IF (T.NE.'N' .AND. T.NE.'S') GOTO 603
      IPAR(3)=1
      IF (T.EQ.'S') IPAR(3)=-1
      WRITE (15,602) IPAR(2)
602   FORMAT (' UTM - GEOGRAPHISCH',/
     *       ,' BEZUGSMERIDIAN (KENNZIFFER) :',I5)
      IF (T.EQ.'N') THEN
       WRITE (15,*) ' NORDHALBKUGEL'
      ELSE
       WRITE (15,*) ' SUEDHALBKUGEL'
      ENDIF
      GOTO 1000
C
C GEOGRAPHISCH - GAUSS-BOAGA
C
700   CONTINUE
      IPAR(1)=1
      SUBTR='COORD(1)'
701   WRITE (6,*) 'WESTLICHER ODER OESTLICHER STREIFEN (W/O)?'
      READ (5,'(A)',ERR=701) T
      IF (T.NE.'W' .AND. T.NE.'O') GOTO 701
      IPAR(2)=1
      IF (T.EQ.'O') IPAR(2)=2
C
702   WRITE (6,*) 'BEZOGEN AUF MONTE MARIO ODER GREENWICH (M/G)?'
      READ (5,'(A)',ERR=702) T
      IF (T.NE.'M' .AND. T.NE.'G') GOTO 702
      IPAR(3)=1
      IF (T.EQ.'G') IPAR(3)=2
C
      WRITE (15,*) 'GEOGRAPHISCH - GAUSS-BOAGA'
      IF (IPAR(2).EQ.1) WRITE (15,*) 'WESTLICHER STREIFEN'
      IF (IPAR(2).EQ.2) WRITE (15,*) 'OESTLICHER STREIFEN'
      IF (IPAR(3).EQ.1) WRITE (15,*) 'BEZOFGEN AUF MONTE MARIO'
      IF (IPAR(3).EQ.2) WRITE (15,*) 'BEZOGEN AUF GREENWICH'
      GOTO 1000
C
C GAUSS-BOAGA - GEOGRAPHISCH
C
800   CONTINUE
      IPAR(1)=2
      SUBTR='COORD(2)'
C
802   WRITE (6,*) 'BEZOGEN AUF MONTE MARIO ODER GREENWICH (M/G)?'
      READ (5,'(A)',ERR=802) T
      IF (T.NE.'M' .AND. T.NE.'G') GOTO 802
      IPAR(3)=1
      IF (T.EQ.'G') IPAR(3)=2
C
      WRITE (15,*) 'GAUSS-BOAGA - GEOGRAPHISCH'
      IF (IPAR(3).EQ.1) WRITE (15,*) 'BEZOFGEN AUF MONTE MARIO'
      IF (IPAR(3).EQ.2) WRITE (15,*) 'BEZOGEN AUF GREENWICH'
      GOTO 1000
C
C GEOGRAPHISCH - SCHWEIZER LANDESKOORDINATEN
C
950   WRITE (15,*) 'GEOGRAPHISCH - SCHWEIZER LANDESKOORDINATEN'
      SUBTR='ELLEB'
      GOTO 1000
C
C SCHWEIZER LANDESKOORDINATEN - GEOGRAPHISCH
C
960   WRITE (15,*) 'SCHWEIZER LANDESKOORDINATEN - GEOGRAPHISCH'
      SUBTR='EBELL'
      GOTO 1000
C
C OHNE TRANSFORMATION
C
970   WRITE (15,*) 'OHNE TRANSFORMATION'
      SUBTR='OHNE'
C
1000  RETURN
      END
